@@ -2,7 +2,7 @@
 
     X Windows PCMCIA device control program
 
-    cardinfo.c 1.27 1999/07/20 16:02:22
+    cardinfo.c 1.31 2000/06/07 22:01:20
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,8 +15,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
@@ -75,7 +86,7 @@ static socket_info_t st[MAX_SOCK];
 static FL_OBJECT *event_log;
 
 static char *pidfile = "/var/run/cardmgr.pid";
-static char *stabfile = "/var/run/stab";
+static char *stabfile;
 
 /*====================================================================*/
 
@@ -430,6 +441,12 @@ int main(int argc, char *argv[])
 	fprintf(stderr, "cardinfo must be setuid root\n");
 	exit(EXIT_FAILURE);
     }
+
+    if (access("/var/lib/pcmcia", R_OK) == 0) {
+	stabfile = "/var/lib/pcmcia/stab";
+    } else {
+	stabfile = "/var/run/stab";
+    }
     
     major = lookup_dev("pcmcia");
     if (major < 0) {
