@@ -2,7 +2,7 @@
 
     PCMCIA Card Manager daemon
 
-    cardmgr.c 1.125 1999/07/20 16:02:23
+    cardmgr.c 1.138 2000/06/07 22:01:20
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,8 +15,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
@@ -57,7 +68,7 @@ typedef struct socket_info_t {
     int			state;
     card_info_t		*card;
     bind_info_t		*bind[MAX_BINDINGS];
-    char		*mtd[2*CISTPL_MAX_DEVICES];
+    mtd_ident_t		*mtd[2*CISTPL_MAX_DEVICES];
 } socket_info_t;
 
 #define SOCKET_PRESENT	0x01
@@ -104,7 +115,7 @@ static char *modpath = NULL;
 #endif
 
 /* Default path for socket info table */
-static char *stabfile = "/var/run/stab";
+static char *stabfile;
 
 /* If set, don't generate beeps when cards are inserted */
 static int be_quiet = 0;
@@ -115,9 +126,6 @@ static int do_modprobe = 0;
 /* If set, configure already inserted cards, then exit */
 static int one_pass = 0;
 
-/* most recent signal */
-static int caught_signal = 0;
-
 /* Extra message logging? */
 static int verbose = 0;
 
@@ -279,7 +287,7 @@ static void write_pid(void)
     FILE *f;
     f = fopen(pidfile, "w");
     if (f == NULL)
-	syslog(LOG_INFO, "could not open %s: %m", pidfile);
+	syslog(LOG_WARNING, "could not open %s: %m", pidfile);
     else {
 	fprintf(f, "%d\n", getpid());
 	fclose(f);
@@ -295,12 +303,12 @@ static void write_stab(void)
 
     f = fopen(stabfile, "w");
     if (f == NULL) {
-	syslog(LOG_INFO, "fopen(stabfile) failed: %m");
+	syslog(LOG_WARNING, "fopen(stabfile) failed: %m");
 	return;
     }
 #ifndef __BEOS__
     if (flock(fileno(f), LOCK_EX) != 0) {
-	syslog(LOG_INFO, "flock(stabfile) failed: %m");
+	syslog(LOG_ERR, "flock(stabfile) failed: %m");
 	return;
     }
 #endif
@@ -405,9 +413,13 @@ static card_info_t *lookup_card(int ns)
     
     /* Try to read VERS_1, MANFID tuples */
     if (has_cis) {
+	/* rule of thumb: cards with no FUNCID, but with common memory
+	   device geometry information, are probably memory cards */
 	if (get_tuple(ns, CISTPL_FUNCID, &arg) == 0)
 	    memcpy(&funcid, &arg.tuple_parse.parse.funcid,
 		   sizeof(funcid));
+	else if (get_tuple(ns, CISTPL_DEVICE_GEO, &arg) == 0)
+	    funcid.func = CISTPL_FUNCID_MEMORY;
 	if (get_tuple(ns, CISTPL_MANFID, &arg) == 0)
 	    memcpy(&manfid, &arg.tuple_parse.parse.manfid,
 		   sizeof(manfid));
@@ -504,34 +516,60 @@ static card_info_t *lookup_card(int ns)
 
 static void load_config(void)
 {
-    if (chdir(configpath) != 0)
-	syslog(LOG_INFO, "chdir to %s failed: %m", configpath);
+    if (chdir(configpath) != 0) {
+	syslog(LOG_ERR, "chdir to %s failed: %m", configpath);
+	exit(EXIT_FAILURE);
+    }
     if (parse_configfile("config") != 0)
 	exit(EXIT_FAILURE);
     if (root_device == NULL)
-	syslog(LOG_INFO, "no device drivers defined");
+	syslog(LOG_WARNING, "no device drivers defined");
     if ((root_card == NULL) && (root_func == NULL))
-	syslog(LOG_INFO, "no cards defined");
+	syslog(LOG_WARNING, "no cards defined");
 }
 
 /*====================================================================*/
 
 static void free_card(card_info_t *card)
 {
-    int i;
-    free(card->name);
-    switch(card->ident_type) {
-    case VERS_1_IDENT:
-	for (i = 0; i < card->id.vers.ns; i++)
-	    free(card->id.vers.pi[i]);
-	break;
-    case TUPLE_IDENT:
-	free(card->id.tuple.info);
-	break;
-    default:
+    if (card && (--card->refs == 0)) {
+	int i;
+	free(card->name);
+	switch(card->ident_type) {
+	case VERS_1_IDENT:
+	    for (i = 0; i < card->id.vers.ns; i++)
+		free(card->id.vers.pi[i]);
 	break;
+	case TUPLE_IDENT:
+	    free(card->id.tuple.info);
+	    break;
+	default:
+	    break;
+	}
+	free(card);
+    }
+}
+
+static void free_device(device_info_t *dev)
+{
+    if (dev && (--dev->refs == 0)) {
+	int i;
+	for (i = 0; i < dev->modules; i++) {
+	    free(dev->module[i]);
+	    if (dev->opts[i]) free(dev->opts[i]);
+	}
+	if (dev->class) free(dev->class);
+	free(dev);
+    }
+}
+
+static void free_mtd(mtd_ident_t *mtd)
+{
+    if (mtd && (--mtd->refs == 0)) {
+	free(mtd->name);
+	free(mtd->module);
+	free(mtd);
     }
-    free(card);
 }
 
 static void free_config(void)
@@ -544,14 +582,8 @@ static void free_config(void)
     
     while (root_device != NULL) {
 	device_info_t *dev = root_device;
-	int i;
 	root_device = root_device->next;
-	for (i = 0; i < dev->modules; i++) {
-	    free(dev->module[i]);
-	    if (dev->opts[i]) free(dev->opts[i]);
-	}
-	if (dev->class) free(dev->class);
-	free(dev);
+	free_device(dev);
     }
 
     while (root_card != NULL) {
@@ -570,9 +602,7 @@ static void free_config(void)
     while (root_mtd != NULL) {
 	mtd_ident_t *mtd = root_mtd;
 	root_mtd = root_mtd->next;
-	free(mtd->name);
-	free(mtd->module);
-	free(mtd);
+	free_mtd(mtd);
     }
     default_mtd = NULL;
 }
@@ -638,12 +668,40 @@ typedef struct module_list_t {
 
 static module_list_t *module_list = NULL;
 
-static int install_module(char *mod, char *opts)
+static int try_insmod(char *mod, char *opts)
 {
     char path[128], cmd[128];
+    if (strchr(mod, '/') != NULL)
+	sprintf(path, "%s/%s.o", modpath, mod);
+    else
+	sprintf(path, "%s/pcmcia/%s.o", modpath, mod);
+    if (access(path, R_OK) != 0) {
+	syslog(LOG_INFO, "module %s not available", path);
+	return -1;
+    }
+    sprintf(cmd, "insmod %s", path);
+    if (opts) {
+	strcat(cmd, " ");
+	strcat(cmd, opts);
+    }
+    return execute("insmod", cmd);
+}
+
+static int try_modprobe(char *mod, char *opts)
+{
+    char cmd[128], *s = strrchr(mod, '/');
+    sprintf(cmd, "modprobe %s", (s) ? s+1 : mod);
+    if (opts) {
+	strcat(cmd, " ");
+	strcat(cmd, opts);
+    }
+    return execute("modprobe", cmd);
+}
+
+static void install_module(char *mod, char *opts)
+{
     module_list_t *ml;
-    int ret;
-    
+
     for (ml = module_list; ml != NULL; ml = ml->next)
 	if (strcmp(mod, ml->mod) == 0) break;
     if (ml == NULL) {
@@ -655,30 +713,34 @@ static int install_module(char *mod, char *opts)
     }
     ml->usage++;
     if (ml->usage != 1)
-	return 0;
-    if (!do_modprobe) {
-	if (strchr(mod, '/') != NULL)
-	    sprintf(path, "%s/%s.o", modpath, mod);
-	else
-	    sprintf(path, "%s/pcmcia/%s.o", modpath, mod);
-	if (access(path, R_OK) != 0) {
-	    syslog(LOG_INFO, "module %s not available", path);
-	    return -1;
+	return;
+
+#ifdef __linux__
+    if (access("/proc/bus/pccard/drivers", R_OK) == 0) {
+	FILE *f = fopen("/proc/bus/pccard/drivers", "r");
+	if (f) {
+	    char a[61], s[33];
+	    while (fgets(a, 60, f)) {
+		int is_kernel;
+		sscanf(a, "%s %d", s, &is_kernel);
+		if (strcmp(s, mod) != 0) continue;
+		/* If it isn't a module, we won't try to rmmod */
+		ml->usage += is_kernel;
+		fclose(f);
+		return;
+	    }
+	    fclose(f);
 	}
-	sprintf(cmd, "insmod %s", path);
-	if (opts) {
-	    strcat(cmd, " ");
-	    strcat(cmd, opts);
-	}
-	ret = execute("insmod", cmd);
-	if (ret == 0) return 0;
     }
-    sprintf(cmd, "modprobe %s", mod);
-    if (opts) {
-	strcat(cmd, " ");
-	strcat(cmd, opts);
+#endif
+
+    if (do_modprobe) {
+	if (try_modprobe(mod, opts) != 0)
+	    try_insmod(mod, opts);
+    } else {
+	if (try_insmod(mod, opts) != 0)
+	    try_modprobe(mod, opts);
     }
-    return execute("modprobe", cmd);
 }
 
 static void remove_module(char *mod)
@@ -693,10 +755,7 @@ static void remove_module(char *mod)
 	if (ml->usage == 0) {
 	    /* Strip off leading path names */
 	    s = strrchr(mod, '/');
-	    if (s == NULL)
-		s = mod;
-	    else
-		s++;
+	    s = (s) ? s+1 : mod;
 	    sprintf(cmd, do_modprobe ? "modprobe -r %s" : "rmmod %s", s);
 	    execute(do_modprobe ? "modprobe" : "rmmod", cmd);
 	}
@@ -769,17 +828,18 @@ static void bind_mtd(int sn)
 	    if (mtd) {
 		/* Have we seen this MTD before? */
 		for (i = 0; i < nr; i++)
-		    if (strcmp(s->mtd[i], mtd->module) == 0) break;
+		    if (s->mtd[i] == mtd) break;
 		if (i == nr) {
 		    install_module(mtd->module, mtd->opts);
-		    s->mtd[nr] = mtd->module;
+		    s->mtd[nr] = mtd;
+		    mtd->refs++;
 		    nr++;
 		}
 		syslog(LOG_INFO, "  %s memory region at 0x%lx: %s",
 		       attr ? "Attribute" : "Common", region.CardOffset,
 		       mtd->name);
 		/* Bind MTD to this region */
-		strcpy(mtd_info.dev_info, s->mtd[i]);
+		strcpy(mtd_info.dev_info, s->mtd[i]->module);
 		mtd_info.Attributes = region.Attributes;
 		mtd_info.CardOffset = region.CardOffset;
 		if (ioctl(s->fd, DS_BIND_MTD, &mtd_info) != 0) {
@@ -795,7 +855,7 @@ static void bind_mtd(int sn)
     
     /* Now bind each unique MTD as a normal client of this socket */
     for (i = 0; i < nr; i++) {
-	strcpy(bind.dev_info, s->mtd[i]);
+	strcpy(bind.dev_info, s->mtd[i]->module);
 	if (ioctl(s->fd, DS_BIND_REQUEST, &bind) != 0)
 	    syslog(LOG_INFO, "bind MTD '%s' to socket %d failed: %m",
 		   (char *)bind.dev_info, sn);
@@ -809,12 +869,12 @@ static void update_cis(socket_info_t *s)
     cisdump_t cis;
     FILE *f = fopen(s->card->cis_file, "r");
     if (f == NULL)
-	syslog(LOG_INFO, "could not open '%s': %m", s->card->cis_file);
+	syslog(LOG_ERR, "could not open '%s': %m", s->card->cis_file);
     else {
 	cis.Length = fread(cis.Data, 1, CISTPL_MAX_CIS_SIZE, f);
 	fclose(f);
 	if (ioctl(s->fd, DS_REPLACE_CIS, &cis) != 0)
-	    syslog(LOG_INFO, "could not replace CIS: %m");
+	    syslog(LOG_ERR, "could not replace CIS: %m");
     }
 }
 
@@ -838,6 +898,7 @@ static void do_insert(int sn)
     if (card == s->card)
 	return;
     s->card = card;
+    card->refs++;
     if (card->cis_file) update_cis(s);
 
     dev = card->device;
@@ -852,6 +913,7 @@ static void do_insert(int sn)
 #ifdef __linux__
     /* Install kernel modules */
     for (i = 0; i < card->bindings; i++) {
+	dev[i]->refs++;
 	for (j = 0; j < dev[i]->modules; j++)
 	    install_module(dev[i]->module[j], dev[i]->opts[j]);
     }
@@ -882,7 +944,7 @@ static void do_insert(int sn)
 	republish_driver(dev[i]->module[0]);
 #endif
 
-	for (j = 0; j < 10; j++) {
+	for (ret = j = 0; j < 10; j++) {
 	    ret = ioctl(s->fd, DS_GET_DEVICE_INFO, bind);
 	    if ((ret == 0) || (errno != EAGAIN))
 		break;
@@ -988,25 +1050,28 @@ static void do_remove(int sn)
     }
     for (i = 0; (s->mtd[i] != NULL); i++) {
 	bind_info_t b;
-	strcpy(b.dev_info, s->mtd[i]);
+	strcpy(b.dev_info, s->mtd[i]->module);
 	if (ioctl(s->fd, DS_UNBIND_REQUEST, &b) != 0)
 	    syslog(LOG_INFO, "unbind MTD '%s' from socket %d failed: %m",
-		   (char *)s->mtd[i], sn);
+		   s->mtd[i]->module, sn);
     }
 
     /* remove kernel modules in inverse order */
     for (i = 0; i < card->bindings; i++) {
 	for (j = dev[i]->modules-1; j >= 0; j--)
 	    remove_module(dev[i]->module[j]);
+	free_device(dev[i]);
     }
     /* Remove any MTD's bound to this socket */
     for (i = 0; (s->mtd[i] != NULL); i++) {
-	remove_module(s->mtd[i]);
+	remove_module(s->mtd[i]->module);
+	free_mtd(s->mtd[i]);
 	s->mtd[i] = NULL;
     }
 
 done:
     beep(BEEP_TIME, BEEP_OK);
+    free_card(card);
     s->card = NULL;
     write_stab();
 }
@@ -1138,10 +1203,9 @@ static void fork_now(void)
 	exit(0);
     }
     if (ret == -1)
-	syslog(LOG_INFO, "forking: %m");
+	syslog(LOG_ERR, "forking: %m");
     if (setsid() < 0)
-	syslog(LOG_INFO, "detaching from tty: %m");
-    write_pid();
+	syslog(LOG_ERR, "detaching from tty: %m");
 }    
 
 static void done(void)
@@ -1153,6 +1217,11 @@ static void done(void)
     }
 }
 
+/*====================================================================*/
+
+/* most recent signal */
+static int caught_signal = 0;
+
 static void catch_signal(int sig)
 {
     caught_signal = sig;
@@ -1197,32 +1266,32 @@ static int init_sockets(void)
     major = lookup_dev("pcmcia");
     if (major < 0) {
 	if (major == -ENODEV)
-	    syslog(LOG_INFO, "no pcmcia driver in /proc/devices");
+	    syslog(LOG_ERR, "no pcmcia driver in /proc/devices");
 	else
-	    syslog(LOG_INFO, "could not open /proc/devices: %m");
+	    syslog(LOG_ERR, "could not open /proc/devices: %m");
 	exit(EXIT_FAILURE);
     }
 #endif
-    for (i = 0; i < MAX_SOCKS; i++) {
+    for (fd = -1, i = 0; i < MAX_SOCKS; i++) {
 	fd = open_sock(i, S_IFCHR|S_IREAD|S_IWRITE);
 	if (fd < 0) break;
 	socket[i].fd = fd;
 	socket[i].state = 0;
     }
     if ((fd < 0) && (errno != ENODEV) && (errno != ENOENT))
-	syslog(LOG_INFO, "open_sock(socket %d) failed: %m", i);
+	syslog(LOG_ERR, "open_sock(socket %d) failed: %m", i);
     sockets = i;
     if (sockets == 0) {
-	syslog(LOG_INFO, "no sockets found!");
+	syslog(LOG_ERR, "no sockets found!");
 	return -1;
     } else
 	syslog(LOG_INFO, "watching %d sockets", sockets);
 
     if (ioctl(socket[0].fd, DS_GET_CARD_SERVICES_INFO, &serv) == 0) {
 	if (serv.Revision != CS_RELEASE_CODE)
-	    syslog(LOG_INFO, "Card Services release does not match!");
+	    syslog(LOG_INFO, "Card Services release does not match");
     } else {
-	syslog(LOG_INFO, "could not get CS revision info!");
+	syslog(LOG_ERR, "could not get CS revision info!");
 	return -1;
     }
     adjust_resources();
@@ -1239,6 +1308,12 @@ int main(int argc, char *argv[])
     struct timeval tv;
     fd_set fds;
 
+    if (access("/var/lib/pcmcia", R_OK) == 0) {
+	stabfile = "/var/lib/pcmcia/stab";
+    } else {
+	stabfile = "/var/run/stab";
+    }
+
     errflg = 0;
     while ((optch = getopt(argc, argv, "Vqdvofc:m:p:s:")) != -1) {
 	switch (optch) {
@@ -1248,8 +1323,6 @@ int main(int argc, char *argv[])
 	    break;
 	case 'q':
 	    be_quiet = 1; break;
-	case 'd':
-	    do_modprobe = 1; break;
 	case 'v':
 	    verbose = 1; break;
 	case 'o':
@@ -1259,6 +1332,8 @@ int main(int argc, char *argv[])
 	case 'c':
 	    configpath = strdup(optarg); break;
 #ifdef __linux__
+	case 'd':
+	    do_modprobe = 1; break;
 	case 'm':
 	    modpath = strdup(optarg); break;
 #endif
@@ -1281,9 +1356,7 @@ int main(int argc, char *argv[])
     openlog("cardmgr", LOG_PID|LOG_PERROR, LOG_DAEMON);
 #else
     openlog("cardmgr", LOG_PID|LOG_CONS, LOG_DAEMON);
-#endif
-
-#ifndef DEBUG
+    close(0); close(1); close(2);
     if (!delay_fork && !one_pass)
 	fork_now();
 #endif
@@ -1299,17 +1372,17 @@ int main(int argc, char *argv[])
 	else {
 	    struct utsname utsname;
 	    if (uname(&utsname) != 0) {
-		syslog(LOG_INFO, "uname(): %m");
+		syslog(LOG_ERR, "uname(): %m");
 		exit(EXIT_FAILURE);
 	    }
-	    modpath = (char *)malloc(32);
+	    modpath = (char *)malloc(strlen(utsname.release)+14);
 	    sprintf(modpath, "/lib/modules/%s", utsname.release);
 	}
     }
-    if (access(modpath, X_OK) != 0) {
+    if (access(modpath, X_OK) != 0)
 	syslog(LOG_INFO, "cannot access %s: %m", modpath);
-	exit(EXIT_FAILURE);
-    }
+    /* We default to using modprobe if it is available */
+    do_modprobe |= (access("/sbin/modprobe", X_OK) == 0);
 #endif /* __linux__ */
     
     load_config();
@@ -1318,18 +1391,19 @@ int main(int argc, char *argv[])
 	exit(EXIT_FAILURE);
 
     /* If we've gotten this far, then clean up pid and stab at exit */
+    write_pid();
     write_stab();
     cleanup_files = 1;
     
     if (signal(SIGHUP, catch_signal) == SIG_ERR)
-	syslog(LOG_INFO, "signal(SIGHUP): %m");
+	syslog(LOG_ERR, "signal(SIGHUP): %m");
     if (signal(SIGTERM, catch_signal) == SIG_ERR)
-	syslog(LOG_INFO, "signal(SIGTERM): %m");
+	syslog(LOG_ERR, "signal(SIGTERM): %m");
     if (signal(SIGINT, catch_signal) == SIG_ERR)
-	syslog(LOG_INFO, "signal(SIGINT): %m");
+	syslog(LOG_ERR, "signal(SIGINT): %m");
 #ifdef SIGPWR
     if (signal(SIGPWR, catch_signal) == SIG_ERR)
-	syslog(LOG_INFO, "signal(SIGPWR): %m");
+	syslog(LOG_ERR, "signal(SIGPWR): %m");
 #endif
     
     for (i = max_fd = 0; i < sockets; i++)
@@ -1352,7 +1426,7 @@ int main(int argc, char *argv[])
 	    if (errno == EINTR) {
 		handle_signal();
 	    } else {
-		syslog(LOG_INFO, "select(): %m");
+		syslog(LOG_ERR, "select(): %m");
 		exit(EXIT_FAILURE);
 	    }
 	}
@@ -1401,8 +1475,10 @@ int main(int argc, char *argv[])
 
 	if (one_pass)
 	    exit(EXIT_SUCCESS);
-	if (delay_fork)
+	if (delay_fork) {
 	    fork_now();
+	    write_pid();
+	}
 	
     } /* repeat */
     return 0;
