@@ -1,5 +1,5 @@
 /*
- * cardmgr.h 1.30 1999/07/20 16:02:24
+ * cardmgr.h 1.34 1999/12/29 00:57:15
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.1 (the "License"); you may not use this file except in
@@ -12,8 +12,19 @@
  * limitations under the License. 
  *
  * The initial developer of the original code is David A. Hinds
- * <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
- * are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+ * <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+ * are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+ *
+ * Alternatively, the contents of this file may be used under the
+ * terms of the GNU Public License version 2 (the "GPL"), in which
+ * case the provisions of the GPL are applicable instead of the
+ * above.  If you wish to allow the use of your version of this file
+ * only under the terms of the GPL and not to allow others to use
+ * your version of this file under the MPL, indicate your decision by
+ * deleting the provisions above and replace them with the notice and
+ * other provisions required by the GPL.  If you do not delete the
+ * provisions above, a recipient may use your version of this file
+ * under either the MPL or the GPL.
  */
 
 #define MAX_SOCKS	8
@@ -52,6 +63,7 @@ typedef struct device_info_t {
     char		*module[MAX_MODULES];
     char		*opts[MAX_MODULES];
     char		*class;
+    int			refs;
     struct device_info_t *next;
 } device_info_t;
 
@@ -59,7 +71,7 @@ typedef struct card_info_t {
     char		*name;
     enum {
 	VERS_1_IDENT=1, MANFID_IDENT, TUPLE_IDENT, FUNC_IDENT,
-	BLANK_IDENT, CARDBUS_IDENT
+	BLANK_IDENT, PCI_IDENT
     } ident_type;
     union {
 	vers_ident_t	vers;
@@ -71,6 +83,7 @@ typedef struct card_info_t {
     device_info_t	*device[MAX_BINDINGS];
     int			dev_fn[MAX_BINDINGS];
     char		*cis_file;
+    int			refs;
     struct card_info_t	*next;
 } card_info_t;
 
@@ -81,6 +94,7 @@ typedef struct mtd_ident_t {
     } mtd_type;
     int			dtype, jedec_mfr, jedec_info;
     char		*module, *opts;
+    int			refs;
     struct mtd_ident_t	*next;
 } mtd_ident_t;
     
