@@ -2,7 +2,7 @@
 
     Utility to select the transceiver type for a network device
 
-    ifport.c 1.9 1999/07/20 16:02:25
+    ifport.c 1.12 1999/10/25 20:00:14
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,8 +15,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
@@ -31,8 +42,7 @@
 #include <unistd.h>
 
 char *if_names[] = {
-    "Auto", "10baseT", "10base2", "AUI",
-    "100baseTx", "100baseFx", "100baseT4"
+    "Auto", "10baseT", "10base2", "AUI", "100baseT"
 };
 #define NR_NAMES (sizeof(if_names)/sizeof(char *))
 
@@ -56,7 +66,7 @@ static int sockets_open(void)
 void usage(char *s)
 {
     fprintf(stderr, "usage: %s interface "
-	    "[auto|10baseT|10base2|aui|100baseTx/Fx/T4|##]\n", s);
+	    "[auto|10baseT|10base2|aui|100baseT|##]\n", s);
     exit(1);
 }
 
