@@ -9,7 +9,7 @@
     The exit code is 0 if any host is using the specified interface,
     and 1 if the interface is not in use (just like fuser).
     
-    ifuser.c 1.9 1999/07/20 16:02:25
+    ifuser.c 1.14 1999/10/25 20:00:14
 
     1998/10/24: Regis "HPReg" Duchesne <regis@via.ecp.fr>
       . Added network names (/etc/networks) management
@@ -27,8 +27,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     Portions created by Regis "HPReg" Duchesne are Copyright (C) 1998
     Regis "HPReg" Duchesne.  All Rights Reserved.
 
@@ -40,6 +51,7 @@
 #include <string.h>
 #include <netdb.h>
 #include <errno.h>
+#include <ctype.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
@@ -107,10 +119,10 @@ int main(int argc, char *argv[])
     
     do {
 	fgets(s, 128, f);
-    } while (strstr(s, "Destination") == NULL);
+    } while (!feof(f) && !isdigit(s[0]));
     
     tail = &tbl;
-    while (fgets(s, 128, f) != NULL) {
+    do {
 	r = malloc(sizeof(route_t));
 	if (r == NULL) {
 	    fprintf(stderr, "%s: out of memory\n", argv[0]);
@@ -121,7 +133,7 @@ int main(int argc, char *argv[])
 	resolv_name(mask, &r->mask);
 	r->match = (strcmp(iface, dev) == 0);
 	*tail = r; tail = &(r->next);
-    }
+    } while (fgets(s, 128, f) != NULL);
     *tail = NULL;
     pclose(f);
 
