@@ -2,7 +2,7 @@
 
     PCMCIA controller probe
 
-    probe.c 1.41 1999/07/20 16:02:27
+    probe.c 1.51 2000/05/22 20:21:38
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,8 +15,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
@@ -28,21 +39,7 @@
 #include <errno.h>
 #include <fcntl.h>
 
-#ifdef __GLIBC__
-#include <sys/io.h>
-#else
-#include <asm/io.h>
-#endif
-
 #include <pcmcia/config.h>
-#include "i82365.h"
-#include "cirrus.h"
-#include "vg468.h"
-#include "tcic.h"
-
-static int i365_base = 0x03e0;
-
-typedef u_short ioaddr_t;
 
 /*====================================================================*/
 
@@ -55,7 +52,7 @@ typedef struct {
 } pci_id_t;
 
 pci_id_t pci_id[] = {
-    { 0x1013, 0x6729, "Cirrus Logic CL 6729", "Cirrus PD6729" },
+    { 0x1013, 0x1100, "Cirrus Logic CL 6729", "Cirrus PD6729" },
     { 0x1013, 0x1110, "Cirrus Logic PD 6832", "Cirrus PD6832" },
     { 0x10b3, 0xb106, "SMC 34C90", "SMC 34C90" },
     { 0x1180, 0x0465, "Ricoh RL5C465", "Ricoh RL5C465" },
@@ -74,13 +71,18 @@ pci_id_t pci_id[] = {
     { 0x104c, 0xac1f, "Texas Instruments PCI1251B", "TI 1251B" },
     { 0x104c, 0xac1b, "Texas Instruments PCI1450", "TI 1450" },
     { 0x104c, 0xac1c, "Texas Instruments PCI1225", "TI 1225" },
+    { 0x104c, 0xac1e, "Texas Instruments PCI1211", "TI 1211" },
+    { 0x104c, 0xac50, "Texas Instruments PCI1410", "TI 1410" },
+    { 0x104c, 0xac51, "Texas Instruments PCI1420", "TI 1420" },
     { 0x1217, 0x6729, "O2 Micro 6729", "O2Micro OZ6729" },
     { 0x1217, 0x673a, "O2 Micro 6730", "O2Micro OZ6730" },
-    { 0x1217, 0x6832, "O2 Micro 6832", "O2Micro OZ6832" },
-    { 0x1217, 0x6836, "O2 Micro 6836", "O2Micro OZ6836" },
+    { 0x1217, 0x6832, "O2 Micro 6832/6833", "O2Micro OZ6832/OZ6833" },
+    { 0x1217, 0x6836, "O2 Micro 6836/6860", "O2Micro OZ6836/OZ6860" },
+    { 0x1217, 0x6872, "O2 Micro 6812", "O2Micro OZ6812" },
     { 0x1179, 0x0603, "Toshiba ToPIC95-A", "Toshiba ToPIC95-A" },
     { 0x1179, 0x060a, "Toshiba ToPIC95-B", "Toshiba ToPIC95-B" },
     { 0x1179, 0x060f, "Toshiba ToPIC97", "Toshiba ToPIC97" },
+    { 0x1179, 0x0617, "Toshiba ToPIC100", "Toshiba ToPIC100" },
     { 0x119b, 0x1221, "Omega Micro 82C092G", "Omega Micro 82C092G" },
     { 0x8086, 0x1221, "Intel 82092AA", "Intel 82092AA" }
 };
@@ -151,6 +153,21 @@ static int pci_probe(int verbose, int module)
 
 /*====================================================================*/
 
+#ifdef CONFIG_ISA
+
+#ifdef __GLIBC__
+#include <sys/io.h>
+#else
+#include <asm/io.h>
+#endif
+typedef u_short ioaddr_t;
+
+#include "i82365.h"
+#include "cirrus.h"
+#include "vg468.h"
+
+static ioaddr_t i365_base = 0x03e0;
+
 static u_char i365_get(u_short sock, u_short reg)
 {
     u_char val = I365_REG(sock, reg);
@@ -178,8 +195,6 @@ static void i365_bclr(u_short sock, u_short reg, u_char mask)
     i365_set(sock, reg, d);
 }
 
-/*====================================================================*/
-
 int i365_probe(int verbose, int module)
 {
     int val, sock, done;
@@ -265,9 +280,17 @@ int i365_probe(int verbose, int module)
     return 0;
     
 } /* i365_probe */
-  
+
+#endif /* CONFIG_ISA */
+
 /*====================================================================*/
 
+#ifdef CONFIG_ISA
+
+#include "tcic.h"
+
+static ioaddr_t tcic_base = TCIC_BASE;
+
 static u_char tcic_getb(ioaddr_t base, u_char reg)
 {
     u_char val = inb(base+reg);
@@ -391,6 +414,8 @@ int tcic_probe(int verbose, int module, ioaddr_t base)
     
 } /* tcic_probe */
 
+#endif /* CONFIG_ISA */
+
 /*====================================================================*/
 
 int main(int argc, char *argv[])
@@ -398,13 +423,14 @@ int main(int argc, char *argv[])
     int optch, errflg;
     extern char *optarg;
     int verbose = 0, module = 0;
-    ioaddr_t tcic_base = TCIC_BASE;
     
     errflg = 0;
     while ((optch = getopt(argc, argv, "t:vxm")) != -1) {
 	switch (optch) {
+#ifdef CONFIG_ISA
 	case 't':
 	    tcic_base = strtoul(optarg, NULL, 0); break;
+#endif
 	case 'v':
 	    verbose = 1; break;
 	case 'm':
@@ -421,13 +447,13 @@ int main(int argc, char *argv[])
 #ifdef CONFIG_PCI
     if (pci_probe(verbose, module) == 0)
 	exit(EXIT_SUCCESS);
-    else
 #endif
+#ifdef CONFIG_ISA
     if (i365_probe(verbose, module) == 0)
 	exit(EXIT_SUCCESS);
     else if (tcic_probe(verbose, module, tcic_base) == 0)
 	exit(EXIT_SUCCESS);
-    else
-	exit(EXIT_FAILURE);
+#endif
+    exit(EXIT_FAILURE);
     return 0;
 }
