@@ -2,7 +2,7 @@
 
     Utility to look up information about SCSI devices
 
-    scsi_info.c 1.12 1999/07/20 16:04:26
+    scsi_info.c 1.15 2000/04/03 19:44:50
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,8 +15,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
@@ -75,7 +86,7 @@ static int get_host_no(int host)
 int main(int argc, char *argv[])
 {
     int i, fd, host, channel, id, lun;
-    u_long arg[2];
+    u_int arg[2];
     char match[128], s[128], vendor[9], model[17], rev[5];
     FILE *f;
 
@@ -101,8 +112,15 @@ int main(int argc, char *argv[])
     channel = (arg[0] >> 16) & 0xff;
     host = ((arg[0] >> 24) & 0xff);
 
+#ifdef SCSI_IOCTL_GET_BUS_NUMBER
+    if (ioctl(fd, SCSI_IOCTL_GET_BUS_NUMBER, arg) == 0)
+	host = arg[0];
+    else
+#endif
+	host = get_host_no(host);
+
     sprintf(match, "Host: scsi%d Channel: %02x Id: %02x Lun: %02x\n",
-	    get_host_no(host), channel, id, lun);
+	    host, channel, id, lun);
 
     printf("SCSI_ID=\"%d,%d,%d\"\n", channel, id, lun);
     
