@@ -2,7 +2,7 @@
 
 	Written 1993-1998 by
 	Donald Becker, becker@cesdis.gsfc.nasa.gov, (driver core) and
-	David Hinds, dhinds@allegro.stanford.edu (derived from his PC card code).
+	David Hinds, dhinds@pcmcia.sourceforge.org (from his PC card code).
 
 	This software may be used and distributed according to the terms of
 	the GNU Public License, incorporated herein by reference.
@@ -74,9 +74,10 @@ earlier 3Com products.
 #include <pcmcia/config.h>
 #include <pcmcia/k_compat.h>
 
+#include <linux/module.h>
 #include <linux/kernel.h>
+#include <linux/init.h>
 #include <linux/sched.h>
-#include <linux/ptrace.h>
 #include <linux/malloc.h>
 #include <linux/string.h>
 #include <linux/timer.h>
@@ -106,9 +107,6 @@ earlier 3Com products.
 MODULE_PARM(irq_mask, "i");
 MODULE_PARM(irq_list, "1-4i");
 MODULE_PARM(max_interrupt_work, "i");
-MODULE_PARM(use_fifo_buffer, "i");
-MODULE_PARM(use_memory_ops, "i");
-MODULE_PARM(no_wait, "i");
 MODULE_PARM(full_duplex, "i");
 
 /* Now-standard PC card module parameters. */
@@ -121,14 +119,7 @@ static int irq_list[4] = { -1 };
 /* Maximum events (Rx packets, etc.) to handle at each interrupt. */
 static int max_interrupt_work = 32;
 
-/* Performance features: best left disabled. */
-/* Set to buffer all Tx/RxFIFO accesses. */
-static int use_fifo_buffer = 0;
-/* Set iff memory ops are faster than I/O ops. */
-static int use_memory_ops = 0;
-/* Set iff disabling the WAIT signal is reliable and faster. */
-static int no_wait = 0;
-/* Advertise full duplex modes? */
+/* Force full duplex modes? */
 static int full_duplex = 0;
 
 /* To minimize the size of the driver source and make the driver more
@@ -205,9 +196,11 @@ enum Window4 {		/* Window 4: Xcvr/media bits. */
 #define MEDIA_TP	0x00C0	/* Enable link beat and jabber for 10baseT. */
 
 struct el3_private {
+	dev_link_t link;
+	struct net_device dev;
 	dev_node_t node;
 	struct net_device_stats stats;
-	u16 advertising;					/* NWay media advertisement */
+	u16 advertising, partner;			/* NWay media advertisement */
 	unsigned char phys[2];				/* MII device addresses. */
 	unsigned int
 	  autoselect:1, default_media:3;	/* Read from the EEPROM/Wn3_Config. */
@@ -240,25 +233,24 @@ static void tc574_release(u_long arg);
 static int tc574_event(event_t event, int priority,
 					   event_callback_args_t *args);
 
-static void mdio_sync(int ioaddr, int bits);
-static int mdio_read(int ioaddr, int phy_id, int location);
-static void mdio_write(int ioaddr, int phy_id, int location, int value);
-static ushort read_eeprom(int ioaddr, int index);
-static void wait_for_completion(struct device *dev, int cmd);
+static void mdio_sync(ioaddr_t ioaddr, int bits);
+static int mdio_read(ioaddr_t ioaddr, int phy_id, int location);
+static void mdio_write(ioaddr_t ioaddr, int phy_id, int location, int value);
+static u_short read_eeprom(ioaddr_t ioaddr, int index);
+static void wait_for_completion(struct net_device *dev, int cmd);
 
-static void tc574_reset(struct device *dev);
+static void tc574_reset(struct net_device *dev);
 static void media_check(u_long arg);
-static int el3_open(struct device *dev);
-static int el3_start_xmit(struct sk_buff *skb, struct device *dev);
-static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs);
-static void update_stats(int addr, struct device *dev);
-static struct net_device_stats *el3_get_stats(struct device *dev);
-static int el3_rx(struct device *dev, int worklimit);
-static int el3_close(struct device *dev);
-#ifdef HAVE_PRIVATE_IOCTL
-static int private_ioctl(struct device *dev, struct ifreq *rq, int cmd);
-#endif
-static void set_rx_mode(struct device *dev);
+static int el3_open(struct net_device *dev);
+static int el3_start_xmit(struct sk_buff *skb, struct net_device *dev);
+static void el3_interrupt(int irq, void *dev_id, struct pt_regs *regs);
+static void update_stats(struct net_device *dev);
+static struct net_device_stats *el3_get_stats(struct net_device *dev);
+static int el3_rx(struct net_device *dev, int worklimit);
+static int el3_close(struct net_device *dev);
+static void el3_tx_timeout(struct net_device *dev);
+static int el3_ioctl(struct net_device *dev, struct ifreq *rq, int cmd);
+static void set_rx_mode(struct net_device *dev);
 
 static dev_info_t dev_info = "3c574_cs";
 
@@ -287,16 +279,6 @@ static void cs_error(client_handle_t handle, int func, int ret)
 #endif
 }
 
-/*
-   We never need to do anything when a tc574 device is "initialized"
-   by the net software, because we only register already-found cards.
-*/
-
-static int tc574_init(struct device *dev)
-{
-	return 0;
-}
-
 /*
 	tc574_attach() creates an "instance" of the driver, allocating
 	local data structures for one device.  The device is registered
@@ -305,22 +287,26 @@ static int tc574_init(struct device *dev)
 
 static dev_link_t *tc574_attach(void)
 {
+	struct el3_private *lp;
 	client_reg_t client_reg;
 	dev_link_t *link;
-	struct device *dev;
+	struct net_device *dev;
 	int i, ret;
 
 	DEBUG(0, "3c574_attach()\n");
 	flush_stale_links();
 
 	/* Create the PC card device object. */
-	link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
-	memset(link, 0, sizeof(struct dev_link_t));
+	lp = kmalloc(sizeof(*lp), GFP_KERNEL);
+	if (!lp) return NULL;
+	memset(lp, 0, sizeof(*lp));
+	link = &lp->link; dev = &lp->dev;
+	link->priv = dev->priv = link->irq.Instance = lp;
+	
 	link->release.function = &tc574_release;
 	link->release.data = (u_long)link;
 	link->io.NumPorts1 = 32;
 	link->io.Attributes1 = IO_DATA_PATH_WIDTH_16;
-	link->io.IOAddrLines = 5;
 	link->irq.Attributes = IRQ_TYPE_EXCLUSIVE | IRQ_HANDLE_PRESENT;
 	link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
 	if (irq_list[0] == -1)
@@ -335,31 +321,18 @@ static dev_link_t *tc574_attach(void)
 	link->conf.ConfigIndex = 1;
 	link->conf.Present = PRESENT_OPTION;
 
-	/* Create the network device object. */
-	dev = kmalloc(sizeof(struct device), GFP_KERNEL);
-	memset(dev, 0, sizeof(struct device));
-
-	/* Make up a Odie-specific data structure. */
-	dev->priv = kmalloc(sizeof(struct el3_private), GFP_KERNEL);
-	memset(dev->priv, 0, sizeof(struct el3_private));
-
 	/* The EL3-specific entries in the device structure. */
 	dev->hard_start_xmit = &el3_start_xmit;
 	dev->get_stats = &el3_get_stats;
-#ifdef HAVE_PRIVATE_IOCTL
-	dev->do_ioctl = &private_ioctl;
-#endif
+	dev->do_ioctl = &el3_ioctl;
 	dev->set_multicast_list = &set_rx_mode;
 	ether_setup(dev);
-	dev->name = ((struct el3_private *)dev->priv)->node.dev_name;
-	dev->init = &tc574_init;
+	init_dev_name(dev, lp->node);
 	dev->open = &el3_open;
 	dev->stop = &el3_close;
-	dev->tbusy = 1;
-
-	link->priv = dev;
-#if CS_RELEASE_CODE > 0x2911
-	link->irq.Instance = dev;
+#ifdef HAVE_NETIF_QUEUE
+	dev->tx_timeout = el3_tx_timeout;
+	dev->watchdog_timeo = TX_TIMEOUT;
 #endif
 
 	/* Register with Card Services */
@@ -395,8 +368,8 @@ static dev_link_t *tc574_attach(void)
 
 static void tc574_detach(dev_link_t *link)
 {
+	struct el3_private *lp = link->priv;
 	dev_link_t **linkp;
-	long flags;
 
 	DEBUG(0, "3c574_detach(0x%p)\n", link);
 
@@ -406,14 +379,7 @@ static void tc574_detach(dev_link_t *link)
 	if (*linkp == NULL)
 	return;
 
-	save_flags(flags);
-	cli();
-	if (link->state & DEV_RELEASE_PENDING) {
-		del_timer(&link->release);
-		link->state &= ~DEV_RELEASE_PENDING;
-	}
-	restore_flags(flags);
-
+	del_timer(&link->release);
 	if (link->state & DEV_CONFIG) {
 		tc574_release((u_long)link);
 		if (link->state & DEV_STALE_CONFIG) {
@@ -427,15 +393,9 @@ static void tc574_detach(dev_link_t *link)
 
 	/* Unlink device structure, free bits */
 	*linkp = link->next;
-	if (link->priv) {
-		struct device *dev = link->priv;
-		if (link->dev != NULL)
-			unregister_netdev(dev);
-		if (dev->priv)
-			kfree_s(dev->priv, sizeof(struct el3_private));
-		kfree_s(link->priv, sizeof(struct device));
-	}
-	kfree_s(link, sizeof(struct dev_link_t));
+	if (link->dev)
+		unregister_netdev(&lp->dev);
+	kfree(lp);
 
 } /* tc574_detach */
 
@@ -450,19 +410,17 @@ while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
 
 static void tc574_config(dev_link_t *link)
 {
-	client_handle_t handle;
-	struct device *dev;
-	struct el3_private *lp;
+	client_handle_t handle = link->handle;
+	struct el3_private *lp = link->priv;
+	struct net_device *dev = &lp->dev;
 	tuple_t tuple;
 	cisparse_t parse;
 	u_short buf[32];
 	int last_fn, last_ret, i, j;
-	int ioaddr;
+	ioaddr_t ioaddr;
 	u16 *phys_addr;
 	char *cardname;
 
-	handle = link->handle;
-	dev = link->priv;
 	phys_addr = (u16 *)dev->dev_addr;
 
 	DEBUG(0, "3c574_config(0x%p)\n", link);
@@ -481,6 +439,7 @@ static void tc574_config(dev_link_t *link)
 	/* Configure card */
 	link->state |= DEV_CONFIG;
 
+	link->io.IOAddrLines = 16;
 	for (i = j = 0; j < 0x400; j += 0x20) {
 		link->io.BasePort1 = j ^ 0x300;
 		i = CardServices(RequestIO, link->handle, &link->io);
@@ -493,39 +452,18 @@ static void tc574_config(dev_link_t *link)
 	CS_CHECK(RequestIRQ, link->handle, &link->irq);
 	CS_CHECK(RequestConfiguration, link->handle, &link->conf);
 
-	dev->mem_start = 0;
-	if (use_memory_ops) {
-		win_req_t req;
-		memreq_t mem;
-		req.Attributes = WIN_DATA_WIDTH_16 | WIN_MEMORY_TYPE_CM |
-			WIN_ENABLE | WIN_USE_WAIT;
-		req.Base = 0;
-		req.Size = 0x1000;
-		req.AccessSpeed = 0;
-		link->win = (window_handle_t)link->handle;
-		i = CardServices(RequestWindow, &link->win, &req);
-		if (i == CS_SUCCESS) {
-			mem.Page = mem.CardOffset = 0;
-			CardServices(MapMemPage, link->win, &mem);
-			dev->mem_start = (long)(ioremap(req.Base, 0x1000)) + 0x800;
-		} else
-			cs_error(link->handle, RequestWindow, i);
-	}
-
 	dev->irq = link->irq.AssignedIRQ;
 	dev->base_addr = link->io.BasePort1;
 
-	dev->tbusy = 0;
 	if (register_netdev(dev) != 0) {
 		printk(KERN_NOTICE "3c574_cs: register_netdev() failed\n");
 		goto failed;
 	}
 
-	link->state &= ~DEV_CONFIG_PENDING;
-
 	ioaddr = dev->base_addr;
-	lp = (struct el3_private *)dev->priv;
+	copy_dev_name(lp->node, dev);
 	link->dev = &lp->node;
+	link->state &= ~DEV_CONFIG_PENDING;
 
 	/* The 3c574 normally uses an EEPROM for configuration info, including
 	   the hardware address.  The future products may include a modem chip
@@ -559,10 +497,6 @@ static void tc574_config(dev_link_t *link)
 	for (i = 0; i < 6; i++)
 		printk("%02X%s", dev->dev_addr[i], ((i<5) ? ":" : ".\n"));
 
-	if (dev->mem_start)
-		printk(KERN_INFO"  Acceleration window at memory base %#lx.\n",
-			   dev->mem_start);
-
 	{
 		u_char mcr, *ram_split[] = {"5:3", "3:1", "1:1", "3:5"};
 		union wn3_config config;
@@ -611,9 +545,9 @@ static void tc574_config(dev_link_t *link)
 		i = mdio_read(ioaddr, lp->phys[0], 16) | 0x40;
 		mdio_write(ioaddr, lp->phys[0], 16, i);
 		lp->advertising = mdio_read(ioaddr, lp->phys[0], 4);
-		if (!full_duplex) {
-			/* Only advertise the HD media types. */
-			lp->advertising &= 0x00af;
+		if (full_duplex) {
+			/* Only advertise the FD media types. */
+			lp->advertising &= ~0x02a0;
 			mdio_write(ioaddr, lp->phys[0], 4, lp->advertising);
 		}
 	}
@@ -637,7 +571,6 @@ failed:
 static void tc574_release(u_long arg)
 {
 	dev_link_t *link = (dev_link_t *)arg;
-	struct device *dev = link->priv;
 
 	DEBUG(0, "3c574_release(0x%p)\n", link);
 
@@ -651,12 +584,8 @@ static void tc574_release(u_long arg)
 	CardServices(ReleaseConfiguration, link->handle);
 	CardServices(ReleaseIO, link->handle, &link->io);
 	CardServices(ReleaseIRQ, link->handle, &link->irq);
-	if (link->win) {
-		iounmap((void *)(dev->mem_start - 0x800));
-		CardServices(ReleaseWindow, link->win);
-	}
 
-	link->state &= ~(DEV_CONFIG | DEV_RELEASE_PENDING);
+	link->state &= ~DEV_CONFIG;
 
 } /* tc574_release */
 
@@ -671,7 +600,8 @@ static int tc574_event(event_t event, int priority,
 					   event_callback_args_t *args)
 {
 	dev_link_t *link = args->client_data;
-	struct device *dev = link->priv;
+	struct el3_private *lp = link->priv;
+	struct net_device *dev = &lp->dev;
 
 	DEBUG(1, "3c574_event(0x%06x)\n", event);
 
@@ -679,9 +609,8 @@ static int tc574_event(event_t event, int priority,
 	case CS_EVENT_CARD_REMOVAL:
 		link->state &= ~DEV_PRESENT;
 		if (link->state & DEV_CONFIG) {
-			dev->tbusy = 1; dev->start = 0;
-			link->release.expires = RUN_AT(HZ/20);
-			add_timer(&link->release);
+			netif_device_detach(dev);
+			mod_timer(&link->release, jiffies + HZ/20);
 		}
 		break;
 	case CS_EVENT_CARD_INSERTION:
@@ -693,9 +622,8 @@ static int tc574_event(event_t event, int priority,
 		/* Fall through... */
 	case CS_EVENT_RESET_PHYSICAL:
 		if (link->state & DEV_CONFIG) {
-			if (link->open) {
-				dev->tbusy = 1; dev->start = 0;
-			}
+			if (link->open)
+				netif_device_detach(dev);
 			CardServices(ReleaseConfiguration, link->handle);
 		}
 		break;
@@ -707,7 +635,7 @@ static int tc574_event(event_t event, int priority,
 			CardServices(RequestConfiguration, link->handle, &link->conf);
 			if (link->open) {
 				tc574_reset(dev);
-				dev->tbusy = 0; dev->start = 1;
+				netif_device_attach(dev);
 			}
 		}
 		break;
@@ -715,9 +643,9 @@ static int tc574_event(event_t event, int priority,
 	return 0;
 } /* tc574_event */
 
-static void dump_status(struct device *dev)
+static void dump_status(struct net_device *dev)
 {
-	int ioaddr = dev->base_addr;
+	ioaddr_t ioaddr = dev->base_addr;
 	EL3WINDOW(1);
     printk(KERN_INFO "  irq status %04x, rx status %04x, tx status "
 		   "%02x, tx free %04x\n", inw(ioaddr+EL3_STATUS),
@@ -733,7 +661,7 @@ static void dump_status(struct device *dev)
 /*
   Use this for commands that may take time to finish
 */
-static void wait_for_completion(struct device *dev, int cmd)
+static void wait_for_completion(struct net_device *dev, int cmd)
 {
     int i = 1500;
     outw(cmd, dev->base_addr + EL3_CMD);
@@ -747,7 +675,7 @@ static void wait_for_completion(struct device *dev, int cmd)
 /* Read a word from the EEPROM using the regular EEPROM access register.
    Assume that we are in register window zero.
  */
-static ushort read_eeprom(int ioaddr, int index)
+static u_short read_eeprom(ioaddr_t ioaddr, int index)
 {
 	int timer;
 	outw(EEPROM_Read + index, ioaddr + Wn0EepromCmd);
@@ -775,7 +703,7 @@ static ushort read_eeprom(int ioaddr, int index)
 
 /* Generate the preamble required for initial synchronization and
    a few older transceivers. */
-static void mdio_sync(int ioaddr, int bits)
+static void mdio_sync(ioaddr_t ioaddr, int bits)
 {
 	int mdio_addr = ioaddr + Wn4_PhysicalMgmt;
 
@@ -786,7 +714,7 @@ static void mdio_sync(int ioaddr, int bits)
 	}
 }
 
-static int mdio_read(int ioaddr, int phy_id, int location)
+static int mdio_read(ioaddr_t ioaddr, int phy_id, int location)
 {
 	int i;
 	int read_cmd = (0xf6 << 10) | (phy_id << 5) | location;
@@ -811,7 +739,7 @@ static int mdio_read(int ioaddr, int phy_id, int location)
 	return (retval>>1) & 0xffff;
 }
 
-static void mdio_write(int ioaddr, int phy_id, int location, int value)
+static void mdio_write(ioaddr_t ioaddr, int phy_id, int location, int value)
 {
 	int write_cmd = 0x50020000 | (phy_id << 23) | (location << 18) | value;
 	int mdio_addr = ioaddr + Wn4_PhysicalMgmt;
@@ -836,24 +764,13 @@ static void mdio_write(int ioaddr, int phy_id, int location, int value)
 }
 
 /* Reset and restore all of the 3c574 registers. */
-static void tc574_reset(struct device *dev)
+static void tc574_reset(struct net_device *dev)
 {
 	struct el3_private *lp = (struct el3_private *)dev->priv;
 	int i, ioaddr = dev->base_addr;
 
 	wait_for_completion(dev, TotalReset|0x10);
- 
-	/* Set the PIO ctrl bits in the PC card LAN COR using Runner window 1. */
-	if (dev->mem_start || no_wait) {
-		u8 lan_cor;
-		outw(1<<11, ioaddr + RunnerRdCtrl);
-		lan_cor = inw(ioaddr) & ~0x30;
-		if (dev->mem_start)		/* Iff use_memory_ops worked! */
-			lan_cor |= 0x10;
-		if (no_wait)
-			lan_cor |= 0x20;
-		outw(lan_cor, ioaddr);
-	}
+
 	/* Clear any transactions in progress. */
 	outw(0, ioaddr + RunnerWrCtrl);
 	outw(0, ioaddr + RunnerRdCtrl);
@@ -865,11 +782,9 @@ static void tc574_reset(struct device *dev)
 	for (; i < 12; i+=2)
 		outw(0, ioaddr + i);
 
-	/* Set the full-duplex bit. */
-	EL3WINDOW(3);
-	outb((full_duplex ? 0x20 : 0) |
-		 (dev->mtu > 1500 ? 0x40 : 0), ioaddr + Wn3_MAC_Ctrl);
 	/* Reset config options */
+	EL3WINDOW(3);
+	outb((dev->mtu > 1500 ? 0x40 : 0), ioaddr + Wn3_MAC_Ctrl);
 	outl((lp->autoselect ? 0x01000000 : 0) | 0x0062001b,
 		 ioaddr + Wn3_Config);
 	
@@ -889,10 +804,10 @@ static void tc574_reset(struct device *dev)
 		inb(ioaddr + i);
 	inw(ioaddr + 10);
 	inw(ioaddr + 12);
-
 	EL3WINDOW(4);
-	/* New: On the Vortex/Odie we must also clear the BadSSD counter.. */
 	inb(ioaddr + 12);
+	inb(ioaddr + 13);
+
 	/* .. enable any extra statistics bits.. */
 	outw(0x0040, ioaddr + Wn4_NetDiag);
 	/* .. re-sync MII and re-fill what NWay is advertising. */
@@ -915,36 +830,35 @@ static void tc574_reset(struct device *dev)
 		 | AdapterFailure | RxEarly, ioaddr + EL3_CMD);
 }
 
-static int el3_open(struct device *dev)
+static int el3_open(struct net_device *dev)
 {
 	struct el3_private *lp = (struct el3_private *)dev->priv;
-	dev_link_t *link;
+	dev_link_t *link = &lp->link;
 
-	for (link = dev_list; link; link = link->next)
-		if (link->priv == dev) break;
 	if (!DEV_OK(link))
 		return -ENODEV;
 	
 	link->open++;
 	MOD_INC_USE_COUNT;
-	dev->interrupt = 0; dev->tbusy = 0; dev->start = 1;
+	netif_start_queue(dev);
+	netif_mark_up(dev);
 	
 	tc574_reset(dev);
 	lp->media.function = &media_check;
-	lp->media.data = (u_long)dev;
-	lp->media.expires = RUN_AT(HZ);
+	lp->media.data = (u_long)lp;
+	lp->media.expires = jiffies + HZ;
 	add_timer(&lp->media);
 	
 	DEBUG(2, "%s: opened, status %4.4x.\n",
 		  dev->name, inw(dev->base_addr + EL3_STATUS));
 	
-	return 0;					/* Always succeed */
+	return 0;
 }
 
-static void el3_tx_timeout(struct device *dev)
+static void el3_tx_timeout(struct net_device *dev)
 {
 	struct el3_private *lp = (struct el3_private *)dev->priv;
-	int ioaddr = dev->base_addr;
+	ioaddr_t ioaddr = dev->base_addr;
 	
 	printk(KERN_NOTICE "%s: Transmit timed out!\n", dev->name);
 	dump_status(dev);
@@ -953,13 +867,13 @@ static void el3_tx_timeout(struct device *dev)
 	/* Issue TX_RESET and TX_START commands. */
 	wait_for_completion(dev, TxReset);
 	outw(TxEnable, ioaddr + EL3_CMD);
-	dev->tbusy = 0;
+	netif_start_queue(dev);
 }
 
-static void pop_tx_status(struct device *dev)
+static void pop_tx_status(struct net_device *dev)
 {
     struct el3_private *lp = (struct el3_private *)dev->priv;
-    int ioaddr = dev->base_addr;
+    ioaddr_t ioaddr = dev->base_addr;
     int i;
     
     /* Clear the Tx status stack. */
@@ -979,55 +893,26 @@ static void pop_tx_status(struct device *dev)
     }
 }
 
-static int el3_start_xmit(struct sk_buff *skb, struct device *dev)
+static int el3_start_xmit(struct sk_buff *skb, struct net_device *dev)
 {
-	int ioaddr = dev->base_addr;
-	long flags = 0;
+	ioaddr_t ioaddr = dev->base_addr;
 
-	/* Transmitter timeout, serious problems. */
-	if (test_and_set_bit(0, (void*)&dev->tbusy) != 0) {
-		if (jiffies - dev->trans_start < TX_TIMEOUT)
-			return 1;
-		el3_tx_timeout(dev);
-	}
+	tx_timeout_check(dev, el3_tx_timeout);
+	skb_tx_check(dev, skb);
 
 	DEBUG(3, "%s: el3_start_xmit(length = %ld) called, "
 		  "status %4.4x.\n", dev->name, (long)skb->len,
 		  inw(ioaddr + EL3_STATUS));
 
-	if (use_fifo_buffer) {
-		/* Avoid other accesses to the chip while RunnerWrCtrl is non-zero. */
-		save_flags(flags);
-		cli();
-		outw((((skb->len + 7)>>2)<<1), ioaddr + RunnerWrCtrl);
-		DEBUG(0, "TxFree %x, tx length %x, RunnerWrCtrl is %4.4x.\n",
-			  inw(ioaddr+TxFree), skb->len, inw(ioaddr+RunnerWrCtrl));
-	}
+	outw(skb->len, ioaddr + TX_FIFO);
+	outw(0, ioaddr + TX_FIFO);
+	outsl_ns(ioaddr + TX_FIFO, skb->data, (skb->len+3)>>2);
 
-	/* Put out the doubleword header... */
-	/* ... and the packet rounded to a doubleword. */
-	if (dev->mem_start) {
-		writew(skb->len, (void *)dev->mem_start);
-		writew(0, (void *)dev->mem_start);
-		copy_to_pc((void*)dev->mem_start, skb->data, (skb->len+3)&~3);
-	} else {
-		outw(skb->len, ioaddr + TX_FIFO);
-		outw(0, ioaddr + TX_FIFO);
-		outsl_ns(ioaddr + TX_FIFO, skb->data, (skb->len+3)>>2);
-	}
-
-	if (use_fifo_buffer) {
-		DEBUG(0, " RunnerWr/RdCtrl is %4.4x/%4.4x, TxFree %x.\n",
-			  inw(ioaddr + RunnerWrCtrl), inw(ioaddr + RunnerRdCtrl),
-			  inw(ioaddr + TxFree));
-		restore_flags(flags);
-	}
-	
 	dev->trans_start = jiffies;
 
 	/* TxFree appears only in Window 1, not offset 0x1c. */
 	if (inw(ioaddr + TxFree) > 1536) {
-		dev->tbusy = 0;
+		netif_start_queue(dev);
 	} else
 		/* Interrupt us when the FIFO has room for max-sized packet. 
 		   The threshold is in units of dwords. */
@@ -1040,31 +925,24 @@ static int el3_start_xmit(struct sk_buff *skb, struct device *dev)
 }
 
 /* The EL3 interrupt handler. */
-static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
+static void el3_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 {
-	struct device *dev = (struct device *)DEV_ID;
-	struct el3_private *lp;
-	int ioaddr, status;
+	struct el3_private *lp = dev_id;
+	struct net_device *dev = &lp->dev;
+	ioaddr_t ioaddr, status;
 	int work_budget = max_interrupt_work;
 
-	if ((dev == NULL) || !dev->start)
+	if (!netif_device_present(dev))
 		return;
-	lp = (struct el3_private *)dev->priv;
 	ioaddr = dev->base_addr;
 
-#ifdef PCMCIA_DEBUG
-	if (test_and_set_bit(0, (void*)&dev->interrupt)) {
-		printk(KERN_NOTICE "%s: re-entering the interrupt handler.\n",
-			   dev->name);
-		return;
-	}
 	DEBUG(3, "%s: interrupt, status %4.4x.\n",
 		  dev->name, inw(ioaddr + EL3_STATUS));
-#endif
 
 	while ((status = inw(ioaddr + EL3_STATUS)) &
 		   (IntLatch | RxComplete | RxEarly | StatsFull)) {
-		if ((dev->start == 0) || ((status & 0xe000) != 0x2000)) {
+		if (!netif_device_present(dev) ||
+			((status & 0xe000) != 0x2000)) {
 			DEBUG(1, "%s: Interrupt from dead card\n", dev->name);
 			break;
 		}
@@ -1076,8 +954,7 @@ static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 			DEBUG(3, "  TX room bit was handled.\n");
 			/* There's room in the FIFO for a full-sized packet. */
 			outw(AckIntr | TxAvailable, ioaddr + EL3_CMD);
-			dev->tbusy = 0;
-			mark_bh(NET_BH);
+			netif_wake_queue(dev);
 		}
 
 		if (status & TxComplete)
@@ -1086,7 +963,7 @@ static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 		if (status & (AdapterFailure | RxEarly | StatsFull)) {
 			/* Handle all uncommon interrupts. */
 			if (status & StatsFull)
-				update_stats(ioaddr, dev);
+				update_stats(dev);
 			if (status & RxEarly) {
 				work_budget = el3_rx(dev, work_budget);
 				outw(AckIntr | RxEarly, ioaddr + EL3_CMD);
@@ -1114,8 +991,8 @@ static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 		}
 
 		if (--work_budget < 0) {
-			printk(KERN_NOTICE "%s: Too much work in interrupt, "
-				   "status %4.4x.\n", dev->name, status);
+			DEBUG(0, "%s: Too much work in interrupt, "
+				  "status %4.4x.\n", dev->name, status);
 			/* Clear all interrupts */
 			outw(AckIntr | 0xFF, ioaddr + EL3_CMD);
 			break;
@@ -1124,11 +1001,8 @@ static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 		outw(AckIntr | IntReq | IntLatch, ioaddr + EL3_CMD);
 	}
 
-#ifdef PCMCIA_DEBUG
 	DEBUG(3, "%s: exiting interrupt, status %4.4x.\n",
 		  dev->name, inw(ioaddr + EL3_STATUS));
-	dev->interrupt = 0;
-#endif
 	return;
 }
 
@@ -1139,13 +1013,14 @@ static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 */
 static void media_check(u_long arg)
 {
-    struct device *dev = (struct device *)(arg);
-    struct el3_private *lp = (struct el3_private *)dev->priv;
-    int ioaddr = dev->base_addr;
+    struct el3_private *lp = (struct el3_private *)arg;
+    struct net_device *dev = &lp->dev;
+    ioaddr_t ioaddr = dev->base_addr;
     u_long flags;
 	u_short /* cable, */ media, partner;
-	
-    if (dev->start == 0) goto reschedule;
+
+	if (!netif_device_present(dev))
+		goto reschedule;
 	
     /* Check for pending interrupt with expired latency timer: with
        this, we can limp along even if the interrupt is blocked */
@@ -1153,12 +1028,12 @@ static void media_check(u_long arg)
 		(inb(ioaddr + Timer) == 0xff)) {
 		if (!lp->fast_poll)
 			printk(KERN_INFO "%s: interrupt(s) dropped!\n", dev->name);
-		el3_interrupt IRQ(dev->irq, dev, NULL);
+		el3_interrupt(dev->irq, lp, NULL);
 		lp->fast_poll = HZ;
     }
     if (lp->fast_poll) {
 		lp->fast_poll--;
-		lp->media.expires = RUN_AT(2);
+		lp->media.expires = jiffies + 2;
 		add_timer(&lp->media);
 		return;
     }
@@ -1187,11 +1062,13 @@ static void media_check(u_long arg)
 			printk(KERN_INFO "%s: %s link beat\n", dev->name,
 				   (lp->media_status & 0x0004) ? "lost" : "found");
 		if ((media ^ lp->media_status) & 0x0020) {
+			lp->partner = 0;
 			if (lp->media_status & 0x0020) {
 				printk(KERN_INFO "%s: autonegotiation restarted\n",
 					   dev->name);
 			} else if (partner) {
 				partner &= lp->advertising;
+				lp->partner = partner;
 				printk(KERN_INFO "%s: autonegotiation complete: "
 					   "%sbaseT-%cD selected\n", dev->name,
 					   ((partner & 0x0180) ? "100" : "10"),
@@ -1200,6 +1077,12 @@ static void media_check(u_long arg)
 				printk(KERN_INFO "%s: link partner did not autonegotiate\n",
 					   dev->name);
 			}
+
+			EL3WINDOW(3);
+			outb((partner & 0x0140 ? 0x20 : 0) |
+				 (dev->mtu > 1500 ? 0x40 : 0), ioaddr + Wn3_MAC_Ctrl);
+			EL3WINDOW(1);
+
 		}
 		if (media & 0x0010)
 			printk(KERN_INFO "%s: remote fault detected\n",
@@ -1210,16 +1093,16 @@ static void media_check(u_long arg)
 	}
 
 reschedule:
-    lp->media.expires = RUN_AT(HZ);
+    lp->media.expires = jiffies + HZ;
     add_timer(&lp->media);
 }
 
-static struct net_device_stats *el3_get_stats(struct device *dev)
+static struct net_device_stats *el3_get_stats(struct net_device *dev)
 {
 	struct el3_private *lp = (struct el3_private *)dev->priv;
 
-	if (dev->start)
-		update_stats(dev->base_addr, dev);
+	if (netif_device_present(dev))
+		update_stats(dev);
 	return &lp->stats;
 }
 
@@ -1227,10 +1110,11 @@ static struct net_device_stats *el3_get_stats(struct device *dev)
 	Suprisingly this need not be run single-threaded, but it effectively is.
 	The counters clear when read, so the adds must merely be atomic.
  */
-static void update_stats(int ioaddr, struct device *dev)
+static void update_stats(struct net_device *dev)
 {
 	struct el3_private *lp = (struct el3_private *)dev->priv;
-	u8 upper_cnt;
+	ioaddr_t ioaddr = dev->base_addr;
+	u8 rx, tx, up;
 
 	DEBUG(2, "%s: updating the statistics.\n", dev->name);
 
@@ -1243,33 +1127,31 @@ static void update_stats(int ioaddr, struct device *dev)
 	lp->stats.tx_carrier_errors 	+= inb(ioaddr + 0);
 	lp->stats.tx_heartbeat_errors	+= inb(ioaddr + 1);
 	/* Multiple collisions. */	   	inb(ioaddr + 2);
-	lp->stats.collisions		+= inb(ioaddr + 3);
-	lp->stats.tx_window_errors	+= inb(ioaddr + 4);
-	lp->stats.rx_fifo_errors	+= inb(ioaddr + 5);
-	lp->stats.tx_packets		+= inb(ioaddr + 6);
-	upper_cnt = inb(ioaddr + 9);
-	lp->stats.tx_packets		+= (upper_cnt&0x30) << 4;
-	/* Rx packets   */			inb(ioaddr + 7);
-	/* Tx deferrals */			inb(ioaddr + 8);
-#if (LINUX_VERSION_CODE >= VERSION(2,1,25))
-	lp->stats.rx_bytes += inw(ioaddr + 10);
-	lp->stats.tx_bytes += inw(ioaddr + 12);
-#else
-	inw(ioaddr + 10);
-	inw(ioaddr + 12);
-#endif
+	lp->stats.collisions			+= inb(ioaddr + 3);
+	lp->stats.tx_window_errors		+= inb(ioaddr + 4);
+	lp->stats.rx_fifo_errors		+= inb(ioaddr + 5);
+	lp->stats.tx_packets			+= inb(ioaddr + 6);
+	up		 						 = inb(ioaddr + 9);
+	lp->stats.tx_packets			+= (up&0x30) << 4;
+	/* Rx packets   */				   inb(ioaddr + 7);
+	/* Tx deferrals */				   inb(ioaddr + 8);
+	rx		 						 = inw(ioaddr + 10);
+	tx								 = inw(ioaddr + 12);
 
-	/* With Vortex and later we must also clear the BadSSD counter. */
 	EL3WINDOW(4);
-	inb(ioaddr + 12);
+	/* BadSSD */					   inb(ioaddr + 12);
+	up								 = inb(ioaddr + 13);
+
+	add_rx_bytes(&lp->stats, rx + ((up & 0x0f) << 16));
+	add_tx_bytes(&lp->stats, tx + ((up & 0xf0) << 12));
 
 	EL3WINDOW(1);
 }
 
-static int el3_rx(struct device *dev, int worklimit)
+static int el3_rx(struct net_device *dev, int worklimit)
 {
 	struct el3_private *lp = (struct el3_private *)dev->priv;
-	int ioaddr = dev->base_addr;
+	ioaddr_t ioaddr = dev->base_addr;
 	short rx_status;
 	
 	DEBUG(3, "%s: in rx_packet(), status %4.4x, rx_status %4.4x.\n",
@@ -1280,12 +1162,12 @@ static int el3_rx(struct device *dev, int worklimit)
 			short error = rx_status & 0x3800;
 			lp->stats.rx_errors++;
 			switch (error) {
-			case 0x0000:		lp->stats.rx_over_errors++; break;
-			case 0x0800:		lp->stats.rx_length_errors++; break;
-			case 0x1000:		lp->stats.rx_frame_errors++; break;
-			case 0x1800:		lp->stats.rx_length_errors++; break;
-			case 0x2000:		lp->stats.rx_frame_errors++; break;
-			case 0x2800:		lp->stats.rx_crc_errors++; break;
+			case 0x0000:	lp->stats.rx_over_errors++; break;
+			case 0x0800:	lp->stats.rx_length_errors++; break;
+			case 0x1000:	lp->stats.rx_frame_errors++; break;
+			case 0x1800:	lp->stats.rx_length_errors++; break;
+			case 0x2000:	lp->stats.rx_frame_errors++; break;
+			case 0x2800:	lp->stats.rx_crc_errors++; break;
 			}
 		} else {
 			short pkt_len = rx_status & 0x7ff;
@@ -1299,28 +1181,9 @@ static int el3_rx(struct device *dev, int worklimit)
 				skb->dev = dev;
 				skb_reserve(skb, 2);
 
-				if (use_fifo_buffer) {
-					outw(((pkt_len+3)>>2)<<1, ioaddr + RunnerRdCtrl);
-					DEBUG(0,"Start Rx %x -- RunnerRdCtrl is %4.4x.\n",
-						  pkt_len, inw(ioaddr + RunnerRdCtrl));
-				}
-				
-				if (dev->mem_start) {
-					copy_from_pc(skb_put(skb, pkt_len),
-								 (void*)dev->mem_start, (pkt_len+3)&~3);
-				} else {
-					insl_ns(ioaddr+RX_FIFO, skb_put(skb, pkt_len),
-							((pkt_len+3)>>2));
-				}
+				insl_ns(ioaddr+RX_FIFO, skb_put(skb, pkt_len),
+						((pkt_len+3)>>2));
 
-				if (use_fifo_buffer) {
-					DEBUG(0," RunnerRdCtrl is now %4.4x.\n",
-						  inw(ioaddr + RunnerRdCtrl));
-					outw(0, ioaddr + RunnerRdCtrl);
-					DEBUG(0, " Rx packet with data %2.2x:%2.2x:%2.2x\n",
-						  skb->head[0], skb->head[1], skb->head[2]);
-				}
-				
 				skb->protocol = eth_type_trans(skb, dev);
 				netif_rx(skb);
 				lp->stats.rx_packets++;
@@ -1336,14 +1199,13 @@ static int el3_rx(struct device *dev, int worklimit)
 	return worklimit;
 }
 
-#ifdef HAVE_PRIVATE_IOCTL
 /* Provide ioctl() calls to examine the MII xcvr state. */
-static int private_ioctl(struct device *dev, struct ifreq *rq, int cmd)
+static int el3_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
 {
-	struct el3_private *vp = (struct el3_private *)dev->priv;
-	int ioaddr = dev->base_addr;
+	struct el3_private *lp = (struct el3_private *)dev->priv;
+	ioaddr_t ioaddr = dev->base_addr;
 	u16 *data = (u16 *)&rq->ifr_data;
-	int phy = vp->phys[0] & 0x1f;
+	int phy = lp->phys[0] & 0x1f;
 
 	DEBUG(2, "%s: In ioct(%-.6s, %#4.4x) %4.4x %4.4x %4.4x %4.4x.\n",
 		  dev->name, rq->ifr_ifrn.ifrn_name, cmd,
@@ -1371,7 +1233,7 @@ static int private_ioctl(struct device *dev, struct ifreq *rq, int cmd)
 			int saved_window;
 			long flags;
 
-			if (!suser())
+			if (!capable(CAP_NET_ADMIN))
 				return -EPERM;
 			save_flags(flags);
 			cli();
@@ -1386,7 +1248,6 @@ static int private_ioctl(struct device *dev, struct ifreq *rq, int cmd)
 		return -EOPNOTSUPP;
 	}
 }
-#endif  /* HAVE_PRIVATE_IOCTL */
 
 /* The Odie chip has a 64 bin multicast filter, but the bit layout is not
    documented.  Until it is we revert to receiving all multicast frames when
@@ -1396,9 +1257,9 @@ static int private_ioctl(struct device *dev, struct ifreq *rq, int cmd)
    typical PC card machines, so we omit the message.
    */
 
-static void set_rx_mode(struct device *dev)
+static void set_rx_mode(struct net_device *dev)
 {
-	int ioaddr = dev->base_addr;
+	ioaddr_t ioaddr = dev->base_addr;
 
 	if (dev->flags & IFF_PROMISC)
 		outw(SetRxFilter | RxStation | RxMulticast | RxBroadcast | RxProm,
@@ -1409,15 +1270,11 @@ static void set_rx_mode(struct device *dev)
 		outw(SetRxFilter | RxStation | RxBroadcast, ioaddr + EL3_CMD);
 }
 
-static int el3_close(struct device *dev)
+static int el3_close(struct net_device *dev)
 {
-	int ioaddr = dev->base_addr;
-	dev_link_t *link;
-
-	for (link = dev_list; link; link = link->next)
-		if (link->priv == dev) break;
-	if (link == NULL)
-		return -ENODEV;
+	ioaddr_t ioaddr = dev->base_addr;
+	struct el3_private *lp = dev->priv;
+	dev_link_t *link = &lp->link;
 
 	DEBUG(2, "%s: shutting down ethercard.\n", dev->name);
 	
@@ -1432,24 +1289,22 @@ static int el3_close(struct device *dev)
 		/* Note: Switching to window 0 may disable the IRQ. */
 		EL3WINDOW(0);
 		
-		update_stats(ioaddr, dev);
+		update_stats(dev);
 	}
 
 	link->open--;
-	dev->start = 0;
-	del_timer(&((struct el3_private *)dev->priv)->media);
-	if (link->state & DEV_STALE_CONFIG) {
-		link->release.expires = RUN_AT(HZ/20);
-		link->state |= DEV_RELEASE_PENDING;
-		add_timer(&link->release);
-	}
+	netif_stop_queue(dev);
+	netif_mark_down(dev);
+	del_timer(&lp->media);
+	if (link->state & DEV_STALE_CONFIG)
+		mod_timer(&link->release, jiffies + HZ/20);
 
 	MOD_DEC_USE_COUNT;
 
 	return 0;
 }
 
-int init_module(void)
+static int __init init_3c574_cs(void)
 {
 	servinfo_t serv;
 
@@ -1462,18 +1317,21 @@ int init_module(void)
 			   "does not match!\n");
 		return -1;
 	}
-	register_pcmcia_driver(&dev_info, &tc574_attach, &tc574_detach);
+	register_pccard_driver(&dev_info, &tc574_attach, &tc574_detach);
 	return 0;
 }
 
-void cleanup_module(void)
+static void __exit exit_3c574_cs(void)
 {
 	DEBUG(0, "3c574_cs: unloading\n");
-	unregister_pcmcia_driver(&dev_info);
+	unregister_pccard_driver(&dev_info);
 	while (dev_list != NULL)
 		tc574_detach(dev_list);
 }
 
+module_init(init_3c574_cs);
+module_exit(exit_3c574_cs);
+
 /*
  * Local variables:
  *  compile-command: "make 3c574_cs.o"
