@@ -2,9 +2,9 @@
 
     A PCMCIA ethernet driver for the 3com 3c589 card.
     
-    Copyright (C) 1998 David A. Hinds -- dhinds@hyper.stanford.edu
+    Copyright (C) 1999 David A. Hinds -- dhinds@pcmcia.sourceforge.org
 
-    3c589_cs.c 1.128 1999/07/30 15:17:50
+    3c589_cs.c 1.152 2000/05/25 03:11:12
 
     The network driver code is based on Donald Becker's 3c589 code:
     
@@ -20,6 +20,8 @@
 #include <pcmcia/config.h>
 #include <pcmcia/k_compat.h>
 
+#include <linux/module.h>
+#include <linux/init.h>
 #include <linux/kernel.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
@@ -100,13 +102,15 @@ enum RxFilter {
 #define TX_TIMEOUT	((400*HZ)/1000)
 
 struct el3_private {
-    dev_node_t node;
+    dev_link_t		link;
+    struct net_device	dev;
+    dev_node_t 		node;
     struct net_device_stats stats;
     /* For transceiver monitoring */
-    struct timer_list media;
-    u_short	media_status;
-    u_short	fast_poll;
-    u_long	last_irq;
+    struct timer_list	media;
+    u_short		media_status;
+    u_short		fast_poll;
+    u_long		last_irq;
 };
 
 static char *if_names[] = { "auto", "10baseT", "10base2", "AUI" };
@@ -116,7 +120,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"3c589_cs.c 1.128 1999/07/30 15:17:50 (David Hinds)";
+"3c589_cs.c 1.152 2000/05/25 03:11:12 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -143,19 +147,19 @@ static void tc589_release(u_long arg);
 static int tc589_event(event_t event, int priority,
 		       event_callback_args_t *args);
 
-static ushort read_eeprom(short ioaddr, int index);
-static void tc589_reset(struct device *dev);
+static u_short read_eeprom(ioaddr_t ioaddr, int index);
+static void tc589_reset(struct net_device *dev);
 static void media_check(u_long arg);
-static int el3_config(struct device *dev, struct ifmap *map);
-static int el3_open(struct device *dev);
-static int el3_start_xmit(struct sk_buff *skb, struct device *dev);
-static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs);
-static void update_stats(int addr, struct device *dev);
-static struct net_device_stats *el3_get_stats(struct device *dev);
-static int el3_rx(struct device *dev);
-static int el3_close(struct device *dev);
-
-static void set_multicast_list(struct device *dev);
+static int el3_config(struct net_device *dev, struct ifmap *map);
+static int el3_open(struct net_device *dev);
+static int el3_start_xmit(struct sk_buff *skb, struct net_device *dev);
+static void el3_interrupt(int irq, void *dev_id, struct pt_regs *regs);
+static void update_stats(struct net_device *dev);
+static struct net_device_stats *el3_get_stats(struct net_device *dev);
+static int el3_rx(struct net_device *dev);
+static int el3_close(struct net_device *dev);
+static void el3_tx_timeout(struct net_device *dev);
+static void set_multicast_list(struct net_device *dev);
 
 static dev_info_t dev_info = "3c589_cs";
 
@@ -190,18 +194,6 @@ static void cs_error(client_handle_t handle, int func, int ret)
     CardServices(ReportError, handle, &err);
 }
 
-/*======================================================================
-
-    We never need to do anything when a tc589 device is "initialized"
-    by the net software, because we only register already-found cards.
-    
-======================================================================*/
-
-static int tc589_init(struct device *dev)
-{
-    return 0;
-}
-
 /*======================================================================
 
     tc589_attach() creates an "instance" of the driver, allocating
@@ -212,22 +204,26 @@ static int tc589_init(struct device *dev)
 
 static dev_link_t *tc589_attach(void)
 {
+    struct el3_private *lp;
     client_reg_t client_reg;
     dev_link_t *link;
-    struct device *dev;
+    struct net_device *dev;
     int i, ret;
 
     DEBUG(0, "3c589_attach()\n");
     flush_stale_links();
     
     /* Create new ethernet device */
-    link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
-    memset(link, 0, sizeof(struct dev_link_t));
+    lp = kmalloc(sizeof(*lp), GFP_KERNEL);
+    if (!lp) return NULL;
+    memset(lp, 0, sizeof(*lp));
+    link = &lp->link; dev = &lp->dev;
+    link->priv = dev->priv = link->irq.Instance = lp;
+    
     link->release.function = &tc589_release;
     link->release.data = (u_long)link;
     link->io.NumPorts1 = 16;
     link->io.Attributes1 = IO_DATA_PATH_WIDTH_16;
-    link->io.IOAddrLines = 4;
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE | IRQ_HANDLE_PRESENT;
     link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
     if (irq_list[0] == -1)
@@ -242,25 +238,19 @@ static dev_link_t *tc589_attach(void)
     link->conf.ConfigIndex = 1;
     link->conf.Present = PRESENT_OPTION;
     
-    dev = kmalloc(sizeof(struct device), GFP_KERNEL);
-    memset(dev, 0, sizeof(struct device));
-    
-    /* Make up a EL3-specific-data structure. */
-    dev->priv = kmalloc(sizeof(struct el3_private), GFP_KERNEL);
-    memset(dev->priv, 0, sizeof(struct el3_private));
-    
     /* The EL3-specific entries in the device structure. */
     dev->hard_start_xmit = &el3_start_xmit;
     dev->set_config = &el3_config;
     dev->get_stats = &el3_get_stats;
     dev->set_multicast_list = &set_multicast_list;
     ether_setup(dev);
-    dev->name = ((struct el3_private *)dev->priv)->node.dev_name;
-    dev->init = &tc589_init;
+    init_dev_name(dev, lp->node);
     dev->open = &el3_open;
     dev->stop = &el3_close;
-    dev->tbusy = 1;
-    link->priv = link->irq.Instance = dev;
+#ifdef HAVE_NETIF_QUEUE
+    dev->tx_timeout = el3_tx_timeout;
+    dev->watchdog_timeo = TX_TIMEOUT;
+#endif
     
     /* Register with Card Services */
     link->next = dev_list;
@@ -295,8 +285,8 @@ static dev_link_t *tc589_attach(void)
 
 static void tc589_detach(dev_link_t *link)
 {
+    struct el3_private *lp = link->priv;
     dev_link_t **linkp;
-    long flags;
     
     DEBUG(0, "3c589_detach(0x%p)\n", link);
     
@@ -306,14 +296,7 @@ static void tc589_detach(dev_link_t *link)
     if (*linkp == NULL)
 	return;
 
-    save_flags(flags);
-    cli();
-    if (link->state & DEV_RELEASE_PENDING) {
-	del_timer(&link->release);
-	link->state &= ~DEV_RELEASE_PENDING;
-    }
-    restore_flags(flags);
-    
+    del_timer(&link->release);
     if (link->state & DEV_CONFIG) {
 	tc589_release((u_long)link);
 	if (link->state & DEV_STALE_CONFIG) {
@@ -327,15 +310,9 @@ static void tc589_detach(dev_link_t *link)
     
     /* Unlink device structure, free bits */
     *linkp = link->next;
-    if (link->priv) {
-	struct device *dev = link->priv;
-	if (link->dev != NULL)
-	    unregister_netdev(dev);
-	if (dev->priv)
-	    kfree_s(dev->priv, sizeof(struct el3_private));
-	kfree_s(link->priv, sizeof(struct device));
-    }
-    kfree_s(link, sizeof(struct dev_link_t));
+    if (link->dev)
+	unregister_netdev(&lp->dev);
+    kfree(lp);
     
 } /* tc589_detach */
 
@@ -352,21 +329,19 @@ while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
 
 static void tc589_config(dev_link_t *link)
 {
-    client_handle_t handle;
-    struct device *dev;
+    client_handle_t handle = link->handle;
+    struct el3_private *lp = link->priv;
+    struct net_device *dev = &lp->dev;
     tuple_t tuple;
     cisparse_t parse;
-    u_short buf[32];
+    u_short buf[32], *phys_addr;
     int last_fn, last_ret, i, j, multi = 0;
-    short ioaddr, *phys_addr;
+    ioaddr_t ioaddr;
     char *ram_split[] = {"5:3", "3:1", "1:1", "3:5"};
     
-    handle = link->handle;
-    dev = link->priv;
-    phys_addr = (short *)dev->dev_addr;
-
     DEBUG(0, "3c589_config(0x%p)\n", link);
 
+    phys_addr = (u_short *)dev->dev_addr;
     tuple.Attributes = 0;
     tuple.DesiredTuple = CISTPL_CONFIG;
     CS_CHECK(GetFirstTuple, handle, &tuple);
@@ -393,6 +368,7 @@ static void tc589_config(dev_link_t *link)
     link->state |= DEV_CONFIG;
 
     /* For the 3c562, the base address must be xx00-xx7f */
+    link->io.IOAddrLines = 16;
     for (i = j = 0; j < 0x400; j += 0x10) {
 	if (multi && (j & 0x80)) continue;
 	link->io.BasePort1 = j ^ 0x300;
@@ -408,13 +384,11 @@ static void tc589_config(dev_link_t *link)
 	
     dev->irq = link->irq.AssignedIRQ;
     dev->base_addr = link->io.BasePort1;
-    dev->tbusy = 0;
     if (register_netdev(dev) != 0) {
 	printk(KERN_NOTICE "3c589_cs: register_netdev() failed\n");
 	goto failed;
     }
     
-    link->state &= ~DEV_CONFIG_PENDING;
     ioaddr = dev->base_addr;
     EL3WINDOW(0);
 
@@ -434,8 +408,10 @@ static void tc589_config(dev_link_t *link)
 	    goto failed;
 	}
     }
-    
-    link->dev = &((struct el3_private *)dev->priv)->node;
+
+    copy_dev_name(lp->node, dev);
+    link->dev = &lp->node;
+    link->state &= ~DEV_CONFIG_PENDING;
     
     /* The address and resource configuration register aren't loaded from
        the EEPROM and *must* be set to 0 and IRQ3 for the PCMCIA version. */
@@ -491,7 +467,7 @@ static void tc589_release(u_long arg)
     CardServices(ReleaseIO, link->handle, &link->io);
     CardServices(ReleaseIRQ, link->handle, &link->irq);
     
-    link->state &= ~(DEV_CONFIG | DEV_RELEASE_PENDING);
+    link->state &= ~DEV_CONFIG;
     
 } /* tc589_release */
 
@@ -508,7 +484,8 @@ static int tc589_event(event_t event, int priority,
 		       event_callback_args_t *args)
 {
     dev_link_t *link = args->client_data;
-    struct device *dev = link->priv;
+    struct el3_private *lp = link->priv;
+    struct net_device *dev = &lp->dev;
     
     DEBUG(1, "3c589_event(0x%06x)\n", event);
     
@@ -516,9 +493,8 @@ static int tc589_event(event_t event, int priority,
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG) {
-	    dev->tbusy = 1; dev->start = 0;
-	    link->release.expires = RUN_AT(HZ/20);
-	    add_timer(&link->release);
+	    netif_device_detach(dev);
+	    mod_timer(&link->release, jiffies + HZ/20);
 	}
 	break;
     case CS_EVENT_CARD_INSERTION:
@@ -530,9 +506,8 @@ static int tc589_event(event_t event, int priority,
 	/* Fall through... */
     case CS_EVENT_RESET_PHYSICAL:
 	if (link->state & DEV_CONFIG) {
-	    if (link->open) {
-		dev->tbusy = 1; dev->start = 0;
-	    }
+	    if (link->open)
+		netif_device_detach(dev);
 	    CardServices(ReleaseConfiguration, link->handle);
 	}
 	break;
@@ -544,7 +519,7 @@ static int tc589_event(event_t event, int priority,
 	    CardServices(RequestConfiguration, link->handle, &link->conf);
 	    if (link->open) {
 		tc589_reset(dev);
-		dev->tbusy = 0; dev->start = 1;
+		netif_device_attach(dev);
 	    }
 	}
 	break;
@@ -557,7 +532,7 @@ static int tc589_event(event_t event, int priority,
 /*
   Use this for commands that may take time to finish
 */
-static void wait_for_completion(struct device *dev, int cmd)
+static void wait_for_completion(struct net_device *dev, int cmd)
 {
     int i = 100;
     outw(cmd, dev->base_addr + EL3_CMD);
@@ -572,7 +547,7 @@ static void wait_for_completion(struct device *dev, int cmd)
   Read a word from the EEPROM using the regular EEPROM access register.
   Assume that we are in register window zero.
 */
-static ushort read_eeprom(short ioaddr, int index)
+static u_short read_eeprom(ioaddr_t ioaddr, int index)
 {
     int i;
     outw(EEPROM_READ + index, ioaddr + 10);
@@ -587,10 +562,10 @@ static ushort read_eeprom(short ioaddr, int index)
   Set transceiver type, perhaps to something other than what the user
   specified in dev->if_port.
 */
-static void tc589_set_xcvr(struct device *dev, int if_port)
+static void tc589_set_xcvr(struct net_device *dev, int if_port)
 {
     struct el3_private *lp = (struct el3_private *)dev->priv;
-    ushort ioaddr = dev->base_addr;
+    ioaddr_t ioaddr = dev->base_addr;
     
     EL3WINDOW(0);
     switch (if_port) {
@@ -610,9 +585,9 @@ static void tc589_set_xcvr(struct device *dev, int if_port)
 	lp->media_status = ((dev->if_port == 0) ? 0x4010 : 0x8800);
 }
 
-static void dump_status(struct device *dev)
+static void dump_status(struct net_device *dev)
 {
-    int ioaddr = dev->base_addr;
+    ioaddr_t ioaddr = dev->base_addr;
     EL3WINDOW(1);
     printk(KERN_INFO "  irq status %04x, rx status %04x, tx status "
 	   "%02x  tx free %04x\n", inw(ioaddr+EL3_STATUS),
@@ -626,9 +601,9 @@ static void dump_status(struct device *dev)
 }
 
 /* Reset and restore all of the 3c589 registers. */
-static void tc589_reset(struct device *dev)
+static void tc589_reset(struct net_device *dev)
 {
-    ushort ioaddr = dev->base_addr;
+    ioaddr_t ioaddr = dev->base_addr;
     int i;
     
     EL3WINDOW(0);
@@ -667,7 +642,7 @@ static void tc589_reset(struct device *dev)
 	 | AdapterFailure, ioaddr + EL3_CMD);
 }
 
-static int el3_config(struct device *dev, struct ifmap *map)
+static int el3_config(struct net_device *dev, struct ifmap *map)
 {
     if ((map->port != (u_char)(-1)) && (map->port != dev->if_port)) {
 	if (map->port <= 3) {
@@ -681,36 +656,35 @@ static int el3_config(struct device *dev, struct ifmap *map)
     return 0;
 }
 
-static int el3_open(struct device *dev)
+static int el3_open(struct net_device *dev)
 {
     struct el3_private *lp = (struct el3_private *)dev->priv;
-    dev_link_t *link;
+    dev_link_t *link = &lp->link;
     
-    for (link = dev_list; link; link = link->next)
-	if (link->priv == dev) break;
     if (!DEV_OK(link))
 	return -ENODEV;
 
     link->open++;
     MOD_INC_USE_COUNT;
-    dev->interrupt = 0; dev->tbusy = 0; dev->start = 1;
+    netif_start_queue(dev);
+    netif_mark_up(dev);
     
     tc589_reset(dev);
     lp->media.function = &media_check;
-    lp->media.data = (u_long)dev;
-    lp->media.expires = RUN_AT(HZ);
+    lp->media.data = (u_long)lp;
+    lp->media.expires = jiffies + HZ;
     add_timer(&lp->media);
 
     DEBUG(1, "%s: opened, status %4.4x.\n",
 	  dev->name, inw(dev->base_addr + EL3_STATUS));
     
-    return 0;					/* Always succeed */
+    return 0;
 }
 
-static void el3_tx_timeout(struct device *dev)
+static void el3_tx_timeout(struct net_device *dev)
 {
     struct el3_private *lp = (struct el3_private *)dev->priv;
-    int ioaddr = dev->base_addr;
+    ioaddr_t ioaddr = dev->base_addr;
     
     printk(KERN_NOTICE "%s: Transmit timed out!\n", dev->name);
     dump_status(dev);
@@ -719,13 +693,13 @@ static void el3_tx_timeout(struct device *dev)
     /* Issue TX_RESET and TX_START commands. */
     wait_for_completion(dev, TxReset);
     outw(TxEnable, ioaddr + EL3_CMD);
-    dev->tbusy = 0;
+    netif_start_queue(dev);
 }
 
-static void pop_tx_status(struct device *dev)
+static void pop_tx_status(struct net_device *dev)
 {
     struct el3_private *lp = (struct el3_private *)dev->priv;
-    int ioaddr = dev->base_addr;
+    ioaddr_t ioaddr = dev->base_addr;
     int i;
     
     /* Clear the Tx status stack. */
@@ -745,52 +719,31 @@ static void pop_tx_status(struct device *dev)
     }
 }
 
-static int el3_start_xmit(struct sk_buff *skb, struct device *dev)
+static int el3_start_xmit(struct sk_buff *skb, struct net_device *dev)
 {
-    struct el3_private *lp = (struct el3_private *)dev->priv;
-    int ioaddr = dev->base_addr;
+    ioaddr_t ioaddr = dev->base_addr;
 
-    /* Transmitter timeout, serious problems. */
-    if (dev->tbusy) {
-	if (jiffies - dev->trans_start < TX_TIMEOUT)
-	    return 1;
-	el3_tx_timeout(dev);
-    }
-
-#if (LINUX_VERSION_CODE < VERSION(2,1,25))
-    if (skb == NULL) {
-	dev_tint(dev);
-	return 0;
-    }
-    if (skb->len <= 0)
-	return 0;
-#endif
+    tx_timeout_check(dev, el3_tx_timeout);
+    skb_tx_check(dev, skb);
 
     DEBUG(3, "%s: el3_start_xmit(length = %ld) called, "
 	  "status %4.4x.\n", dev->name, (long)skb->len,
 	  inw(ioaddr + EL3_STATUS));
 
-    /* Avoid timer-based retransmission conflicts. */
-    if (test_and_set_bit(0, (void*)&dev->tbusy) != 0)
-	printk(KERN_NOTICE "%s: transmitter access conflict.\n",
-	       dev->name);
-    else {
-#if (LINUX_VERSION_CODE >= VERSION(2,1,25))
-	lp->stats.tx_bytes += skb->len;
-#endif
-	/* Put out the doubleword header... */
-	outw(skb->len, ioaddr + TX_FIFO);
-	outw(0x00, ioaddr + TX_FIFO);
-	/* ... and the packet rounded to a doubleword. */
-	outsl_ns(ioaddr + TX_FIFO, skb->data, (skb->len + 3) >> 2);
-	
-	dev->trans_start = jiffies;
-	if (inw(ioaddr + TX_FREE) > 1536) {
-	    dev->tbusy = 0;
-	} else
-	    /* Interrupt us when the FIFO has room for max-sized packet. */
-	    outw(SetTxThreshold + 1536, ioaddr + EL3_CMD);
-    }
+    add_tx_bytes(&((struct el3_private *)dev->priv)->stats, skb->len);
+
+    /* Put out the doubleword header... */
+    outw(skb->len, ioaddr + TX_FIFO);
+    outw(0x00, ioaddr + TX_FIFO);
+    /* ... and the packet rounded to a doubleword. */
+    outsl_ns(ioaddr + TX_FIFO, skb->data, (skb->len + 3) >> 2);
+
+    dev->trans_start = jiffies;
+    if (inw(ioaddr + TX_FREE) > 1536) {
+	netif_start_queue(dev);
+    } else
+	/* Interrupt us when the FIFO has room for max-sized packet. */
+	outw(SetTxThreshold + 1536, ioaddr + EL3_CMD);
 
     DEV_KFREE_SKB(skb);
     pop_tx_status(dev);
@@ -799,32 +752,24 @@ static int el3_start_xmit(struct sk_buff *skb, struct device *dev)
 }
 
 /* The EL3 interrupt handler. */
-static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
+static void el3_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 {
-    struct device *dev = (struct device *)DEV_ID;
-    struct el3_private *lp;
-    int ioaddr, status;
+    struct el3_private *lp = dev_id;
+    struct net_device *dev = &lp->dev;
+    ioaddr_t ioaddr, status;
     int i = 0;
     
-    if ((dev == NULL) || !dev->start)
+    if (!netif_device_present(dev))
 	return;
-    lp = (struct el3_private *)dev->priv;
     ioaddr = dev->base_addr;
 
-#ifdef PCMCIA_DEBUG
-    if (dev->interrupt) {
-	printk(KERN_NOTICE "%s: re-entering the interrupt handler.\n",
-	       dev->name);
-	return;
-    }
-    dev->interrupt = 1;
     DEBUG(3, "%s: interrupt, status %4.4x.\n",
 	  dev->name, inw(ioaddr + EL3_STATUS));
-#endif
     
     while ((status = inw(ioaddr + EL3_STATUS)) &
 	(IntLatch | RxComplete | StatsFull)) {
-	if ((dev->start == 0) || ((status & 0xe000) != 0x2000)) {
+	if (!netif_device_present(dev) ||
+	    ((status & 0xe000) != 0x2000)) {
 	    DEBUG(1, "%s: interrupt from dead card\n", dev->name);
 	    break;
 	}
@@ -836,8 +781,7 @@ static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 	    DEBUG(3, "    TX room bit was handled.\n");
 	    /* There's room in the FIFO for a full-sized packet. */
 	    outw(AckIntr | TxAvailable, ioaddr + EL3_CMD);
-	    dev->tbusy = 0;
-	    mark_bh(NET_BH);
+	    netif_wake_queue(dev);
 	}
 	
 	if (status & TxComplete)
@@ -846,7 +790,7 @@ static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 	if (status & (AdapterFailure | RxEarly | StatsFull)) {
 	    /* Handle all uncommon interrupts. */
 	    if (status & StatsFull)		/* Empty statistics. */
-		update_stats(ioaddr, dev);
+		update_stats(dev);
 	    if (status & RxEarly) {		/* Rx early is unused. */
 		el3_rx(dev);
 		outw(AckIntr | RxEarly, ioaddr + EL3_CMD);
@@ -885,24 +829,21 @@ static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
     }
 
     lp->last_irq = jiffies;
-#ifdef PCMCIA_DEBUG
     DEBUG(3, "%s: exiting interrupt, status %4.4x.\n",
 	  dev->name, inw(ioaddr + EL3_STATUS));
-    dev->interrupt = 0;
-#endif
     return;
 }
 
 static void media_check(u_long arg)
 {
-    struct device *dev = (struct device *)(arg);
-    struct el3_private *lp = (struct el3_private *)dev->priv;
-    int ioaddr = dev->base_addr;
+    struct el3_private *lp = (struct el3_private *)(arg);
+    struct net_device *dev = &lp->dev;
+    ioaddr_t ioaddr = dev->base_addr;
     u_short media, errs;
     u_long flags;
 
-    if (dev->start == 0) goto reschedule;
-    
+    if (!netif_device_present(dev)) goto reschedule;
+
     EL3WINDOW(1);
     /* Check for pending interrupt with expired latency timer: with
        this, we can limp along even if the interrupt is blocked */
@@ -910,12 +851,12 @@ static void media_check(u_long arg)
 	(inb(ioaddr + EL3_TIMER) == 0xff)) {
 	if (!lp->fast_poll)
 	    printk(KERN_INFO "%s: interrupt(s) dropped!\n", dev->name);
-	el3_interrupt IRQ(dev->irq, dev, NULL);
+	el3_interrupt(dev->irq, lp, NULL);
 	lp->fast_poll = HZ;
     }
     if (lp->fast_poll) {
 	lp->fast_poll--;
-	lp->media.expires = RUN_AT(2);
+	lp->media.expires = jiffies + 1;
 	add_timer(&lp->media);
 	return;
     }
@@ -969,22 +910,20 @@ static void media_check(u_long arg)
     restore_flags(flags);
 
 reschedule:
-    lp->media.expires = RUN_AT(HZ);
+    lp->media.expires = jiffies + HZ;
     add_timer(&lp->media);
 }
 
-static struct net_device_stats *el3_get_stats(struct device *dev)
+static struct net_device_stats *el3_get_stats(struct net_device *dev)
 {
     struct el3_private *lp = (struct el3_private *)dev->priv;
     unsigned long flags;
-    dev_link_t *link;
+    dev_link_t *link = &lp->link;
 
-    for (link = dev_list; link; link = link->next)
-	if (link->priv == dev) break;
     if (DEV_OK(link)) {
 	save_flags(flags);
 	cli();
-	update_stats(dev->base_addr, dev);
+	update_stats(dev);
 	restore_flags(flags);
     }
     return &lp->stats;
@@ -996,10 +935,11 @@ static struct net_device_stats *el3_get_stats(struct device *dev)
   operation, and it's simpler for the rest of the driver to assume that
   window 1 is always valid rather than use a special window-state variable.
 */
-static void update_stats(int ioaddr, struct device *dev)
+static void update_stats(struct net_device *dev)
 {
     struct el3_private *lp = (struct el3_private *)dev->priv;
-    
+    ioaddr_t ioaddr = dev->base_addr;
+
     DEBUG(2, "%s: updating the statistics.\n", dev->name);
     /* Turn off statistics updates while reading. */
     outw(StatsDisable, ioaddr + EL3_CMD);
@@ -1014,18 +954,18 @@ static void update_stats(int ioaddr, struct device *dev)
     lp->stats.tx_packets		+= inb(ioaddr + 6);
     /* Rx packets   */			inb(ioaddr + 7);
     /* Tx deferrals */			inb(ioaddr + 8);
-    inw(ioaddr + 10);	/* Total Rx and Tx octets. */
-    inw(ioaddr + 12);
+    /* Rx octets */			inw(ioaddr + 10);
+    /* Tx octets */			inw(ioaddr + 12);
     
     /* Back to window 1, and turn statistics back on. */
     EL3WINDOW(1);
     outw(StatsEnable, ioaddr + EL3_CMD);
 }
 
-static int el3_rx(struct device *dev)
+static int el3_rx(struct net_device *dev)
 {
     struct el3_private *lp = (struct el3_private *)dev->priv;
-    int ioaddr = dev->base_addr;
+    ioaddr_t ioaddr = dev->base_addr;
     int worklimit = 32;
     short rx_status;
     
@@ -1037,12 +977,12 @@ static int el3_rx(struct device *dev)
 	    short error = rx_status & 0x3800;
 	    lp->stats.rx_errors++;
 	    switch (error) {
-	    case 0x0000:		lp->stats.rx_over_errors++; break;
-	    case 0x0800:		lp->stats.rx_length_errors++; break;
-	    case 0x1000:		lp->stats.rx_frame_errors++; break;
-	    case 0x1800:		lp->stats.rx_length_errors++; break;
-	    case 0x2000:		lp->stats.rx_frame_errors++; break;
-	    case 0x2800:		lp->stats.rx_crc_errors++; break;
+	    case 0x0000:	lp->stats.rx_over_errors++; break;
+	    case 0x0800:	lp->stats.rx_length_errors++; break;
+	    case 0x1000:	lp->stats.rx_frame_errors++; break;
+	    case 0x1800:	lp->stats.rx_length_errors++; break;
+	    case 0x2000:	lp->stats.rx_frame_errors++; break;
+	    case 0x2800:	lp->stats.rx_crc_errors++; break;
 	    }
 	} else {
 	    short pkt_len = rx_status & 0x7ff;
@@ -1062,9 +1002,7 @@ static int el3_rx(struct device *dev)
 		
 		netif_rx(skb);
 		lp->stats.rx_packets++;
-#if (LINUX_VERSION_CODE >= VERSION(2,1,25))
-		lp->stats.rx_bytes += skb->len;
-#endif
+		add_rx_bytes(&lp->stats, skb->len);
 	    } else {
 		DEBUG(1, "%s: couldn't allocate a sk_buff of"
 		      " size %d.\n", dev->name, pkt_len);
@@ -1085,12 +1023,12 @@ static int el3_rx(struct device *dev)
    num_addrs > 0	Multicast mode, receive normal and MC packets, and do
 			best-effort filtering.
  */
-static void set_multicast_list(struct device *dev)
+static void set_multicast_list(struct net_device *dev)
 {
-    short ioaddr = dev->base_addr;
-    dev_link_t *link;
-    for (link = dev_list; link; link = link->next)
-	if (link->priv == dev) break;
+    struct el3_private *lp = dev->priv;
+    dev_link_t *link = &lp->link;
+    ioaddr_t ioaddr = dev->base_addr;
+
     if (!(DEV_OK(link))) return;
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 2) {
@@ -1111,15 +1049,11 @@ static void set_multicast_list(struct device *dev)
 	outw(SetRxFilter | RxStation | RxBroadcast, ioaddr + EL3_CMD);
 }
 
-static int el3_close(struct device *dev)
+static int el3_close(struct net_device *dev)
 {
-    int ioaddr = dev->base_addr;
-    dev_link_t *link;
-
-    for (link = dev_list; link; link = link->next)
-	if (link->priv == dev) break;
-    if (link == NULL)
-	return -ENODEV;
+    struct el3_private *lp = dev->priv;
+    dev_link_t *link = &lp->link;
+    ioaddr_t ioaddr = dev->base_addr;
     
     DEBUG(1, "%s: shutting down ethercard.\n", dev->name);
 
@@ -1147,17 +1081,15 @@ static int el3_close(struct device *dev)
         
 	/* Check if the card still exists */
 	if ((inw(ioaddr+EL3_STATUS) & 0xe000) == 0x2000)
-	    update_stats(ioaddr, dev);
+	    update_stats(dev);
     }
 
     link->open--;
-    dev->start = 0;
-    del_timer(&((struct el3_private *)dev->priv)->media);
-    if (link->state & DEV_STALE_CONFIG) {
-	link->release.expires = RUN_AT(HZ/20);
-	link->state |= DEV_RELEASE_PENDING;
-	add_timer(&link->release);
-    }
+    netif_stop_queue(dev);
+    netif_mark_down(dev);
+    del_timer(&lp->media);
+    if (link->state & DEV_STALE_CONFIG)
+	mod_timer(&link->release, jiffies + HZ/20);
     
     MOD_DEC_USE_COUNT;
     
@@ -1166,7 +1098,7 @@ static int el3_close(struct device *dev)
 
 /*====================================================================*/
 
-int init_module(void)
+static int __init init_3c589_cs(void)
 {
     servinfo_t serv;
     DEBUG(0, "%s\n", version);
@@ -1176,14 +1108,17 @@ int init_module(void)
 	       "does not match!\n");
 	return -1;
     }
-    register_pcmcia_driver(&dev_info, &tc589_attach, &tc589_detach);
+    register_pccard_driver(&dev_info, &tc589_attach, &tc589_detach);
     return 0;
 }
 
-void cleanup_module(void)
+static void __exit exit_3c589_cs(void)
 {
     DEBUG(0, "3c589_cs: unloading\n");
-    unregister_pcmcia_driver(&dev_info);
+    unregister_pccard_driver(&dev_info);
     while (dev_list != NULL)
 	tc589_detach(dev_list);
 }
+
+module_init(init_3c589_cs);
+module_exit(exit_3c589_cs);
