@@ -6,7 +6,7 @@
     As written, it will function as a sort of generic point enabler,
     configuring any card as that card's CIS specifies.
     
-    dummy_cs.c 1.14 1999/07/20 15:59:33
+    dummy_cs.c 1.26 2000/05/16 21:31:37
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -19,8 +19,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
@@ -28,11 +39,14 @@
 #include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
+#include <linux/module.h>
+#include <linux/init.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
 #include <linux/malloc.h>
 #include <linux/string.h>
 #include <linux/timer.h>
+#include <linux/ioport.h>
 #include <asm/io.h>
 #include <asm/system.h>
 
@@ -54,9 +68,9 @@
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
-#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args);
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"dummy_cs.c 1.14 1999/07/20 15:59:33 (David Hinds)";
+"dummy_cs.c 1.26 2000/05/16 21:31:37 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -65,12 +79,16 @@ static char *version =
 
 /* Parameters that can be set with 'insmod' */
 
+/* Release IO ports after configuration? */
+static int free_ports = 0;
+
 /* The old way: bit map of interrupts to choose from */
 /* This means pick from 15, 14, 12, 11, 10, 9, 7, 5, 4, and 3 */
 static u_int irq_mask = 0xdeb8;
 /* Newer, simpler way of listing specific interrupts */
 static int irq_list[4] = { -1 };
 
+MODULE_PARM(free_ports, "i");
 MODULE_PARM(irq_mask, "i");
 MODULE_PARM(irq_list, "1-4i");
 
@@ -101,7 +119,7 @@ static void dummy_detach(dev_link_t *);
 
 /*
    You'll also need to prototype all the functions that will actually
-   be used to talk to your device.  See 'pcmem_cs' for a good example
+   be used to talk to your device.  See 'memory_cs' for a good example
    of a fully self-sufficient driver; the other drivers rely more or
    less on other parts of the kernel.
 */
@@ -133,6 +151,9 @@ static dev_link_t *dev_list = NULL;
    'priv' pointer in a dev_link_t structure can be used to point to
    a device-specific private data structure, like this.
 
+   To simplify the data structure handling, we actually include the
+   dev_link_t structure in the device's private data structure.
+
    A driver needs to provide a dev_node_t structure for each device
    on a card.  In some cases, there is only one device per card (for
    example, ethernet cards, modems).  In other cases, there may be
@@ -153,6 +174,7 @@ static dev_link_t *dev_list = NULL;
 */
    
 typedef struct local_info_t {
+    dev_link_t		link;
     dev_node_t		node;
     int			stop;
     struct bus_operations *bus;
@@ -180,16 +202,20 @@ static void cs_error(client_handle_t handle, int func, int ret)
 
 static dev_link_t *dummy_attach(void)
 {
-    client_reg_t client_reg;
-    dev_link_t *link;
     local_info_t *local;
+    dev_link_t *link;
+    client_reg_t client_reg;
     int ret, i;
     
     DEBUG(0, "dummy_attach()\n");
 
+    /* Allocate space for private device-specific data */
+    local = kmalloc(sizeof(local_info_t), GFP_KERNEL);
+    if (!local) return NULL;
+    memset(local, 0, sizeof(local_info_t));
+    link = &local->link; link->priv = local;
+    
     /* Initialize the dev_link_t structure */
-    link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
-    memset(link, 0, sizeof(struct dev_link_t));
     link->release.function = &dummy_release;
     link->release.data = (u_long)link;
 
@@ -214,11 +240,6 @@ static dev_link_t *dummy_attach(void)
     link->conf.Vcc = 50;
     link->conf.IntType = INT_MEMORY_AND_IO;
 
-    /* Allocate space for private device-specific data */
-    local = kmalloc(sizeof(local_info_t), GFP_KERNEL);
-    memset(local, 0, sizeof(local_info_t));
-    link->priv = local;
-    
     /* Register with Card Services */
     link->next = dev_list;
     dev_list = link;
@@ -232,7 +253,7 @@ static dev_link_t *dummy_attach(void)
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
     ret = CardServices(RegisterClient, &link->handle, &client_reg);
-    if (ret != 0) {
+    if (ret != CS_SUCCESS) {
 	cs_error(link->handle, RegisterClient, ret);
 	dummy_detach(link);
 	return NULL;
@@ -281,12 +302,10 @@ static void dummy_detach(dev_link_t *link)
     if (link->handle)
 	CardServices(DeregisterClient, link->handle);
     
-    /* Unlink device structure, free pieces */
+    /* Unlink device structure, and free it */
     *linkp = link->next;
-    if (link->priv) {
-	kfree_s(link->priv, sizeof(local_info_t));
-    }
-    kfree_s(link, sizeof(struct dev_link_t));
+    /* This points to the parent local_info_t struct */
+    kfree(link->priv);
     
 } /* dummy_detach */
 
@@ -306,19 +325,16 @@ if (CardServices(fn, args) != 0) goto next_entry
 
 static void dummy_config(dev_link_t *link)
 {
-    client_handle_t handle;
+    client_handle_t handle = link->handle;
+    local_info_t *dev = link->priv;
     tuple_t tuple;
     cisparse_t parse;
-    local_info_t *dev;
     int last_fn, last_ret;
     u_char buf[64];
     config_info_t conf;
     win_req_t req;
     memreq_t map;
     
-    handle = link->handle;
-    dev = link->priv;
-
     DEBUG(0, "dummy_config(0x%p)\n", link);
 
     /*
@@ -403,6 +419,7 @@ static void dummy_config(dev_link_t *link)
 		link->io.Attributes1 = IO_DATA_PATH_WIDTH_16;
 	    if (!(io->flags & CISTPL_IO_16BIT))
 		link->io.Attributes1 = IO_DATA_PATH_WIDTH_8;
+	    link->io.IOAddrLines = io->flags & CISTPL_IO_LINES_MASK;
 	    link->io.BasePort1 = io->win[0].base;
 	    link->io.NumPorts1 = io->win[0].len;
 	    if (io->nwin > 1) {
@@ -461,6 +478,18 @@ static void dummy_config(dev_link_t *link)
     */
     CS_CHECK(RequestConfiguration, link->handle, &link->conf);
 
+    /*
+      We can release the IO port allocations here, if some other
+      driver for the card is going to loaded, and will expect the
+      ports to be available.
+    */
+    if (free_ports) {
+	if (link->io.BasePort1)
+	    release_region(link->io.BasePort1, link->io.NumPorts1);
+	if (link->io.BasePort2)
+	    release_region(link->io.BasePort2, link->io.NumPorts2);
+    }
+
     /*
       At this point, the dev_node_t structure(s) need to be
       initialized and arranged in a linked list at link->dev.
@@ -571,8 +600,7 @@ static int dummy_event(event_t event, int priority,
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG) {
 	    ((local_info_t *)link->priv)->stop = 1;
-	    link->release.expires = RUN_AT(HZ/20);
-	    add_timer(&link->release);
+	    mod_timer(&link->release, jiffies + HZ/20);
 	}
 	break;
     case CS_EVENT_CARD_INSERTION:
@@ -607,7 +635,7 @@ static int dummy_event(event_t event, int priority,
 
 /*====================================================================*/
 
-int init_module(void)
+static int __init init_dummy_cs(void)
 {
     servinfo_t serv;
     DEBUG(0, "%s\n", version);
@@ -617,17 +645,21 @@ int init_module(void)
 	       "does not match!\n");
 	return -1;
     }
-    register_pcmcia_driver(&dev_info, &dummy_attach, &dummy_detach);
+    register_pccard_driver(&dev_info, &dummy_attach, &dummy_detach);
     return 0;
 }
 
-void cleanup_module(void)
+static void __exit exit_dummy_cs(void)
 {
     DEBUG(0, "dummy_cs: unloading\n");
-    unregister_pcmcia_driver(&dev_info);
+    unregister_pccard_driver(&dev_info);
     while (dev_list != NULL) {
+	del_timer(&dev_list->release);
 	if (dev_list->state & DEV_CONFIG)
 	    dummy_release((u_long)dev_list);
 	dummy_detach(dev_list);
     }
 }
+
+module_init(init_dummy_cs);
+module_exit(exit_dummy_cs);
