@@ -30,7 +30,9 @@
 #include <pcmcia/config.h>
 #include <pcmcia/k_compat.h>
 
+#include <linux/module.h>
 #include <linux/kernel.h>
+#include <linux/init.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
 #include <linux/malloc.h>
@@ -70,16 +72,6 @@ MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...)
 #endif
 
-/*
-   For debugging this driver you may need more information.
-   To enable printing registers or status, set 'fmvj18x_debug=#' option .
- */
-#ifdef FMVJ18X_DEBUG
-static int fmvj18x_debug = FMVJ18X_DEBUG;
-#else
-static int fmvj18x_debug = 2;
-#endif /* FMVJ18X_DEBUG */
-
 /* Bit map of interrupts to choose from */
 /* This means pick from 15, 14, 12, 11, 10, 9, 7, 5, 4, and 3 */
 static u_int irq_mask = 0xdeb8;
@@ -110,22 +102,22 @@ static void fmvj18x_config(dev_link_t *link);
 static void fmvj18x_release(u_long arg);
 static int fmvj18x_event(event_t event, int priority,
 			  event_callback_args_t *args);
-static int fmvj18x_init(struct device *dev);
 static dev_link_t *fmvj18x_attach(void);
 static void fmvj18x_detach(dev_link_t *);
 
 /*
     LAN controler(MBH86960A) specific routines
  */
-static int fjn_config(struct device *dev, struct ifmap *map);
-static int fjn_open(struct device *dev);
-static int fjn_close(struct device *dev);
-static int fjn_start_xmit(struct sk_buff *skb, struct device *dev);
-static void fjn_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs);
-static void fjn_rx(struct device *dev);
-static void fjn_reset(struct device *dev);
-static struct net_device_stats *fjn_get_stats(struct device *dev);
-static void set_rx_mode(struct device *dev);
+static int fjn_config(struct net_device *dev, struct ifmap *map);
+static int fjn_open(struct net_device *dev);
+static int fjn_close(struct net_device *dev);
+static int fjn_start_xmit(struct sk_buff *skb, struct net_device *dev);
+static void fjn_interrupt(int irq, void *dev_id, struct pt_regs *regs);
+static void fjn_rx(struct net_device *dev);
+static void fjn_reset(struct net_device *dev);
+static struct net_device_stats *fjn_get_stats(struct net_device *dev);
+static void set_rx_mode(struct net_device *dev);
+static void fjn_tx_timeout(struct net_device *dev);
 
 static dev_info_t dev_info = "fmvj18x_cs";
 static dev_link_t *dev_list = NULL;
@@ -139,14 +131,16 @@ typedef enum { MBH10302, MBH10304, TDK, CONTEC, LA501 } cardtype_t;
     driver specific data structure
 */
 typedef struct local_info_t {
+    dev_link_t link;
+    struct net_device dev;
     dev_node_t node;
     struct net_device_stats stats;
     long open_time;
     uint tx_started:1;
     uint tx_queue;
-    ushort tx_queue_len;
+    u_short tx_queue_len;
     cardtype_t cardtype;
-    ushort sent;
+    u_short sent;
     u_char mc_filter[8];
 } local_info_t;
 
@@ -244,6 +238,8 @@ typedef struct local_info_t {
 #define INTR_OFF             0x0d /* LAN controler ignores interrupts */
 #define INTR_ON              0x1d /* LAN controler will catch interrupts */
 
+#define TX_TIMEOUT		((400*HZ)/1000)
+
 /*======================================================================
 
     This bit of code is used to avoid unregistering network devices
@@ -274,17 +270,21 @@ static void cs_error(client_handle_t handle, int func, int ret)
 
 static dev_link_t *fmvj18x_attach(void)
 {
-    client_reg_t client_reg;
+    local_info_t *lp;
     dev_link_t *link;
-    struct device *dev;
+    struct net_device *dev;
+    client_reg_t client_reg;
     int i, ret;
     
     DEBUG(0, "fmvj18x_attach()\n");
     flush_stale_links();
 
-    /* Initialize the dev_link_t structure */
-    link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
-    memset(link, 0, sizeof(struct dev_link_t));
+    /* Make up a FMVJ18x specific data structure */
+    lp = kmalloc(sizeof(*lp), GFP_KERNEL);
+    if (!lp) return NULL;
+    memset(lp, 0, sizeof(*lp));
+    link = &lp->link; dev = &lp->dev;
+    link->priv = dev->priv = link->irq.Instance = lp;
 
     link->release.function = &fmvj18x_release;
     link->release.data = (u_long)link;
@@ -309,24 +309,19 @@ static dev_link_t *fmvj18x_attach(void)
     link->conf.Vcc = 50;
     link->conf.IntType = INT_MEMORY_AND_IO;
 
-    /* Make up a FMVJ18x specific data structure */
-    dev = kmalloc(sizeof(struct device), GFP_KERNEL);
-    memset(dev, 0, sizeof(struct device));
-    dev->priv = kmalloc(sizeof(local_info_t), GFP_KERNEL);
-    memset(dev->priv, 0, sizeof(local_info_t));
-
     /* The FMVJ18x specific entries in the device structure. */
     dev->hard_start_xmit = &fjn_start_xmit;
     dev->set_config = &fjn_config;
     dev->get_stats = &fjn_get_stats;
     dev->set_multicast_list = &set_rx_mode;
     ether_setup(dev);
-    dev->name = ((local_info_t *)dev->priv)->node.dev_name;
-    dev->init = &fmvj18x_init;
+    init_dev_name(dev, lp->node);
     dev->open = &fjn_open;
     dev->stop = &fjn_close;
-    dev->tbusy = 0xFF;
-    link->priv = link->irq.Instance = dev;
+#ifdef HAVE_NETIF_QUEUE
+    dev->tx_timeout = fjn_tx_timeout;
+    dev->watchdog_timeo = TX_TIMEOUT;
+#endif
     
     /* Register with Card Services */
     link->next = dev_list;
@@ -354,8 +349,8 @@ static dev_link_t *fmvj18x_attach(void)
 
 static void fmvj18x_detach(dev_link_t *link)
 {
+    local_info_t *lp = link->priv;
     dev_link_t **linkp;
-    long flags;
     
     DEBUG(0, "fmvj18x_detach(0x%p)\n", link);
     
@@ -365,14 +360,7 @@ static void fmvj18x_detach(dev_link_t *link)
     if (*linkp == NULL)
 	return;
 
-    save_flags(flags);
-    cli();
-    if (link->state & DEV_RELEASE_PENDING) {
-	del_timer(&link->release);
-	link->state &= ~DEV_RELEASE_PENDING;
-    }
-    restore_flags(flags);
-    
+    del_timer(&link->release);
     if (link->state & DEV_CONFIG) {
 	fmvj18x_release((u_long)link);
 	if (link->state & DEV_STALE_CONFIG) {
@@ -387,15 +375,9 @@ static void fmvj18x_detach(dev_link_t *link)
     
     /* Unlink device structure, free pieces */
     *linkp = link->next;
-    if (link->priv) {
-	struct device *dev = link->priv;
-	if (link->dev != NULL)
-	    unregister_netdev(dev);
-	if (dev->priv) 
-	    kfree_s(dev->priv, sizeof(local_info_t));
-	kfree_s(dev, sizeof(struct device));
-    }
-    kfree_s(link, sizeof(struct dev_link_t));
+    if (link->dev)
+	unregister_netdev(&lp->dev);
+    kfree(lp);
     
 } /* fmvj18x_detach */
 
@@ -406,20 +388,18 @@ while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
 
 static void fmvj18x_config(dev_link_t *link)
 {
-    client_handle_t handle;
+    client_handle_t handle = link->handle;
+    local_info_t *lp = link->priv;
+    struct net_device *dev = &lp->dev;
     tuple_t tuple;
     cisparse_t parse;
-    struct device *dev;
     u_short buf[32];
     int i, last_fn, last_ret;
-    short ioaddr;
+    ioaddr_t ioaddr;
     cardtype_t cardtype;
     char *card_name = "unknown";
     u_char *node_id;
     
-    handle = link->handle;
-    dev =link->priv;
-
     DEBUG(0, "fmvj18x_config(0x%p)\n", link);
 
     /*
@@ -481,7 +461,6 @@ static void fmvj18x_config(dev_link_t *link)
     CS_CHECK(RequestConfiguration, link->handle, &link->conf);
     dev->irq = link->irq.AssignedIRQ;
     dev->base_addr = link->io.BasePort1;
-    dev->tbusy = 0;
     if (register_netdev(dev) != 0) {
 	printk(KERN_NOTICE "fmvj18x_cs: register_netdev() failed\n");
 	goto failed;
@@ -539,10 +518,11 @@ static void fmvj18x_config(dev_link_t *link)
 	break;
     }
 
-    link->dev = &((local_info_t *)dev->priv)->node;
+    copy_dev_name(lp->node, dev);
+    link->dev = &lp->node;
     link->state &= ~DEV_CONFIG_PENDING;
 
-    ((struct local_info_t *)dev->priv)->cardtype = cardtype ;
+    lp->cardtype = cardtype;
     /* print current configuration */
     printk(KERN_INFO "%s: %s, sram %s, port %#3lx, irq %d, hw_addr ", 
 	   dev->name, card_name, sram_config == 0 ? "4K TX*2" : "8K TX*2", 
@@ -585,7 +565,7 @@ static void fmvj18x_release(u_long arg)
     CardServices(ReleaseIO, link->handle, &link->io);
     CardServices(ReleaseIRQ, link->handle, &link->irq);
     
-    link->state &= ~(DEV_CONFIG | DEV_RELEASE_PENDING);
+    link->state &= ~DEV_CONFIG;
     
 } /* fmvj18x_release */
 
@@ -595,7 +575,8 @@ static int fmvj18x_event(event_t event, int priority,
 			  event_callback_args_t *args)
 {
     dev_link_t *link = args->client_data;
-    struct device *dev = link->priv;
+    local_info_t *lp = link->priv;
+    struct net_device *dev = &lp->dev;
 
     DEBUG(1, "fmvj18x_event(0x%06x)\n", event);
     
@@ -603,10 +584,8 @@ static int fmvj18x_event(event_t event, int priority,
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG) {
-	    dev->tbusy = 0xFF; 
-	    dev->start = 0;
-	    link->release.expires = RUN_AT(HZ/20);
-	    add_timer(&link->release);
+	    netif_device_detach(dev);
+	    mod_timer(&link->release, jiffies + HZ/20);
 	}
 	break;
     case CS_EVENT_CARD_INSERTION:
@@ -618,10 +597,8 @@ static int fmvj18x_event(event_t event, int priority,
 	/* Fall through... */
     case CS_EVENT_RESET_PHYSICAL:
 	if (link->state & DEV_CONFIG) {
-	    if (link->open) {
-		dev->tbusy = 0xFF; 
-		dev->start = 0;
-	    }
+	    if (link->open)
+		netif_device_detach(dev);
 	    CardServices(ReleaseConfiguration, link->handle);
 	}
 	break;
@@ -632,9 +609,8 @@ static int fmvj18x_event(event_t event, int priority,
 	if (link->state & DEV_CONFIG) {
 	    CardServices(RequestConfiguration, link->handle, &link->conf);
 	    if (link->open) {
-		dev->tbusy = 0;
-		dev->start = 1;
 		fjn_reset(dev);
+		netif_device_attach(dev);
 	    }
 	}
 	break;
@@ -642,14 +618,9 @@ static int fmvj18x_event(event_t event, int priority,
     return 0;
 } /* fmvj18x_event */
 
-static int fmvj18x_init(struct device *dev)
-{
-    return 0;
-} /* fmvj18x_init */
-
 /*====================================================================*/
 
-int init_module(void)
+static int __init init_fmvj18x_cs(void)
 {
     servinfo_t serv;
     DEBUG(0, "%s\n", version);
@@ -659,41 +630,35 @@ int init_module(void)
 	       "does not match!\n");
 	return -1;
     }
-    register_pcmcia_driver(&dev_info, &fmvj18x_attach, &fmvj18x_detach);
+    register_pccard_driver(&dev_info, &fmvj18x_attach, &fmvj18x_detach);
     return 0;
 }
 
-/*====================================================================*/
-
-void cleanup_module(void)
+static void __exit exit_fmvj18x_cs(void)
 {
     DEBUG(0, "fmvj18x_cs: unloading\n");
-    unregister_pcmcia_driver(&dev_info);
+    unregister_pccard_driver(&dev_info);
     while (dev_list != NULL)
 	fmvj18x_detach(dev_list);
 }
 
+module_init(init_fmvj18x_cs);
+module_exit(exit_fmvj18x_cs);
+
 /*====================================================================*/
 
-static void fjn_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
+static void fjn_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 {
-    struct device *dev = (struct device *)DEV_ID;
-    int ioaddr;
-    local_info_t *lp;
+    local_info_t *lp = dev_id;
+    struct net_device *dev = &lp->dev;
+    ioaddr_t ioaddr;
     unsigned short tx_stat, rx_stat;
 
-    if (dev == NULL) {
+    if (lp == NULL) {
         printk(KERN_NOTICE "fjn_interrupt(): irq %d for "
 	       "unknown device.\n", irq);
         return;
     }
-    if (dev->interrupt) {
-        printk(KERN_NOTICE "%s: re-entering the interrupt handler.\n",
-	       dev->name);
-        return;
-    }
-    dev->interrupt = 1;
-    lp = (struct local_info_t *)dev->priv;
     ioaddr = dev->base_addr;
 
     /* avoid multiple interrupts */
@@ -710,12 +675,8 @@ static void fjn_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
     outb(tx_stat, ioaddr + TX_STATUS);
     outb(rx_stat, ioaddr + RX_STATUS);
     
-    if (fmvj18x_debug > 4) {
-        printk(KERN_DEBUG "%s: interrupt, rx_status %02x.\n",
-	       dev->name, rx_stat);
-        printk(KERN_DEBUG "               tx_status %02x.\n",
-	       tx_stat);
-    }
+    DEBUG(4, "%s: interrupt, rx_status %02x.\n", dev->name, rx_stat);
+    DEBUG(4, "               tx_status %02x.\n", tx_stat);
     
     if (rx_stat || (inb(ioaddr + RX_MODE) & F_BUF_EMP) == 0) {
 	/* there is packet(s) in rx buffer */
@@ -730,97 +691,73 @@ static void fjn_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 	    lp->tx_queue = 0;
 	    lp->tx_queue_len = 0;
 	    dev->trans_start = jiffies;
-	    dev->tbusy = 0;
-	    mark_bh(NET_BH);	/* Inform upper layers. */
 	} else {
 	    lp->tx_started = 0;
-	    dev->tbusy = 0;
-	    mark_bh(NET_BH);	/* Inform upper layers. */
 	}
+	netif_wake_queue(dev);
     }
-    if (fmvj18x_debug > 4) {
-        printk(KERN_DEBUG "%s: exiting interrupt,\n", dev->name);
-        printk(KERN_DEBUG "    tx_status %02x, rx_status %02x.\n",
-	       tx_stat, rx_stat);
-    }
+    DEBUG(4, "%s: exiting interrupt,\n", dev->name);
+    DEBUG(4, "    tx_status %02x, rx_status %02x.\n", tx_stat, rx_stat);
 
-    dev->interrupt = 0;
     outb(D_TX_INTR, ioaddr + TX_INTR);
     outb(D_RX_INTR, ioaddr + RX_INTR);
 
-    return;
 } /* fjn_interrupt */
 
 /*====================================================================*/
 
-static int fjn_start_xmit(struct sk_buff *skb, struct device *dev)
+static void fjn_tx_timeout(struct net_device *dev)
 {
     struct local_info_t *lp = (struct local_info_t *)dev->priv;
-    int ioaddr = dev->base_addr;
+    ioaddr_t ioaddr = dev->base_addr;
 
-    if (dev->tbusy) {
-	/* If we get here, some higher level has decided we are broken.
-	   There should really be a "kick me" function call instead. */
-	int tickssofar = jiffies - dev->trans_start;
-	if (tickssofar < 10)
-	    return 1;
-	printk(KERN_NOTICE "%s: transmit timed out with status %04x, %s?\n",
-	       dev->name, htons(inw(ioaddr + TX_STATUS)),
-	       inb(ioaddr + TX_STATUS) & F_TMT_RDY
-	       ? "IRQ conflict" : "network cable problem");
-	printk(KERN_NOTICE "%s: timeout registers: %04x %04x %04x "
-	       "%04x %04x %04x %04x %04x.\n",
-	       dev->name, htons(inw(ioaddr + 0)),
-	       htons(inw(ioaddr + 2)), htons(inw(ioaddr + 4)),
-	       htons(inw(ioaddr + 6)), htons(inw(ioaddr + 8)),
-	       htons(inw(ioaddr +10)), htons(inw(ioaddr +12)),
-	       htons(inw(ioaddr +14)));
-	lp->stats.tx_errors++;
-	/* ToDo: We should try to restart the adaptor... */
-	cli();
+    printk(KERN_NOTICE "%s: transmit timed out with status %04x, %s?\n",
+	   dev->name, htons(inw(ioaddr + TX_STATUS)),
+	   inb(ioaddr + TX_STATUS) & F_TMT_RDY
+	   ? "IRQ conflict" : "network cable problem");
+    printk(KERN_NOTICE "%s: timeout registers: %04x %04x %04x "
+	   "%04x %04x %04x %04x %04x.\n",
+	   dev->name, htons(inw(ioaddr + 0)),
+	   htons(inw(ioaddr + 2)), htons(inw(ioaddr + 4)),
+	   htons(inw(ioaddr + 6)), htons(inw(ioaddr + 8)),
+	   htons(inw(ioaddr +10)), htons(inw(ioaddr +12)),
+	   htons(inw(ioaddr +14)));
+    lp->stats.tx_errors++;
+    /* ToDo: We should try to restart the adaptor... */
+    cli();
 
-	fjn_reset(dev);
+    fjn_reset(dev);
 
-	lp->tx_started = 0;
-	lp->tx_queue = 0;
-	lp->tx_queue_len = 0;
-	lp->sent = 0;
-	lp->open_time = jiffies;
-	dev->interrupt = 0;
-	dev->tbusy = 0;
-	dev->start = 1;
-    
-	sti();
-    }
+    lp->tx_started = 0;
+    lp->tx_queue = 0;
+    lp->tx_queue_len = 0;
+    lp->sent = 0;
+    lp->open_time = jiffies;
+    sti();
+    netif_start_queue(dev);
+}
 
-#if (LINUX_VERSION_CODE < VERSION(2,1,25))
-    if (skb == NULL) {
-	dev_tint(dev);
-	return 0;
-    }
-#endif
-    
-    /* Block a timer-based transmit from overlapping.  This could better be
-       done with atomic_swap(1, dev->tbusy), but set_bit() works as well. */
-    if (test_and_set_bit(0, (void*)&dev->tbusy) != 0)
-	printk(KERN_NOTICE "%s: Transmitter access conflict.\n", dev->name);
-    else {
+static int fjn_start_xmit(struct sk_buff *skb, struct net_device *dev)
+{
+    struct local_info_t *lp = (struct local_info_t *)dev->priv;
+    ioaddr_t ioaddr = dev->base_addr;
+
+    tx_timeout_check(dev, fjn_tx_timeout);
+    skb_tx_check(dev, skb);
+
+    {
 	short length = ETH_ZLEN < skb->len ? skb->len : ETH_ZLEN;
 	unsigned char *buf = skb->data;
 
 	if (length > ETH_FRAME_LEN) {
-	    if (fmvj18x_debug)
-		printk(KERN_NOTICE "%s: Attempting to send a large packet"
-		       " (%d bytes).\n", dev->name, length);
+	    printk(KERN_NOTICE "%s: Attempting to send a large packet"
+		   " (%d bytes).\n", dev->name, length);
 	    return 1;
 	}
 
-	if (fmvj18x_debug > 4)
-	    printk(KERN_DEBUG "%s: Transmitting a packet of length %lu.\n",
-		   dev->name, (unsigned long)skb->len);
-#if (LINUX_VERSION_CODE >= VERSION(2,1,25))
-	lp->stats.tx_bytes += skb->len;
-#endif
+	DEBUG(4, "%s: Transmitting a packet of length %lu.\n",
+	      dev->name, (unsigned long)skb->len);
+	add_tx_bytes(&lp->stats, skb->len);
 
 	/* Disable both interrupts. */
 	outw(0x0000, ioaddr + TX_INTR);
@@ -842,17 +779,17 @@ static int fjn_start_xmit(struct sk_buff *skb, struct device *dev)
 	    lp->tx_queue_len = 0;
 	    dev->trans_start = jiffies;
 	    lp->tx_started = 1;
-	    dev->tbusy = 0;
+	    netif_start_queue(dev);
 	} else {
 	    if( sram_config == 0 ) {
 		if (lp->tx_queue_len < (4096 - (ETH_FRAME_LEN +2)) )
 		    /* Yes, there is room for one more packet. */
-		    dev->tbusy = 0;
+		    netif_start_queue(dev);
 	    } else {
 		if (lp->tx_queue_len < (8192 - (ETH_FRAME_LEN +2)) && 
 						lp->tx_queue < 127 )
 		    /* Yes, there is room for one more packet. */
-		    dev->tbusy = 0;
+		    netif_start_queue(dev);
 	    }
 	}
 
@@ -867,15 +804,13 @@ static int fjn_start_xmit(struct sk_buff *skb, struct device *dev)
 
 /*====================================================================*/
 
-static void fjn_reset(struct device *dev)
+static void fjn_reset(struct net_device *dev)
 {
     struct local_info_t *lp = (struct local_info_t *)dev->priv;
-    int ioaddr = dev->base_addr;
+    ioaddr_t ioaddr = dev->base_addr;
     int i;
 
-    if (fmvj18x_debug > 4) {
-	printk(KERN_DEBUG "fjn_reset(%s) called.\n",dev->name);
-    }
+    DEBUG(4, "fjn_reset(%s) called.\n",dev->name);
 
     /* Power On chip and select bank 0 */
     outb(BANK_0, ioaddr + CONFIG_1);
@@ -894,12 +829,6 @@ static void fjn_reset(struct device *dev)
     for (i = 0; i < 6; i++) 
         outb(dev->dev_addr[i], ioaddr + NODE_ID + i);
 
-    if (fmvj18x_debug > 4) {
-	printk(KERN_DEBUG "node id: ");
-	for (i = 0; i < 6; i++) 
-	    printk("%02X ",inb(ioaddr + NODE_ID + i));
-	printk("\n");
-    }
     /* Switch to bank 1 */
     outb(BANK_1, ioaddr + CONFIG_1);
 
@@ -951,22 +880,20 @@ static void fjn_reset(struct device *dev)
 
 /*====================================================================*/
 
-static void fjn_rx(struct device *dev)
+static void fjn_rx(struct net_device *dev)
 {
     struct local_info_t *lp = (struct local_info_t *)dev->priv;
-    int ioaddr = dev->base_addr;
+    ioaddr_t ioaddr = dev->base_addr;
     int boguscount = 10;	/* 5 -> 10: by agy 19940922 */
 
-    if (fmvj18x_debug > 4)
-        printk(KERN_DEBUG "%s: in rx_packet(), rx_status %02x.\n",
-               dev->name, inb(ioaddr + RX_STATUS));
+    DEBUG(4, "%s: in rx_packet(), rx_status %02x.\n",
+	  dev->name, inb(ioaddr + RX_STATUS));
 
     while ((inb(ioaddr + RX_MODE) & F_BUF_EMP) == 0) {
-	ushort status = inw(ioaddr + DATAPORT);
+	u_short status = inw(ioaddr + DATAPORT);
 
-	if (fmvj18x_debug > 4)
-	    printk(KERN_DEBUG "%s: Rxing packet mode %02x status %04x.\n",
-		   dev->name, inb(ioaddr + RX_MODE), status);
+	DEBUG(4, "%s: Rxing packet mode %02x status %04x.\n",
+	      dev->name, inb(ioaddr + RX_MODE), status);
 #ifndef final_version
 	if (status == 0) {
 	    outb(F_SKP_PKT, ioaddr + RX_SKIP);
@@ -980,7 +907,7 @@ static void fjn_rx(struct device *dev)
 	    if (status & F_CRC_ERR) lp->stats.rx_crc_errors++;
 	    if (status & F_OVR_FLO) lp->stats.rx_over_errors++;
 	} else {
-	    ushort pkt_len = inw(ioaddr + DATAPORT);
+	    u_short pkt_len = inw(ioaddr + DATAPORT);
 	    /* Malloc up new buffer. */
 	    struct sk_buff *skb;
 
@@ -1006,7 +933,8 @@ static void fjn_rx(struct device *dev)
 		    (pkt_len + 1) >> 1);
 	    skb->protocol = eth_type_trans(skb, dev);
 
-	    if (fmvj18x_debug > 5) {
+#ifdef PCMCIA_DEBUG
+	    if (pc_debug > 5) {
 		int i;
 		printk(KERN_DEBUG "%s: Rxed packet of length %d: ",
 		       dev->name, pkt_len);
@@ -1014,19 +942,18 @@ static void fjn_rx(struct device *dev)
 		    printk(" %02x", skb->data[i]);
 		printk(".\n");
 	    }
+#endif
 
 	    netif_rx(skb);
 	    lp->stats.rx_packets++;
-#if (LINUX_VERSION_CODE >= VERSION(2,1,25))
-	    lp->stats.rx_bytes += skb->len;
-#endif
+	    add_rx_bytes(&lp->stats, skb->len);
 	}
 	if (--boguscount <= 0)
 	    break;
     }
 
     /* If any worth-while packets have been received, dev_rint()
-	   has done a mark_bh(NET_BH) for us and will work on them
+	   has done a netif_wake_queue() for us and will work on them
 	   when we get to the bottom-half routine. */
 /*
     if( lp->cardtype != TDK ) {
@@ -1038,9 +965,9 @@ static void fjn_rx(struct device *dev)
 	    outb(F_SKP_PKT, ioaddr + RX_SKIP);
 	}
 
-	if (fmvj18x_debug > 5 && i > 0)
-	    printk(KERN_DEBUG "%s: Exint Rx packet with mode %02x after"
-		   " %d ticks.\n", dev->name, inb(ioaddr + RX_MODE), i);
+	if (i > 0)
+	    DEBUG(5, "%s: Exint Rx packet with mode %02x after "
+		  "%d ticks.\n", dev->name, inb(ioaddr + RX_MODE), i);
     }
 */
 
@@ -1049,20 +976,17 @@ static void fjn_rx(struct device *dev)
 
 /*====================================================================*/
 
-static int fjn_config(struct device *dev, struct ifmap *map){
+static int fjn_config(struct net_device *dev, struct ifmap *map){
     return 0;
-} /* fjn_config */
+}
 
-static int fjn_open(struct device *dev)
+static int fjn_open(struct net_device *dev)
 {
     struct local_info_t *lp = (struct local_info_t *)dev->priv;
-    dev_link_t *link;
+    dev_link_t *link = &lp->link;
 
-    if (fmvj18x_debug > 4)
-        printk(KERN_DEBUG "fjn_open('%s').\n", dev->name);
+    DEBUG(4, "fjn_open('%s').\n", dev->name);
 
-    for (link = dev_list; link; link = link->next)
-	if (link->priv == dev) break;
     if (!DEV_OK(link))
 	return -ENODEV;
     
@@ -1074,9 +998,8 @@ static int fjn_open(struct device *dev)
     lp->tx_queue = 0;
     lp->tx_queue_len = 0;
     lp->open_time = jiffies;
-    dev->interrupt = 0;
-    dev->tbusy = 0;
-    dev->start = 1;
+    netif_mark_up(dev);
+    netif_start_queue(dev);
     
     MOD_INC_USE_COUNT;
 
@@ -1085,27 +1008,17 @@ static int fjn_open(struct device *dev)
 
 /*====================================================================*/
 
-static int fjn_close(struct device *dev)
+static int fjn_close(struct net_device *dev)
 {
-    int ioaddr = dev->base_addr;
     struct local_info_t *lp = (struct local_info_t *)dev->priv;
-    dev_link_t *link;
+    dev_link_t *link = &lp->link;
+    ioaddr_t ioaddr = dev->base_addr;
 
-    if (fmvj18x_debug > 4)
-        printk(KERN_DEBUG "fjn_open('%s').\n", dev->name);
+    DEBUG(4, "fjn_close('%s').\n", dev->name);
 
-    for (link = dev_list; link; link = link->next)
-	if (link->priv == dev) break;
-    if (link == NULL)
-	return -ENODEV;
-    
-    if (fmvj18x_debug > 2)
-	printk(KERN_DEBUG "%s: shutting down ethercard.\n", dev->name);
-
-    ((struct local_info_t *)dev->priv)->open_time = 0;
-
-    dev->tbusy = 1;
-    dev->start = 0;
+    lp->open_time = 0;
+    netif_stop_queue(dev);
+    netif_mark_down(dev);
 
     /* Set configuration register 0 to disable Tx and Rx. */
     if( sram_config == 0 ) 
@@ -1120,15 +1033,11 @@ static int fjn_close(struct device *dev)
 
     /* Set the ethernet adaptor disable IRQ */
     if( lp->cardtype != TDK ) 
-    		outb(INTR_OFF, ioaddr + LAN_CTRL);
+	outb(INTR_OFF, ioaddr + LAN_CTRL);
 
     link->open--;
-    dev->start = 0;
-    if (link->state & DEV_STALE_CONFIG) {
-	link->release.expires = RUN_AT(HZ/20);
-	link->state |= DEV_RELEASE_PENDING;
-	add_timer(&link->release);
-    }
+    if (link->state & DEV_STALE_CONFIG)
+	mod_timer(&link->release, jiffies + HZ/20);
     MOD_DEC_USE_COUNT;
 
     return 0;
@@ -1136,7 +1045,7 @@ static int fjn_close(struct device *dev)
 
 /*====================================================================*/
 
-static struct net_device_stats *fjn_get_stats(struct device *dev)
+static struct net_device_stats *fjn_get_stats(struct net_device *dev)
 {
     local_info_t *lp = (local_info_t *)dev->priv;
     return &lp->stats;
@@ -1169,9 +1078,9 @@ static inline unsigned ether_crc_le(int length, unsigned char *data)
     return crc;
 }
 
-static void set_rx_mode(struct device *dev)
+static void set_rx_mode(struct net_device *dev)
 {
-    int ioaddr = dev->base_addr;
+    ioaddr_t ioaddr = dev->base_addr;
     struct local_info_t *lp = (struct local_info_t *)dev->priv;
     unsigned char mc_filter[8];		 /* Multicast hash filter */
     long flags;
