@@ -5,7 +5,7 @@
     This driver implements a disk-like block device driver with an
     apparent block size of 512 bytes for flash memory cards.
 
-    ftl_cs.c 1.50 1999/07/30 03:49:13
+    ftl_cs.c 1.65 2000/05/16 21:31:37
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -18,8 +18,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
 
     LEGAL NOTE: The FTL format is patented by M-Systems.  They have
     granted a license for its use with PCMCIA devices:
@@ -43,6 +54,8 @@
 /* #define PSYCHO_DEBUG */
 
 #include <linux/kernel.h>
+#include <linux/module.h>
+#include <linux/init.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
 #include <linux/malloc.h>
@@ -55,6 +68,7 @@
 #include <asm/io.h>
 #include <asm/system.h>
 #include <asm/segment.h>
+#include <asm/uaccess.h>
 #include <stdarg.h>
 
 #if (LINUX_VERSION_CODE >= VERSION(2,1,0))
@@ -105,7 +119,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"ftl_cs.c 1.50 1999/07/30 03:49:13 (David Hinds)";
+"ftl_cs.c 1.65 2000/05/16 21:31:37 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -136,9 +150,6 @@ static int ftl_event(event_t event, int priority,
 static dev_link_t *ftl_attach(void);
 static void ftl_detach(dev_link_t *);
 
-/* Block IO request handler stuff */
-static void do_ftl_request(void);
-
 /* Each memory region corresponds to a minor device */
 typedef struct partition_t {
     dev_node_t		dev;
@@ -179,6 +190,7 @@ typedef struct partition_t {
 #define XFER_FAILED	0x04
 
 typedef struct ftl_dev_t {
+    dev_link_t		link;
     eraseq_handle_t	eraseq_handle;
     eraseq_entry_t	eraseq[MAX_ERASE];
     wait_queue_head_t	erase_pending;
@@ -195,17 +207,16 @@ static int ftl_blocksizes[MINOR_NR(MAX_DEV, 0, 0)];
 static wait_queue_head_t ftl_wait_open;
 
 static struct gendisk ftl_gendisk = {
-    0,			/* Major device number */
-    "ftl",
-    PART_BITS,		/* Bits in partition # */
-    MAX_PART,		/* Partitions per device */
-    MAX_DEV*MAX_PART,	/* maximum number of devices */
-    NULL,		/* init function */
-    ftl_hd,		/* hd_struct */
-    ftl_sizes,		/* block sizes */
-    MAX_DEV*MAX_PART,	/* number of "real" devices */
-    NULL,
-    NULL
+    major:		0,
+    major_name:		"ftl",
+    minor_shift:	PART_BITS,
+    max_p:		MAX_PART,
+#if (LINUX_VERSION_CODE < VERSION(2,3,40))
+    max_nr:		MAX_DEV*MAX_PART,
+#endif
+    part:		ftl_hd,
+    sizes:		ftl_sizes,
+    nr_real:		MAX_DEV*MAX_PART
 };
 
 /*====================================================================*/
@@ -216,18 +227,15 @@ static int ftl_open(struct inode *inode, struct file *file);
 static FS_RELEASE_T ftl_close(struct inode *inode, struct file *file);
 static int ftl_reread_partitions(int minor);
 
-static struct file_operations ftl_blk_fops = {
-    NULL,		/* lseek */
-    block_read,		/* read */
-    block_write,	/* write */
-    NULL,		/* readdir */
-    NULL,		/* select */
-    ftl_ioctl,		/* ioctl */
-    NULL,		/* mmap */
-    ftl_open,		/* open */
-    NULL_FLUSH		/* flush */
-    ftl_close,		/* release */
-    block_fsync		/* fsync */
+static struct block_device_operations ftl_blk_fops = {
+    open:	ftl_open,
+    release:	ftl_close,
+    ioctl:	ftl_ioctl,
+#ifdef block_device_operations
+    read:	block_read,
+    write:	block_write,
+    fsync:	block_fsync
+#endif
 };
 
 /*====================================================================*/
@@ -268,16 +276,15 @@ static dev_link_t *ftl_attach(void)
     }
     
     /* Create new memory card device */
-    link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
-    dev_table[i] = link;
-    memset(link, 0, sizeof(struct dev_link_t));
+    dev = kmalloc(sizeof(*dev), GFP_KERNEL);
+    if (!dev) return NULL;
+    memset(dev, 0, sizeof(*dev));
+    link = &dev->link; link->priv = dev;
+
     link->release.function = &ftl_release;
     link->release.data = (u_long)link;
-    
-    dev = kmalloc(sizeof(struct ftl_dev_t), GFP_KERNEL);
-    memset(dev, 0, sizeof(ftl_dev_t));
+    dev_table[i] = link;
     init_waitqueue_head(&dev->erase_pending);
-    link->priv = dev;
 
     /* Register with Card Services */
     client_reg.dev_info = &dev_info;
@@ -323,7 +330,7 @@ static dev_link_t *ftl_attach(void)
 
 static void ftl_detach(dev_link_t *link)
 {
-    ftl_dev_t *dev;
+    ftl_dev_t *dev = link->priv;
     int i;
 
     DEBUG(0, "ftl_cs: ftl_detach(0x%p)\n", link);
@@ -334,6 +341,7 @@ static void ftl_detach(dev_link_t *link)
     if (i == MAX_DEV)
 	return;
 
+    del_timer(&link->release);
     if (link->state & DEV_CONFIG) {
 	ftl_release((u_long)link);
 	if (link->state & DEV_STALE_CONFIG) {
@@ -342,7 +350,6 @@ static void ftl_detach(dev_link_t *link)
 	}
     }
 
-    dev = (ftl_dev_t *)link->priv;
     if (dev->eraseq_handle)
 	CardServices(DeregisterEraseQueue, dev->eraseq_handle);
     if (link->handle)
@@ -350,8 +357,7 @@ static void ftl_detach(dev_link_t *link)
     
     /* Unlink device structure, free bits */
     dev_table[i] = NULL;
-    kfree_s(dev, sizeof(struct ftl_dev_t));
-    kfree_s(link, sizeof(struct dev_link_t));
+    kfree(dev);
     
 } /* ftl_detach */
 
@@ -365,7 +371,7 @@ static void ftl_detach(dev_link_t *link)
 
 static void ftl_config(dev_link_t *link)
 {
-    ftl_dev_t *dev;
+    ftl_dev_t *dev = link->priv;
     partition_t *minor;
     region_info_t region;
     dev_node_t **tail;
@@ -378,7 +384,6 @@ static void ftl_config(dev_link_t *link)
 
     for (i = 0; i < MAX_DEV; i++)
 	if (dev_table[i] == link) break;
-    dev = (ftl_dev_t *)link->priv;
     tail = &link->dev;
     minor = dev->minor;
     nr = 0;
@@ -459,17 +464,15 @@ static int ftl_event(event_t event, int priority,
 		     event_callback_args_t *args)
 {
     dev_link_t *link = args->client_data;
-    ftl_dev_t *dev;
+    ftl_dev_t *dev = link->priv;
 
     DEBUG(1, "ftl_cs: ftl_event()\n");
     
     switch (event) {
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
-	if (link->state & DEV_CONFIG) {
-	    link->release.expires = RUN_AT(HZ/20);
-	    add_timer(&link->release);
-	}
+	if (link->state & DEV_CONFIG)
+	    mod_timer(&link->release, jiffies + HZ/20);
 	break;
     case CS_EVENT_CARD_INSERTION:
 	link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
@@ -477,7 +480,6 @@ static int ftl_event(event_t event, int priority,
 	break;
     case CS_EVENT_ERASE_COMPLETE:
 	save_status((eraseq_entry_t *)(args->info));
-	dev = (ftl_dev_t *)link->priv;
 	wake_up(&dev->erase_pending);
 	break;
     case CS_EVENT_PM_SUSPEND:
@@ -509,7 +511,8 @@ static int scan_header(partition_t *part)
     erase_unit_header_t header;
     mem_op_t req;
     int ret;
-    
+
+    part->header.FormattedSize = 0;
     /* Search first megabyte for a valid FTL header */
     req.Attributes = MEM_OP_BUFFER_KERNEL;
     req.Count = sizeof(header);
@@ -550,11 +553,13 @@ static int build_maps(partition_t *part)
 	part->header.NumTransferUnits;
     part->EUNInfo = kmalloc(part->DataUnits * sizeof(struct eun_info_t),
 			    GFP_KERNEL);
+    if (!part->EUNInfo) return -1;
     for (i = 0; i < part->DataUnits; i++)
 	part->EUNInfo[i].Offset = 0xffffffff;
     part->XferInfo =
 	kmalloc(part->header.NumTransferUnits * sizeof(struct xfer_info_t),
 		GFP_KERNEL);
+    if (!part->XferInfo) return -1;
 
     req.Attributes = MEM_OP_BUFFER_KERNEL;
     req.Count = sizeof(header);
@@ -612,6 +617,7 @@ static int build_maps(partition_t *part)
 
     part->bam_cache = kmalloc(part->BlocksPerUnit * sizeof(u_int),
 			      GFP_KERNEL);
+    if (!part->bam_cache) return -1;
     part->bam_index = 0xffff;
     part->FreeTotal = 0;
     for (i = 0; i < part->DataUnits; i++) {
@@ -1049,20 +1055,21 @@ static int ftl_open(struct inode *inode, struct file *file)
     partition_t *partition;
     open_mem_t open;
     int ret;
-    
+
+    MOD_INC_USE_COUNT;
     DEBUG(0, "ftl_cs: ftl_open(%d)\n", minor);
 
     link = dev_table[DEVICE_NR(minor)];
     if (!DEV_OK(link))
-	return -ENODEV;
-    
+	goto failed;
+
     dev = (ftl_dev_t *)link->priv;
     partition = &dev->minor[REGION_NR(minor)];
     if (partition->region.RegionSize == 0)
-	return -ENODEV;
+	goto failed;
     while (partition->locked)
 	sleep_on(&ftl_wait_open);
-    
+
     if (partition->handle == NULL) {
 	partition->handle = (memory_handle_t)link->handle;
 	open.Attributes = partition->region.Attributes;
@@ -1070,7 +1077,7 @@ static int ftl_open(struct inode *inode, struct file *file)
 	ret = CardServices(OpenMemory, &partition->handle, &open);
 	if (ret != CS_SUCCESS) {
 	    cs_error(OpenMemory, ret);
-	    return -ENODEV;
+	    goto failed;
 	}
 	if ((scan_header(partition) == 0) &&
 	    (build_maps(partition) == 0)) {
@@ -1084,14 +1091,16 @@ static int ftl_open(struct inode *inode, struct file *file)
 	    CardServices(CloseMemory, partition->handle);
 	    partition->handle = NULL;
 	    printk(KERN_NOTICE "ftl_cs: FTL partition is invalid.\n");
-	    return -ENODEV;
+	    goto failed;
 	}
     }
-    
+
     partition->open++;
     link->open++;
-    MOD_INC_USE_COUNT;
     return 0;
+failed:
+    MOD_DEC_USE_COUNT;
+    return -ENODEV;
 } /* ftl_open */
 
 /*====================================================================*/
@@ -1383,15 +1392,14 @@ static int ftl_ioctl(struct inode *inode, struct file *file,
     case BLKGETSIZE:
 	ret = verify_area(VERIFY_WRITE, (long *)arg, sizeof(long));
 	if (ret) return ret;
-	put_user(part->header.FormattedSize/SECTOR_SIZE,
-		 (long *)arg);
+	put_user(ftl_hd[minor].nr_sects, (long *)arg);
 	break;
     case BLKRRPART:
 	ret = ftl_reread_partitions(minor);
 	break;
 #if (LINUX_VERSION_CODE < VERSION(2,3,3))
     case BLKFLSBUF:
-	if (!suser()) return -EACCES;
+	if (!capable(CAP_SYS_ADMIN)) return -EACCES;
 	fsync_dev(inode->i_rdev);
 	invalidate_buffers(inode->i_rdev);
 	break;
@@ -1440,13 +1448,10 @@ static int ftl_reread_partitions(int minor)
 	ftl_hd[whole+i].nr_sects = 0;
     }
 
-    if (scan_header(part) == 0)
-	ftl_hd[whole].nr_sects =
-	    part->header.FormattedSize/SECTOR_SIZE;
-    else
-	ftl_hd[whole].start_sect = -1;
-    resetup_one_dev(&ftl_gendisk, whole >> PART_BITS);
-    
+    scan_header(part);
+    register_disk(&ftl_gendisk, whole >> PART_BITS, MAX_PART,
+		  &ftl_blk_fops, part->header.FormattedSize/SECTOR_SIZE);
+
 #ifdef PCMCIA_DEBUG
     for (i = 0; i < MAX_PART; i++) {
 	if (ftl_hd[whole+i].nr_sects > 0)
@@ -1467,7 +1472,7 @@ static int ftl_reread_partitions(int minor)
 
 ======================================================================*/
 
-static void do_ftl_request(void)
+static void do_ftl_request(request_arg_t)
 {
     int ret, minor;
     dev_link_t *link;
@@ -1511,7 +1516,7 @@ static void do_ftl_request(void)
 
 /*====================================================================*/
 
-int init_module(void)
+static int __init init_ftl_cs(void)
 {
     servinfo_t serv;
     int i;
@@ -1525,7 +1530,7 @@ int init_module(void)
 	return -1;
     }
     
-    register_pcmcia_driver(&dev_info, &ftl_attach, &ftl_detach);
+    register_pccard_driver(&dev_info, &ftl_attach, &ftl_detach);
 
     major_dev = register_blkdev(major_dev, "ftl", &ftl_blk_fops);
     if (major_dev == 0) {
@@ -1538,11 +1543,11 @@ int init_module(void)
 	ftl_blocksizes[i] = 1024;
     for (i = 0; i < MAX_DEV*MAX_PART; i++) {
 	ftl_hd[i].nr_sects = 0;
-	ftl_hd[i].start_sect = -1;
+	ftl_hd[i].start_sect = 0;
     }
     blksize_size[major_dev] = ftl_blocksizes;
     ftl_gendisk.major = major_dev;
-    blk_dev[major_dev].request_fn = DEVICE_REQUEST;
+    blk_init_queue(BLK_DEFAULT_QUEUE(major_dev), &do_ftl_request);
     ftl_gendisk.next = gendisk_head;
     gendisk_head = &ftl_gendisk;
     init_waitqueue_head(&ftl_wait_open);
@@ -1550,17 +1555,17 @@ int init_module(void)
     return 0;
 }
 
-void cleanup_module(void)
+static void __exit exit_ftl_cs(void)
 {
     int i;
     dev_link_t *link;
     struct gendisk *gd, **gdp;
 
     DEBUG(0, "ftl_cs: unloading\n");
-    unregister_pcmcia_driver(&dev_info);
+    unregister_pccard_driver(&dev_info);
     if (major_dev != 0) {
 	unregister_blkdev(major_dev, "ftl");
-	blk_dev[major_dev].request_fn = NULL;
+	blk_cleanup_queue(BLK_DEFAULT_QUEUE(major_dev));
 	blksize_size[major_dev] = NULL;
     }
     for (i = 0; i < MAX_DEV; i++) {
@@ -1577,3 +1582,6 @@ void cleanup_module(void)
 	    break;
 	}
 }
+
+module_init(init_ftl_cs);
+module_exit(exit_ftl_cs);
