@@ -56,6 +56,7 @@
 #include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
+#include <linux/init.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
 #include <linux/malloc.h>
@@ -67,7 +68,7 @@
 
 #include <linux/netdevice.h>
 #include <linux/trdevice.h>
-#include <drivers/net/ibmtr.h>
+#include <linux/ibmtr.h>
 
 #include <pcmcia/version.h>
 #include <pcmcia/cs_types.h>
@@ -116,7 +117,7 @@ MODULE_PARM(ringspeed, "i");
 /*====================================================================*/
 
 static void ibmtr_config(dev_link_t *link);
-static void ibmtr_hw_setup(struct device *dev);
+static void ibmtr_hw_setup(struct net_device *dev);
 static void ibmtr_release(u_long arg);
 static int ibmtr_event(event_t event, int priority,
                        event_callback_args_t *args);
@@ -129,13 +130,13 @@ static void ibmtr_detach(dev_link_t *);
 static dev_link_t *dev_list = NULL;
 
 #if (LINUX_VERSION_CODE >= VERSION(2,2,0))
-extern int ibmtr_probe(struct device *dev);
+extern int ibmtr_probe(struct net_device *dev);
 unsigned char pcmcia_reality_check(unsigned char gss);
 #endif
 
-extern int trdev_init(struct device *dev);
+extern int trdev_init(struct net_device *dev);
 extern void tok_interrupt(int irq, struct pt_regs *regs);
-extern int tok_init_card(struct device *dev);
+extern int tok_init_card(struct net_device *dev);
 extern unsigned char get_sram_size(struct tok_info *ti);
 #if (LINUX_VERSION_CODE < VERSION(2,1,100))
 extern struct timer_list tr_timer;
@@ -144,9 +145,11 @@ extern struct timer_list tr_timer;
 /*====================================================================*/
 
 typedef struct ibmtr_dev_t {
-    struct device       *dev; /* Changed for 2.2.0 */ 
+    dev_link_t		link;
+    struct net_device	*dev; /* Changed for 2.2.0 */
     dev_node_t          node;
     window_handle_t     sram_win_handle;
+    struct tok_info	ti;
 } ibmtr_dev_t;
 
 /*======================================================================
@@ -185,19 +188,21 @@ static void cs_error(client_handle_t handle, int func, int ret)
 
 static dev_link_t *ibmtr_attach(void)
 {
-    client_reg_t client_reg;
-    dev_link_t *link;
     ibmtr_dev_t *info;
-    struct device *dev;
-    struct tok_info *ti;
+    dev_link_t *link;
+    struct net_device *dev;
+    client_reg_t client_reg;
     int i, ret;
     
     DEBUG(0, "ibmtr_attach()\n");
     flush_stale_links();
 
     /* Create new token-ring device */
-    link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
-    memset(link, 0, sizeof(struct dev_link_t));
+    info = kmalloc(sizeof(*info), GFP_KERNEL);
+    if (!info) return NULL;
+    memset(info, 0, sizeof(*info));
+    link = &info->link; link->priv = info;
+
     link->release.function = &ibmtr_release;
     link->release.data = (u_long)link;
     link->io.Attributes1 = IO_DATA_PATH_WIDTH_8;
@@ -216,25 +221,13 @@ static dev_link_t *ibmtr_attach(void)
     link->conf.IntType = INT_MEMORY_AND_IO;
     link->conf.Present = PRESENT_OPTION;
 
-    info = kmalloc(sizeof(struct ibmtr_dev_t), GFP_KERNEL);
-    memset(info, 0, sizeof(struct ibmtr_dev_t));
-
 #if (LINUX_VERSION_CODE < VERSION(2,2,0))
-    dev = kmalloc(sizeof(struct device), GFP_KERNEL);
-    memset(dev,0,sizeof(struct device));
+    dev = kmalloc(sizeof(struct net_device), GFP_KERNEL);
+    memset(dev,0,sizeof(struct net_device));
 #else
-    dev = NULL; /* Pass NULL pointers or kmallloc'ed pointers only */ 
-    dev = init_trdev(dev,0);
+    dev = init_trdev(NULL,0);
 #endif
-
-    /* Believe it or not, init_trdev does not set up the tok_info memory
-       allocation. Although it would do it we asked it to.  Why don't
-       we ? (ibmtr.c doesn't either)  */
-
-    ti = kmalloc(sizeof(struct tok_info), GFP_KERNEL);
-    memset(ti,0,sizeof(struct tok_info));
-    dev->priv = ti;
-
+    dev->priv = &info->ti;
     link->irq.Instance = info->dev = dev;
     
     if (dev == NULL) {
@@ -249,9 +242,6 @@ static dev_link_t *ibmtr_attach(void)
     dev->name = info->node.dev_name;
 #endif
 
-    dev->tbusy = 1; 
-    link->priv = info;
-
     /* Register with Card Services */
     link->next = dev_list;
     dev_list = link;
@@ -287,8 +277,7 @@ static void ibmtr_detach(dev_link_t *link)
 {
     struct ibmtr_dev_t *info = link->priv;
     dev_link_t **linkp;
-    struct device *dev; 
-    long flags;
+    struct net_device *dev; 
 
     DEBUG(0, "ibmtr_detach(0x%p)\n", link);
 
@@ -299,22 +288,15 @@ static void ibmtr_detach(dev_link_t *link)
         return;
 
     dev = info->dev;
-    save_flags(flags);
-    cli();
 #if (LINUX_VERSION_CODE < VERSION(2,1,100))
-    if (tr_timer.next) del_timer(&tr_timer);
+    del_timer(&tr_timer);
 #else
     {
 	struct tok_info *ti = (struct tok_info *)dev->priv;
-	if (ti->tr_timer.next) del_timer(&(ti->tr_timer));
+	del_timer(&(ti->tr_timer));
     }
 #endif
-    if (link->state & DEV_RELEASE_PENDING) {
-        del_timer(&link->release);
-        link->state &= ~DEV_RELEASE_PENDING;
-    }
-    restore_flags(flags);
-
+    del_timer(&link->release);
     if (link->state & DEV_CONFIG) {
         ibmtr_release((u_long)link);
         if (link->state & DEV_STALE_CONFIG) {
@@ -328,23 +310,13 @@ static void ibmtr_detach(dev_link_t *link)
 
     /* Unlink device structure, free bits */
     *linkp = link->next;
-    if (link->priv) {
-	dev = info->dev;
-	if (link->dev != NULL) {
+    if (link->dev)
 #if (LINUX_VERSION_CODE <= VERSION(2,1,16))
-	    unregister_netdev(dev);
+	unregister_netdev(info->dev);
 #else
-	    unregister_trdev(dev);
+	unregister_trdev(info->dev);
 #endif
-	}
-	if (dev) {
-	    if (dev->priv)
-		kfree_s(dev->priv, sizeof(struct tok_info));
-	    kfree_s(dev, sizeof(struct device));
-	}
-	kfree_s(info, sizeof(struct ibmtr_dev_t));
-    }
-    kfree_s(link, sizeof(struct dev_link_t));
+    kfree(info);
 
 } /* ibmtr_detach */
 
@@ -361,23 +333,18 @@ while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
 
 static void ibmtr_config(dev_link_t *link)
 {
-    client_handle_t handle;
+    client_handle_t handle = link->handle;
+    ibmtr_dev_t *info = link->priv;
+    struct net_device *dev = info->dev;
+    struct tok_info *ti = dev->priv;
     tuple_t tuple;
     cisparse_t parse;
     win_req_t req;
     memreq_t mem;
-    ibmtr_dev_t *info;
-    struct device *dev;
-    struct tok_info *ti;
     int i, last_ret, last_fn;
     u_char buf[64];
     unsigned char Shared_Ram_Base;
 
-    handle = link->handle;
-    info = link->priv;
-    dev = info->dev;
-    ti = dev->priv;
-
     DEBUG(0, "ibmtr_config(0x%p)\n", link);
 
     tuple.Attributes = 0;
@@ -417,7 +384,7 @@ static void ibmtr_config(dev_link_t *link)
 
     /* Allocate the MMIO memory window */
     req.Attributes = WIN_DATA_WIDTH_16|WIN_MEMORY_TYPE_CM|WIN_ENABLE;
-    req.Attributes |= WIN_USE_WAIT;
+    req.Attributes |= WIN_USE_WAIT|WIN_STRICT_ALIGN;
     req.Base = mmiobase;
     req.Size = 0x2000;
     req.AccessSpeed = 250;
@@ -431,7 +398,7 @@ static void ibmtr_config(dev_link_t *link)
 
     /* Allocate the SRAM memory window */
     req.Attributes = WIN_DATA_WIDTH_16|WIN_MEMORY_TYPE_CM|WIN_ENABLE;
-    req.Attributes |= WIN_USE_WAIT | WIN_MAP_BELOW_1MB;
+    req.Attributes |= WIN_USE_WAIT|WIN_MAP_BELOW_1MB;
     req.Base = srambase;
     req.Size = sramsize * 1024;
     req.AccessSpeed = 250;
@@ -453,8 +420,6 @@ static void ibmtr_config(dev_link_t *link)
         Adapters Technical Reference"  SC30-3585 for this info.  */
     ibmtr_hw_setup(dev);
 
-    dev->tbusy = 0;
-
 #if (LINUX_VERSION_CODE <= VERSION(2,1,16))
     i = register_netdev(dev);
 #else
@@ -501,7 +466,7 @@ static void ibmtr_release(u_long arg)
 {
     dev_link_t *link = (dev_link_t *)arg;
     ibmtr_dev_t *info = link->priv;
-    struct device *dev = info->dev;
+    struct net_device *dev = info->dev;
 
     DEBUG(0, "ibmtr_release(0x%p)\n", link);
 
@@ -522,7 +487,7 @@ static void ibmtr_release(u_long arg)
 	CardServices(ReleaseWindow, info->sram_win_handle);
     }
 
-    link->state &= ~(DEV_CONFIG | DEV_RELEASE_PENDING);
+    link->state &= ~DEV_CONFIG;
 
 } /* ibmtr_release */
 
@@ -540,7 +505,7 @@ static int ibmtr_event(event_t event, int priority,
 {
     dev_link_t *link = args->client_data;
     ibmtr_dev_t *info = link->priv;
-    struct device *dev = info->dev;
+    struct net_device *dev = info->dev;
 
     DEBUG(1, "ibmtr_event(0x%06x)\n", event);
     
@@ -548,10 +513,8 @@ static int ibmtr_event(event_t event, int priority,
     case CS_EVENT_CARD_REMOVAL:
         link->state &= ~DEV_PRESENT;
         if (link->state & DEV_CONFIG) {
-            dev->tbusy = 1; dev->start = 0;
-            link->release.expires = RUN_AT(HZ/20);
-            link->state |= DEV_RELEASE_PENDING;
-            add_timer(&link->release);
+	    netif_device_detach(dev);
+	    mod_timer(&link->release, jiffies + HZ/20);
         }
         break;
     case CS_EVENT_CARD_INSERTION:
@@ -563,9 +526,8 @@ static int ibmtr_event(event_t event, int priority,
         /* Fall through... */
     case CS_EVENT_RESET_PHYSICAL:
         if (link->state & DEV_CONFIG) {
-            if (link->open) {
-                dev->tbusy = 1; dev->start = 0;
-            }
+            if (link->open)
+		netif_device_detach(dev);
             CardServices(ReleaseConfiguration, link->handle);
         }
         break;
@@ -577,7 +539,7 @@ static int ibmtr_event(event_t event, int priority,
             CardServices(RequestConfiguration, link->handle, &link->conf);
             if (link->open) {
 		(dev->init)(dev);
-                dev->tbusy = 0; dev->start = 1;
+		netif_device_attach(dev);
             }
         }
         break;
@@ -587,17 +549,15 @@ static int ibmtr_event(event_t event, int priority,
 
 /*====================================================================*/
 
-static void ibmtr_hw_setup(struct device *dev)
+static void ibmtr_hw_setup(struct net_device *dev)
 {
-    struct tok_info *ti;
+    struct tok_info *ti = dev->priv;
     int i; 
 #if (LINUX_VERSION_CODE < VERSION(2,2,0))
     int j;
-    unsigned char  temp;
+    unsigned char temp;
 #endif
 
-    ti = dev->priv;
-
 #if (LINUX_VERSION_CODE >= VERSION(2,2,0)) 
 
     /* Bizarre IBM behavior, there are 16 bits of information we
@@ -715,7 +675,7 @@ unsigned char pcmcia_reality_check(unsigned char gss)
 
 /*====================================================================*/
 
-int init_module(void)
+static int __init init_ibmtr_cs(void)
 {
     servinfo_t serv;
     DEBUG(0, "%s\n", version);
@@ -725,14 +685,17 @@ int init_module(void)
 	       "does not match!\n");
         return -1;
     }
-    register_pcmcia_driver(&dev_info, &ibmtr_attach, &ibmtr_detach);
+    register_pccard_driver(&dev_info, &ibmtr_attach, &ibmtr_detach);
     return 0;
 }
 
-void cleanup_module(void)
+static void __exit exit_ibmtr_cs(void)
 {
     DEBUG(0, "ibmtr_cs: unloading\n");
-    unregister_pcmcia_driver(&dev_info);
+    unregister_pccard_driver(&dev_info);
     while (dev_list != NULL)
         ibmtr_detach(dev_list);
 }
+
+module_init(init_ibmtr_cs);
+module_exit(exit_ibmtr_cs);
