@@ -2,7 +2,7 @@
 
     A driver for PCMCIA IDE/ATA disk cards
 
-    ide_cs.c 1.18 1999/07/20 15:59:40
+    ide_cs.c 1.29 2000/05/16 21:31:36
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,8 +15,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
@@ -24,6 +35,8 @@
 #include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
+#include <linux/module.h>
+#include <linux/init.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
 #include <linux/malloc.h>
@@ -50,7 +63,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"ide_cs.c 1.18 1999/07/20 15:59:40 (David Hinds)";
+"ide_cs.c 1.29 2000/05/16 21:31:36 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -76,12 +89,12 @@ static const char ide_major[] = {
 };
 
 typedef struct ide_info_t {
+    dev_link_t	link;
     int		ndev;
     dev_node_t	node;
     int		hd;
 } ide_info_t;
 
-static void ide_config(dev_link_t *link);
 static void ide_release(u_long arg);
 static int ide_event(event_t event, int priority,
 		     event_callback_args_t *args);
@@ -111,15 +124,19 @@ static void cs_error(client_handle_t handle, int func, int ret)
 
 static dev_link_t *ide_attach(void)
 {
-    client_reg_t client_reg;
+    ide_info_t *info;
     dev_link_t *link;
+    client_reg_t client_reg;
     int i, ret;
     
     DEBUG(0, "ide_attach()\n");
 
     /* Create new ide device */
-    link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
-    memset(link, 0, sizeof(struct dev_link_t));
+    info = kmalloc(sizeof(*info), GFP_KERNEL);
+    if (!info) return NULL;
+    memset(info, 0, sizeof(*info));
+    link = &info->link; link->priv = info;
+
     link->release.function = &ide_release;
     link->release.data = (u_long)link;
     link->io.Attributes1 = IO_DATA_PATH_WIDTH_AUTO;
@@ -135,8 +152,6 @@ static dev_link_t *ide_attach(void)
     link->conf.Attributes = CONF_ENABLE_IRQ;
     link->conf.Vcc = 50;
     link->conf.IntType = INT_MEMORY_AND_IO;
-    link->priv = kmalloc(sizeof(struct ide_info_t), GFP_KERNEL);
-    memset(link->priv, 0, sizeof(struct ide_info_t));
     
     /* Register with Card Services */
     link->next = dev_list;
@@ -172,7 +187,6 @@ static dev_link_t *ide_attach(void)
 static void ide_detach(dev_link_t *link)
 {
     dev_link_t **linkp;
-    long flags;
     int ret;
 
     DEBUG(0, "ide_detach(0x%p)\n", link);
@@ -183,14 +197,7 @@ static void ide_detach(dev_link_t *link)
     if (*linkp == NULL)
 	return;
 
-    save_flags(flags);
-    cli();
-    if (link->state & DEV_RELEASE_PENDING) {
-	del_timer(&link->release);
-	link->state &= ~DEV_RELEASE_PENDING;
-    }
-    restore_flags(flags);
-    
+    del_timer(&link->release);
     if (link->state & DEV_CONFIG)
 	ide_release((u_long)link);
     
@@ -200,10 +207,9 @@ static void ide_detach(dev_link_t *link)
 	    cs_error(link->handle, DeregisterClient, ret);
     }
     
-    /* Unlink device structure, free bits */
+    /* Unlink, free device structure */
     *linkp = link->next;
-    kfree_s(link->priv, sizeof(ide_info_t));
-    kfree_s(link, sizeof(struct dev_link_t));
+    kfree(link->priv);
     
 } /* ide_detach */
 
@@ -223,8 +229,8 @@ if (CardServices(fn, args) != 0) goto next_entry
 
 void ide_config(dev_link_t *link)
 {
-    client_handle_t handle;
-    ide_info_t *info;
+    client_handle_t handle = link->handle;
+    ide_info_t *info = link->priv;
     tuple_t tuple;
     u_short buf[128];
     cisparse_t parse;
@@ -233,9 +239,6 @@ void ide_config(dev_link_t *link)
     cistpl_cftable_entry_t dflt = { 0 };
     int i, pass, last_ret, last_fn, hd, io_base, ctl_base;
 
-    handle = link->handle;
-    info = link->priv;
-    
     DEBUG(0, "ide_config(0x%p)\n", link);
     
     tuple.TupleData = (cisdata_t *)buf;
@@ -285,6 +288,7 @@ void ide_config(dev_link_t *link)
 	    cistpl_io_t *io = (cfg->io.nwin) ? &cfg->io : &dflt.io;
 	    link->conf.ConfigIndex = cfg->index;
 	    link->io.BasePort1 = io->win[0].base;
+	    link->io.IOAddrLines = io->flags & CISTPL_IO_LINES_MASK;
 	    if (!(io->flags & CISTPL_IO_16BIT))
 		link->io.Attributes1 = IO_DATA_PATH_WIDTH_8;
 	    if (io->nwin == 2) {
@@ -325,7 +329,7 @@ void ide_config(dev_link_t *link)
 	release_region(link->io.BasePort2, link->io.NumPorts2);
 
     /* retry registration in case device is still spinning up */
-    for (i = 0; i < 10; i++) {
+    for (hd = -1, i = 0; i < 10; i++) {
 	hd = ide_register(io_base, ctl_base, link->irq.AssignedIRQ);
 	if (hd >= 0) break;
 	if (link->io.NumPorts1 == 0x20) {
@@ -336,7 +340,7 @@ void ide_config(dev_link_t *link)
 		break;
 	    }
 	}
-	current->state = TASK_INTERRUPTIBLE;
+	__set_current_state(TASK_UNINTERRUPTIBLE);
 	schedule_timeout(HZ/10);
     }
     
@@ -417,11 +421,8 @@ int ide_event(event_t event, int priority,
     switch (event) {
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
-	if (link->state & DEV_CONFIG) {
-	    link->release.expires = RUN_AT(HZ/20);
-	    link->state |= DEV_RELEASE_PENDING;
-	    add_timer(&link->release);
-	}
+	if (link->state & DEV_CONFIG)
+	    mod_timer(&link->release, jiffies + HZ/20);
 	break;
     case CS_EVENT_CARD_INSERTION:
 	link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
@@ -447,7 +448,7 @@ int ide_event(event_t event, int priority,
 
 /*====================================================================*/
 
-int init_module(void)
+static int __init init_ide_cs(void)
 {
     servinfo_t serv;
     DEBUG(0, "%s\n", version);
@@ -457,14 +458,17 @@ int init_module(void)
 	       "does not match!\n");
 	return -1;
     }
-    register_pcmcia_driver(&dev_info, &ide_attach, &ide_detach);
+    register_pccard_driver(&dev_info, &ide_attach, &ide_detach);
     return 0;
 }
 
-void cleanup_module(void)
+static void __exit exit_ide_cs(void)
 {
     DEBUG(0, "ide_cs: unloading\n");
-    unregister_pcmcia_driver(&dev_info);
+    unregister_pccard_driver(&dev_info);
     while (dev_list != NULL)
 	ide_detach(dev_list);
 }
+
+module_init(init_ide_cs);
+module_exit(exit_ide_cs);
