@@ -2,7 +2,7 @@
 
     A simple MTD for Intel Series 2+ Flash devices
 
-    iflash2+_mtd.c 1.51 1999/07/20 15:59:09
+    iflash2+_mtd.c 1.64 2000/05/16 21:31:36
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,8 +15,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
     For efficiency and simplicity, this driver is very block oriented.
     Reads and writes must not span erase block boundaries.  Erases
@@ -30,6 +41,8 @@
 
 #ifdef __LINUX__
 #include <linux/kernel.h>
+#include <linux/module.h>
+#include <linux/init.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
 #include <linux/malloc.h>
@@ -38,6 +51,7 @@
 #include <linux/timex.h>
 #include <linux/major.h>
 #include <linux/fs.h>
+#include <linux/delay.h>
 #include <asm/io.h>
 #include <asm/system.h>
 #include <asm/segment.h>
@@ -57,41 +71,29 @@
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
-#ifdef __LINUX__
-#define _printk(args...) printk(KERN_DEBUG args)
-#else
-#define _printk printk
-#endif
-#define DEBUG(n, args) do { if (pc_debug>(n)) _printk args; } while (0)
+#define DEBUG(n, args...) do { if (pc_debug>(n)) printk(KERN_INFO args); } while (0)
 static char *version =
-"iflash2+_mtd.c 1.51 1999/07/20 15:59:09 (David Hinds)";
+"iflash2+_mtd.c 1.64 2000/05/16 21:31:36 (David Hinds)";
 #else
-#define DEBUG(n, args) do { } while (0)
+#define DEBUG(n, args...) do { } while (0)
 #endif
 
 /*====================================================================*/
 
 /* Parameters that can be set with 'insmod' */
 
-static int word_width = 1;			/* 1 = 16-bit */
-static int mem_speed = 0;			/* in ns */
-static int vpp_timeout_period	= 1000;		/* in ms */
-static int vpp_settle		= 100;		/* in ms */
-static int write_timeout	= 100;		/* in ms */
-static int erase_timeout	= 100;		/* in ms */
-static int erase_limit		= 10000;	/* in ms */
-static int retry_limit		= 4;		/* write retries */
-static u_int max_tries       	= 4096;		/* status polling */
+#define INT_MODULE_PARM(n, v) static int n = v; MODULE_PARM(n, "i")
 
-MODULE_PARM(word_width, "i");
-MODULE_PARM(mem_speed, "i");
-MODULE_PARM(vpp_timeout_period, "i");
-MODULE_PARM(vpp_settle, "i");
-MODULE_PARM(write_timeout, "i");
-MODULE_PARM(erase_timeout, "i");
-MODULE_PARM(erase_limit, "i");
-MODULE_PARM(retry_limit, "i");
-MODULE_PARM(max_tries, "i");
+INT_MODULE_PARM(word_width, 1);			/* 1 = 16-bit */
+INT_MODULE_PARM(mem_speed, 0);			/* in ns */
+INT_MODULE_PARM(vpp_timeout_period, 1000);	/* in ms */
+INT_MODULE_PARM(vpp_settle, 100);		/* in ms */
+INT_MODULE_PARM(write_timeout, 100);		/* in ms */
+INT_MODULE_PARM(erase_timeout, 100);		/* in ms */
+INT_MODULE_PARM(erase_limit, 10000);		/* in ms */
+INT_MODULE_PARM(retry_limit, 8);		/* write retries */
+INT_MODULE_PARM(max_tries, 4096);		/* status polling */
+INT_MODULE_PARM(do_sleep, 1);			/* spin vs sleep? */
 
 /*====================================================================*/
 
@@ -119,6 +121,7 @@ typedef struct flash_region_t {
 } flash_region_t;
 
 typedef struct flash_dev_t {
+    dev_link_t		link;
     caddr_t		Base;
     u_int		Size;
     window_handle_t	ESRwin;
@@ -165,6 +168,14 @@ static inline k_time_t uticks(void)
     
 ======================================================================*/
 
+static void sleep_or_spin(wait_queue_head_t *queue)
+{
+    if (do_sleep)
+	wsleeptimeout(queue, 1);
+    else
+	udelay(50);
+}
+
 static void log_esr(char *s, volatile u_short *esr)
 {
     u_short CSR, GSR, BSR;
@@ -174,9 +185,8 @@ static void log_esr(char *s, volatile u_short *esr)
     writew(IF_READ_ESR, esr);
     BSR = readw(esr+2);
     GSR = readw(esr+4);
-    printk(KERN_NOTICE "%s%s", s ? s : "", s ? ": " : "");
-    printk("CSR = 0x%04x, BSR = 0x%04x, GSR = 0x%04x\n",
-	   CSR, BSR, GSR);
+    printk("%sCSR = 0x%04x, BSR = 0x%04x, GSR = 0x%04x\n",
+	   (s ? s : KERN_NOTICE), CSR, BSR, GSR);
 }
 
 static void abort_cmd(volatile u_short *esr)
@@ -195,70 +205,67 @@ static void abort_cmd(volatile u_short *esr)
 
 static int set_rdy_mode(volatile u_short *esr, u_short mode)
 {
-    u_short i;
+    u_short i, j;
 
-    writew(IF_READ_ESR, esr);
-    for (i = 0; i < max_tries; i++)
-	if (!(readw(esr+2) & BSR_QUEUE_FULL)) break;
-    if (i == max_tries) {
-	printk(KERN_NOTICE "iflash2+_mtd: set_rdy_mode timed out!\n");
-	log_esr(NULL, esr);
-	return CS_GENERAL_FAILURE;
+    DEBUG(3, "iflash2+_mtd: set_rdy_mode(%04x)\n", mode);
+    for (j = 0; j < retry_limit; j++) {
+	writew(IF_READ_ESR, esr);
+	for (i = 0; i < max_tries; i++)
+	    if (!(readw(esr+4) & GSR_QUEUE_FULL)) break;
+	if (i == max_tries)
+	    goto failed;
+	writew(IF_RDY_MODE, esr);
+	writew(mode, esr);
+	writew(IF_READ_ESR, esr);
+	for (i = 0; i < max_tries; i++)
+	    if ((readw(esr+4) & GSR_WR_READY) == GSR_WR_READY) break;
+	if (i == max_tries)
+	    goto failed;
+	if (!(readw(esr+4) & GSR_OP_ERR))
+	    return CS_SUCCESS;
+	writew(IF_READ_ARRAY, esr);
+	writew(IF_CLEAR_CSR, esr);
     }
-    writew(IF_RDY_MODE, esr);
-    writew(mode, esr);
-    writew(IF_READ_ESR, esr);
-    for (i = 0; i < max_tries; i++) {
-	if ((readw(esr+4) & GSR_WR_READY) == GSR_WR_READY) break;
-    }
-    if (i == max_tries) {
-	printk(KERN_NOTICE "iflash2+_mtd: set_rdy_mode timed out!\n");
-	log_esr(NULL, esr);
-	return CS_GENERAL_FAILURE;
-    }
-    if (readw(esr+4) & GSR_OP_ERR) {
-	printk(KERN_NOTICE "iflash2+_mtd: set_rdy_mode failed!\n");
-	return CS_GENERAL_FAILURE;
-    }
-    return CS_SUCCESS;
+failed:
+    printk(KERN_NOTICE "iflash2+_mtd: set_rdy_mode failed!\n");
+    log_esr(NULL, esr);
+    return CS_GENERAL_FAILURE;
 }
 
 static int check_write(wait_queue_head_t *queue, volatile u_short *esr)
 {
+    u_long end = jiffies + write_timeout;
     writew(IF_READ_ESR, esr);
+    while (((readw(esr+2) & BSR_READY) != BSR_READY) && 
+	   (jiffies < end))
+	sleep_or_spin(queue);
     if ((readw(esr+2) & BSR_READY) != BSR_READY) {
-	int to = 1;
-	while (to && ((readw(esr+2) & BSR_READY) != BSR_READY))
-	    to = wsleeptimeout(queue, write_timeout);
-	if ((readw(esr+2) & BSR_READY) != BSR_READY) {
-	    printk(KERN_NOTICE "iflash2+_mtd: check_write: timed out!\n");
-	    log_esr(NULL, esr);
-	    return CS_GENERAL_FAILURE;
-	}
+	printk(KERN_NOTICE "iflash2+_mtd: check_write: timed out!\n");
+	log_esr(NULL, esr);
+	return CS_GENERAL_FAILURE;
     }
     if (readw(esr+2) & BSR_FAILED) {
-	log_esr("write error", esr);
+	log_esr(KERN_DEBUG "write error: ", esr);
 	return CS_WRITE_FAILURE;
     } else
 	return CS_SUCCESS;
 }
 
 static int page_setup(wait_queue_head_t *queue, volatile u_short *esr,
-		      volatile u_short *address,  u_short count)
+		      volatile u_short *address, u_short count)
 {
+    u_long end = jiffies + write_timeout;
     u_short nw;
-
     writew(IF_READ_ESR, esr);
+    while (((readw(esr+4) & GSR_PAGE_AVAIL) != GSR_PAGE_AVAIL) &&
+	   (jiffies < end))
+	sleep_or_spin(queue);
     if ((readw(esr+4) & GSR_PAGE_AVAIL) != GSR_PAGE_AVAIL) {
-	int to = 1;
-	while (to && ((readw(esr+4) & GSR_PAGE_AVAIL) != GSR_PAGE_AVAIL))
-	    to = wsleeptimeout(queue, write_timeout);
-	if ((readw(esr+4) & GSR_PAGE_AVAIL) != GSR_PAGE_AVAIL) {
-	    printk(KERN_NOTICE "iflash2+_mtd: page_setup timed out\n");
-	    log_esr(NULL, esr);
-	    return CS_GENERAL_FAILURE;
-	}
+	printk(KERN_NOTICE "iflash2+_mtd: page_setup timed out\n");
+	log_esr(NULL, esr);
+	return CS_GENERAL_FAILURE;
     }
+
     nw = count >> 1;
     if (nw == 0) {
 	/* Special case of single byte write */
@@ -326,7 +333,7 @@ static int check_erase(volatile u_short *address)
     if ((CSR & CSR_WR_READY) != CSR_WR_READY) {
 	return CS_BUSY;
     } else if (CSR & (CSR_ERA_ERR | CSR_VPP_LOW | CSR_WR_ERR)) {
-	log_esr("erase error", address);
+	log_esr(KERN_NOTICE "erase error: ", address);
 	return CS_WRITE_FAILURE;
     } else
 	return CS_SUCCESS;
@@ -336,7 +343,7 @@ static int suspend_erase(volatile u_short *esr)
 {
     u_short i;
 
-    DEBUG(1, ("iflash2+_mtd: suspending erase...\n"));
+    DEBUG(1, "iflash2+_mtd: suspending erase...\n");
     writew(IF_ERASE_SUSPEND, esr);
     writew(IF_READ_ESR, esr);
     for (i = 0; i < max_tries; i++)
@@ -352,7 +359,7 @@ static int suspend_erase(volatile u_short *esr)
 
 static void resume_erase(volatile u_short *esr)
 {
-    DEBUG(1, ("iflash2+_mtd: resuming erase...\n"));
+    DEBUG(1, "iflash2+_mtd: resuming erase...\n");
     writew(IF_READ_ESR, esr);
     /* Only give resume signal if the erase is really suspended */
     if (readw(esr+4) & GSR_OP_SUSPEND)
@@ -402,13 +409,10 @@ static int vpp_setup(dev_link_t *link, mtd_request_t *req)
 
     /* First time for this request? */
     if (!(req->Function & MTD_REQ_TIMEOUT)) {
-	dev->vpp_usage++;
 	/* If no other users, kill shutdown timer and apply power */
-	if (dev->vpp_usage == 1) {
-	    if (dev->vpp_timeout.expires)
-		del_timer(&dev->vpp_timeout);
-	    else {
-		DEBUG(1, ("iflash2+_mtd: raising Vpp...\n"));
+	if (++dev->vpp_usage == 1) {
+	    if (!del_timer(&dev->vpp_timeout)) {
+		DEBUG(1, "iflash2+_mtd: raising Vpp...\n");
 		dev->vpp_start = jiffies;
 		vpp_req.Vpp1 = vpp_req.Vpp2 = 120;
 		MTDHelperEntry(MTDSetVpp, link->handle, &vpp_req);
@@ -430,7 +434,7 @@ static void vpp_off(u_long arg)
     flash_dev_t *dev;
     mtd_vpp_req_t req;
 
-    DEBUG(1, ("iflash2+_mtd: lowering Vpp...\n"));
+    DEBUG(1, "iflash2+_mtd: lowering Vpp...\n");
     dev = (flash_dev_t *)link->priv;
     dev->vpp_timeout.expires = 0;
     req.Vpp1 = req.Vpp2 = 0;
@@ -443,7 +447,7 @@ static void vpp_shutdown(dev_link_t *link)
     dev = (flash_dev_t *)link->priv;
     dev->vpp_usage--;
     if (dev->vpp_usage == 0) {
-	dev->vpp_timeout.expires = RUN_AT(vpp_timeout_period);
+	dev->vpp_timeout.expires = jiffies + vpp_timeout_period;
 	add_timer(&dev->vpp_timeout);
     }
 }
@@ -463,19 +467,20 @@ static dev_link_t *flash_attach(void)
     flash_dev_t *dev;
     int ret;
     
-    DEBUG(0, ("iflash2+_mtd: flash_attach()\n"));
+    DEBUG(0, "iflash2+_mtd: flash_attach()\n");
 
     /* Create new memory card device */
-    link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
-    memset(link, 0, sizeof(*link));
+    dev = kmalloc(sizeof(*dev), GFP_KERNEL);
+    if (!dev) return NULL;
+    memset(dev, 0, sizeof(*dev));
+    link = &dev->link; link->priv = dev;
+    
     link->release.function = &flash_release;
     link->release.data = (u_long)link;
     init_waitqueue_head(&link->pending);
-    dev = kmalloc(sizeof(struct flash_dev_t), GFP_KERNEL);
-    memset(dev, 0, sizeof(*dev));
+    
     dev->vpp_timeout.function = vpp_off;
     dev->vpp_timeout.data = (u_long)link;
-    link->priv = dev;
 
     /* Register with Card Services */
     link->next = dev_list;
@@ -514,7 +519,7 @@ static void flash_detach(dev_link_t *link)
     dev_link_t **linkp;
     int ret;
 
-    DEBUG(0, ("iflash2+_mtd: flash_detach(0x%p)\n", link));
+    DEBUG(0, "iflash2+_mtd: flash_detach(0x%p)\n", link);
     
     /* Locate device structure */
     for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
@@ -522,9 +527,7 @@ static void flash_detach(dev_link_t *link)
     if (*linkp == NULL)
 	return;
 
-    if (link->state & DEV_RELEASE_PENDING)
-	del_timer(&link->release);
-    
+    del_timer(&link->release);
     if (link->state & DEV_CONFIG)
 	flash_release((u_long)link);
 
@@ -536,8 +539,7 @@ static void flash_detach(dev_link_t *link)
     
     /* Unlink device structure, free bits */
     *linkp = link->next;
-    kfree_s(link->priv, sizeof(struct flash_dev_t));
-    kfree_s(link, sizeof(struct dev_link_t));
+    kfree(link->priv);
     
 } /* flash_detach */
 
@@ -560,13 +562,14 @@ static void printk_size(u_int sz)
 
 static void flash_config(dev_link_t *link)
 {
-    flash_dev_t *dev;
+    client_handle_t handle = link->handle;
+    flash_dev_t *dev = link->priv;
     win_req_t req;
     mtd_reg_t reg;
     region_info_t region;
     int i, attr, ret;
 
-    DEBUG(0, ("iflash2+_mtd: flash_config(0x%p)\n", link));
+    DEBUG(0, "iflash2+_mtd: flash_config(0x%p)\n", link);
 
     /* Allocate a small memory window */
     if (word_width)
@@ -575,24 +578,23 @@ static void flash_config(dev_link_t *link)
 	req.Attributes = WIN_DATA_WIDTH_8;
     req.Base = req.Size = 0;
     req.AccessSpeed = mem_speed;
-    link->win = (window_handle_t)link->handle;
+    link->win = (window_handle_t)handle;
     ret = MTDHelperEntry(MTDRequestWindow, &link->win, &req);
     if (ret != 0) {
-	cs_error(link->handle, RequestWindow, ret);
+	cs_error(handle, RequestWindow, ret);
 	link->state &= ~DEV_CONFIG_PENDING;
 	flash_release((u_long)link);
 	return;
     }
-    dev = link->priv;
     dev->Base = ioremap(req.Base, req.Size);
     dev->Size = req.Size;
 
     /* Allocate a memory window for ESR accesses*/
     req.Base = 0;
-    dev->ESRwin = (window_handle_t)link->handle;
+    dev->ESRwin = (window_handle_t)handle;
     ret = MTDHelperEntry(MTDRequestWindow, &dev->ESRwin, &req);
     if (ret != 0) {
-	cs_error(link->handle, RequestWindow, ret);
+	cs_error(handle, RequestWindow, ret);
 	link->state &= ~DEV_CONFIG_PENDING;
 	flash_release((u_long)link);
 	return;
@@ -605,22 +607,25 @@ static void flash_config(dev_link_t *link)
     i = 0;
     for (attr = 0; attr < 2; attr++) {
 	region.Attributes = attr ? REGION_TYPE_AM : REGION_TYPE_CM;
-	ret = CardServices(GetFirstRegion, link->handle, &region);
+	ret = CardServices(GetFirstRegion, handle, &region);
 	while (ret == CS_SUCCESS) {
 	    reg.Attributes = region.Attributes;
 	    reg.Offset = region.CardOffset;
 	    dev->flash[i] = kmalloc(sizeof(struct flash_region_t),
 				    GFP_KERNEL);
+	    if (!dev->flash[i]) break;
 	    reg.MediaID = (u_long)dev->flash[i];
-	    ret = CardServices(RegisterMTD, link->handle, &reg);
+	    ret = CardServices(RegisterMTD, handle, &reg);
 	    if (ret != 0) {
-		kfree_s(dev->flash[i], sizeof(struct flash_region_t));
+		kfree(dev->flash[i]);
 		break;
 	    }
 	    printk(KERN_INFO "iflash2+_mtd: %s at 0x%x, ",
 		   attr ? "attr" : "common", region.CardOffset);
 	    printk_size(region.RegionSize);
-	    printk(", %u ns\n", region.AccessSpeed);
+	    printk(", ");
+	    printk_size(region.BlockSize);
+	    printk(" blocks, %u ns\n", region.AccessSpeed);
 	    memset(dev->flash[i], 0, sizeof(struct flash_region_t));
 	    dev->flash[i]->region = region;
 	    /* Distinguish between 4MB..20MB cards and 40MB cards */
@@ -629,7 +634,7 @@ static void flash_config(dev_link_t *link)
 	    else
 		dev->flash[i]->cell_size = 0x400000; /* 4MB components */
 	    i++;
-	    ret = CardServices(GetNextRegion, link->handle, &region);
+	    ret = CardServices(GetNextRegion, handle, &region);
 	}
     }
     dev->flash[i] = NULL;
@@ -646,13 +651,12 @@ static void flash_config(dev_link_t *link)
 static void flash_release(u_long arg)
 {
     dev_link_t *link = (dev_link_t *)arg;
-    flash_dev_t *dev;
+    flash_dev_t *dev = link->priv;
     int i;
 
-    DEBUG(0, ("iflash2+_mtd: flash_release(0x%p)\n", link));
+    DEBUG(0, "iflash2+_mtd: flash_release(0x%p)\n", link);
 
     link->state &= ~DEV_CONFIG;
-    dev = link->priv;
     if (link->win) {
 	iounmap(dev->Base);
 	i = MTDHelperEntry(MTDReleaseWindow, link->win);
@@ -668,10 +672,8 @@ static void flash_release(u_long arg)
     if (dev->vpp_usage == 0)
 	del_timer(&dev->vpp_timeout);
     vpp_off((u_long)link);
-    for (i = 0; i < 2*CISTPL_MAX_DEVICES; i++)
-	if (dev->flash[i])
-	    kfree_s(dev->flash[i], sizeof(struct flash_region_t));
-	else break;
+    for (i = 0; (i < 2*CISTPL_MAX_DEVICES) && dev->flash[i]; i++)
+	kfree(dev->flash[i]);
     
     if (link->state & DEV_STALE_LINK)
 	flash_detach(link);
@@ -698,9 +700,9 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
     k_time_t time;
 #endif
     
-    DEBUG(2, ("iflash2+_mtd: flash_read(0x%p, 0x%lx, 0x%p, 0x%x, "
-	      "0x%x)\n", link, req->MediaID, buf, req->SrcCardOffset,
-	      req->TransferLength));
+    DEBUG(2, "iflash2+_mtd: flash_read(0x%p, 0x%lx, 0x%p, 0x%x, "
+	  "0x%x)\n", link, req->MediaID, buf, req->SrcCardOffset,
+	  req->TransferLength);
 
     flash = (flash_region_t *)(req->MediaID);
     region = &flash->region;
@@ -718,7 +720,7 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
     if (flash->cell[cell].state & FLASH_ERASING) {
 	if ((flash->cell[cell].erase_addr / region->BlockSize) ==
 	    (req->SrcCardOffset / region->BlockSize)) {
-	    DEBUG(1, ("iflash2+_mtd: delaying read...\n"));
+	    DEBUG(1, "iflash2+_mtd: delaying read...\n");
 	    req->Status = MTD_WAITREQ;
 	    return CS_BUSY;
 	}
@@ -758,10 +760,10 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
 #ifdef BENCHMARK
     time = uticks() - time;
     if (time < 10000000)
-	DEBUG(3, ("iflash2+_mtd: read complete, time = %ld,"
-		  " avg = %ld ns/word, rate = %ld kb/sec\n", time,
-		  time*2000/req->TransferLength,
-		  req->TransferLength*977/time));
+	DEBUG(3, "iflash2+_mtd: read complete, time = %ld,"
+	      " avg = %ld ns/word, rate = %ld kb/sec\n", time,
+	      time*2000/req->TransferLength,
+	      req->TransferLength*977/time);
 #endif
     
 done:
@@ -788,12 +790,12 @@ static int basic_write(wait_queue_head_t *queue, char *esr, char *dest,
     int ret;
 
     /* Enable interrupts on write complete */
-    ret = set_rdy_mode((u_short *)esr, IF_RDY_PULSE_WRITE);
+    ret = set_rdy_mode((u_short *)esr, IF_RDY_LEVEL);
     if (ret != CS_SUCCESS) return ret;
     
     /* Fix for mis-aligned writes */
     if ((u_long)dest & 1) {
-	DEBUG(2, ("iflash2+_mtd: odd address fixup at 0x%p\n", dest));
+	DEBUG(2, "iflash2+_mtd: odd address fixup at 0x%p\n", dest);
 	ret = page_setup(queue, (u_short *)esr, (u_short *)dest, 1);
 	if (ret != CS_SUCCESS) return ret;
 	if (is_krnl)
@@ -809,7 +811,6 @@ static int basic_write(wait_queue_head_t *queue, char *esr, char *dest,
     }
     
     for (; nb > 0; nb -= npb) {
-	    
 	/* npb = # of bytes to write to page buffer */
 	npb = (nb > 512) ? 512 : nb;
 	/* sleep until page buffer is free */
@@ -823,6 +824,7 @@ static int basic_write(wait_queue_head_t *queue, char *esr, char *dest,
 	
 	ret = page_write((u_short *)esr, (u_short *)dest, npb);
 	if (ret != CS_SUCCESS) return ret;
+	check_write(queue, (u_short *)esr);
 	
 	dest += npb;
 	buf += npb;
@@ -853,9 +855,9 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
     k_time_t time;
 #endif
 
-    DEBUG(2, ("iflash2+_mtd: flash_write(0x%p, 0x%lx, "
-	      "0x%p, 0x%x, 0x%x)\n", link, req->MediaID, buf,
-	      req->DestCardOffset, req->TransferLength));
+    DEBUG(2, "iflash2+_mtd: flash_write(0x%p, 0x%lx, "
+	  "0x%p, 0x%x, 0x%x)\n", link, req->MediaID, buf,
+	  req->DestCardOffset, req->TransferLength);
 
     /* Check card write protect status */
     ret = CardServices(GetStatus, link->handle, &status);
@@ -878,7 +880,7 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
     /* Is this cell being erased or written? */
     cell = (req->DestCardOffset - region->CardOffset) / flash->cell_size;
     if (flash->cell[cell].state & FLASH_ERASING) {
-	DEBUG(1, ("iflash2+_mtd: delaying write...\n"));
+	DEBUG(1, "iflash2+_mtd: delaying write...\n");
 	req->Status = MTD_WAITREQ;
 	return CS_BUSY;
     }
@@ -919,8 +921,8 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
 	    abort_cmd((u_short *)dev->ESRbase);
 	}
 	if (retry == retry_limit) {
-	    printk(KERN_DEBUG "iflash2+_mtd: write failed: "
-		   "too many retries!\n");
+	    printk(KERN_NOTICE "iflash2+_mtd: write at 0x%06x failed:"
+		   " too many retries!\n", mod.CardOffset);
 	    goto done;
 	}
 	
@@ -932,10 +934,10 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
 #ifdef BENCHMARK
     time = uticks() - time;
     if (time < 10000000)
-	DEBUG(3, ("iflash2+_mtd: write complete, time = %ld,"
-		  " avg = %ld us/word, rate = %ld kb/sec\n", time,
-		  time*2/req->TransferLength,
-		  req->TransferLength*977/time));
+	DEBUG(3, "iflash2+_mtd: write complete, time = %ld,"
+	      " avg = %ld us/word, rate = %ld kb/sec\n", time,
+	      time*2/req->TransferLength,
+	      req->TransferLength*977/time);
 #endif
     
 done:
@@ -964,9 +966,9 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
     mtd_mod_win_t mod;
     int i, ret;
 
-    DEBUG(2, ("iflash2+_mtd: flash_erase(0x%p, 0x%lx, 0x%x, 0x%x)\n",
-	      link, req->MediaID, req->DestCardOffset,
-	      req->TransferLength));
+    DEBUG(2, "iflash2+_mtd: flash_erase(0x%p, 0x%lx, 0x%x, 0x%x)\n",
+	  link, req->MediaID, req->DestCardOffset,
+	  req->TransferLength);
 
     flash = (flash_region_t *)(req->MediaID);
     region = &flash->region;
@@ -977,7 +979,7 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
     
     if (!(req->Function & MTD_REQ_TIMEOUT)) {
 	if (flash->cell[i].state & (FLASH_ERASING|FLASH_PENDING)) {
-	    DEBUG(1, ("iflash2+_mtd: delaying erase...\n"));
+	    DEBUG(1, "iflash2+_mtd: delaying erase...\n");
 	    req->Status = MTD_WAITREQ;
 	    return CS_BUSY;
 	}
@@ -1047,8 +1049,8 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
     return CS_BUSY;
     
 done:
-    DEBUG(2, ("iflash2+_mtd: erase complete, time = %ld\n",
-	      jiffies - flash->cell[i].erase_time));
+    DEBUG(2, "iflash2+_mtd: erase complete, time = %ld\n",
+	  jiffies - flash->cell[i].erase_time);
     flash->cell[i].state &= ~(FLASH_ERASING|FLASH_PENDING);
     reset_block((u_short *)dev->ESRbase);
     set_global_lock(dev->ESRwin, dev->ESRbase, 1);
@@ -1110,17 +1112,14 @@ static int flash_event(event_t event, int priority,
 {
     dev_link_t *link = args->client_data;
 
-    DEBUG(2, ("iflash2+_mtd: flash_event(0x%06x)\n", event));
+    DEBUG(3, "iflash2+_mtd: flash_event(0x%06x)\n", event);
     
     switch (event) {
 	
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
-	if (link->state & DEV_CONFIG) {
-	    link->release.expires = RUN_AT(HZ/20);
-	    link->state |= DEV_RELEASE_PENDING;
-	    add_timer(&link->release);
-	}
+	if (link->state & DEV_CONFIG)
+	    mod_timer(&link->release, jiffies + HZ/20);
 	break;
 	
     case CS_EVENT_CARD_INSERTION:
@@ -1156,11 +1155,11 @@ static int flash_event(event_t event, int priority,
 
 #ifdef __LINUX__
 
-int init_module(void)
+static int __init init_iflash2x_mtd(void)
 {
     servinfo_t serv;
     
-    DEBUG(0, ("%s\n", version));
+    DEBUG(0, "%s\n", version);
     
     /* Rescale parameters */
     vpp_timeout_period = (vpp_timeout_period * HZ) / 1000;
@@ -1175,17 +1174,20 @@ int init_module(void)
 	return -1;
     }
     
-    register_pcmcia_driver(&dev_info, &flash_attach, &flash_detach);
+    register_pccard_driver(&dev_info, &flash_attach, &flash_detach);
 
     return 0;
 }
 
-void cleanup_module(void)
+static void __exit exit_iflash2x_mtd(void)
 {
-    DEBUG(0, ("iflash2+_mtd: unloading\n"));
-    unregister_pcmcia_driver(&dev_info);
+    DEBUG(0, "iflash2+_mtd: unloading\n");
+    unregister_pccard_driver(&dev_info);
     while (dev_list != NULL)
 	flash_detach(dev_list);
 }
 
+module_init(init_iflash2x_mtd);
+module_exit(exit_iflash2x_mtd);
+
 #endif /* __LINUX__ */
