@@ -2,7 +2,7 @@
 
     A simple MTD for Intel Series 2 and Series 100 Flash devices
 
-    iflash2_mtd.c 1.43 1999/07/20 15:59:13
+    iflash2_mtd.c 1.55 2000/05/16 21:31:36
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,8 +15,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
     For efficiency and simplicity, this driver is very block oriented.
     Reads and writes must not span erase block boundaries.  Erases
@@ -30,6 +41,8 @@
 
 #ifdef __LINUX__
 #include <linux/kernel.h>
+#include <linux/module.h>
+#include <linux/init.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
 #include <linux/malloc.h>
@@ -58,16 +71,11 @@
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
-#ifdef __LINUX__
-#define _printk(args...) printk(KERN_DEBUG args)
-#else
-#define _printk printk
-#endif
-#define DEBUG(n, args) do { if (pc_debug>(n)) _printk args; } while (0)
+#define DEBUG(n, args...) do { if (pc_debug>(n)) printk(KERN_INFO args); } while (0)
 static char *version =
-"iflash2_mtd.c 1.43 1999/07/20 15:59:13 (David Hinds)";
+"iflash2_mtd.c 1.55 2000/05/16 21:31:36 (David Hinds)";
 #else
-#define DEBUG(n, args) do { } while (0)
+#define DEBUG(n, args...) do { } while (0)
 #endif
 
 /*====================================================================*/
@@ -122,6 +130,7 @@ typedef struct flash_region_t {
 } flash_region_t;
 
 typedef struct flash_dev_t {
+    dev_link_t		link;
     caddr_t		Base;
     u_int		Size;
     u_int		vpp;
@@ -171,7 +180,7 @@ static void abort_cmd(dev_link_t *link, volatile caddr_t base,
     u_char *addr = base + CISREG_SLEEP + ((cell>>3)<<1);
     /* Map the CIS register page into memory */
     mtd_mod_win_t mod = { WIN_MEMORY_TYPE_AM, 250, 0x4000 };
-    DEBUG(1, ("iflash2_mtd: abort\n"));
+    DEBUG(1, "iflash2_mtd: abort\n");
     MTDHelperEntry(MTDModifyWindow, link->win, &mod);
     /* Put the cell to sleep, then wake it up again */
     writeb(readb(addr) | (1 << (cell & 7)), addr);
@@ -332,13 +341,10 @@ static int vpp_setup(dev_link_t *link, mtd_request_t *req)
     
     /* First time for this request? */
     if (!(req->Function & MTD_REQ_TIMEOUT)) {
-	dev->vpp_usage++;
 	/* If no other users, kill shutdown timer and apply power */
-	if (dev->vpp_usage == 1) {
-	    if (dev->vpp_timeout.expires)
-		del_timer(&dev->vpp_timeout);
-	    else {
-		DEBUG(1, ("iflash2_mtd: raising Vpp...\n"));
+	if (++dev->vpp_usage == 1) {
+	    if (!del_timer(&dev->vpp_timeout)) {
+		DEBUG(1, "iflash2_mtd: raising Vpp...\n");
 		dev->vpp_start = jiffies;
 		vpp_req.Vpp1 = vpp_req.Vpp2 = dev->vpp;
 		MTDHelperEntry(MTDSetVpp, link->handle, &vpp_req);
@@ -361,7 +367,7 @@ static void vpp_off(u_long arg)
     mtd_vpp_req_t req;
 
     if (!dev->vpp) return;
-    DEBUG(1, ("iflash2_mtd: lowering Vpp...\n"));
+    DEBUG(1, "iflash2_mtd: lowering Vpp...\n");
     dev->vpp_timeout.expires = 0;
     req.Vpp1 = req.Vpp2 = 0;
     MTDHelperEntry(MTDSetVpp, link->handle, &req);
@@ -373,7 +379,7 @@ static void vpp_shutdown(dev_link_t *link)
     if (!dev->vpp) return;
     dev->vpp_usage--;
     if (dev->vpp_usage == 0) {
-	dev->vpp_timeout.expires = RUN_AT(vpp_timeout_period);
+	dev->vpp_timeout.expires = jiffies + vpp_timeout_period;
 	add_timer(&dev->vpp_timeout);
     }
 }
@@ -393,18 +399,19 @@ static dev_link_t *flash_attach(void)
     flash_dev_t *dev;
     int ret;
     
-    DEBUG(0, ("iflash2_mtd: flash_attach()\n"));
+    DEBUG(0, "iflash2_mtd: flash_attach()\n");
 
     /* Create new memory card device */
-    link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
-    memset(link, 0, sizeof(*link));
+    dev = kmalloc(sizeof(*dev), GFP_KERNEL);
+    if (!dev) return NULL;
+    memset(dev, 0, sizeof(*dev));
+    link = &dev->link; link->priv = dev;
+
     link->release.function = &flash_release;
     link->release.data = (u_long)link;
-    dev = kmalloc(sizeof(struct flash_dev_t), GFP_KERNEL);
-    memset(dev, 0, sizeof(*dev));
+
     dev->vpp_timeout.function = vpp_off;
     dev->vpp_timeout.data = (u_long)link;
-    link->priv = dev;
 
     /* Register with Card Services */
     link->next = dev_list;
@@ -442,7 +449,7 @@ static void flash_detach(dev_link_t *link)
     dev_link_t **linkp;
     int ret;
 
-    DEBUG(0, ("iflash2_mtd: flash_detach(0x%p)\n", link));
+    DEBUG(0, "iflash2_mtd: flash_detach(0x%p)\n", link);
     
     /* Locate device structure */
     for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
@@ -450,9 +457,7 @@ static void flash_detach(dev_link_t *link)
     if (*linkp == NULL)
 	return;
 
-    if (link->state & DEV_RELEASE_PENDING)
-	del_timer(&link->release);
-    
+    del_timer(&link->release);
     if (link->state & DEV_CONFIG)
 	flash_release((u_long)link);
 
@@ -464,8 +469,7 @@ static void flash_detach(dev_link_t *link)
     
     /* Unlink device structure, free bits */
     *linkp = link->next;
-    kfree_s(link->priv, sizeof(struct flash_dev_t));
-    kfree_s(link, sizeof(struct dev_link_t));
+    kfree(link->priv);
     
 } /* flash_detach */
 
@@ -488,13 +492,14 @@ static void printk_size(u_int sz)
 
 static void flash_config(dev_link_t *link)
 {
-    flash_dev_t *dev;
+    client_handle_t handle = link->handle;
+    flash_dev_t *dev = link->priv;
     win_req_t req;
     mtd_reg_t reg;
     region_info_t region;
     int i, attr, ret;
 
-    DEBUG(0, ("iflash2_mtd: flash_config(0x%p)\n", link));
+    DEBUG(0, "iflash2_mtd: flash_config(0x%p)\n", link);
 
     /* Allocate a small memory window */
     if (word_width)
@@ -503,15 +508,14 @@ static void flash_config(dev_link_t *link)
 	req.Attributes = WIN_DATA_WIDTH_8;
     req.Base = req.Size = 0;
     req.AccessSpeed = mem_speed;
-    link->win = (window_handle_t)link->handle;
+    link->win = (window_handle_t)handle;
     ret = MTDHelperEntry(MTDRequestWindow, &link->win, &req);
     if (ret != 0) {
-	cs_error(link->handle, RequestWindow, ret);
+	cs_error(handle, RequestWindow, ret);
 	link->state &= ~DEV_CONFIG_PENDING;
 	flash_release((u_long)link);
 	return;
     }
-    dev = link->priv;
     dev->Base = ioremap(req.Base, req.Size);
     dev->Size = req.Size;
 
@@ -521,22 +525,25 @@ static void flash_config(dev_link_t *link)
     i = 0;
     for (attr = 0; attr < 2; attr++) {
 	region.Attributes = attr ? REGION_TYPE_AM : REGION_TYPE_CM;
-	ret = CardServices(GetFirstRegion, link->handle, &region);
+	ret = CardServices(GetFirstRegion, handle, &region);
 	while (ret == CS_SUCCESS) {
 	    reg.Attributes = region.Attributes;
 	    reg.Offset = region.CardOffset;
 	    dev->flash[i] = kmalloc(sizeof(struct flash_region_t),
 				    GFP_KERNEL);
+	    if (!dev->flash[i]) break;
 	    reg.MediaID = (u_long)dev->flash[i];
-	    ret = CardServices(RegisterMTD, link->handle, &reg);
+	    ret = CardServices(RegisterMTD, handle, &reg);
 	    if (ret != 0) {
-		kfree_s(dev->flash[i], sizeof(struct flash_region_t));
+		kfree(dev->flash[i]);
 		break;
 	    }
 	    printk(KERN_INFO "iflash2_mtd: %s at 0x%x, ",
 		   attr ? "attr" : "common", region.CardOffset);
 	    printk_size(region.RegionSize);
-	    printk(", %u ns\n", region.AccessSpeed);
+	    printk(", ");
+	    printk_size(region.BlockSize);
+	    printk(" blocks, %u ns\n", region.AccessSpeed);
 	    memset(dev->flash[i], 0, sizeof(struct flash_region_t));
 	    /* Assume 128K blocks, if no geometry info present */
 	    if (region.BlockSize == 1)
@@ -547,7 +554,7 @@ static void flash_config(dev_link_t *link)
 	    /* All Series 2 cards have 2MB component pairs */
 	    dev->flash[i]->cell_size = 0x200000;
 	    i++;
-	    ret = CardServices(GetNextRegion, link->handle, &region);
+	    ret = CardServices(GetNextRegion, handle, &region);
 	}
     }
     dev->flash[i] = NULL;
@@ -564,13 +571,12 @@ static void flash_config(dev_link_t *link)
 static void flash_release(u_long arg)
 {
     dev_link_t *link = (dev_link_t *)arg;
-    flash_dev_t *dev;
+    flash_dev_t *dev = link->priv;
     int i;
 
-    DEBUG(0, ("iflash2_mtd: flash_release(0x%p)\n", link));
+    DEBUG(0, "iflash2_mtd: flash_release(0x%p)\n", link);
 
     link->state &= ~DEV_CONFIG;
-    dev = link->priv;
     if (link->win) {
 	iounmap(dev->Base);
 	i = MTDHelperEntry(MTDReleaseWindow, link->win);
@@ -580,10 +586,8 @@ static void flash_release(u_long arg)
     if (dev->vpp_usage == 0)
 	del_timer(&dev->vpp_timeout);
     vpp_off((u_long)link);
-    for (i = 0; i < 2*CISTPL_MAX_DEVICES; i++)
-	if (dev->flash[i])
-	    kfree_s(dev->flash[i], sizeof(struct flash_region_t));
-	else break;
+    for (i = 0; (i < 2*CISTPL_MAX_DEVICES) && dev->flash[i]; i++)
+	kfree(dev->flash[i]);
     
     if (link->state & DEV_STALE_LINK)
 	flash_detach(link);
@@ -608,9 +612,9 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
     u_long time;
     int ret;
     
-    DEBUG(2, ("iflash2_mtd: flash_read(0x%p, 0x%lx, 0x%p, 0x%x, "
-	      "0x%x)\n", link, req->MediaID, buf, req->SrcCardOffset,
-	      req->TransferLength));
+    DEBUG(2, "iflash2_mtd: flash_read(0x%p, 0x%lx, 0x%p, 0x%x, "
+	  "0x%x)\n", link, req->MediaID, buf, req->SrcCardOffset,
+	  req->TransferLength);
 
     flash = (flash_region_t *)(req->MediaID);
     region = &flash->region;
@@ -665,10 +669,10 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
 #ifdef PCMCIA_DEBUG
     time = jiffies - time;
     if (time > 1)
-	DEBUG(3, ("iflash2_mtd: read complete, time = %ld, "
-		  "avg = %ld ns/word, rate = %ld kb/sec\n", time,
-		  time*20000000/req->TransferLength,
-		  req->TransferLength*100/(time*1024)));
+	DEBUG(3, "iflash2_mtd: read complete, time = %ld, "
+	      "avg = %ld ns/word, rate = %ld kb/sec\n", time,
+	      time*20000000/req->TransferLength,
+	      req->TransferLength*100/(time*1024));
 #endif
     
 done:
@@ -744,9 +748,9 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
     cs_status_t status;
     int ret;
 
-    DEBUG(2, ("iflash2_mtd: flash_write(0x%p, 0x%lx, 0x%p, 0x%x, "
-	      "0x%x)\n", link, req->MediaID, buf, req->DestCardOffset,
-	      req->TransferLength));
+    DEBUG(2, "iflash2_mtd: flash_write(0x%p, 0x%lx, 0x%p, 0x%x, "
+	  "0x%x)\n", link, req->MediaID, buf, req->DestCardOffset,
+	  req->TransferLength);
 
     /* Check card write protect status */
     ret = CardServices(GetStatus, link->handle, &status);
@@ -811,10 +815,10 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
 #ifdef PCMCIA_DEBUG
     time = jiffies - time;
     if (time > 1)
-	DEBUG(3, ("iflash2_mtd: write complete, time = %ld, "
-		  "avg = %ld us/word, rate = %ld kb/sec\n", time,
-		  time*20000/req->TransferLength,
-		  req->TransferLength*100/(time*1024)));
+	DEBUG(3, "iflash2_mtd: write complete, time = %ld, "
+	      "avg = %ld us/word, rate = %ld kb/sec\n", time,
+	      time*20000/req->TransferLength,
+	      req->TransferLength*100/(time*1024));
 #endif
     
 done:
@@ -841,9 +845,9 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
     mtd_mod_win_t mod;
     int i, ret;
 
-    DEBUG(2, ("iflash2_mtd: flash_erase(0x%p, 0x%lx, 0x%x, 0x%x)\n",
-	      link, req->MediaID, req->DestCardOffset,
-	      req->TransferLength));
+    DEBUG(2, "iflash2_mtd: flash_erase(0x%p, 0x%lx, 0x%x, 0x%x)\n",
+	  link, req->MediaID, req->DestCardOffset,
+	  req->TransferLength);
 
     flash = (flash_region_t *)(req->MediaID);
     region = &flash->region;
@@ -920,8 +924,8 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
     return CS_BUSY;
     
 done:
-    DEBUG(2, ("iflash2_mtd: erase complete, time = %ld\n",
-	      jiffies - flash->cell[i].erase_time));
+    DEBUG(2, "iflash2_mtd: erase complete, time = %ld\n",
+	  jiffies - flash->cell[i].erase_time);
     flash->cell[i].state &= ~(FLASH_ERASING|FLASH_PENDING);
     reset_block((u_short *)dev->Base);
     vpp_shutdown(link);
@@ -982,17 +986,14 @@ static int flash_event(event_t event, int priority,
 {
     dev_link_t *link = args->client_data;
 
-    DEBUG(1, ("iflash2_mtd: flash_event(0x%06x)\n", event));
+    DEBUG(1, "iflash2_mtd: flash_event(0x%06x)\n", event);
     
     switch (event) {
 	
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
-	if (link->state & DEV_CONFIG) {
-	    link->release.expires = RUN_AT(HZ/20);
-	    link->state |= DEV_RELEASE_PENDING;
-	    add_timer(&link->release);
-	}
+	if (link->state & DEV_CONFIG)
+	    mod_timer(&link->release, jiffies + HZ/20);
 	break;
 	
     case CS_EVENT_CARD_INSERTION:
@@ -1024,11 +1025,11 @@ static int flash_event(event_t event, int priority,
 
 #ifdef __LINUX__
 
-int init_module(void)
+static int __init init_iflash2_mtd(void)
 {
     servinfo_t serv;
     
-    DEBUG(0, ("%s\n", version));
+    DEBUG(0, "%s\n", version);
 
     /* Rescale parameters */
     vpp_timeout_period = (vpp_timeout_period * HZ) / 1000;
@@ -1042,19 +1043,22 @@ int init_module(void)
 	return -1;
     }
     
-    register_pcmcia_driver(&dev_info, &flash_attach, &flash_detach);
+    register_pccard_driver(&dev_info, &flash_attach, &flash_detach);
 
     return 0;
 }
 
-void cleanup_module(void)
+static void __exit exit_iflash2_mtd(void)
 {
-    DEBUG(0, ("iflash2_mtd: unloading\n"));
-    unregister_pcmcia_driver(&dev_info);
+    DEBUG(0, "iflash2_mtd: unloading\n");
+    unregister_pccard_driver(&dev_info);
     while (dev_list != NULL)
 	flash_detach(dev_list);
 }
 
+module_init(init_iflash2_mtd);
+module_exit(exit_iflash2_mtd);
+
 #endif /* __LINUX__ */
 
 /*====================================================================*/
@@ -1064,7 +1068,7 @@ void cleanup_module(void)
 static status_t std_ops(int32 op)
 {
     int ret;
-    DEBUG(0, ("iflash2_mtd: std_ops(%d)\n", op));
+    DEBUG(0, "iflash2_mtd: std_ops(%d)\n", op);
     switch (op) {
     case B_MODULE_INIT:
 	vpp_timeout_period = (vpp_timeout_period * HZ) / 1000;
@@ -1076,10 +1080,10 @@ static status_t std_ops(int32 op)
 	if (ret != B_OK) return ret;
 	ret = get_module(B_ISA_MODULE_NAME, (struct module_info **)&isa);
 	if (ret != B_OK) return ret;
-	register_pcmcia_driver(&dev_info, &flash_attach, &flash_detach);
+	register_pccard_driver(&dev_info, &flash_attach, &flash_detach);
 	break;
     case B_MODULE_UNINIT:
-	unregister_pcmcia_driver(&dev_info);
+	unregister_pccard_driver(&dev_info);
 	while (dev_list != NULL)
 	    flash_detach(dev_list);
 	if (isa) put_module(B_ISA_MODULE_NAME);
