@@ -2,7 +2,7 @@
 
     A direct memory interface driver for CardBus cards
 
-    memory_cb.c 1.6 1999/07/30 03:49:13
+    memory_cb.c 1.15 2000/05/16 21:31:36
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,8 +15,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
@@ -24,12 +35,15 @@
 #include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
+#include <linux/module.h>
+#include <linux/init.h>
 #include <linux/sched.h>
 #include <linux/malloc.h>
 #include <linux/string.h>
-#include <linux/timer.h>
 #include <linux/ioport.h>
 #include <linux/major.h>
+#include <linux/pci.h>
+#include <asm/io.h>
 
 #include <pcmcia/driver_ops.h>
 #include <pcmcia/mem_op.h>
@@ -39,7 +53,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"memory_cb.c 1.6 1999/07/30 03:49:13 (David Hinds)";
+"memory_cb.c 1.15 2000/05/16 21:31:36 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -51,12 +65,12 @@ static char *version =
 /*====================================================================*/
 
 typedef struct memory_dev_t {
-    dev_node_t	node;
-    u_char	bus, devfn;
-    u_int	open, stopped;
-    u_int	base[8];
-    u_int	size[8];
-    u_char	*virt[8];
+    dev_node_t		node;
+    struct pci_dev	*pdev;
+    u_int		open, stopped;
+    u_int		base[8];
+    u_int		size[8];
+    u_char		*virt[8];
 } memory_dev_t;
 
 #define MAX_DEV 8
@@ -78,29 +92,29 @@ static dev_node_t *memory_attach(dev_locator_t *loc)
     
     if (loc->bus != LOC_PCI) return NULL;
     bus = loc->b.pci.bus; devfn = loc->b.pci.devfn;
-    printk(KERN_INFO "memory_attach(bus %d, function %d)\n",
-	   bus, devfn);
+    printk(KERN_INFO "memory_attach(device %02x:%02x.%d)\n",
+	   bus, PCI_SLOT(devfn), PCI_FUNC(devfn));
 
     for (i = 0; i < MAX_DEV; i++)
 	if (dev_table[i] == NULL) break;
     if (i == MAX_DEV) return NULL;
     dev_table[i] = dev = kmalloc(sizeof(memory_dev_t), GFP_KERNEL);
     memset(dev, 0, sizeof(memory_dev_t));
-    dev->bus = bus; dev->devfn = devfn;
+    dev->pdev = pci_find_slot(bus, devfn);
     sprintf(dev->node.dev_name, "cbmem%d", i);
     dev->node.major = major_dev;
     dev->node.minor = i<<3;
     
     dev->size[0] = 0x100;
     printk(KERN_INFO "memory_cb: cbmem%d: 0 [256 b]", i);
-    pcibios_read_config_byte(bus, devfn, PCI_COMMAND, &cmd);
-    pcibios_write_config_byte(bus, devfn, PCI_COMMAND, 0);
+    pci_read_config_byte(dev->pdev, PCI_COMMAND, &cmd);
+    pci_write_config_byte(dev->pdev, PCI_COMMAND, 0);
     for (i = 1; i < 8; i++) {
 	br = (i == 7) ? CB_ROM_BASE : CB_BAR(i-1);
-	pcibios_read_config_dword(bus, devfn, br, &dev->base[i]);
-	pcibios_write_config_dword(bus, devfn, br, 0xffffffff);
-	pcibios_read_config_dword(bus, devfn, br, &dev->size[i]);
-	pcibios_write_config_dword(bus, devfn, br, dev->base[i]);
+	pci_read_config_dword(dev->pdev, br, &dev->base[i]);
+	pci_write_config_dword(dev->pdev, br, 0xffffffff);
+	pci_read_config_dword(dev->pdev, br, &dev->size[i]);
+	pci_write_config_dword(dev->pdev, br, dev->base[i]);
 	dev->size[i] &= PCI_BASE_ADDRESS_MEM_MASK;
 	dev->size[i] = FIND_FIRST_BIT(dev->size[i]);
 	if (dev->size[i] == 0) continue;
@@ -116,7 +130,7 @@ static dev_node_t *memory_attach(dev_locator_t *loc)
 	    printk(", %d [%d kb]", i, dev->size[i]>>10);
     }
     printk("\n");
-    pcibios_write_config_byte(bus, devfn, PCI_COMMAND, cmd);
+    pci_write_config_byte(dev->pdev, PCI_COMMAND, cmd);
     MOD_INC_USE_COUNT;
     return &dev->node;
 }
@@ -150,8 +164,7 @@ static int memory_open(struct inode *inode, struct file *file)
     return 0;
 }
 
-static FS_RELEASE_T memory_close(struct inode *inode,
-				 struct file *file)
+static FS_RELEASE_T memory_close(struct inode *inode, struct file *file)
 {
     int minor = MINOR(F_INODE(file)->i_rdev);
     memory_dev_t *dev = dev_table[minor>>3];
@@ -164,14 +177,14 @@ static FS_RELEASE_T memory_close(struct inode *inode,
     return (FS_RELEASE_T)0;
 }
 
-static FS_SIZE_T memory_read FOPS(struct inode *inode,
-				  struct file *file, char *buf,
-				  U_FS_SIZE_T count, loff_t *ppos)
+static ssize_t memory_read FOPS(struct inode *inode,
+				struct file *file, char *buf,
+				size_t count, loff_t *ppos)
 {
     int minor = MINOR(F_INODE(file)->i_rdev);
     memory_dev_t *dev = dev_table[minor>>3];
     int space = minor & 7;
-    U_FS_SIZE_T i, pos = FPOS;
+    size_t i, odd, pos = FPOS;
     
     DEBUG(2, "memory_read(%d, 0x%lx, 0x%lx)\n", minor,
 	  (u_long)pos, (u_long)count);
@@ -183,27 +196,58 @@ static FS_SIZE_T memory_read FOPS(struct inode *inode,
     if (count > dev->size[space] - pos)
 	count = dev->size[space] - pos;
 
+    odd = count & 3; count &= ~3;
+
     if (space == 0) {
-	for (i = 0; i < count; i++, pos++, buf++)
-	    pcibios_read_config_byte(dev->bus, dev->devfn, pos, buf);
+
+	for (i = 0; i < count; i += 4, pos += 4, buf += 4)
+	    pci_read_config_dword(dev->pdev, pos, (u32 *)buf);
+	if (odd & 2) {
+	    pci_read_config_word(dev->pdev, pos, (u16 *)buf);
+	    pos += 2; buf += 2;
+	}
+	if (odd & 1) {
+	    pci_read_config_byte(dev->pdev, pos, buf);
+	}
+
     } else if (dev->virt[space] != NULL) {
-	copy_pc_to_user(buf, dev->virt[space]+pos, count);
+
+	for (i = 0; i < count; i += 4, pos += 4, buf += 4)
+	    *(u32 *)buf = readl_ns(dev->virt[space]+pos);
+	if (odd & 2) {
+	    *(u16 *)buf = readw_ns(dev->virt[space]+pos);
+	    pos += 2; buf += 2;
+	}
+	if (odd & 1) {
+	    *buf = readb(dev->virt[space]+pos);
+	}
+
     } else {
-	for (i = 0; i < count; i++, pos++, buf++)
+
+	for (i = 0; i < count; i += 4, pos += 4, buf += 4)
+	    *(u32 *)buf = inl(dev->base[space]+pos);
+	if (odd & 2) {
+	    *(u16 *)buf = inw(dev->base[space]+pos);
+	    pos += 2; buf += 2;
+	}
+	if (odd & 1) {
 	    *buf = inb(dev->base[space]+pos);
+	}
+
     }
-    FPOS += count;
-    return count;
+
+    FPOS += count+odd;
+    return count+odd;
 }
 
-static FS_SIZE_T memory_write FOPS(struct inode *inode,
-				   struct file *file, const char *buf,
-				   U_FS_SIZE_T count, loff_t *ppos)
+static ssize_t memory_write FOPS(struct inode *inode,
+				 struct file *file, const char *buf,
+				 size_t count, loff_t *ppos)
 {
     int minor = MINOR(F_INODE(file)->i_rdev);
     memory_dev_t *dev = dev_table[minor>>3];
     int space = minor & 7;
-    U_FS_SIZE_T i, pos = FPOS;
+    size_t i, odd, pos = FPOS;
     
     DEBUG(2, "memory_read(%d, 0x%lx, 0x%lx)\n", minor,
 	  (u_long)pos, (u_long)count);
@@ -215,40 +259,65 @@ static FS_SIZE_T memory_write FOPS(struct inode *inode,
     if (count > dev->size[space] - pos)
 	count = dev->size[space] - pos;
 
+    odd = count & 3; count &= ~3;
+
     if (space == 0) {
-	for (i = 0; i < count; i++, pos++, buf++)
-	    pcibios_write_config_byte(dev->bus, dev->devfn, pos, *buf);
+
+	for (i = 0; i < count; i += 4, pos += 4, buf += 4)
+	    pci_write_config_dword(dev->pdev, pos, *(u32 *)buf);
+	if (odd & 2) {
+	    pci_write_config_word(dev->pdev, pos, *(u16 *)buf);
+	    pos += 2; buf += 2;
+	}
+	if (odd & 1) {
+	    pci_write_config_byte(dev->pdev, pos, *buf);
+	}
+
     } else if (dev->virt[space] != NULL) {
-	copy_user_to_pc(dev->virt[space]+pos, buf, count);
+
+	for (i = 0; i < count; i += 4, pos += 4, buf += 4)
+	    writel_ns(*(u32 *)buf, dev->virt[space]+pos);
+	if (odd & 2) {
+	    writew_ns(*(u16 *)buf, dev->virt[space]+pos);
+	    pos += 2; buf += 2;
+	}
+	if (odd & 1) {
+	    writeb(*buf, dev->virt[space]+pos);
+	}
+
     } else {
-	for (i = 0; i < count; i++, pos++, buf++)
+
+	for (i = 0; i < count; i += 4, pos += 4, buf += 4)
+	    outl(*(u32 *)buf, dev->base[space]+pos);
+	if (odd & 2) {
+	    outw(*(u16 *)buf, dev->base[space]+pos);
+	    pos += 2; buf += 2;
+	}
+	if (odd & 1) {
 	    outb(*buf, dev->base[space]+pos);
+	}
+
     }
-    FPOS += count;
-    return count;
+
+    FPOS += count+odd;
+    return count+odd;
 }
 
 /*====================================================================*/
 
 static struct file_operations memory_fops = {
-    NULL,
-    memory_read,
-    memory_write,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    memory_open,
-    NULL_FLUSH
-    memory_close,
-    NULL
+    open:	memory_open,
+    release:	memory_close,
+    read:	memory_read,
+    write:	memory_write,
 };
 
 struct driver_operations memory_ops = {
     "memory_cb", memory_attach, NULL, NULL, memory_detach
 };
 
-int init_module(void) {
+static int __init init_memory_cb(void)
+{
     DEBUG(0, "%s\n", version);
     major_dev = register_chrdev(major_dev, "memory_cb", &memory_fops);
     if (major_dev == 0) {
@@ -260,9 +329,13 @@ int init_module(void) {
     return 0;
 }
 
-void cleanup_module(void) {
+static void exit_memory_cb(void)
+{
     DEBUG(0, "memory_cb: unloading\n");
     unregister_driver(&memory_ops);
     if (major_dev != 0)
 	unregister_chrdev(major_dev, "memory_cb");
 }
+
+module_init(init_memory_cb);
+module_exit(exit_memory_cb);
