@@ -7,7 +7,7 @@
     card's attribute and common memory.  It includes character
     and block device support.
 
-    memory_cs.c 1.59 1999/07/30 05:36:15
+    memory_cs.c 1.71 2000/06/09 22:15:05
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -20,17 +20,29 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
 #include <pcmcia/config.h>
 #include <pcmcia/k_compat.h>
 
+#include <linux/module.h>
+#include <linux/init.h>
 #include <linux/kernel.h>
 #include <linux/sched.h>
-#include <linux/ptrace.h>
 #include <linux/malloc.h>
 #include <linux/string.h>
 #include <linux/timer.h>
@@ -79,7 +91,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"memory_cs.c 1.59 1999/07/30 05:36:15 (David Hinds)";
+"memory_cs.c 1.71 2000/06/09 22:15:05 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -94,8 +106,12 @@ static int word_width = 1;
 /* Speed of memory accesses, in ns */
 static int mem_speed = 0;
 
+/* Force the size of an SRAM card */
+static int force_size = 0;
+
 MODULE_PARM(word_width, "i");
 MODULE_PARM(mem_speed, "i");
+MODULE_PARM(force_size, "i");
 
 /*====================================================================*/
 
@@ -134,10 +150,11 @@ typedef struct direct_dev_t {		/* For direct access */
     int			open;
     caddr_t		Base;
     u_int		Size;
-    u_long		cardsize;
+    u_int		cardsize;
 } direct_dev_t;
 
 typedef struct memory_dev_t {
+    dev_link_t		link;
     dev_node_t		node;
     eraseq_handle_t	eraseq_handle;
     eraseq_entry_t	eraseq[MAX_ERASE];
@@ -158,12 +175,12 @@ static int memory_blocksizes[MINOR_NR(MAX_DEV, 0, 0, 0)] =
 
 static int memory_ioctl(struct inode *inode, struct file *file,
 			u_int cmd, u_long arg);
-static FS_SIZE_T memory_read FOPS(struct inode *inode,
-				  struct file *file, char *buf,
-				  U_FS_SIZE_T count, loff_t *ppos);
-static FS_SIZE_T memory_write FOPS(struct inode *inode,
-				   struct file *file, const char *buf,
-				   U_FS_SIZE_T count, loff_t *ppos);
+static ssize_t memory_read FOPS(struct inode *inode,
+				struct file *file, char *buf,
+				size_t count, loff_t *ppos);
+static ssize_t memory_write FOPS(struct inode *inode,
+				 struct file *file, const char *buf,
+				 size_t count, loff_t *ppos);
 static int memory_open(struct inode *inode, struct file *file);
 static FS_RELEASE_T memory_close(struct inode *inode,
 				 struct file *file);
@@ -171,31 +188,22 @@ static FS_RELEASE_T memory_blk_close(struct inode *inode,
 				     struct file *file);
 
 static struct file_operations memory_chr_fops = {
-    NULL,		/* lseek */
-    memory_read,	/* read */
-    memory_write,      	/* write */
-    NULL,		/* readdir */
-    NULL,		/* select */
-    memory_ioctl,	/* ioctl */
-    NULL,		/* mmap */
-    memory_open,	/* open */
-    NULL_FLUSH		/* flush */
-    memory_close,	/* release */
-    NULL		/* fsync */
+    open:	memory_open,
+    release:	memory_close,
+    read:	memory_read,
+    write:	memory_write,
+    ioctl:	memory_ioctl,
 };
 
-static struct file_operations memory_blk_fops = {
-    NULL,		/* lseek */
-    block_read,		/* read */
-    block_write,	/* write */
-    NULL,		/* readdir */
-    NULL,		/* select */
-    memory_ioctl,	/* ioctl */
-    NULL,		/* mmap */
-    memory_open,	/* open */
-    NULL_FLUSH		/* flush */
-    memory_blk_close,	/* release */
-    block_fsync		/* fsync */
+static struct block_device_operations memory_blk_fops = {
+    open:	memory_open,
+    release:	memory_blk_close,
+    ioctl:	memory_ioctl,
+#ifdef block_device_operations
+    read:	block_read,
+    write:	block_write,
+    fsync:	block_fsync
+#endif
 };
 
 /*====================================================================*/
@@ -216,9 +224,9 @@ static void cs_error(client_handle_t handle, int func, int ret)
 
 static dev_link_t *memory_attach(void)
 {
-    client_reg_t client_reg;
-    dev_link_t *link;
     memory_dev_t *dev;
+    dev_link_t *link;
+    client_reg_t client_reg;
     eraseq_hdr_t eraseq_hdr;
     int i, ret;
     
@@ -232,16 +240,15 @@ static dev_link_t *memory_attach(void)
     }
     
     /* Create new memory card device */
-    link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
-    dev_table[i] = link;
-    memset(link, 0, sizeof(struct dev_link_t));
+    dev = kmalloc(sizeof(*dev), GFP_KERNEL);
+    if (!dev) return NULL;
+    memset(dev, 0, sizeof(*dev));
+    link = &dev->link; link->priv = dev;
+
     link->release.function = &memory_release;
     link->release.data = (u_long)link;
-    
-    dev = kmalloc(sizeof(struct memory_dev_t), GFP_KERNEL);
-    memset(dev, 0, sizeof(memory_dev_t));
+    dev_table[i] = link;
     init_waitqueue_head(&dev->erase_pending);
-    link->priv = dev;
 
     /* Register with Card Services */
     client_reg.dev_info = &dev_info;
@@ -287,7 +294,7 @@ static dev_link_t *memory_attach(void)
 
 static void memory_detach(dev_link_t *link)
 {
-    memory_dev_t *dev;
+    memory_dev_t *dev = link->priv;
     int nd;
 
     DEBUG(0, "memory_detach(0x%p)\n", link);
@@ -298,6 +305,7 @@ static void memory_detach(dev_link_t *link)
     if (nd == MAX_DEV)
 	return;
 
+    del_timer(&link->release);
     if (link->state & DEV_CONFIG) {
 	memory_release((u_long)link);
 	if (link->state & DEV_STALE_CONFIG) {
@@ -306,7 +314,6 @@ static void memory_detach(dev_link_t *link)
 	}
     }
 
-    dev = (memory_dev_t *)link->priv;
     if (dev->eraseq_handle)
 	CardServices(DeregisterEraseQueue, dev->eraseq_handle);
     if (link->handle)
@@ -314,8 +321,7 @@ static void memory_detach(dev_link_t *link)
     
     /* Unlink device structure, free bits */
     dev_table[nd] = NULL;
-    kfree_s(dev, sizeof(struct memory_dev_t));
-    kfree_s(link, sizeof(struct dev_link_t));
+    kfree(dev);
     
 } /* memory_detach */
 
@@ -325,59 +331,51 @@ static void memory_detach(dev_link_t *link)
     
 ======================================================================*/
 
-static void get_size(dev_link_t *link, direct_dev_t *direct)
+static u_int get_size(dev_link_t *link, direct_dev_t *direct)
 {
     modwin_t mod;
     memreq_t mem;
-    u_char buf[26];
-    int s, t, ret;
+    u_char b0, b1;
+    int s, ret;
 
     mod.Attributes = WIN_ENABLE | WIN_MEMORY_TYPE_CM;
-    mod.AccessSpeed = 0;
+    mod.AccessSpeed = mem_speed;
     ret = CardServices(ModifyWindow, link->win, &mod);
     if (ret != CS_SUCCESS)
 	cs_error(link->handle, ModifyWindow, ret);
 
     /* Look for wrap-around or dead end */
-    mem.Page = 0;
+    mem.Page = mem.CardOffset = 0;
+    CardServices(MapMemPage, link->win, &mem);
+    b0 = readb(direct->Base);
     for (s = 12; s < 26; s++) {
 	mem.CardOffset = 1<<s;
 	CardServices(MapMemPage, link->win, &mem);
-	buf[s] = readb(direct->Base);
-	writeb(~buf[s], direct->Base);
-	for (t = 12; t < s; t++) {
-	    mem.CardOffset = 1<<t;
-	    CardServices(MapMemPage, link->win, &mem);
-	    if (readb(direct->Base) != buf[t]) {
-		writeb(buf[t], direct->Base);
-		break;
-	    }
+	b1 = readb(direct->Base);
+	writeb(~b1, direct->Base);
+	mem.CardOffset = 0;
+	CardServices(MapMemPage, link->win, &mem);
+	if (readb(direct->Base) != b0) {
+	    writeb(b0, direct->Base);
+	    break;
 	}
-	if (t < s) break;
 	mem.CardOffset = 1<<s;
 	CardServices(MapMemPage, link->win, &mem);
-	if (readb(direct->Base) != (0xff & ~buf[s])) break;
-	writeb(buf[s], direct->Base);
+	if (readb(direct->Base) != (0xff & ~b1)) break;
+	writeb(b1, direct->Base);
     }
 
-    /* Restore that last byte on wrap-around */
-    if (t < s) {
-	mem.CardOffset = 1<<t;
-	CardServices(MapMemPage, link->win, &mem);
-	writeb(buf[t], direct->Base);
-    }
-
-    direct->cardsize = (t > 15) ? (1<<t) : 0;
+    return (s > 15) ? (1<<s) : 0;
 } /* get_size */
 
-static void print_size(u_long sz)
+static void print_size(u_int sz)
 {
     if (sz & 0x03ff)
-	printk("%ld bytes", sz);
+	printk("%d bytes", sz);
     else if (sz & 0x0fffff)
-	printk("%ld kb", sz >> 10);
+	printk("%d kb", sz >> 10);
     else
-	printk("%ld mb", sz >> 20);
+	printk("%d mb", sz >> 20);
 }
 
 /*======================================================================
@@ -393,7 +391,7 @@ while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
 
 static void memory_config(dev_link_t *link)
 {
-    memory_dev_t *dev;
+    memory_dev_t *dev = link->priv;
     minor_dev_t *minor;
     region_info_t region;
     cs_status_t status;
@@ -408,8 +406,6 @@ static void memory_config(dev_link_t *link)
     for (nd = 0; nd < MAX_DEV; nd++)
 	if (dev_table[nd] == link) break;
     
-    dev = (memory_dev_t *)link->priv;
-
     /* Allocate a small memory window for direct access */
     if (word_width)
 	req.Attributes = WIN_DATA_WIDTH_16;
@@ -450,7 +446,8 @@ static void memory_config(dev_link_t *link)
 	cisinfo_t cisinfo;
 	if ((CardServices(ValidateCIS, link->handle, &cisinfo)
 	     == CS_SUCCESS) && (cisinfo.Chains == 0)) {
-	    get_size(link, &dev->direct);
+	    dev->direct.cardsize =
+		force_size ? force_size : get_size(link,&dev->direct);
 	    printk(" anonymous: ");
 	    if (dev->direct.cardsize == 0) {
 		dev->direct.cardsize = HIGH_ADDR;
@@ -533,7 +530,7 @@ static int memory_event(event_t event, int priority,
 		       event_callback_args_t *args)
 {
     dev_link_t *link = args->client_data;
-    memory_dev_t *dev;
+    memory_dev_t *dev = link->priv;
     eraseq_entry_t *erase;
 
     DEBUG(1, "memory_event(0x%06x)\n", event);
@@ -541,10 +538,8 @@ static int memory_event(event_t event, int priority,
     switch (event) {
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
-	if (link->state & DEV_CONFIG) {
-	    link->release.expires = RUN_AT(HZ/20);
-	    add_timer(&link->release);
-	}
+	if (link->state & DEV_CONFIG)
+	    mod_timer(&link->release, jiffies + HZ/20);
 	break;
     case CS_EVENT_CARD_INSERTION:
 	link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
@@ -553,7 +548,6 @@ static int memory_event(event_t event, int priority,
     case CS_EVENT_ERASE_COMPLETE:
 	erase = (eraseq_entry_t *)(args->info);
 	wake_up((wait_queue_head_t *)&erase->Optional);
-	dev = (memory_dev_t *)(link->priv);
 	wake_up_interruptible(&dev->erase_pending);
 	break;
     case CS_EVENT_PM_SUSPEND:
@@ -667,15 +661,15 @@ static FS_RELEASE_T memory_blk_close(struct inode *inode,
     
 ======================================================================*/
 
-static FS_SIZE_T direct_read FOPS(struct inode *inode,
-				  struct file *file, char *buf,
-				  U_FS_SIZE_T count, loff_t *ppos)
+static ssize_t direct_read FOPS(struct inode *inode,
+				struct file *file, char *buf,
+				size_t count, loff_t *ppos)
 {
     int minor = MINOR(F_INODE(file)->i_rdev);
     dev_link_t *link;
     memory_dev_t *dev;
     direct_dev_t *direct;
-    U_FS_SIZE_T size, pos, read, from, nb;
+    size_t size, pos, read, from, nb;
     int ret;
     modwin_t mod;
     memreq_t mem;
@@ -702,7 +696,7 @@ static FS_SIZE_T direct_read FOPS(struct inode *inode,
 
     mod.Attributes = WIN_ENABLE;
     mod.Attributes |= (REGION_AM(minor)) ? WIN_MEMORY_TYPE_AM : 0;
-    mod.AccessSpeed = 0;
+    mod.AccessSpeed = mem_speed;
     ret = CardServices(ModifyWindow, link->win, &mod);
     if (ret != CS_SUCCESS) {
 	cs_error(link->handle, ModifyWindow, ret);
@@ -729,9 +723,9 @@ static FS_SIZE_T direct_read FOPS(struct inode *inode,
     return read;
 } /* direct_read */
 
-static FS_SIZE_T memory_read FOPS(struct inode *inode,
-				  struct file *file, char *buf,
-				  U_FS_SIZE_T count, loff_t *ppos)
+static ssize_t memory_read FOPS(struct inode *inode,
+				struct file *file, char *buf,
+				size_t count, loff_t *ppos)
 {
     minor_dev_t *minor;
     mem_op_t req;
@@ -765,13 +759,16 @@ static FS_SIZE_T memory_read FOPS(struct inode *inode,
     
 ======================================================================*/
 
-static int memory_erase(int minor, u_long f_pos, U_FS_SIZE_T count)
+static int memory_erase(int minor, u_long f_pos, size_t count)
 {
     dev_link_t *link = dev_table[DEVICE_NR(minor)];
     memory_dev_t *dev = link->priv;
     minor_dev_t *minor_dev = &dev->minor[REGION_NR(minor)];
     int i, ret;
 
+    DEBUG(2, "memory_erase(%d, 0x%lx, %ld)\n", minor,
+	  f_pos, (u_long)count);
+    
     /* Find a free erase slot, or wait for one to become available */
     for (;;) {
 	for (i = 0; i < MAX_ERASE; i++)
@@ -809,15 +806,15 @@ static int memory_erase(int minor, u_long f_pos, U_FS_SIZE_T count)
     
 ======================================================================*/
 
-static FS_SIZE_T direct_write FOPS(struct inode *inode,
-				   struct file *file, const char *buf,
-				   U_FS_SIZE_T count, loff_t *ppos)
+static ssize_t direct_write FOPS(struct inode *inode,
+				 struct file *file, const char *buf,
+				 size_t count, loff_t *ppos)
 {
     int minor = MINOR(F_INODE(file)->i_rdev);
     dev_link_t *link;
     memory_dev_t *dev;
     direct_dev_t *direct;
-    U_FS_SIZE_T size, pos, wrote, to, nb;
+    size_t size, pos, wrote, to, nb;
     int ret;
     modwin_t mod;
     memreq_t mem;
@@ -849,7 +846,7 @@ static FS_SIZE_T direct_write FOPS(struct inode *inode,
 
     mod.Attributes = WIN_ENABLE;
     mod.Attributes |= (REGION_AM(minor)) ? WIN_MEMORY_TYPE_AM : 0;
-    mod.AccessSpeed = 0;
+    mod.AccessSpeed = mem_speed;
     ret = CardServices(ModifyWindow, link->win, &mod);
     if (ret != CS_SUCCESS) {
 	cs_error(link->handle, ModifyWindow, ret);
@@ -876,9 +873,9 @@ static FS_SIZE_T direct_write FOPS(struct inode *inode,
     return wrote;
 } /* direct_write */
 
-static FS_SIZE_T memory_write FOPS(struct inode *inode,
-				   struct file *file, const char *buf,
-				   U_FS_SIZE_T count, loff_t *ppos)
+static ssize_t memory_write FOPS(struct inode *inode,
+				 struct file *file, const char *buf,
+				 size_t count, loff_t *ppos)
 {
     minor_dev_t *minor;
     mem_op_t req;
@@ -966,7 +963,7 @@ static int memory_ioctl(struct inode *inode, struct file *file,
 	break;
 #if (LINUX_VERSION_CODE < VERSION(2,3,3))
     case BLKFLSBUF:
-	if (!suser()) return -EACCES;
+	if (!capable(CAP_SYS_ADMIN)) return -EACCES;
 	if (!(inode->i_rdev)) return -EINVAL;
 	fsync_dev(inode->i_rdev);
 	invalidate_buffers(inode->i_rdev);
@@ -991,17 +988,15 @@ static int memory_ioctl(struct inode *inode, struct file *file,
     
 ======================================================================*/
 
-
 static void do_direct_request(dev_link_t *link)
 {
+    memory_dev_t *dev = link->priv;
     int addr, len, from, nb, ret;
     char *buf;
-    memory_dev_t *dev;
     direct_dev_t *direct;
     modwin_t mod;
     memreq_t mem;
-    
-    dev = (memory_dev_t *)link->priv;
+
     direct = &dev->direct;
     
     addr = CURRENT->sector * SECTOR_SIZE;
@@ -1011,8 +1006,8 @@ static void do_direct_request(dev_link_t *link)
 	return;
     }
     
-    mod.Attributes = WIN_ENABLE;
-    mod.AccessSpeed = 0;
+    mod.Attributes = WIN_ENABLE | WIN_MEMORY_TYPE_CM;
+    mod.AccessSpeed = mem_speed;
     ret = CardServices(ModifyWindow, link->win, &mod);
     if (ret != CS_SUCCESS) {
 	cs_error(link->handle, ModifyWindow, ret);
@@ -1047,7 +1042,7 @@ static void do_direct_request(dev_link_t *link)
     }
 } /* do_direct_request */
 
-static void do_memory_request(void)
+static void do_memory_request(request_arg_t)
 {
     int ret, minor;
     char *buf;
@@ -1099,7 +1094,7 @@ static void do_memory_request(void)
 
 /*====================================================================*/
 
-int init_module(void)
+static int __init init_memory_cs(void)
 {
     servinfo_t serv;
     int i;
@@ -1113,7 +1108,7 @@ int init_module(void)
 	return -1;
     }
     
-    register_pcmcia_driver(&dev_info, &memory_attach, &memory_detach);
+    register_pccard_driver(&dev_info, &memory_attach, &memory_detach);
 
     for (i = MAX_CHRDEV-1; i > 0; i--) {
 	if (register_chrdev(i, "memory", &memory_chr_fops) == 0) {
@@ -1129,7 +1124,7 @@ int init_module(void)
     }
 
     major_dev = i;
-    blk_dev[major_dev].request_fn = DEVICE_REQUEST;
+    blk_init_queue(BLK_DEFAULT_QUEUE(MAJOR_NR), &do_memory_request);
     for (i = 0; i < MINOR_NR(MAX_DEV, 0, 0, 0); i++)
 	memory_blocksizes[i] = 1024;
     blksize_size[major_dev] = memory_blocksizes;
@@ -1137,17 +1132,17 @@ int init_module(void)
     return 0;
 }
 
-void cleanup_module(void)
+static void __exit exit_memory_cs(void)
 {
     int i;
     dev_link_t *link;
 
     DEBUG(0, "memory_cs: unloading\n");
-    unregister_pcmcia_driver(&dev_info);
+    unregister_pccard_driver(&dev_info);
     if (major_dev != 0) {
 	unregister_chrdev(major_dev, "memory");
 	unregister_blkdev(major_dev, "memory");
-	blk_dev[major_dev].request_fn = NULL;
+	blk_cleanup_queue(BLK_DEFAULT_QUEUE(MAJOR_NR));
 	blksize_size[major_dev] = NULL;
     }
     for (i = 0; i < MAX_DEV; i++) {
@@ -1159,3 +1154,6 @@ void cleanup_module(void)
 	}
     }
 }
+
+module_init(init_memory_cs);
+module_exit(exit_memory_cs);
