@@ -5,7 +5,7 @@
     (specifically, for the Quatech SPP-100 EPP card: other cards will
     probably require driver tweaks)
     
-    parport_cs.c 1.7 1999/07/20 16:00:01
+    parport_cs.c 1.17 2000/05/16 21:31:36
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -18,8 +18,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
@@ -27,6 +38,8 @@
 #include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
+#include <linux/module.h>
+#include <linux/init.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
 #include <linux/malloc.h>
@@ -50,7 +63,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"ide_cs.c 1.7 1999/07/20 16:00:01 (David Hinds)";
+"parport_cs.c 1.17 2000/05/16 21:31:36 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -73,6 +86,7 @@ MODULE_PARM(epp_mode, "i");
 #define FORCE_EPP_MODE	0x08
 
 typedef struct parport_info_t {
+    dev_link_t		link;
     int			ndev;
     dev_node_t		node;
     struct parport	*port;
@@ -109,20 +123,23 @@ static void cs_error(client_handle_t handle, int func, int ret)
 
 static dev_link_t *parport_attach(void)
 {
-    client_reg_t client_reg;
+    parport_info_t *info;
     dev_link_t *link;
+    client_reg_t client_reg;
     int i, ret;
     
     DEBUG(0, "parport_attach()\n");
 
     /* Create new parport device */
-    link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
-    memset(link, 0, sizeof(struct dev_link_t));
+    info = kmalloc(sizeof(*info), GFP_KERNEL);
+    if (!info) return NULL;
+    memset(info, 0, sizeof(*info));
+    link = &info->link; link->priv = info;
+
     link->release.function = &parport_cs_release;
     link->release.data = (u_long)link;
     link->io.Attributes1 = IO_DATA_PATH_WIDTH_8;
     link->io.Attributes2 = IO_DATA_PATH_WIDTH_8;
-    link->io.IOAddrLines = 3;
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE;
     link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
     if (irq_list[0] == -1)
@@ -133,8 +150,6 @@ static dev_link_t *parport_attach(void)
     link->conf.Attributes = CONF_ENABLE_IRQ;
     link->conf.Vcc = 50;
     link->conf.IntType = INT_MEMORY_AND_IO;
-    link->priv = kmalloc(sizeof(struct parport_info_t), GFP_KERNEL);
-    memset(link->priv, 0, sizeof(struct parport_info_t));
     
     /* Register with Card Services */
     link->next = dev_list;
@@ -170,7 +185,6 @@ static dev_link_t *parport_attach(void)
 static void parport_detach(dev_link_t *link)
 {
     dev_link_t **linkp;
-    long flags;
     int ret;
 
     DEBUG(0, "parport_detach(0x%p)\n", link);
@@ -181,14 +195,7 @@ static void parport_detach(dev_link_t *link)
     if (*linkp == NULL)
 	return;
 
-    save_flags(flags);
-    cli();
-    if (link->state & DEV_RELEASE_PENDING) {
-	del_timer(&link->release);
-	link->state &= ~DEV_RELEASE_PENDING;
-    }
-    restore_flags(flags);
-    
+    del_timer(&link->release);
     if (link->state & DEV_CONFIG)
 	parport_cs_release((u_long)link);
     
@@ -198,10 +205,9 @@ static void parport_detach(dev_link_t *link)
 	    cs_error(link->handle, DeregisterClient, ret);
     }
     
-    /* Unlink device structure, free bits */
+    /* Unlink, free device structure */
     *linkp = link->next;
-    kfree_s(link->priv, sizeof(parport_info_t));
-    kfree_s(link, sizeof(struct dev_link_t));
+    kfree(link->priv);
     
 } /* parport_detach */
 
@@ -229,8 +235,8 @@ static struct { u_int flag; char *name; } mode[] = {
 
 void parport_config(dev_link_t *link)
 {
-    client_handle_t handle;
-    parport_info_t *info;
+    client_handle_t handle = link->handle;
+    parport_info_t *info = link->priv;
     tuple_t tuple;
     u_short buf[128];
     cisparse_t parse;
@@ -239,9 +245,6 @@ void parport_config(dev_link_t *link)
     cistpl_cftable_entry_t dflt = { 0 };
     struct parport *p;
     int i, last_ret, last_fn;
-
-    handle = link->handle;
-    info = link->priv;
     
     DEBUG(0, "parport_config(0x%p)\n", link);
     
@@ -275,6 +278,7 @@ void parport_config(dev_link_t *link)
 		link->conf.ConfigIndex |= FORCE_EPP_MODE;
 	    link->io.BasePort1 = io->win[0].base;
 	    link->io.NumPorts1 = io->win[0].len;
+	    link->io.IOAddrLines = io->flags & CISTPL_IO_LINES_MASK;
 	    if (io->nwin == 2) {
 		link->io.BasePort2 = io->win[1].base;
 		link->io.NumPorts2 = io->win[1].len;
@@ -387,7 +391,7 @@ void parport_cs_release(u_long arg)
 ======================================================================*/
 
 int parport_event(event_t event, int priority,
-	      event_callback_args_t *args)
+		  event_callback_args_t *args)
 {
     dev_link_t *link = args->client_data;
 
@@ -396,11 +400,8 @@ int parport_event(event_t event, int priority,
     switch (event) {
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
-	if (link->state & DEV_CONFIG) {
-	    link->release.expires = RUN_AT(HZ/20);
-	    link->state |= DEV_RELEASE_PENDING;
-	    add_timer(&link->release);
-	}
+	if (link->state & DEV_CONFIG)
+	    mod_timer(&link->release, jiffies + HZ/20);
 	break;
     case CS_EVENT_CARD_INSERTION:
 	link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
@@ -440,7 +441,7 @@ static void dec_use_count(void)
 
 /*====================================================================*/
 
-int init_module(void)
+static int __init init_parport_cs(void)
 {
     servinfo_t serv;
     DEBUG(0, "%s\n", version);
@@ -456,14 +457,17 @@ int init_module(void)
     parport_cs_ops.inc_use_count = &inc_use_count;
     parport_cs_ops.dec_use_count = &dec_use_count;
     
-    register_pcmcia_driver(&dev_info, &parport_attach, &parport_detach);
+    register_pccard_driver(&dev_info, &parport_attach, &parport_detach);
     return 0;
 }
 
-void cleanup_module(void)
+static void __exit exit_parport_cs(void)
 {
     DEBUG(0, "parport_cs: unloading\n");
-    unregister_pcmcia_driver(&dev_info);
+    unregister_pccard_driver(&dev_info);
     while (dev_list != NULL)
 	parport_detach(dev_list);
 }
+
+module_init(init_parport_cs);
+module_exit(exit_parport_cs);
