@@ -9,9 +9,9 @@
     Conrad ethernet card, and the Kingston KNE-PCM/x in shared-memory
     mode.  It will also handle the Socket EA card in either mode.
 
-    Copyright (C) 1998 David A. Hinds -- dhinds@hyper.stanford.edu
+    Copyright (C) 1999 David A. Hinds -- dhinds@pcmcia.sourceforge.org
 
-    pcnet_cs.c 1.94 1999/07/29 06:04:49
+    pcnet_cs.c 1.122 2000/05/31 22:24:20
     
     The network driver code is based on Donald Becker's NE2000 code:
 
@@ -32,6 +32,8 @@
 #include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
+#include <linux/module.h>
+#include <linux/init.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
 #include <linux/malloc.h>
@@ -42,7 +44,7 @@
 #include <asm/system.h>
 
 #include <linux/netdevice.h>
-#include <drivers/net/8390.h>
+#include <../drivers/net/8390.h>
 
 #include <pcmcia/version.h>
 #include <pcmcia/cs_types.h>
@@ -73,7 +75,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"pcnet_cs.c 1.94 1999/07/29 06:04:49 (David Hinds)";
+"pcnet_cs.c 1.122 2000/05/31 22:24:20 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -123,39 +125,39 @@ static void pcnet_config(dev_link_t *link);
 static void pcnet_release(u_long arg);
 static int pcnet_event(event_t event, int priority,
 		       event_callback_args_t *args);
-
-static int pcnet_open(struct device *dev);
-static int pcnet_close(struct device *dev);
-
-static void pcnet_reset_8390(struct device *dev);
-
-static int set_config(struct device *dev, struct ifmap *map);
-
+static int pcnet_open(struct net_device *dev);
+static int pcnet_close(struct net_device *dev);
+static int do_ioctl(struct net_device *dev, struct ifreq *rq, int cmd);
+static void ei_irq_wrapper(int irq, void *dev_id, struct pt_regs *regs);
+static void ei_watchdog(u_long arg);
+static void pcnet_reset_8390(struct net_device *dev);
+static int set_config(struct net_device *dev, struct ifmap *map);
 static int setup_shmem_window(dev_link_t *link, int start_pg,
 			      int stop_pg, int cm_offset);
 static int setup_dma_config(dev_link_t *link, int start_pg,
 			    int stop_pg);
 
-static dev_info_t dev_info = "pcnet_cs";
-
 static dev_link_t *pcnet_attach(void);
 static void pcnet_detach(dev_link_t *);
 
+static dev_info_t dev_info = "pcnet_cs";
 static dev_link_t *dev_list;
 
 /*====================================================================*/
 
 typedef struct hw_info_t {
-    u_long	offset;
+    u_int	offset;
     u_char	a0, a1, a2;
-    u_long	flags;
+    u_int	flags;
 } hw_info_t;
 
 #define DELAY_OUTPUT	0x01
 #define HAS_MISC_REG	0x02
 #define USE_BIG_BUF	0x04
 #define HAS_IBM_MISC	0x08
-#define IS_DL10019A	0x10
+#define IS_DL10019	0x10
+#define IS_DL10022	0x20
+#define IS_AX88190	0x40
 #define USE_SHMEM	0x80	/* autodetected */
 
 static hw_info_t hw_info[] = {
@@ -186,6 +188,8 @@ static hw_info_t hw_info[] = {
       HAS_MISC_REG | HAS_IBM_MISC },
     { /* Kansai KLA-PCM/T */ 0x0ff0, 0x00, 0x60, 0x87,
       HAS_MISC_REG | HAS_IBM_MISC },
+    { /* NSC DP83903 */ 0x0374, 0x08, 0x00, 0x17,
+      HAS_MISC_REG | HAS_IBM_MISC },
     { /* NSC DP83903 */ 0x0374, 0x00, 0xc0, 0xa8,
       HAS_MISC_REG | HAS_IBM_MISC },
     { /* NSC DP83903 */ 0x0374, 0x00, 0xa0, 0xb0,
@@ -212,21 +216,29 @@ static hw_info_t hw_info[] = {
     { /* Socket EA */ 0x4000, 0x00, 0xc0, 0x1b,
       DELAY_OUTPUT | HAS_MISC_REG | USE_BIG_BUF },
     { /* SuperSocket RE450T */ 0x0110, 0x00, 0xe0, 0x98, 0 },
-    { /* Volktek NPL-402CT */ 0x0060, 0x00, 0x40, 0x05, 0 }
+    { /* Volktek NPL-402CT */ 0x0060, 0x00, 0x40, 0x05, 0 },
+    { /* NEC PC-9801N-J12 */ 0x0ff0, 0x00, 0x00, 0x4c, 0 },
+    { /* PCMCIA Technology OEM */ 0x01c8, 0x00, 0xa0, 0x0c, 0 }
 };
 
 #define NR_INFO		(sizeof(hw_info)/sizeof(hw_info_t))
 
-static hw_info_t default_info =
-{ /* Unknown NE2000 Clone */ 0x00, 0x00, 0x00, 0x00, 0 };
-static hw_info_t dl_fast_info =
-{ /* D-Link EtherFast */ 0x00, 0x00, 0x00, 0x00, IS_DL10019A };
+static hw_info_t default_info = { 0, 0, 0, 0, 0 };
+static hw_info_t dl10019_info = { 0, 0, 0, 0, IS_DL10019 };
+static hw_info_t dl10022_info = { 0, 0, 0, 0, IS_DL10019|IS_DL10022 };
+#if 0
+static hw_info_t ax88190_info = { 0, 0, 0, 0, IS_AX88190 };
+#endif
 
 typedef struct pcnet_dev_t {
-    struct device	dev;
+    struct net_device	dev;	/* so &dev == &pcnet_dev_t */
+    dev_link_t		link;
     dev_node_t		node;
-    u_long		flags;
+    u_int		flags;
     caddr_t		base;
+    struct timer_list	watchdog;
+    int			stale, fast_poll;
+    u_short		link_status;
 } pcnet_dev_t;
 
 /*======================================================================
@@ -262,7 +274,7 @@ static void cs_error(client_handle_t handle, int func, int ret)
 
 ======================================================================*/
 
-static int pcnet_init(struct device *dev)
+static int pcnet_init(struct net_device *dev)
 {
     return 0;
 }
@@ -277,18 +289,22 @@ static int pcnet_init(struct device *dev)
 
 static dev_link_t *pcnet_attach(void)
 {
-    client_reg_t client_reg;
-    dev_link_t *link;
     pcnet_dev_t *info;
-    struct device *dev;
+    dev_link_t *link;
+    struct net_device *dev;
+    client_reg_t client_reg;
     int i, ret;
 
     DEBUG(0, "pcnet_attach()\n");
     flush_stale_links();
 
     /* Create new ethernet device */
-    link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
-    memset(link, 0, sizeof(struct dev_link_t));
+    info = kmalloc(sizeof(*info), GFP_KERNEL);
+    if (!info) return NULL;
+    memset(info, 0, sizeof(*info));
+    link = &info->link; dev = &info->dev;
+    link->priv = info;
+    
     link->release.function = &pcnet_release;
     link->release.data = (u_long)link;
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE;
@@ -299,20 +315,14 @@ static dev_link_t *pcnet_attach(void)
 	for (i = 0; i < 4; i++)
 	    link->irq.IRQInfo2 |= 1 << irq_list[i];
     link->conf.Attributes = CONF_ENABLE_IRQ;
-    link->conf.Vcc = 50;
     link->conf.IntType = INT_MEMORY_AND_IO;
 
-    info = kmalloc(sizeof(struct pcnet_dev_t), GFP_KERNEL);
-    memset(info, 0, sizeof(struct pcnet_dev_t));
-    dev = &info->dev;
     ethdev_init(dev);
-    dev->name = info->node.dev_name;
+    init_dev_name(dev, info->node);
     dev->init = &pcnet_init;
     dev->open = &pcnet_open;
     dev->stop = &pcnet_close;
     dev->set_config = &set_config;
-    dev->tbusy = 1;
-    link->priv = info;
 
     /* Register with Card Services */
     link->next = dev_list;
@@ -347,8 +357,8 @@ static dev_link_t *pcnet_attach(void)
 
 static void pcnet_detach(dev_link_t *link)
 {
+    pcnet_dev_t *info = link->priv;
     dev_link_t **linkp;
-    long flags;
 
     DEBUG(0, "pcnet_detach(0x%p)\n", link);
 
@@ -358,14 +368,7 @@ static void pcnet_detach(dev_link_t *link)
     if (*linkp == NULL)
 	return;
 
-    save_flags(flags);
-    cli();
-    if (link->state & DEV_RELEASE_PENDING) {
-	del_timer(&link->release);
-	link->state &= ~DEV_RELEASE_PENDING;
-    }
-    restore_flags(flags);
-
+    del_timer(&link->release);
     if (link->state & DEV_CONFIG) {
 	pcnet_release((u_long)link);
 	if (link->state & DEV_STALE_CONFIG) {
@@ -379,39 +382,12 @@ static void pcnet_detach(dev_link_t *link)
 
     /* Unlink device structure, free bits */
     *linkp = link->next;
-    if (link->priv) {
-	struct device *dev = link->priv;
-	if (link->dev != NULL)
-	    unregister_netdev(dev);
-	if (dev->priv)
-	    kfree_s(dev->priv, sizeof(struct ei_device));
-	kfree_s(dev, sizeof(struct pcnet_dev_t));
-    }
-    kfree_s(link, sizeof(struct dev_link_t));
+    if (link->dev)
+	unregister_netdev(&info->dev);
+    kfree(info);
 
 } /* pcnet_detach */
 
-/*======================================================================
-
-    For the Linksys EtherFast 10/100 card
-
-======================================================================*/
-
-static hw_info_t *get_dl_fast(dev_link_t *link)
-{
-    struct device *dev = link->priv;
-    int i;
-    u_char sum;
-
-    for (sum = 0, i = 0x14; i < 0x1c; i++)
-	sum += inb_p(dev->base_addr + i);
-    if (sum != 0xff)
-	return NULL;
-    for (i = 0; i < 6; i++)
-	dev->dev_addr[i] = inb_p(dev->base_addr + 0x14 + i);
-    return &dl_fast_info;
-}
-
 /*======================================================================
 
     This probes for a card's hardware address, for card types that
@@ -421,7 +397,7 @@ static hw_info_t *get_dl_fast(dev_link_t *link)
 
 static hw_info_t *get_hwinfo(dev_link_t *link)
 {
-    struct device *dev = link->priv;
+    struct net_device *dev = link->priv;
     win_req_t req;
     memreq_t mem;
     u_char *base, *virt;
@@ -471,13 +447,14 @@ static hw_info_t *get_hwinfo(dev_link_t *link)
 
 static hw_info_t *get_prom(dev_link_t *link)
 {
-    struct device *dev = link->priv;
-    unsigned char prom[32];
-    int i, j, ioaddr;
+    struct net_device *dev = link->priv;
+    ioaddr_t ioaddr = dev->base_addr;
+    u_char prom[32];
+    int i, j;
 
     /* This is lifted straight from drivers/net/ne.c */
     struct {
-	unsigned char value, offset;
+	u_char value, offset;
     } program_seq[] = {
 	{E8390_NODMA+E8390_PAGE0+E8390_STOP, E8390_CMD}, /* Select page 0*/
 	{0x48,	EN0_DCFG},	/* Set byte-wide (0x48) access. */
@@ -494,8 +471,6 @@ static hw_info_t *get_prom(dev_link_t *link)
 	{E8390_RREAD+E8390_START, E8390_CMD},
     };
 
-    ioaddr = dev->base_addr;
-
     pcnet_reset_8390(dev);
     udelay(10000);
 
@@ -518,6 +493,63 @@ static hw_info_t *get_prom(dev_link_t *link)
     return NULL;
 } /* get_prom */
 
+/*======================================================================
+
+    For DL10019 based cards, like the Linksys EtherFast
+
+======================================================================*/
+
+static hw_info_t *get_dl10019(dev_link_t *link)
+{
+    struct net_device *dev = link->priv;
+    int i;
+    u_char sum;
+
+    for (sum = 0, i = 0x14; i < 0x1c; i++)
+	sum += inb_p(dev->base_addr + i);
+    if (sum != 0xff)
+	return NULL;
+    for (i = 0; i < 6; i++)
+	dev->dev_addr[i] = inb_p(dev->base_addr + 0x14 + i);
+    i = inb(dev->base_addr + 0x1f);
+    return ((i == 0x91)||(i == 0x99)) ? &dl10022_info : &dl10019_info;
+}
+
+/*======================================================================
+
+    For Asix AX88190 based cards
+
+======================================================================*/
+
+static hw_info_t *get_ax88190(dev_link_t *link)
+{
+    struct net_device *dev = link->priv;
+    ioaddr_t ioaddr = dev->base_addr;
+    int i, j;
+
+    /* Not much of a test, but the alternatives are messy */
+    if (link->conf.ConfigBase != 0x03c0)
+	return NULL;
+
+    outb_p(0x01, EN0_DCFG);	/* Set word-wide access. */
+    outb_p(0x00, EN0_RSARLO);	/* DMA starting at 0x0400. */
+    outb_p(0x04, EN0_RSARHI);
+    outb_p(E8390_RREAD+E8390_START, E8390_CMD);
+
+    for (i = 0; i < 6; i += 2) {
+	j = inw(ioaddr + PCNET_DATAPORT);
+	dev->dev_addr[i] = j & 0xff;
+	dev->dev_addr[i+1] = j >> 8;
+    }
+#if 0
+    return &ax88190_info;
+#else
+    printk(KERN_INFO "pcnet_cs: sorry, the AX88190 chipset is not "
+	   "supported.\n");
+    return NULL;
+#endif
+}
+
 /*======================================================================
 
     This should be totally unnecessary... but when we can't figure
@@ -528,7 +560,7 @@ static hw_info_t *get_prom(dev_link_t *link)
 
 static hw_info_t *get_hwired(dev_link_t *link)
 {
-    struct device *dev = link->priv;
+    struct net_device *dev = link->priv;
     int i;
 
     for (i = 0; i < 6; i++)
@@ -573,6 +605,7 @@ static int try_io_port(dev_link_t *link)
 	link->io.Attributes2 = IO_DATA_PATH_WIDTH_16;
     }
     if (link->io.BasePort1 == 0) {
+	link->io.IOAddrLines = 16;
 	for (j = 0; j < 0x400; j += 0x20) {
 	    link->io.BasePort1 = j ^ 0x300;
 	    link->io.BasePort2 = (j ^ 0x300) + 0x10;
@@ -587,20 +620,17 @@ static int try_io_port(dev_link_t *link)
 
 static void pcnet_config(dev_link_t *link)
 {
-    client_handle_t handle;
+    client_handle_t handle = link->handle;
+    pcnet_dev_t *info = link->priv;
+    struct net_device *dev = &info->dev;
     tuple_t tuple;
     cisparse_t parse;
-    pcnet_dev_t *info;
-    struct device *dev;
     int i, last_ret, last_fn, start_pg, stop_pg, cm_offset;
     int manfid = 0, prodid = 0, has_shmem = 0;
     u_short buf[64];
+    config_info_t conf;
     hw_info_t *hw_info;
 
-    handle = link->handle;
-    info = link->priv;
-    dev = &info->dev;
-
     DEBUG(0, "pcnet_config(0x%p)\n", link);
 
     tuple.Attributes = 0;
@@ -617,6 +647,10 @@ static void pcnet_config(dev_link_t *link)
     /* Configure card */
     link->state |= DEV_CONFIG;
 
+    /* Look up current Vcc */
+    CS_CHECK(GetConfigurationInfo, handle, &conf);
+    link->conf.Vcc = conf.Vcc;
+
     tuple.DesiredTuple = CISTPL_MANFID;
     tuple.Attributes = TUPLE_RETURN_COMMON;
     if ((CardServices(GetFirstTuple, handle, &tuple) == CS_SUCCESS) &&
@@ -651,6 +685,7 @@ static void pcnet_config(dev_link_t *link)
 		     (cfg->mem.win[0].len >= 0x4000));
 	link->io.BasePort1 = io->win[i].base;
 	link->io.NumPorts1 = io->win[i].len;
+	link->io.IOAddrLines = io->flags & CISTPL_IO_LINES_MASK;
 	if (link->io.NumPorts1 + link->io.NumPorts2 >= 32) {
 	    last_ret = try_io_port(link);
 	    if (last_ret == CS_SUCCESS) break;
@@ -676,11 +711,14 @@ static void pcnet_config(dev_link_t *link)
     CS_CHECK(RequestConfiguration, handle, &link->conf);
     dev->irq = link->irq.AssignedIRQ;
     dev->base_addr = link->io.BasePort1;
-    if ((if_port == 1) || (if_port == 2))
-	dev->if_port = if_port;
-    else
-	printk(KERN_NOTICE "pcnet_cs: invalid if_port requested\n");
-    dev->tbusy = 0;
+    if (info->flags & HAS_MISC_REG) {
+	if ((if_port == 1) || (if_port == 2))
+	    dev->if_port = if_port;
+	else
+	    printk(KERN_NOTICE "pcnet_cs: invalid if_port requested\n");
+    } else {
+	dev->if_port = 0;
+    }
     if (register_netdev(dev) != 0) {
 	printk(KERN_NOTICE "pcnet_cs: register_netdev() failed\n");
 	goto failed;
@@ -690,19 +728,25 @@ static void pcnet_config(dev_link_t *link)
     if (hw_info == NULL)
 	hw_info = get_prom(link);
     if (hw_info == NULL)
-	hw_info = get_dl_fast(link);
+	hw_info = get_dl10019(link);
+    if (hw_info == NULL)
+	hw_info = get_ax88190(link);
     if (hw_info == NULL)
 	hw_info = get_hwired(link);
     
     if (hw_info == NULL) {
-	printk(KERN_NOTICE "pcnet_cs: unable to read hardware net address\n");
-	goto config_undo;
+	printk(KERN_NOTICE "pcnet_cs: unable to read hardware net"
+	       " address for io base %#3lx\n", dev->base_addr);
+	unregister_netdev(dev);
+	goto failed;
     }
 
     info->flags = hw_info->flags;
     /* Check for user overrides */
     info->flags |= (delay_output) ? DELAY_OUTPUT : 0;
-    if ((manfid == MANFID_SOCKET) && (prodid == PRODID_SOCKET_LPE))
+    if ((manfid == MANFID_SOCKET) &&
+	((prodid == PRODID_SOCKET_LPE) ||
+	 (prodid == PRODID_SOCKET_EIO)))
 	info->flags &= ~USE_BIG_BUF;
     if (!use_big_buf)
 	info->flags &= ~USE_BIG_BUF;
@@ -726,11 +770,20 @@ static void pcnet_config(dev_link_t *link)
     ei_status.word16 = 1;
     ei_status.reset_8390 = &pcnet_reset_8390;
 
+    copy_dev_name(info->node, dev);
     link->dev = &info->node;
     link->state &= ~DEV_CONFIG_PENDING;
 
-    printk(KERN_INFO "%s: NE2000 Compatible: io %#3lx, irq %d,",
-	   dev->name, dev->base_addr, dev->irq);
+    if (info->flags & IS_DL10019) {
+	dev->do_ioctl = &do_ioctl;
+	printk(KERN_INFO "%s: NE2000 (DL100%d rev %02x): ",
+	       dev->name, ((info->flags & IS_DL10022) ? 22 : 19),
+	       inb(dev->base_addr + 0x1a));
+    } else if (info->flags & IS_AX88190) {
+	printk(KERN_INFO "%s: NE2000 (AX88190): ", dev->name);
+    } else
+	printk(KERN_INFO "%s: NE2000 Compatible: ", dev->name);
+    printk("io %#3lx, irq %d,", dev->base_addr, dev->irq);
     if (info->flags & USE_SHMEM)
 	printk (" mem %#5lx,", dev->mem_start);
     if (info->flags & HAS_MISC_REG)
@@ -740,9 +793,6 @@ static void pcnet_config(dev_link_t *link)
 	printk("%02X%s", dev->dev_addr[i], ((i<5) ? ":" : "\n"));
     return;
 
-config_undo:
-    unregister_netdev(dev);
-    goto failed;
 cs_failed:
     cs_error(link->handle, last_fn, last_ret);
 failed:
@@ -780,7 +830,7 @@ static void pcnet_release(u_long arg)
     CardServices(ReleaseIO, link->handle, &link->io);
     CardServices(ReleaseIRQ, link->handle, &link->irq);
 
-    link->state &= ~(DEV_CONFIG | DEV_RELEASE_PENDING);
+    link->state &= ~DEV_CONFIG;
 
 } /* pcnet_release */
 
@@ -805,10 +855,8 @@ static int pcnet_event(event_t event, int priority,
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG) {
-	    info->dev.tbusy = 1; info->dev.start = 0;
-	    link->release.expires = RUN_AT(HZ/20);
-	    link->state |= DEV_RELEASE_PENDING;
-	    add_timer(&link->release);
+	    netif_device_detach(&info->dev);
+	    mod_timer(&link->release, jiffies + HZ/20);
 	}
 	break;
     case CS_EVENT_CARD_INSERTION:
@@ -820,9 +868,8 @@ static int pcnet_event(event_t event, int priority,
 	/* Fall through... */
     case CS_EVENT_RESET_PHYSICAL:
 	if (link->state & DEV_CONFIG) {
-	    if (link->open) {
-		info->dev.tbusy = 1; info->dev.start = 0;
-	    }
+	    if (link->open)
+		netif_device_detach(&info->dev);
 	    CardServices(ReleaseConfiguration, link->handle);
 	}
 	break;
@@ -835,7 +882,7 @@ static int pcnet_event(event_t event, int priority,
 	    if (link->open) {
 		pcnet_reset_8390(&info->dev);
 		NS8390_init(&info->dev, 1);
-		info->dev.tbusy = 0; info->dev.start = 1;
+		netif_device_attach(&info->dev);
 	    }
 	}
 	break;
@@ -843,11 +890,84 @@ static int pcnet_event(event_t event, int priority,
     return 0;
 } /* pcnet_event */
 
+/*======================================================================
+
+    MII interface support for DL10019 and DL10022 based cards
+
+    On the DL10019, the MII IO direction bit is 0x10; on  the DL10022
+    it is 0x20.  Setting both bits seems to work on both card types.
+
+======================================================================*/
+
+#define DLINK_GPIO		0x1c
+#define DLINK_DIAG		0x1d
+#define MDIO_SHIFT_CLK		0x80
+#define MDIO_DATA_OUT		0x40
+#define MDIO_DIR_WRITE		0x30
+#define MDIO_DATA_WRITE0	(MDIO_DIR_WRITE)
+#define MDIO_DATA_WRITE1	(MDIO_DIR_WRITE | MDIO_DATA_OUT)
+#define MDIO_DATA_READ		0x10
+#define MDIO_MASK		0x0f
+
+static void mdio_sync(ioaddr_t addr)
+{
+    int bits, mask = inb(addr) & MDIO_MASK;
+    for (bits = 0; bits < 32; bits++) {
+	outb(mask | MDIO_DATA_WRITE1, addr);
+	outb(mask | MDIO_DATA_WRITE1 | MDIO_SHIFT_CLK, addr);
+    }
+}
+
+static int mdio_read(ioaddr_t addr, int phy_id, int loc)
+{
+    u_int cmd = (0x06<<10)|(phy_id<<5)|loc;
+    int i, retval = 0, mask = inb(addr) & MDIO_MASK;
+
+    mdio_sync(addr);
+    for (i = 13; i >= 0; i--) {
+	int dat = (cmd&(1<<i)) ? MDIO_DATA_WRITE1 : MDIO_DATA_WRITE0;
+	outb(mask | dat, addr);
+	outb(mask | dat | MDIO_SHIFT_CLK, addr);
+    }
+    for (i = 19; i > 0; i--) {
+	outb(mask, addr);
+	retval = (retval << 1) | ((inb(addr) & MDIO_DATA_READ) != 0);
+	outb(mask | MDIO_SHIFT_CLK, addr);
+    }
+    return (retval>>1) & 0xffff;
+}
+
+static void mdio_write(ioaddr_t addr, int phy_id, int loc, int value)
+{
+    u_int cmd = (0x05<<28)|(phy_id<<23)|(loc<<18)|(1<<17)|value;
+    int i, mask = inb(addr) & MDIO_MASK;
+
+    mdio_sync(addr);
+    for (i = 31; i >= 0; i--) {
+	int dat = (cmd&(1<<i)) ? MDIO_DATA_WRITE1 : MDIO_DATA_WRITE0;
+	outb(mask | dat, addr);
+	outb(mask | dat | MDIO_SHIFT_CLK, addr);
+    }
+    for (i = 1; i >= 0; i--) {
+	outb(mask, addr);
+	outb(mask | MDIO_SHIFT_CLK, addr);
+    }
+}
+
+static void mdio_reset(ioaddr_t addr, int phy_id)
+{
+    outb_p(0x08, addr);
+    outb_p(0x0c, addr);
+    outb_p(0x08, addr);
+    outb_p(0x0c, addr);
+    outb_p(0x00, addr);
+}
+
 /*====================================================================*/
 
-static void set_misc_reg(struct device *dev)
+static void set_misc_reg(struct net_device *dev)
 {
-    int nic_base = dev->base_addr;
+    ioaddr_t nic_base = dev->base_addr;
     pcnet_dev_t *info = (pcnet_dev_t *)dev;
     u_char tmp;
     
@@ -861,60 +981,58 @@ static void set_misc_reg(struct device *dev)
 	    tmp |= 8;
 	outb_p(tmp, nic_base + PCNET_MISC);
     }
+    if (info->flags & IS_DL10022) {
+	mdio_reset(nic_base + DLINK_GPIO, 0);
+	/* Restart MII autonegotiation */
+	mdio_write(nic_base + DLINK_GPIO, 0, 0, 0x0000);
+	mdio_write(nic_base + DLINK_GPIO, 0, 0, 0x1200);
+    }
 }
 
 /*====================================================================*/
 
-static int pcnet_open(struct device *dev)
+static int pcnet_open(struct net_device *dev)
 {
     pcnet_dev_t *info = (pcnet_dev_t *)dev;
-    dev_link_t *link;
+    dev_link_t *link = &info->link;
     
     DEBUG(2, "pcnet_open('%s')\n", dev->name);
 
-    for (link = dev_list; link; link = link->next)
-	if (link->priv == dev) break;
     if (!DEV_OK(link))
 	return -ENODEV;
 
     link->open++;
     MOD_INC_USE_COUNT;
 
-    /* For D-Link EtherFast, wait for something(?) to happen */
-    if (info->flags & IS_DL10019A) {
-	int i;
-	for (i = 0; i < 20; i++) {
-	    if ((inb(dev->base_addr+0x1c) & 0x01) == 0) break;
-	    current->state = TASK_INTERRUPTIBLE;
-	    schedule_timeout(HZ/10);
-	}
-    }
-    
     set_misc_reg(dev);
-    request_irq(dev->irq, ei_interrupt, SA_SHIRQ, dev_info, dev);
+    request_irq(dev->irq, ei_irq_wrapper, SA_SHIRQ, dev_info, dev);
+
+    info->link_status = 0x00;
+    info->watchdog.function = &ei_watchdog;
+    info->watchdog.data = (u_long)info;
+    info->watchdog.expires = jiffies + HZ;
+    add_timer(&info->watchdog);
+
     return ei_open(dev);
 } /* pcnet_open */
 
 /*====================================================================*/
 
-static int pcnet_close(struct device *dev)
+static int pcnet_close(struct net_device *dev)
 {
-    dev_link_t *link;
+    pcnet_dev_t *info = (pcnet_dev_t *)dev;
+    dev_link_t *link = &info->link;
 
     DEBUG(2, "pcnet_close('%s')\n", dev->name);
 
-    for (link = dev_list; link; link = link->next)
-	if (link->priv == dev) break;
-    if (link == NULL)
-	return -ENODEV;
     free_irq(dev->irq, dev);
     
-    link->open--; dev->start = 0;
-    if (link->state & DEV_STALE_CONFIG) {
-	link->release.expires = RUN_AT(HZ/20);
-	link->state |= DEV_RELEASE_PENDING;
-	add_timer(&link->release);
-    }
+    link->open--;
+    netif_stop_queue(dev);
+    netif_mark_down(dev);
+    del_timer(&info->watchdog);
+    if (link->state & DEV_STALE_CONFIG)
+	mod_timer(&link->release, jiffies + HZ/20);
 
     MOD_DEC_USE_COUNT;
 
@@ -928,9 +1046,9 @@ static int pcnet_close(struct device *dev)
 
 ======================================================================*/
 
-static void pcnet_reset_8390(struct device *dev)
+static void pcnet_reset_8390(struct net_device *dev)
 {
-    int nic_base = dev->base_addr;
+    ioaddr_t nic_base = dev->base_addr;
     int i;
 
     ei_status.txing = ei_status.dmaing = 0;
@@ -951,40 +1069,113 @@ static void pcnet_reset_8390(struct device *dev)
     
 } /* pcnet_reset_8390 */
 
-/* ======================================================================= */
+/*====================================================================*/
 
-static int set_config(struct device *dev, struct ifmap *map)
+static int set_config(struct net_device *dev, struct ifmap *map)
 {
     pcnet_dev_t *info = (pcnet_dev_t *)dev;
     if ((map->port != (u_char)(-1)) && (map->port != dev->if_port)) {
-	if ((map->port != 0) && !(info->flags & HAS_MISC_REG)) {
-	    printk(KERN_NOTICE "%s: transceiver selection not "
-		   "implemented\n", dev->name);
-	    return -EINVAL;
-	}
-	if ((map->port == 1) || (map->port == 2)) {
-	    dev->if_port = map->port;
-	    printk(KERN_INFO "%s: switched to %s port\n",
-		   dev->name, if_names[dev->if_port]);
-	} else
+	if (!(info->flags & HAS_MISC_REG))
+	    return -EOPNOTSUPP;
+	else if ((map->port < 1) || (map->port > 2))
 	    return -EINVAL;
+	dev->if_port = map->port;
+	printk(KERN_INFO "%s: switched to %s port\n",
+	       dev->name, if_names[dev->if_port]);
+	NS8390_init(dev, 1);
     }
     return 0;
 }
 
-/* ======================================================================= */
+/*====================================================================*/
 
-static void dma_get_8390_hdr(struct device *dev,
+static void ei_irq_wrapper(int irq, void *dev_id, struct pt_regs *regs)
+{
+    pcnet_dev_t *info = dev_id;
+    info->stale = 0;
+    ei_interrupt(irq, dev_id, regs);
+}
+
+static void ei_watchdog(u_long arg)
+{
+    pcnet_dev_t *info = (pcnet_dev_t *)(arg);
+    struct net_device *dev = &info->dev;
+    ioaddr_t nic_base = dev->base_addr;
+    u_short link;
+
+    if (!netif_device_present(dev)) goto reschedule;
+
+    /* Check for pending interrupt with expired latency timer: with
+       this, we can limp along even if the interrupt is blocked */
+    outb_p(E8390_NODMA+E8390_PAGE0, nic_base + E8390_CMD);
+    if (info->stale++ && (inb_p(nic_base + EN0_ISR) & ENISR_ALL)) {
+	if (!info->fast_poll)
+	    printk(KERN_INFO "%s: interrupt(s) dropped!\n", dev->name);
+	ei_irq_wrapper(dev->irq, dev, NULL);
+	info->fast_poll = HZ;
+    }
+    if (info->fast_poll) {
+	info->fast_poll--;
+	info->watchdog.expires = jiffies + 1;
+	add_timer(&info->watchdog);
+	return;
+    }
+
+    if (!(info->flags & IS_DL10019))
+	goto reschedule;
+
+    link = mdio_read(dev->base_addr + DLINK_GPIO, 0, 1) & 0x0004;
+    if (link != info->link_status) {
+	printk(KERN_INFO "%s: %s link beat\n", dev->name,
+	       (link) ? "found" : "lost");
+	if (link && info->flags & IS_DL10022) {
+	    /* Disable collision detection on full duplex links */
+	    u_short p = mdio_read(dev->base_addr + DLINK_GPIO, 0, 5);
+	    outb((p & 0x0140) ? 4 : 0, dev->base_addr + DLINK_DIAG);
+	}
+	if (link)
+	    NS8390_init(dev, 1);
+	info->link_status = link;
+    }
+
+reschedule:
+    info->watchdog.expires = jiffies + HZ;
+    add_timer(&info->watchdog);
+}
+
+/*====================================================================*/
+
+static int do_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
+{
+    u16 *data = (u16 *)&rq->ifr_data;
+    ioaddr_t addr = dev->base_addr + DLINK_GPIO;
+    switch (cmd) {
+    case SIOCDEVPRIVATE:
+	data[0] = 0;
+    case SIOCDEVPRIVATE+1:
+	data[3] = mdio_read(addr, data[0], data[1] & 0x1f);
+	return 0;
+    case SIOCDEVPRIVATE+2:
+	if (!capable(CAP_NET_ADMIN))
+	    return -EPERM;
+	mdio_write(addr, data[0], data[1] & 0x1f, data[2]);
+	return 0;
+    }
+    return -EOPNOTSUPP;
+}
+
+/*====================================================================*/
+
+static void dma_get_8390_hdr(struct net_device *dev,
 			     struct e8390_pkt_hdr *hdr,
 			     int ring_page)
 {
-    int nic_base = dev->base_addr;
+    ioaddr_t nic_base = dev->base_addr;
 
     if (ei_status.dmaing) {
 	printk(KERN_NOTICE "%s: DMAing conflict in dma_block_input."
-	       "[DMAstat:%1x][irqlock:%1x][intr:%ld]\n",
-	       dev->name, ei_status.dmaing, ei_status.irqlock,
-	       (long)dev->interrupt);
+	       "[DMAstat:%1x][irqlock:%1x]\n",
+	       dev->name, ei_status.dmaing, ei_status.irqlock);
 	return;
     }
     
@@ -1005,12 +1196,12 @@ static void dma_get_8390_hdr(struct device *dev,
     ei_status.dmaing &= ~0x01;
 }
 
-/* ======================================================================= */
+/*====================================================================*/
 
-static void dma_block_input(struct device *dev, int count,
+static void dma_block_input(struct net_device *dev, int count,
 			    struct sk_buff *skb, int ring_offset)
 {
-    int nic_base = dev->base_addr;
+    ioaddr_t nic_base = dev->base_addr;
     int xfer_count = count;
     char *buf = skb->data;
 
@@ -1020,9 +1211,8 @@ static void dma_block_input(struct device *dev, int count,
 #endif
     if (ei_status.dmaing) {
 	printk(KERN_NOTICE "%s: DMAing conflict in dma_block_input."
-	       "[DMAstat:%1x][irqlock:%1x][intr:%ld]\n",
-	       dev->name, ei_status.dmaing, ei_status.irqlock,
-	       (long)dev->interrupt);
+	       "[DMAstat:%1x][irqlock:%1x]\n",
+	       dev->name, ei_status.dmaing, ei_status.irqlock);
 	return;
     }
     ei_status.dmaing |= 0x01;
@@ -1063,11 +1253,10 @@ static void dma_block_input(struct device *dev, int count,
 
 /*====================================================================*/
 
-static void dma_block_output(struct device *dev, int count,
-			     const unsigned char *buf,
-			     const int start_page)
+static void dma_block_output(struct net_device *dev, int count,
+			     const u_char *buf, const int start_page)
 {
-    int nic_base = dev->base_addr;
+    ioaddr_t nic_base = dev->base_addr;
     pcnet_dev_t *info = (pcnet_dev_t *)dev;
 #ifdef PCMCIA_DEBUG
     int retries = 0;
@@ -1086,9 +1275,8 @@ static void dma_block_output(struct device *dev, int count,
 	count++;
     if (ei_status.dmaing) {
 	printk(KERN_NOTICE "%s: DMAing conflict in dma_block_output."
-	       "[DMAstat:%1x][irqlock:%1x][intr:%ld]\n",
-	       dev->name, ei_status.dmaing, ei_status.irqlock,
-	       (long)dev->interrupt);
+	       "[DMAstat:%1x][irqlock:%1x]\n",
+	       dev->name, ei_status.dmaing, ei_status.irqlock);
 	return;
     }
     ei_status.dmaing |= 0x01;
@@ -1154,7 +1342,7 @@ static void dma_block_output(struct device *dev, int count,
 static int setup_dma_config(dev_link_t *link, int start_pg,
 			    int stop_pg)
 {
-    struct device *dev = link->priv;
+    struct net_device *dev = link->priv;
 
     ei_status.tx_start_page = start_pg;
     ei_status.rx_start_page = start_pg + TX_PAGES;
@@ -1170,33 +1358,31 @@ static int setup_dma_config(dev_link_t *link, int start_pg,
 
 /*====================================================================*/
 
-static void copyin(unsigned char *dest, unsigned char *src, int c)
+static void copyin(u_char *dest, u_char *src, int c)
 {
-    unsigned short *d = (unsigned short *) dest;
-    unsigned short *s = (unsigned short *) src;
+    u_short *d = (u_short *)dest, *s = (u_short *)src;
     int odd;
 
     if (c <= 0)
 	return;
-    odd = (c & 01); c >>= 1;
+    odd = (c & 1); c >>= 1;
 
     if (c) {
 	do { *d++ = readw_ns(s++); } while (--c);
     }
     /* get last byte by fetching a word and masking */
     if (odd)
-	*((unsigned char *)d) = readw(s) & 0xff;
+	*((u_char *)d) = readw(s) & 0xff;
 }
 
-static void copyout(unsigned char *dest, const unsigned char *src, int c)
+static void copyout(u_char *dest, const u_char *src, int c)
 {
-    volatile unsigned short *d = (unsigned short *) dest;
-    unsigned short *s = (unsigned short *) src;
+    u_short *d = (u_short *)dest, *s = (u_short *)src;
     int odd;
 
     if (c <= 0)
 	return;
-    odd = (c & 01); c >>= 1;
+    odd = (c & 1); c >>= 1;
 
     if (c) {
 	do { writew_ns(*s++, d++); } while (--c);
@@ -1208,7 +1394,7 @@ static void copyout(unsigned char *dest, const unsigned char *src, int c)
 
 /*====================================================================*/
 
-static void shmem_get_8390_hdr(struct device *dev,
+static void shmem_get_8390_hdr(struct net_device *dev,
 			       struct e8390_pkt_hdr *hdr,
 			       int ring_page)
 {
@@ -1222,7 +1408,7 @@ static void shmem_get_8390_hdr(struct device *dev,
 
 /*====================================================================*/
 
-static void shmem_block_input(struct device *dev, int count,
+static void shmem_block_input(struct net_device *dev, int count,
 			      struct sk_buff *skb, int ring_offset)
 {
     void *xfer_start = (void *)(dev->rmem_start + ring_offset
@@ -1243,16 +1429,11 @@ static void shmem_block_input(struct device *dev, int count,
 
 /*====================================================================*/
 
-static void shmem_block_output(struct device *dev, int count,
-			       const unsigned char *buf,
-			       const int start_page)
+static void shmem_block_output(struct net_device *dev, int count,
+			       const u_char *buf, const int start_page)
 {
     void *shmem = (void *)dev->mem_start + (start_page << 8);
     shmem -= ei_status.tx_start_page << 8;
-
-    if (ei_debug > 4)
-	printk(KERN_DEBUG "[bo=%d @ %x]\n", count, start_page);
-
     copyout(shmem, buf, count);
 }
 
@@ -1261,7 +1442,7 @@ static void shmem_block_output(struct device *dev, int count,
 static int setup_shmem_window(dev_link_t *link, int start_pg,
 			      int stop_pg, int cm_offset)
 {
-    struct device *dev = link->priv;
+    struct net_device *dev = link->priv;
     pcnet_dev_t *info = link->priv;
     win_req_t req;
     memreq_t mem;
@@ -1328,7 +1509,7 @@ failed:
 
 /*====================================================================*/
 
-int init_module(void)
+static int __init init_pcnet_cs(void)
 {
     servinfo_t serv;
     DEBUG(0, "%s\n", version);
@@ -1338,14 +1519,17 @@ int init_module(void)
 	       "does not match!\n");
 	return -1;
     }
-    register_pcmcia_driver(&dev_info, &pcnet_attach, &pcnet_detach);
+    register_pccard_driver(&dev_info, &pcnet_attach, &pcnet_detach);
     return 0;
 }
 
-void cleanup_module(void)
+static void __exit exit_pcnet_cs(void)
 {
     DEBUG(0, "pcnet_cs: unloading\n");
-    unregister_pcmcia_driver(&dev_info);
+    unregister_pccard_driver(&dev_info);
     while (dev_list != NULL)
 	pcnet_detach(dev_list);
 }
+
+module_init(init_pcnet_cs);
+module_exit(exit_pcnet_cs);
