@@ -2,7 +2,7 @@
 
     A driver for PCMCIA serial devices
 
-    serial_cs.c 1.106 1999/07/20 16:00:02
+    serial_cs.c 1.120 2000/06/05 23:02:12
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,15 +15,28 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
 #include <pcmcia/config.h>
 #include <pcmcia/k_compat.h>
 
+#include <linux/module.h>
 #include <linux/kernel.h>
+#include <linux/init.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
 #include <linux/malloc.h>
@@ -48,7 +61,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"serial_cs.c 1.106 1999/07/20 16:00:02 (David Hinds)";
+"serial_cs.c 1.120 2000/06/05 23:02:12 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -90,6 +103,7 @@ static multi_id_t multi_id[] = {
 #define MULTI_COUNT (sizeof(multi_id)/sizeof(multi_id_t))
 
 typedef struct serial_info_t {
+    dev_link_t	link;
     int		ndev;
     int		multi;
     int		slave;
@@ -128,6 +142,7 @@ static void cs_error(client_handle_t handle, int func, int ret)
 
 static dev_link_t *serial_attach(void)
 {
+    serial_info_t *info;
     client_reg_t client_reg;
     dev_link_t *link;
     int i, ret;
@@ -135,13 +150,15 @@ static dev_link_t *serial_attach(void)
     DEBUG(0, "serial_attach()\n");
 
     /* Create new serial device */
-    link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
-    memset(link, 0, sizeof(struct dev_link_t));
+    info = kmalloc(sizeof(*info), GFP_KERNEL);
+    if (!info) return NULL;
+    memset(info, 0, sizeof(*info));
+    link = &info->link; link->priv = info;
+
     link->release.function = &serial_release;
     link->release.data = (u_long)link;
     link->io.Attributes1 = IO_DATA_PATH_WIDTH_8;
     link->io.NumPorts1 = 8;
-    link->io.IOAddrLines = 3;
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE;
     link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
     if (irq_list[0] == -1)
@@ -156,8 +173,6 @@ static dev_link_t *serial_attach(void)
 	link->conf.Status = CCSR_AUDIO_ENA;
     }
     link->conf.IntType = INT_MEMORY_AND_IO;
-    link->priv = kmalloc(sizeof(struct serial_info_t), GFP_KERNEL);
-    memset(link->priv, 0, sizeof(struct serial_info_t));
     
     /* Register with Card Services */
     link->next = dev_list;
@@ -192,8 +207,8 @@ static dev_link_t *serial_attach(void)
 
 static void serial_detach(dev_link_t *link)
 {
+    serial_info_t *info = link->priv;
     dev_link_t **linkp;
-    long flags;
     int ret;
 
     DEBUG(0, "serial_detach(0x%p)\n", link);
@@ -204,14 +219,7 @@ static void serial_detach(dev_link_t *link)
     if (*linkp == NULL)
 	return;
 
-    save_flags(flags);
-    cli();
-    if (link->state & DEV_RELEASE_PENDING) {
-	del_timer(&link->release);
-	link->state &= ~DEV_RELEASE_PENDING;
-    }
-    restore_flags(flags);
-    
+    del_timer(&link->release);
     if (link->state & DEV_CONFIG)
 	serial_release((u_long)link);
     
@@ -223,8 +231,7 @@ static void serial_detach(dev_link_t *link)
     
     /* Unlink device structure, free bits */
     *linkp = link->next;
-    kfree_s(link->priv, sizeof(serial_info_t));
-    kfree_s(link, sizeof(struct dev_link_t));
+    kfree(info);
     
 } /* serial_detach */
 
@@ -235,14 +242,14 @@ static int setup_serial(serial_info_t *info, ioaddr_t port, int irq)
     struct serial_struct serial;
     int line;
     
+    memset(&serial, 0, sizeof(serial));
     serial.port = port;
     serial.irq = irq;
-    serial.flags = ASYNC_SKIP_TEST;
-    serial.flags |= (info->multi || info->slave) ? ASYNC_SHARE_IRQ : 0;
+    serial.flags = ASYNC_SKIP_TEST | ASYNC_SHARE_IRQ;
     line = register_serial(&serial);
     if (line < 0) {
-	printk(KERN_NOTICE "serial_cs: register_serial() at 0x%04x, "
-	       "irq %d failed\n", serial.port, serial.irq);
+	printk(KERN_NOTICE "serial_cs: register_serial() at 0x%04lx,"
+	       " irq %d failed\n", (u_long)serial.port, serial.irq);
 	return -1;
     }
     
@@ -316,9 +323,11 @@ static int simple_config(dev_link_t *link)
 	if (cf->vpp1.present & (1<<CISTPL_POWER_VNOM))
 	    link->conf.Vpp1 = link->conf.Vpp2 =
 		cf->vpp1.param[CISTPL_POWER_VNOM]/10000;
-	if ((cf->io.nwin > 0) && ((cf->io.win[0].base & 0xf) == 8)) {
+	if ((cf->io.nwin > 0) && (cf->io.win[0].len == 8) &&
+	    (cf->io.win[0].base != 0)) {
 	    link->conf.ConfigIndex = cf->index;
 	    link->io.BasePort1 = cf->io.win[0].base;
+	    link->io.IOAddrLines = cf->io.flags & CISTPL_IO_LINES_MASK;
 	    i = CardServices(RequestIO, link->handle, &link->io);
 	    if (i == CS_SUCCESS) goto found_port;
 	}
@@ -336,6 +345,7 @@ static int simple_config(dev_link_t *link)
 	    link->conf.ConfigIndex = cf->index;
 	    for (j = 0; j < 5; j++) {
 		link->io.BasePort1 = base[j];
+		link->io.IOAddrLines = base[j] ? 16 : 3;
 		i = CardServices(RequestIO, link->handle,
 				 &link->io);
 		if (i == CS_SUCCESS) goto found_port;
@@ -391,6 +401,7 @@ static int multi_config(dev_link_t *link)
 	    (cf->io.win[0].len > 8)) {
 	    link->conf.ConfigIndex = cf->index;
 	    link->io.BasePort1 = cf->io.win[0].base;
+	    link->io.IOAddrLines = cf->io.flags & CISTPL_IO_LINES_MASK;
 	    i = CardServices(RequestIO, link->handle, &link->io);
 	    base2 = link->io.BasePort1 + 8;
 	    if (i == CS_SUCCESS) break;
@@ -408,6 +419,7 @@ static int multi_config(dev_link_t *link)
 		link->conf.ConfigIndex = cf->index;
 		link->io.BasePort1 = cf->io.win[0].base;
 		link->io.BasePort2 = cf->io.win[1].base;
+		link->io.IOAddrLines = cf->io.flags & CISTPL_IO_LINES_MASK;
 		i = CardServices(RequestIO, link->handle, &link->io);
 		base2 = link->io.BasePort2;
 		if (i == CS_SUCCESS) break;
@@ -457,18 +469,14 @@ while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
 
 void serial_config(dev_link_t *link)
 {
-    client_handle_t handle;
-    serial_info_t *info;
+    client_handle_t handle = link->handle;
+    serial_info_t *info = link->priv;
     tuple_t tuple;
     u_short buf[128];
     cisparse_t parse;
     cistpl_cftable_entry_t *cf = &parse.cftable_entry;
     int i, last_ret, last_fn;
 
-    sti();
-    handle = link->handle;
-    info = link->priv;
-    
     DEBUG(0, "serial_config(0x%p)\n", link);
     
     tuple.TupleData = (cisdata_t *)buf;
@@ -559,8 +567,6 @@ void serial_release(u_long arg)
     serial_info_t *info = link->priv;
     int i;
     
-    sti();
-    
     DEBUG(0, "serial_release(0x%p)\n", link);
 
     for (i = 0; i < info->ndev; i++) {
@@ -598,11 +604,8 @@ static int serial_event(event_t event, int priority,
     switch (event) {
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
-	if (link->state & DEV_CONFIG) {
-	    link->release.expires = RUN_AT(HZ/20);
-	    link->state |= DEV_RELEASE_PENDING;
-	    add_timer(&link->release);
-	}
+	if (link->state & DEV_CONFIG)
+	    mod_timer(&link->release, jiffies + HZ/20);
 	break;
     case CS_EVENT_CARD_INSERTION:
 	link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
@@ -628,7 +631,7 @@ static int serial_event(event_t event, int priority,
 
 /*====================================================================*/
 
-int init_module(void)
+static int __init init_serial_cs(void)
 {
     servinfo_t serv;
     DEBUG(0, "%s\n", version);
@@ -638,14 +641,17 @@ int init_module(void)
 	       "does not match!\n");
 	return -1;
     }
-    register_pcmcia_driver(&dev_info, &serial_attach, &serial_detach);
+    register_pccard_driver(&dev_info, &serial_attach, &serial_detach);
     return 0;
 }
 
-void cleanup_module(void)
+static void __exit exit_serial_cs(void)
 {
     DEBUG(0, "serial_cs: unloading\n");
-    unregister_pcmcia_driver(&dev_info);
+    unregister_pccard_driver(&dev_info);
     while (dev_list != NULL)
 	serial_detach(dev_list);
 }
+
+module_init(init_serial_cs);
+module_exit(exit_serial_cs);
