@@ -3,7 +3,7 @@
     A utility to convert a plain text description of a Card
     Information Structure into its packed binary representation.
 
-    pack_cis.c 1.10 1999/07/20 16:03:50
+    pack_cis.c 1.15 2000/01/25 02:29:52
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -16,8 +16,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
 
     Usage:
 
@@ -88,9 +99,7 @@ static int pack_io(cistpl_io_t *p, u_char *b)
 {
     u_char *c = b;
     u_int i, j, ml, ma;
-    *c = 0;
-    *c |= (p->flags & CISTPL_IO_8BIT) ? 0x20 : 0;
-    *c |= (p->flags & CISTPL_IO_16BIT) ? 0x40 : 0;
+    *c = p->flags & (CISTPL_IO_8BIT|CISTPL_IO_16BIT);
     if ((p->nwin == 1) && (p->win[0].base == 0)) {
 	for (i = 1, j = 0; i < p->win[0].len; i *= 2, j++) ;
 	*c |= j; c++;
@@ -101,7 +110,7 @@ static int pack_io(cistpl_io_t *p, u_char *b)
 	}
 	ma = (ma > 0xffff) ? 3 : ((ma > 0xff) ? 2 : 1);
 	ml = (ml > 0xffff) ? 3 : ((ml > 0xff) ? 2 : 1);
-	*c |= 0x80; c++;
+	*c |= 0x80 | (p->flags & CISTPL_IO_LINES_MASK); c++;
 	*c = (p->nwin-1) | (ma<<4) | (ml<<6); c++;
 	if (ma == 3) ma++; if (ml == 3) ml++;
 	for (i = 0; i < p->nwin; i++) {
@@ -189,6 +198,10 @@ static void pack_cftable(cistpl_cftable_entry_t *p, u_char *b)
 	b[4] |= 0x60;
 	c += pack_mem(&p->mem, c);
     }
+    if (p->flags >> 8) {
+	b[4] |= 0x80;
+	*c++ = p->flags >> 8;
+    }
     b[1] = c-b-2;
 }
 
@@ -250,6 +263,7 @@ static int pack_tuple(tuple_info_t *t, u_char *b)
     *b = t->type;
     switch (t->type) {
     case CISTPL_DEVICE:
+    case CISTPL_DEVICE_A:
 	if (p) {
 	    pack_device(&p->device, b);
 	} else {
