@@ -2,7 +2,7 @@
 
     PCMCIA Bulk Memory Services
 
-    bulkmem.c 1.28 1999/07/20 16:01:19
+    bulkmem.c 1.36 2000/05/09 20:53:56
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,8 +15,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
@@ -25,6 +36,7 @@
 #include <pcmcia/k_compat.h>
 
 #ifdef __LINUX__
+#include <linux/module.h>
 #include <linux/kernel.h>
 #include <linux/string.h>
 #include <linux/errno.h>
@@ -63,7 +75,7 @@ static int do_mtd_request(memory_handle_t handle, mtd_request_t *req,
 	return CS_GENERAL_FAILURE;
     s = SOCKET(mtd);
     wacquire(&mtd->mtd_req);
-    for (tries = 0; tries < 100; tries++) {
+    for (ret = tries = 0; tries < 100; tries++) {
 	mtd->event_callback_args.mtdrequest = req;
 	mtd->event_callback_args.buffer = buf;
 	ret = EVENT(mtd, CS_EVENT_MTD_REQUEST, CS_EVENT_PRI_LOW);
@@ -106,7 +118,7 @@ static int do_mtd_request(memory_handle_t handle, mtd_request_t *req,
 
 static void insert_queue(erase_busy_t *head, erase_busy_t *entry)
 {
-    DEBUG(2, ("cs: adding 0x%p to queue 0x%p\n", entry, head));
+    DEBUG(2, "cs: adding 0x%p to queue 0x%p\n", entry, head);
     entry->next = head;
     entry->prev = head->prev;
     head->prev->next = entry;
@@ -115,7 +127,7 @@ static void insert_queue(erase_busy_t *head, erase_busy_t *entry)
 
 static void remove_queue(erase_busy_t *entry)
 {
-    DEBUG(2, ("cs: unqueueing 0x%p\n", entry));
+    DEBUG(2, "cs: unqueueing 0x%p\n", entry);
     entry->next->prev = entry->prev;
     entry->prev->next = entry->next;
 }
@@ -128,7 +140,7 @@ static void retry_erase(erase_busy_t *busy, u_int cause)
     socket_info_t *s;
     int ret;
 
-    DEBUG(2, ("cs: trying erase request 0x%p...\n", busy));
+    DEBUG(2, "cs: trying erase request 0x%p...\n", busy);
     if (busy->next)
 	remove_queue(busy);
     req.Function = MTD_REQ_ERASE | cause;
@@ -142,7 +154,7 @@ static void retry_erase(erase_busy_t *busy, u_int cause)
     ret = EVENT(mtd, CS_EVENT_MTD_REQUEST, CS_EVENT_PRI_LOW);
     wrelease(&mtd->mtd_req);
     if (ret == CS_BUSY) {
-	DEBUG(2, ("  Status = %d, requeueing.\n", req.Status));
+	DEBUG(2, "  Status = %d, requeueing.\n", req.Status);
 	switch (req.Status) {
 	case MTD_WAITREQ:
 	case MTD_WAITPOWER:
@@ -152,13 +164,13 @@ static void retry_erase(erase_busy_t *busy, u_int cause)
 	case MTD_WAITRDY:
 	    if (req.Status == MTD_WAITRDY)
 		insert_queue(&s->erase_busy, busy);
-	    busy->timeout.expires = RUN_AT(req.Timeout*HZ/1000);
+	    busy->timeout.expires = jiffies + req.Timeout*HZ/1000;
 	    add_timer(&busy->timeout);
 	    break;
 	}
     } else {
 	/* update erase queue status */
-	DEBUG(2, ("  Ret = %d\n", ret));
+	DEBUG(2, "  Ret = %d\n", ret);
 	switch (ret) {
 	case CS_SUCCESS:
 	    erase->State = ERASE_PASSED; break;
@@ -175,7 +187,7 @@ static void retry_erase(erase_busy_t *busy, u_int cause)
 	}
 	busy->client->event_callback_args.info = erase;
 	EVENT(busy->client, CS_EVENT_ERASE_COMPLETE, CS_EVENT_PRI_LOW);
-	kfree_s(busy, sizeof(*busy));
+	kfree(busy);
 	/* Resubmit anything waiting for a request to finish */
 	wakeup(&mtd->mtd_req);
 	retry_erase_list(&mtd->erase_busy, 0);
@@ -186,7 +198,7 @@ void retry_erase_list(erase_busy_t *list, u_int cause)
 {
     erase_busy_t tmp = *list;
 
-    DEBUG(2, ("cs: rescanning erase queue list 0x%p\n", list));
+    DEBUG(2, "cs: rescanning erase queue list 0x%p\n", list);
     if (list->next == list)
 	return;
     /* First, truncate the original list */
@@ -203,7 +215,7 @@ void retry_erase_list(erase_busy_t *list, u_int cause)
 
 static void handle_erase_timeout(u_long arg)
 {
-    DEBUG(0, ("cs: erase timeout for entry 0x%lx\n", arg));
+    DEBUG(0, "cs: erase timeout for entry 0x%lx\n", arg);
     retry_erase((erase_busy_t *)arg, MTD_REQ_TIMEOUT);
 }
 
@@ -227,7 +239,7 @@ static void setup_erase_request(client_handle_t handle, eraseq_entry_t *erase)
 	    busy = kmalloc(sizeof(erase_busy_t), GFP_KERNEL);
 	    busy->erase = erase;
 	    busy->client = handle;
-	    busy->timeout.prev = busy->timeout.next = NULL;
+	    init_timer(&busy->timeout);
 	    busy->timeout.data = (u_long)busy;
 	    busy->timeout.function = &handle_erase_timeout;
 	    busy->prev = busy->next = NULL;
@@ -322,8 +334,8 @@ static void setup_regions(client_handle_t handle, int attr,
     cistpl_device_geo_t geo;
     memory_handle_t r;
 
-    DEBUG(1, ("cs: setup_regions(0x%p, %d, 0x%p)\n",
-	      handle, attr, list));
+    DEBUG(1, "cs: setup_regions(0x%p, %d, 0x%p)\n",
+	  handle, attr, list);
 
     code = (attr) ? CISTPL_DEVICE_A : CISTPL_DEVICE;
     if (read_tuple(handle, code, &device) != CS_SUCCESS)
@@ -443,8 +455,8 @@ int register_mtd(client_handle_t handle, mtd_reg_t *reg)
 	list = s->a_region;
     else
 	list = s->c_region;
-    DEBUG(1, ("cs: register_mtd(0x%p, '%s', 0x%x)\n",
-	      handle, handle->dev_info, reg->Offset));
+    DEBUG(1, "cs: register_mtd(0x%p, '%s', 0x%x)\n",
+	  handle, handle->dev_info, reg->Offset);
     while (list) {
 	if (list->info.CardOffset == reg->Offset) break;
 	list = list->info.next;
@@ -462,7 +474,7 @@ int register_mtd(client_handle_t handle, mtd_reg_t *reg)
 
 /*======================================================================
 
-    
+    Erase queue management functions
     
 ======================================================================*/
 
@@ -473,6 +485,7 @@ int register_erase_queue(client_handle_t *handle, eraseq_hdr_t *header)
     if ((handle == NULL) || CHECK_HANDLE(*handle))
 	return CS_BAD_HANDLE;
     queue = kmalloc(sizeof(*queue), GFP_KERNEL);
+    if (!queue) return CS_OUT_OF_RESOURCE;
     queue->eraseq_magic = ERASEQ_MAGIC;
     queue->handle = *handle;
     queue->count = header->QueueEntryCnt;
@@ -491,7 +504,7 @@ int deregister_erase_queue(eraseq_handle_t eraseq)
     if (i < eraseq->count)
 	return CS_BUSY;
     eraseq->eraseq_magic = 0;
-    kfree_s(eraseq, sizeof(*eraseq));
+    kfree(eraseq);
     return CS_SUCCESS;
 } /* deregister_erase_queue */
 
@@ -540,8 +553,8 @@ int open_memory(client_handle_t *handle, open_mem_t *open)
 #endif
     if (region && region->mtd) {
 	*handle = (client_handle_t)region;
-	DEBUG(1, ("cs: open_memory(0x%p, 0x%x) = 0x%p\n",
-		  handle, open->Offset, region));
+	DEBUG(1, "cs: open_memory(0x%p, 0x%x) = 0x%p\n",
+	      handle, open->Offset, region);
 	return CS_SUCCESS;
     } else
 	return CS_BAD_OFFSET;
@@ -557,7 +570,7 @@ int open_memory(client_handle_t *handle, open_mem_t *open)
 
 int close_memory(memory_handle_t handle)
 {
-    DEBUG(1, ("cs: close_memory(0x%p)\n", handle));
+    DEBUG(1, "cs: close_memory(0x%p)\n", handle);
     if (CHECK_REGION(handle))
 	return CS_BAD_HANDLE;
 #ifdef __BEOS__
@@ -624,6 +637,7 @@ int write_memory(memory_handle_t handle, mem_op_t *req, caddr_t buf)
 
 /*======================================================================
 
+    This isn't needed for anything I could think of.
     
 ======================================================================*/
 
@@ -632,5 +646,5 @@ int copy_memory(memory_handle_t handle, copy_op_t *req)
     if (CHECK_REGION(handle))
 	return CS_BAD_HANDLE;
     return CS_UNSUPPORTED_FUNCTION;
-} /* close_memory */
+}
 
