@@ -2,7 +2,7 @@
   
     Cardbus device configuration
     
-    cardbus.c 1.52 1999/07/20 16:01:21
+    cardbus.c 1.74 2000/05/20 03:16:31
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,8 +15,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
     These routines handle allocating resources for Cardbus cards, as
     well as setting up and shutting down Cardbus sockets.  They are
@@ -29,8 +40,9 @@
 #define __NO_VERSION__
 #include <pcmcia/k_compat.h>
 
-#ifdef __LINUX__
+#include <linux/module.h>
 #include <linux/kernel.h>
+#include <linux/config.h>
 #include <linux/string.h>
 #include <linux/malloc.h>
 #include <linux/mm.h>
@@ -38,7 +50,6 @@
 #include <linux/ioport.h>
 #include <asm/irq.h>
 #include <asm/io.h>
-#endif
 
 #ifndef PCMCIA_DEBUG
 #define PCMCIA_DEBUG 1
@@ -59,12 +70,12 @@ static int pc_debug = PCMCIA_DEBUG;
 
 #define FIND_FIRST_BIT(n)	((n) - ((n) & ((n)-1)))
 
-#define pci_readb		pcibios_read_config_byte
-#define pci_writeb		pcibios_write_config_byte
-#define pci_readw		pcibios_read_config_word
-#define pci_writew		pcibios_write_config_word
-#define pci_readl		pcibios_read_config_dword
-#define pci_writel		pcibios_write_config_dword
+#define pci_readb		pci_read_config_byte
+#define pci_writeb		pci_write_config_byte
+#define pci_readw		pci_read_config_word
+#define pci_writew		pci_write_config_word
+#define pci_readl		pci_read_config_dword
+#define pci_writel		pci_write_config_dword
 
 #define CB_BAR(n)		(PCI_BASE_ADDRESS_0+(4*(n)))
 #define CB_ROM_BASE		0x0030
@@ -83,28 +94,46 @@ static int pc_debug = PCMCIA_DEBUG;
 #define PCDATA_CODE_TYPE	0x0014
 #define PCDATA_INDICATOR	0x0015
 
-#ifdef __LINUX__
 #if (LINUX_VERSION_CODE >= VERSION(2,1,103))
 #define NEW_LINUX_PCI
 #endif
+#if (LINUX_VERSION_CODE >= VERSION(2,3,40))
+#define NEWER_LINUX_PCI
 #endif
 
 typedef struct cb_config_t {
     u_int		size[7];
-#ifdef NEW_LINUX_PCI
     struct pci_dev	dev;
-#else
+#ifndef NEW_LINUX_PCI
     struct {
-	unsigned int	irq;
-	unsigned long	base_address[6];
-	unsigned long	rom_address;
-    } dev;
+	u_long		base_address[6];
+	u_long		rom_address;
+    } extra;
 #endif
 } cb_config_t;
 
+#if (LINUX_VERSION_CODE >= VERSION(2,3,15))
+#define BASE(cb,n)	((cb).dev.resource[n].start)
+#define ROM(cb)		((cb).dev.resource[6].start)
+#else
+#ifdef NEW_LINUX_PCI
+#define BASE(cb,n)	((cb).dev.base_address[n])
+#define ROM(cb)		((cb).dev.rom_address)
+#else
+#define BASE(cb,n)	((cb).extra.base_address[n])
+#define ROM(cb)		((cb).extra.rom_address)
+#endif
+#endif
+
 /* There are three classes of bridge maps: IO ports,
    non-prefetchable memory, and prefetchable memory */
-typedef enum { B_IO, B_M1, B_M2 } bridge_type;
+typedef enum { B_IO, B_M1, B_M2 } map_type;
+
+static const u_int map_flags[] = {
+    PCI_BASE_ADDRESS_SPACE_IO,
+    PCI_BASE_ADDRESS_SPACE_MEMORY,
+    PCI_BASE_ADDRESS_SPACE_MEMORY | PCI_BASE_ADDRESS_MEM_PREFETCH
+};
 
 /*=====================================================================
 
@@ -120,18 +149,18 @@ static int check_rom(u_char *b, u_long len)
 {
     u_int img = 0, ofs = 0, sz;
     u_short data;
-    DEBUG(0, ("ROM image dump:\n"));
+    DEBUG(0, "ROM image dump:\n");
     while ((readb(b) == 0x55) && (readb(b+1) == 0xaa)) {
 	data = readb(b+ROM_DATA_PTR) +
 	    (readb(b+ROM_DATA_PTR+1) << 8);
 	sz = 512 * (readb(b+data+PCDATA_IMAGE_SZ) +
 		    (readb(b+data+PCDATA_IMAGE_SZ+1) << 8));
-	DEBUG(0, ("  image %d: 0x%06x-0x%06x, signature %c%c%c%c\n",
-		  img, ofs, ofs+sz-1,
-		  readb(b+data+PCDATA_SIGNATURE),
-		  readb(b+data+PCDATA_SIGNATURE+1),
-		  readb(b+data+PCDATA_SIGNATURE+2),
-		  readb(b+data+PCDATA_SIGNATURE+3)));
+	DEBUG(0, "  image %d: 0x%06x-0x%06x, signature %c%c%c%c\n",
+	      img, ofs, ofs+sz-1,
+	      readb(b+data+PCDATA_SIGNATURE),
+	      readb(b+data+PCDATA_SIGNATURE+1),
+	      readb(b+data+PCDATA_SIGNATURE+2),
+	      readb(b+data+PCDATA_SIGNATURE+3));
 	ofs += sz; img++;
 	if ((readb(b+data+PCDATA_INDICATOR) & 0x80) ||
 	    (sz == 0) || (ofs >= len)) break;
@@ -168,6 +197,7 @@ static u_int xlate_rom_addr(u_char *b, u_int addr)
 int cb_setup_cis_mem(socket_info_t *s, int space)
 {
     cb_bridge_map *m = &s->cb_cis_map;
+    cb_config_t *c = s->cb_config;
     u_long base = 0;
     u_int sz, br;
 
@@ -175,11 +205,11 @@ int cb_setup_cis_mem(socket_info_t *s, int space)
 	return CS_SUCCESS;
     else if (s->cb_cis_space != 0)
 	cb_release_cis_mem(s);
-    DEBUG(1, ("cs: cb_setup_cis_mem(space %d)\n", space));
+    DEBUG(1, "cs: cb_setup_cis_mem(space %d)\n", space);
     /* If socket is configured, then use existing memory mapping */
     if (s->lock_count) {
 	s->cb_cis_virt =
-	    ioremap(s->cb_config[0].dev.base_address[space-1],
+	    ioremap(BASE(s->cb_config[0], space-1),
 		    s->cb_config[0].size[space-1] & ~3);
 	s->cb_cis_space = space;
 	return CS_SUCCESS;
@@ -187,22 +217,22 @@ int cb_setup_cis_mem(socket_info_t *s, int space)
     
     /* Not configured?  Then set up temporary map */
     br = (space == 7) ? CB_ROM_BASE : CB_BAR(space-1);
-    pci_writel(s->cap.cardbus, 0, br, 0xffffffff);
-    pci_readl(s->cap.cardbus, 0, br, &sz);
+    pci_writel(&c[0].dev, br, 0xffffffff);
+    pci_readl(&c[0].dev, br, &sz);
     sz &= PCI_BASE_ADDRESS_MEM_MASK;
     sz = FIND_FIRST_BIT(sz);
     if (sz < PAGE_SIZE) sz = PAGE_SIZE;
-    if (find_mem_region(&base, sz, "cb_enabler", sz, 0) != 0) {
+    if (find_mem_region(&base, sz, sz, 0, "cb_enabler") != 0) {
 	printk(KERN_NOTICE "cs: could not allocate %dK memory for"
 	       " CardBus socket %d\n", sz/1024, s->sock);
 	return CS_OUT_OF_RESOURCE;
     }
     s->cb_cis_space = space;
     s->cb_cis_virt = ioremap(base, sz);
-    DEBUG(1, ("  phys 0x%08lx-0x%08lx, virt 0x%08lx\n",
-	      base, base+sz-1, (u_long)s->cb_cis_virt));
-    pci_writel(s->cap.cardbus, 0, br, base | 1);
-    pci_writeb(s->cap.cardbus, 0, PCI_COMMAND, PCI_COMMAND_MEMORY);
+    DEBUG(1, "  phys 0x%08lx-0x%08lx, virt 0x%08lx\n",
+	  base, base+sz-1, (u_long)s->cb_cis_virt);
+    pci_writel(&c[0].dev, br, base | 1);
+    pci_writeb(&c[0].dev, PCI_COMMAND, PCI_COMMAND_MEMORY);
     m->map = 0; m->flags = MAP_ACTIVE;
     m->start = base; m->stop = base+sz-1;
     s->ss_entry(s->sock, SS_SetBridge, m);
@@ -218,21 +248,22 @@ void cb_release_cis_mem(socket_info_t *s)
     cb_bridge_map *m = &s->cb_cis_map;
     u_int br;
     if (s->cb_cis_virt) {
-	DEBUG(1, ("cs: cb_release_cis_mem()\n"));
+	DEBUG(1, "cs: cb_release_cis_mem()\n");
 	iounmap(s->cb_cis_virt);
 	s->cb_cis_virt = NULL;
 	s->cb_cis_space = 0;
     }
     if (m->start) {
+	cb_config_t *c = s->cb_config;
 	/* This is overkill: we probably only need to release the
 	   memory region, but the rest should be safe */
 	br = (s->cb_cis_space == 7) ?
 	    CB_ROM_BASE : CB_BAR(s->cb_cis_space-1);
 	m->map = 0; m->flags = 0;
 	s->ss_entry(s->sock, SS_SetBridge, m);
-	pci_writeb(s->cap.cardbus, 0, PCI_COMMAND, 0);
-	pci_writel(s->cap.cardbus, 0, br, 0);
-	vacate_mem_region(m->start, m->stop - m->start + 1);
+	pci_writeb(&c[0].dev, PCI_COMMAND, 0);
+	pci_writel(&c[0].dev, br, 0);
+	release_mem_region(m->start, m->stop - m->start + 1);
 	m->start = 0;
     }
 }
@@ -247,25 +278,136 @@ void cb_release_cis_mem(socket_info_t *s)
 void read_cb_mem(socket_info_t *s, u_char fn, int space,
 		 u_int addr, u_int len, void *ptr)
 {
-    DEBUG(3, ("cs: read_cb_mem(%d, %#x, %u)\n", space, addr, len));
+    cb_config_t *c = s->cb_config;
+    DEBUG(3, "cs: read_cb_mem(%d, %#x, %u)\n", space, addr, len);
     if (space == 0) {
+	if (addr+len > 0x100) goto fail;
 	for (; len; addr++, ptr++, len--)
-	    pci_readb(s->cap.cardbus, fn, addr, (u_char *)ptr);
+	    pci_readb(&c[fn].dev, addr, (u_char *)ptr);
     } else {
-	if (cb_setup_cis_mem(s, space) != 0) {
-	    memset(ptr, 0xff, len);
-	    return;
-	}
+	if (cb_setup_cis_mem(s, space) != 0) goto fail;
 	if (space == 7) {
 	    addr = xlate_rom_addr(s->cb_cis_virt, addr);
-	    if (addr == 0) {
-		memset(ptr, 0xff, len);
-		return;
-	    }
+	    if (addr == 0) goto fail;
 	}
-	if (s->cb_cis_virt != NULL)
-	    for (; len; addr++, ptr++, len--)
-		*(u_char *)ptr = readb(s->cb_cis_virt+addr);
+	if (addr+len > s->cb_cis_map.stop - s->cb_cis_map.start)
+	    goto fail;
+	for (; len; addr++, ptr++, len--)
+	    *(u_char *)ptr = readb(s->cb_cis_virt+addr);
+    }
+    return;
+ fail:
+    memset(ptr, 0xff, len);
+    return;
+}
+
+/*=====================================================================
+
+    cb_alloc() and cb_free() allocate and free the kernel data
+    structures for a Cardbus device, and handle the lowest level PCI
+    device setup issues.
+    
+=====================================================================*/
+
+int cb_alloc(socket_info_t *s)
+{
+    u_short vend, v, dev;
+    u_char hdr, fn, bus = s->cap.cardbus;
+    cb_config_t *c;
+
+    struct pci_dev tmp;
+    int i;
+    tmp.bus = s->cap.cb_bus; tmp.devfn = 0;
+#ifdef NEWER_LINUX_PCI
+    list_add(&tmp.global_list, &pci_devices);
+#else
+    tmp.next = pci_devices; pci_devices = &tmp;
+#endif
+
+    /* The APA1480 did not like reading the vendor ID first */
+    pci_readw(&tmp, PCI_DEVICE_ID, &dev);
+    pci_readw(&tmp, PCI_VENDOR_ID, &vend);
+    printk(KERN_INFO "cs: cb_alloc(bus %d): vendor 0x%04x, "
+	   "device 0x%04x\n", bus, vend, dev);
+
+    pci_readb(&tmp, PCI_HEADER_TYPE, &hdr);
+    if (hdr & 0x80) {
+	/* Count functions */
+	for (fn = 0; fn < 8; fn++) {
+	    tmp.devfn = fn;
+	    pci_readw(&tmp, PCI_VENDOR_ID, &v);
+	    if (v != vend) break;
+	}
+    } else fn = 1;
+    s->functions = fn;
+    
+    c = kmalloc(fn * sizeof(struct cb_config_t), GFP_ATOMIC);
+    if (!c) return CS_OUT_OF_RESOURCE;
+    memset(c, 0, fn * sizeof(struct cb_config_t));
+    s->cb_config = c;
+
+    for (i = 0; i < fn; i++) {
+	c[i].dev.bus = s->cap.cb_bus;
+	c[i].dev.devfn = i;
+#ifdef NEWER_LINUX_PCI
+	list_add_tail(&c[i].dev.bus_list, &s->cap.cb_bus->devices); 
+	list_add_tail(&c[i].dev.global_list, &pci_devices);
+#else
+	if (i < fn-1) {
+	    c[i].dev.sibling = c[i].dev.next = &c[i+1].dev;
+	}
+#endif
+    }
+#ifdef NEWER_LINUX_PCI
+    list_del(&tmp.global_list);
+#else
+    s->cap.cb_bus->devices = &c[0].dev;
+    /* Link into PCI device chain */
+    c[fn-1].dev.next = tmp.next;
+    pci_devices = &c[0].dev;
+#endif
+    for (i = 0; i < fn; i++) {
+	c[i].dev.vendor = vend;
+	pci_readw(&c[i].dev, PCI_DEVICE_ID, &c[i].dev.device);
+	pci_readl(&c[i].dev, PCI_CLASS_REVISION, &c[i].dev.class);
+	c[i].dev.class >>= 8;
+#ifdef NEW_LINUX_PCI
+	c[i].dev.hdr_type = hdr;
+#endif
+#if defined(HAS_PROC_BUS) && !defined(NEWER_LINUX_PCI)
+	pci_proc_attach_device(&c[i].dev);
+#endif
+    }
+    return CS_SUCCESS;
+}
+
+void cb_free(socket_info_t *s)
+{
+    cb_config_t *c = s->cb_config;
+
+    if (c) {
+#ifdef NEWER_LINUX_PCI
+	int i;
+	for (i = 0; i < s->functions; i++)
+	    list_del(&c[i].dev.global_list);
+	INIT_LIST_HEAD(&s->cap.cb_bus->devices);
+#else
+	struct pci_dev **p, *q;
+	/* Unlink from PCI device chain */
+	for (p = &pci_devices; *p; p = &((*p)->next))
+	    if (*p == &c[0].dev) break;
+	for (q = *p; q; q = q->next) {
+	    if (q->bus != (*p)->bus) break;
+#ifdef HAS_PROC_BUS
+	    pci_proc_detach_device(q);
+#endif
+	}
+	if (*p) *p = q;
+	s->cap.cb_bus->devices = NULL;
+#endif
+	kfree(s->cb_config);
+	s->cb_config = NULL;
+	printk(KERN_INFO "cs: cb_free(bus %d)\n", s->cap.cardbus);
     }
 }
 
@@ -283,54 +425,21 @@ void read_cb_mem(socket_info_t *s, u_char fn, int space,
 
 int cb_config(socket_info_t *s)
 {
-    u_short vend, v, dev;
-    u_char i, j, fn, bus = s->cap.cardbus, *name;
+    cb_config_t *c = s->cb_config;
+    u_char fn = s->functions;
+    u_char i, j, *name;
     u_int sz, align, m, mask[3], num[3], base[3];
-    cb_config_t *c;
-    int irq, try, ret;
-    
-    pci_readw(bus, 0, PCI_VENDOR_ID, &vend);
-    pci_readw(bus, 0, PCI_DEVICE_ID, &dev);
-    printk(KERN_INFO "cs: cb_config(bus %d): vendor 0x%04x, "
-	   "device 0x%04x\n", bus, vend, dev);
+    int irq;
 
-    pci_readb(bus, 0, PCI_HEADER_TYPE, &fn);
-    if (fn & 0x80) {
-	/* Count functions */
-	for (fn = 0; fn < 8; fn++) {
-	    pci_readw(bus, fn, PCI_VENDOR_ID, &v);
-	    if (v != vend) break;
-	}
-    } else fn = 1;
-    s->functions = fn;
-    
-    c = kmalloc(fn * sizeof(struct cb_config_t), GFP_KERNEL);
-    memset(c, 0, fn * sizeof(struct cb_config_t));
-    s->cb_config = c;
+    printk(KERN_INFO "cs: cb_config(bus %d)\n", s->cap.cardbus);
 
-#ifdef NEW_LINUX_PCI
-    for (i = 0; i < fn; i++) {
-	c[i].dev.bus = s->cap.cb_bus;
-	if (i < fn-1) {
-	    c[i].dev.sibling = c[i].dev.next = &c[i+1].dev;
-	}
-	c[i].dev.devfn = i;
-	c[i].dev.vendor = vend; c[i].dev.device = dev;
-	pci_readl(bus, i, PCI_CLASS_REVISION, &c[i].dev.class);
-	c[i].dev.class >>= 8;
-	pci_readb(bus, i, PCI_HEADER_TYPE, &j);
-	c[i].dev.hdr_type = j;
-    }
-    s->cap.cb_bus->devices = &c[0].dev;
-#endif
-    
     /* Determine IO and memory space needs */
     num[B_IO] = num[B_M1] = num[B_M2] = 0;
     mask[B_IO] = mask[B_M1] = mask[B_M2] = 0;
     for (i = 0; i < fn; i++) {
 	for (j = 0; j < 6; j++) {
-	    pci_writel(bus, i, CB_BAR(j), 0xffffffff);
-	    pci_readl(bus, i, CB_BAR(j), &sz);
+	    pci_writel(&c[i].dev, CB_BAR(j), 0xffffffff);
+	    pci_readl(&c[i].dev, CB_BAR(j), &sz);
 	    if (sz == 0) continue;
 	    if (sz & PCI_BASE_ADDRESS_SPACE) {
 		m = B_IO;
@@ -344,8 +453,8 @@ int cb_config(socket_info_t *s)
 	    if (m && (sz < PAGE_SIZE)) sz = PAGE_SIZE;
 	    num[m] += sz; mask[m] |= sz;
 	}
-	pci_writel(bus, i, CB_ROM_BASE, 0xffffffff);
-	pci_readl(bus, i, CB_ROM_BASE, &sz);
+	pci_writel(&c[i].dev, CB_ROM_BASE, 0xffffffff);
+	pci_readl(&c[i].dev, CB_ROM_BASE, &sz);
 	if (sz != 0) {
 	    sz = FIND_FIRST_BIT(sz & ~0x00000001);
 	    c[i].size[6] = sz | B_M1;
@@ -359,7 +468,9 @@ int cb_config(socket_info_t *s)
     s->io[0].NumPorts = num[B_IO];
     s->io[0].BasePort = 0;
     if (num[B_IO]) {
-	if (find_io_region(&s->io[0].BasePort, num[B_IO], name) != 0) {
+	for (align = 1; align < mask[B_IO]; align <<= 1) ;
+	if (find_io_region(&s->io[0].BasePort, num[B_IO],
+			   align, name) != 0) {
 	    printk(KERN_NOTICE "cs: could not allocate %d IO ports for"
 		   " CardBus socket %d\n", num[B_IO], s->sock);
 	    goto failed;
@@ -369,8 +480,9 @@ int cb_config(socket_info_t *s)
     s->win[0].size = num[B_M1];
     s->win[0].base = 0;
     if (num[B_M1]) {
-	if (find_mem_region(&s->win[0].base, num[B_M1],
-			    name, num[B_M1], 0) != 0) {
+	for (align = 1; align < mask[B_M1]; align <<= 1) ;
+	if (find_mem_region(&s->win[0].base, num[B_M1], align,
+			    0, name) != 0) {
 	    printk(KERN_NOTICE "cs: could not allocate %dK memory for"
 		   " CardBus socket %d\n", num[B_M1]/1024, s->sock);
 	    goto failed;
@@ -380,8 +492,9 @@ int cb_config(socket_info_t *s)
     s->win[1].size = num[B_M2];
     s->win[1].base = 0;
     if (num[B_M2]) {
-	if (find_mem_region(&s->win[1].base, num[B_M2],
-			    name, num[B_M2], 0) != 0) {
+	for (align = 1; align < mask[B_M2]; align <<= 1) ;
+	if (find_mem_region(&s->win[1].base, num[B_M2], align,
+			    0, name) != 0) {
 	    printk(KERN_NOTICE "cs: could not allocate %dK memory for"
 		   " CardBus socket %d\n", num[B_M2]/1024, s->sock);
 	    goto failed;
@@ -407,42 +520,43 @@ int cb_config(socket_info_t *s)
 			printk(KERN_INFO "  fn %d rom: ", i);
 		    printk("%s 0x%x-0x%x\n", (m) ? "mem" : "io",
 			   base[m], base[m]+sz-1);
-		    c[i].dev.base_address[j] = base[m];
+		    BASE(c[i], j) = base[m] | map_flags[m];
 		}
 	    }
 	}
     }
     
     /* Allocate interrupt if needed */
-    s->irq.AssignedIRQ = irq = 0; ret = -1;
+    s->irq.AssignedIRQ = irq = 0;
     for (i = 0; i < fn; i++) {
-	pci_readb(bus, i, PCI_INTERRUPT_PIN, &j);
+	pci_readb(&c[i].dev, PCI_INTERRUPT_PIN, &j);
 	if (j == 0) continue;
-	if (irq == 0) {
-	    if (s->cap.irq_mask & (1 << s->cap.pci_irq)) {
-		irq = s->cap.pci_irq;
-		ret = 0;
-	    }
+	if ((irq == 0) && (s->cap.irq_mask & (1 << s->cap.pci_irq)))
+	    irq = s->cap.pci_irq;
 #ifdef CONFIG_ISA
-	    else
-		for (try = 0; try < 2; try++) {
-		    for (irq = 0; irq < 32; irq++)
-			if ((s->cap.irq_mask >> irq) & 1) {
-			    ret = try_irq(IRQ_TYPE_EXCLUSIVE, irq, try);
-			    if (ret == 0) break;
-			}
-		    if (ret == 0) break;
-		}
-	    if (ret != 0) {
-		printk(KERN_NOTICE "cs: could not allocate interrupt"
-		       " for CardBus socket %d\n", s->sock);
-		goto failed;
+	if (irq == 0) {
+	    int try;
+	    for (try = 0; try < 2; try++) {
+		for (irq = 0; irq < 32; irq++)
+		    if (((s->cap.irq_mask >> irq) & 1) &&
+			(try_irq(IRQ_TYPE_EXCLUSIVE, irq, try) == 0))
+			break;
+		if (irq < 32) break;
 	    }
-#endif
-	    s->irq.AssignedIRQ = irq;
+	    if (irq == 32) irq = 0;
 	}
+	if (irq == 0) {
+	    printk(KERN_NOTICE "cs: could not allocate interrupt"
+		   " for CardBus socket %d\n", s->sock);
+	    goto failed;
+	}
+#endif
     }
-    c[0].dev.irq = irq;
+    if (irq)
+	printk(KERN_INFO "  irq %d\n", irq);
+    s->irq.AssignedIRQ = irq;
+    for (i = 0; i < fn; i++)
+	c[i].dev.irq = irq;
     
     return CS_SUCCESS;
 
@@ -465,21 +579,19 @@ void cb_release(socket_info_t *s)
 {
     cb_config_t *c = s->cb_config;
     
-    DEBUG(0, ("cs: cb_release(bus %d)\n", s->cap.cardbus));
+    DEBUG(0, "cs: cb_release(bus %d)\n", s->cap.cardbus);
     
     if (s->win[0].size > 0)
-	vacate_mem_region(s->win[0].base, s->win[0].size);
+	release_mem_region(s->win[0].base, s->win[0].size);
     if (s->win[1].size > 0)
-	vacate_mem_region(s->win[1].base, s->win[1].size);
+	release_mem_region(s->win[1].base, s->win[1].size);
     if (s->io[0].NumPorts > 0)
-	vacate_region(s->io[0].BasePort, s->io[0].NumPorts);
+	release_region(s->io[0].BasePort, s->io[0].NumPorts);
     s->io[0].NumPorts = 0;
 #ifdef CONFIG_ISA
     if ((c[0].dev.irq != 0) && (c[0].dev.irq != s->cap.pci_irq))
 	undo_irq(IRQ_TYPE_EXCLUSIVE, c[0].dev.irq);
 #endif
-    kfree(s->cb_config);
-    s->cb_config = NULL;
 }
 
 /*=====================================================================
@@ -502,7 +614,7 @@ void cb_enable(socket_info_t *s)
     u_char i, j, bus = s->cap.cardbus;
     cb_config_t *c = s->cb_config;
     
-    DEBUG(0, ("cs: cb_enable(bus %d)\n", bus));
+    DEBUG(0, "cs: cb_enable(bus %d)\n", bus);
     
     /* Configure bridge */
     if (s->cb_cis_map.start)
@@ -527,43 +639,38 @@ void cb_enable(socket_info_t *s)
 	    break;
 	}
 	if (m.start == 0) continue;
-	DEBUG(0, ("  bridge %s map %d (flags 0x%x): 0x%x-0x%x\n",
-		  (m.flags & MAP_IOSPACE) ? "io" : "mem",
-		  m.map, m.flags, m.start, m.stop));
+	DEBUG(0, "  bridge %s map %d (flags 0x%x): 0x%x-0x%x\n",
+	      (m.flags & MAP_IOSPACE) ? "io" : "mem",
+	      m.map, m.flags, m.start, m.stop);
 	s->ss_entry(s->sock, SS_SetBridge, &m);
     }
 
     /* Set up base address registers */
     for (i = 0; i < s->functions; i++) {
 	for (j = 0; j < 6; j++) {
-	    if (c[i].dev.base_address[j] != 0)
-		pci_writel(bus, i, CB_BAR(j), c[i].dev.base_address[j]);
+	    if (BASE(c[i], j) != 0)
+		pci_writel(&c[i].dev, CB_BAR(j), BASE(c[i], j));
 	}
-	if (c[i].dev.rom_address != 0)
-	    pci_writel(bus, i, CB_ROM_BASE, c[i].dev.rom_address | 1);
+	if (ROM(c[i]) != 0)
+	    pci_writel(&c[i].dev, CB_ROM_BASE, ROM(c[i]) | 1);
     }
 
-    /* Set up PCI interrupt and command registers */
+    /* Set up a few basic PCI registers */
     for (i = 0; i < s->functions; i++) {
-	pci_writeb(bus, i, PCI_COMMAND, PCI_COMMAND_MASTER |
+	pci_writeb(&c[i].dev, PCI_COMMAND, PCI_COMMAND_MASTER |
 		   PCI_COMMAND_IO | PCI_COMMAND_MEMORY);
-	pci_writeb(bus, i, PCI_CACHE_LINE_SIZE, 8);
+	/* These are lame but I don't know how to do any better */
+	pci_writeb(&c[i].dev, PCI_CACHE_LINE_SIZE, 8);
+	pci_writeb(&c[i].dev, PCI_LATENCY_TIMER, 64);
     }
     
     if (s->irq.AssignedIRQ) {
 	for (i = 0; i < s->functions; i++)
-	    pci_writeb(bus, i, PCI_INTERRUPT_LINE,
+	    pci_writeb(&c[i].dev, PCI_INTERRUPT_LINE,
 		       s->irq.AssignedIRQ);
 	s->socket.io_irq = s->irq.AssignedIRQ;
 	s->ss_entry(s->sock, SS_SetSocket, &s->socket);
     }
-
-#ifdef NEW_LINUX_PCI
-    /* Link into PCI device chain */
-    c[s->functions-1].dev.next = pci_devices;
-    pci_devices = &c[0].dev;
-    pci_proc_attach_device(&c[0].dev);
-#endif
 }
 
 /*======================================================================
@@ -579,21 +686,8 @@ void cb_disable(socket_info_t *s)
 {
     u_char i;
     cb_bridge_map m = { 0, 0, 0, 0xffff };
-#ifdef NEW_LINUX_PCI
-    struct pci_dev **p, *q;
-    cb_config_t *c = s->cb_config;
     
-    /* Unlink from PCI device chain */
-    pci_proc_detach_device(&c[0].dev);
-    for (p = &pci_devices; *p; p = &((*p)->next))
-	if (*p == &c[0].dev) break;
-    for (q = *p; q; q = q->next)
-	if (q->bus != (*p)->bus) break;
-    if (*p) *p = q;
-    s->cap.cb_bus->devices = NULL;
-#endif
-    
-    DEBUG(0, ("cs: cb_disable(bus %d)\n", s->cap.cardbus));
+    DEBUG(0, "cs: cb_disable(bus %d)\n", s->cap.cardbus);
     
     /* Turn off bridge windows */
     if (s->cb_cis_map.start)
