@@ -1,8 +1,8 @@
 /*======================================================================
 
-    Cardbus device enabler
+    CardBus device enabler
 
-    cb_enabler.c 1.19 1999/07/24 15:48:06
+    cb_enabler.c 1.30 2000/05/23 22:57:03
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,8 +15,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
     The general idea:
 
@@ -32,6 +43,8 @@
 #include <pcmcia/k_compat.h>
 
 #ifdef __LINUX__
+#include <linux/module.h>
+#include <linux/init.h>
 #include <linux/kernel.h>
 #include <linux/sched.h>
 #include <linux/malloc.h>
@@ -50,11 +63,14 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"cb_enabler.c 1.19 1999/07/24 15:48:06 (David Hinds)";
+"cb_enabler.c 1.30 2000/05/23 22:57:03 (David Hinds)";
 #else
 #define DEBUG(n, args...) do { } while (0)
 #endif
 
+MODULE_AUTHOR("David Hinds <dhinds@pcmcia.sourceforge.org>");
+MODULE_DESCRIPTION("CardBus stub enabler module");
+
 /*====================================================================*/
 
 /* Parameters that can be set with 'insmod' */
@@ -120,15 +136,17 @@ struct dev_link_t *cb_attach(int n)
     client_reg_t client_reg;
     dev_link_t *link;
     int ret;
-    
-    DEBUG(0, "cb_attach(%d)\n", n);
 
     MOD_INC_USE_COUNT;
-    link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
-    memset(link, 0, sizeof(struct dev_link_t));
-    link->release.function = &cb_release;
-    link->release.data = (u_long)link;
+    DEBUG(0, "cb_attach(%d)\n", n);
 
+    link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
+    if (!link) {
+	MOD_DEC_USE_COUNT;
+	return NULL;
+    }
+
+    memset(link, 0, sizeof(struct dev_link_t));
     link->conf.IntType = INT_CARDBUS;
     link->conf.Vcc = 33;
 
@@ -141,7 +159,7 @@ struct dev_link_t *cb_attach(int n)
     client_reg.dev_info = &driver[n].dev_info;
     client_reg.Attributes = INFO_IO_CLIENT | INFO_CARD_SHARE;
     client_reg.event_handler = &cb_event;
-    client_reg.EventMask =
+    client_reg.EventMask = CS_EVENT_RESET_PHYSICAL |
 	CS_EVENT_RESET_REQUEST | CS_EVENT_CARD_RESET |
 	CS_EVENT_CARD_INSERTION | CS_EVENT_CARD_REMOVAL |
 	CS_EVENT_PM_SUSPEND | CS_EVENT_PM_RESUME;
@@ -163,38 +181,29 @@ static void cb_detach(dev_link_t *link)
     driver_info_t *dev = link->priv;
     dev_link_t **linkp;
     bus_info_t *b = (void *)link->win;
-    u_long flags;
-    
+
     DEBUG(0, "cb_detach(0x%p)\n", link);
-    
+
     /* Locate device structure */
     for (linkp = &dev->dev_list; *linkp; linkp = &(*linkp)->next)
 	if (*linkp == link) break;
     if (*linkp == NULL)
 	return;
 
-    save_flags(flags);
-    cli();
-    if (link->state & DEV_RELEASE_PENDING) {
-	del_timer(&link->release);
-	link->state &= ~DEV_RELEASE_PENDING;
-    }
-    restore_flags(flags);
-    
     if (link->state & DEV_CONFIG)
 	cb_release((u_long)link);
-    
+
     /* Don't drop Card Services connection if we are the bus owner */
-    if ((b->flags != 0) && (link == b->owner)) {
+    if (b && (b->flags != 0) && (link == b->owner)) {
 	link->state |= DEV_STALE_LINK;
 	return;
     }
-    
+
     if (link->handle)
 	CardServices(DeregisterClient, link->handle);
-    
+
     *linkp = link->next;
-    kfree_s(link, sizeof(struct dev_link_t));
+    kfree(link);
     MOD_DEC_USE_COUNT;
 }
 
@@ -306,11 +315,8 @@ static int cb_event(event_t event, int priority,
     switch (event) {
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
-	if (link->state & DEV_CONFIG) {
-	    link->release.expires = RUN_AT(HZ/20);
-	    link->state |= DEV_RELEASE_PENDING;
-	    add_timer(&link->release);
-	}
+	if (link->state & DEV_CONFIG)
+	    cb_release((u_long)link);
 	break;
     case CS_EVENT_CARD_INSERTION:
 	link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
@@ -362,7 +368,7 @@ int register_driver(struct driver_operations *ops)
     MOD_INC_USE_COUNT;
     driver[i].ops = ops;
     strcpy(driver[i].dev_info, ops->name);
-    register_pcmcia_driver(&driver[i].dev_info, driver[i].attach,
+    register_pccard_driver(&driver[i].dev_info, driver[i].attach,
 			   &cb_detach);
     return 0;
 }
@@ -375,7 +381,7 @@ void unregister_driver(struct driver_operations *ops)
     for (i = 0; i < MAX_DRIVER; i++)
 	if (driver[i].ops == ops) break;
     if (i < MAX_DRIVER) {
-	unregister_pcmcia_driver(&driver[i].dev_info);
+	unregister_pccard_driver(&driver[i].dev_info);
 	driver[i].ops = NULL;
 	MOD_DEC_USE_COUNT;
     }
@@ -385,7 +391,27 @@ void unregister_driver(struct driver_operations *ops)
 
 #ifdef __LINUX__
 
-int init_module(void) {
+#if (LINUX_VERSION_CODE <= VERSION(2,1,17))
+
+#undef CONFIG_MODVERSIONS
+static struct symbol_table cb_enabler_symtab = {
+#include <linux/symtab_begin.h>
+#undef X
+#define X(sym) { (void *)&sym, SYMBOL_NAME_STR(sym) }
+    X(register_driver),
+    X(unregister_driver),
+#include <linux/symtab_end.h>
+};
+
+#else
+
+EXPORT_SYMBOL(register_driver);
+EXPORT_SYMBOL(unregister_driver);
+
+#endif
+
+static int __init init_cb_enabler(void)
+{
     servinfo_t serv;
     DEBUG(0, "%s\n", version);
     CardServices(GetCardServicesInfo, &serv);
@@ -394,13 +420,18 @@ int init_module(void) {
 	       "does not match!\n");
 	return -1;
     }
+    register_symtab(&cb_enabler_symtab);
     return 0;
 }
 
-void cleanup_module(void) {
+static void __exit exit_cb_enabler(void)
+{
     DEBUG(0, "cb_enabler: unloading\n");
 }
 
+module_init(init_cb_enabler);
+module_exit(exit_cb_enabler);
+
 #endif /* __LINUX__ */
 
 /*====================================================================*/
