@@ -2,7 +2,7 @@
 
     PCMCIA Card Services -- core services
 
-    cs.c 1.222 1999/07/20 16:01:24
+    cs.c 1.259 2000/05/10 19:26:32
     
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,8 +15,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
@@ -24,7 +35,10 @@
 #include <pcmcia/k_compat.h>
 
 #ifdef __LINUX__
+#include <linux/module.h>
+#include <linux/init.h>
 #include <linux/kernel.h>
+#include <linux/config.h>
 #include <linux/string.h>
 #include <linux/major.h>
 #include <linux/errno.h>
@@ -35,6 +49,7 @@
 #include <linux/ioport.h>
 #include <linux/delay.h>
 #include <linux/proc_fs.h>
+#include <linux/pm.h>
 #include <asm/system.h>
 #include <asm/irq.h>
 #endif
@@ -51,16 +66,38 @@
 #include "cs_internal.h"
 #include "rsrc_mgr.h"
 
-#ifdef CONFIG_APM
-#include <linux/apm_bios.h>
-static int handle_apm_event(apm_event_t event);
-#endif
-
 #ifdef PCMCIA_DEBUG
 int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 static const char *version =
-"cs.c 1.222 1999/07/20 16:01:24 (David Hinds)";
+"cs.c 1.259 2000/05/10 19:26:32 (David Hinds)";
+#endif
+
+#ifdef CONFIG_PCI
+#define PCI_OPT " [pci]"
+#else
+#define PCI_OPT ""
+#endif
+#ifdef CONFIG_CARDBUS
+#define CB_OPT " [cardbus]"
+#else
+#define CB_OPT ""
+#endif
+#ifdef CONFIG_PM
+#define PM_OPT " [apm]"
+#else
+#define PM_OPT ""
+#endif
+#ifdef CONFIG_PNP_BIOS
+#define PNP_OPT " [pnp]"
+#else
+#define PNP_OPT ""
+#endif
+#if !defined(CONFIG_CARDBUS) && !defined(CONFIG_PCI) && \
+    !defined(CONFIG_PM) && !defined(CONFIG_PNP_BIOS)
+#define OPTIONS " none"
+#else
+#define OPTIONS PCI_OPT CB_OPT PM_OPT PNP_OPT
 #endif
 
 #ifdef __BEOS__
@@ -68,53 +105,44 @@ static const char *release = "BeOS PCMCIA Card Services " CS_RELEASE;
 #endif
 #ifdef __LINUX__
 static const char *release = "Linux PCMCIA Card Services " CS_RELEASE;
+#ifdef UTS_RELEASE
 static const char *kernel = "kernel build: " UTS_RELEASE " " UTS_VERSION;
 #endif
-static const char *options = "options: "
-#ifdef CONFIG_PCI
-" [pci]"
 #endif
-#ifdef CONFIG_CARDBUS
-" [cardbus]"
-#endif
-#ifdef CONFIG_APM
-" [apm]"
-#else
-#if !defined(CONFIG_CARDBUS) && !defined(CONFIG_PCI)
-" none"
-#endif
-#endif
-;
+static const char *options = "options: " OPTIONS;
+
+MODULE_AUTHOR("David Hinds <dhinds@pcmcia.sourceforge.org>");
+MODULE_DESCRIPTION("Linux PCMCIA Card Services " CS_RELEASE
+		   "\n  options:" OPTIONS);
 
 /*====================================================================*/
 
 /* Parameters that can be set with 'insmod' */
 
-static int setup_delay		= HZ/20;	/* ticks */
-static int resume_delay		= HZ/5;		/* ticks */
-static int shutdown_delay	= HZ/40;	/* ticks */
-static int vcc_settle		= HZ*3/10;	/* ticks */
-static int reset_time		= 10;		/* usecs */
-static int unreset_delay	= HZ/10;	/* ticks */
-static int unreset_check	= HZ/10;	/* ticks */
-static int unreset_limit	= 30;		/* unreset_check's */
+#define INT_MODULE_PARM(n, v) static int n = v; MODULE_PARM(n, "i")
+
+INT_MODULE_PARM(setup_delay,	HZ/20);		/* ticks */
+INT_MODULE_PARM(resume_delay,	HZ/5);		/* ticks */
+INT_MODULE_PARM(shutdown_delay,	HZ/40);		/* ticks */
+INT_MODULE_PARM(vcc_settle,	HZ*4/10);	/* ticks */
+INT_MODULE_PARM(reset_time,	10);		/* usecs */
+INT_MODULE_PARM(unreset_delay,	HZ/10);		/* ticks */
+INT_MODULE_PARM(unreset_check,	HZ/10);		/* ticks */
+INT_MODULE_PARM(unreset_limit,	30);		/* unreset_check's */
 
 /* Access speed for attribute memory windows */
-static int cis_speed		= 300;	/* ns */
+INT_MODULE_PARM(cis_speed,	300);		/* ns */
 
 /* Access speed for IO windows */
-static int io_speed		= 0;	/* ns */
+INT_MODULE_PARM(io_speed,	0);		/* ns */
 
-MODULE_PARM(setup_delay, "i");
-MODULE_PARM(resume_delay, "i");
-MODULE_PARM(shutdown_delay, "i");
-MODULE_PARM(vcc_settle, "i");
-MODULE_PARM(reset_time, "i");
-MODULE_PARM(unreset_delay, "i");
-MODULE_PARM(unreset_check, "i");
-MODULE_PARM(unreset_limit, "i");
-MODULE_PARM(cis_speed, "i");
-MODULE_PARM(io_speed, "i");
+/* Optional features */
+#ifdef CONFIG_PM
+INT_MODULE_PARM(do_apm, 1);
+#endif
+#ifdef CONFIG_PNP_BIOS
+INT_MODULE_PARM(do_pnp, 1);
+#endif
 
 /*====================================================================*/
 
@@ -255,8 +283,8 @@ static void init_socket(socket_info_t *s)
 /*====================================================================*/
 
 #if defined(HAS_PROC_BUS) && defined(PCMCIA_DEBUG)
-int proc_read_clients(char *buf, char **start, off_t pos,
-		      int count, int *eof, void *data)
+static int proc_read_clients(char *buf, char **start, off_t pos,
+			     int count, int *eof, void *data)
 {
     socket_info_t *s = data;
     client_handle_t c;
@@ -287,7 +315,7 @@ int register_ss_entry(int nsock, ss_entry_t ss_entry)
     int i, ns;
     socket_info_t *s;
 
-    DEBUG(0, ("cs: register_ss_entry(%d, 0x%p)\n", nsock, ss_entry));
+    DEBUG(0, "cs: register_ss_entry(%d, 0x%p)\n", nsock, ss_entry);
 
     for (ns = 0; ns < nsock; ns++) {
 	s = kmalloc(sizeof(struct socket_info_t), GFP_KERNEL);
@@ -303,6 +331,7 @@ int register_ss_entry(int nsock, ss_entry_t ss_entry)
 	s->cis_mem.flags = 0;
 	s->cis_mem.speed = cis_speed;
 	s->erase_busy.next = s->erase_busy.prev = &s->erase_busy;
+	spin_lock_init(&s->lock);
 	
 	for (i = 0; i < sockets; i++)
 	    if (socket_table[i] == NULL) break;
@@ -314,17 +343,15 @@ int register_ss_entry(int nsock, ss_entry_t ss_entry)
 #ifdef HAS_PROC_BUS
 	if (proc_pccard) {
 	    char name[3];
-#ifdef PCMCIA_DEBUG
-	    struct proc_dir_entry *ent;
-#endif
 	    sprintf(name, "%02d", i);
-	    s->proc = create_proc_entry(name, S_IFDIR, proc_pccard);
+	    s->proc = proc_mkdir(name, proc_pccard);
+	    if (s->proc)
+		ss_entry(ns, SS_ProcSetup, s->proc);
 #ifdef PCMCIA_DEBUG
-	    ent = create_proc_entry("clients", 0, s->proc);
-	    ent->read_proc = proc_read_clients;
-	    ent->data = s;
+	    if (s->proc)
+		create_proc_read_entry("clients", 0, s->proc,
+				       proc_read_clients, s);
 #endif
-	    ss_entry(ns, SS_ProcSetup, s->proc);
 	}
 #endif
     }
@@ -340,41 +367,41 @@ void unregister_ss_entry(ss_entry_t ss_entry)
     socket_info_t *s = NULL;
     client_t *client;
 
+#ifdef HAS_PROC_BUS
+    for (i = 0; i < sockets; i++) {
+	s = socket_table[i];
+	if (s->ss_entry != ss_entry) continue;
+	if (proc_pccard) {
+	    char name[3];
+	    sprintf(name, "%02d", i);
+#ifdef PCMCIA_DEBUG
+	    remove_proc_entry("clients", s->proc);
+#endif
+	    remove_proc_entry(name, proc_pccard);
+	}
+    }
+#endif
+
     for (;;) {
 	for (i = 0; i < sockets; i++) {
 	    s = socket_table[i];
 	    if (s->ss_entry == ss_entry) break;
 	}
-	if (i == sockets) {
+	if (i == sockets)
 	    break;
-	} else {
-#ifdef HAS_PROC_BUS
-	    if (proc_pccard) {
-		char name[3];
-		sprintf(name, "%02d", i);
-#ifdef PCMCIA_DEBUG
-		remove_proc_entry("clients", s->proc);
-#endif
-		remove_proc_entry(name, proc_pccard);
-	    }
-#endif
-	    while (s->clients) {
-		client = s->clients;
-		s->clients = s->clients->next;
-		kfree_s(client, sizeof(*client));
-	    }
-	    init_socket(s);
-	    release_cis_mem(s);
-#ifdef CONFIG_CARDBUS
-	    cb_release_cis_mem(s);
-#endif
-	    s->ss_entry = NULL;
-	    kfree_s(s, sizeof(struct socket_info_t));
-	    socket_table[i] = NULL;
-	    for (j = i; j < sockets-1; j++)
-		socket_table[j] = socket_table[j+1];
-	    sockets--;
+	shutdown_socket(i);
+	release_cis_mem(s);
+	while (s->clients) {
+	    client = s->clients;
+	    s->clients = s->clients->next;
+	    kfree(client);
 	}
+	s->ss_entry = NULL;
+	kfree(s);
+	socket_table[i] = NULL;
+	for (j = i; j < sockets-1; j++)
+	    socket_table[j] = socket_table[j+1];
+	sockets--;
     }
     
 } /* unregister_ss_entry */
@@ -396,7 +423,7 @@ static void free_regions(memory_handle_t *list)
 	tmp = *list;
 	*list = tmp->info.next;
 	tmp->region_magic = 0;
-	kfree_s(tmp, sizeof(*tmp));
+	kfree(tmp);
     }
 }
 
@@ -407,13 +434,12 @@ static void shutdown_socket(u_long i)
     socket_info_t *s = socket_table[i];
     client_t **c;
     
-    DEBUG(1, ("cs: shutdown_socket(%ld)\n", i));
+    DEBUG(1, "cs: shutdown_socket(%ld)\n", i);
 
     /* Blank out the socket state */
     s->state &= SOCKET_PRESENT|SOCKET_SETUP_PENDING;
     init_socket(s);
     s->irq.AssignedIRQ = s->irq.Config = 0;
-    s->functions = 0;
     s->lock_count = 0;
     s->cis_used = 0;
     if (s->fake_cis) {
@@ -422,7 +448,9 @@ static void shutdown_socket(u_long i)
     }
 #ifdef CONFIG_CARDBUS
     cb_release_cis_mem(s);
+    cb_free(s);
 #endif
+    s->functions = 0;
     if (s->config) {
 	kfree(s->config);
 	s->config = NULL;
@@ -431,7 +459,7 @@ static void shutdown_socket(u_long i)
 	if ((*c)->state & CLIENT_UNBOUND) {
 	    client_t *d = *c;
 	    *c = (*c)->next;
-	    kfree_s(d, sizeof(*d));
+	    kfree(d);
 	} else {
 	    c = &((*c)->next);
 	}
@@ -446,8 +474,18 @@ static void setup_socket(u_long i)
     socket_info_t *s = socket_table[i];
 
     s->ss_entry(s->sock, SS_GetStatus, &val);
-    if (val & SS_DETECT) {
-	DEBUG(1, ("cs: setup_socket(%ld): applying power\n", i));
+    if (val & SS_PENDING) {
+	/* Does the socket need more time? */
+	DEBUG(2, "cs: setup_socket(%ld): status pending\n", i);
+	if (++s->setup_timeout > 100) {
+	    printk(KERN_NOTICE "cs: socket %ld voltage interrogation"
+		   " timed out\n", i);
+	} else {
+	    s->setup.expires = jiffies + HZ/10;
+	    add_timer(&s->setup);
+	}
+    } else if (val & SS_DETECT) {
+	DEBUG(1, "cs: setup_socket(%ld): applying power\n", i);
 	s->state |= SOCKET_PRESENT;
 	s->socket.flags = 0;
 	if (val & SS_3VCARD)
@@ -467,10 +505,10 @@ static void setup_socket(u_long i)
 	}
 	s->ss_entry(s->sock, SS_SetSocket, &s->socket);
 	s->setup.function = &reset_socket;
-	s->setup.expires = RUN_AT(vcc_settle);
+	s->setup.expires = jiffies + vcc_settle;
 	add_timer(&s->setup);
     } else
-	DEBUG(0, ("cs: setup_socket(%ld): no card!\n", i));
+	DEBUG(0, "cs: setup_socket(%ld): no card!\n", i);
 } /* setup_socket */
 
 /*======================================================================
@@ -486,14 +524,14 @@ static void reset_socket(u_long i)
 {
     socket_info_t *s = socket_table[i];
 
-    DEBUG(1, ("cs: resetting socket %ld\n", i));
+    DEBUG(1, "cs: resetting socket %ld\n", i);
     s->socket.flags |= SS_OUTPUT_ENA | SS_RESET;
     s->ss_entry(s->sock, SS_SetSocket, &s->socket);
     udelay((long)reset_time);
     s->socket.flags &= ~SS_RESET;
     s->ss_entry(s->sock, SS_SetSocket, &s->socket);
-    s->unreset_timeout = 0;
-    s->setup.expires = RUN_AT(unreset_delay);
+    s->setup_timeout = 0;
+    s->setup.expires = jiffies + unreset_delay;
     s->setup.function = &unreset_socket;
     add_timer(&s->setup);
 } /* reset_socket */
@@ -508,7 +546,7 @@ static void unreset_socket(u_long i)
 
     s->ss_entry(s->sock, SS_GetStatus, &val);
     if (val & SS_READY) {
-	DEBUG(1, ("cs: reset done on socket %ld\n", i));
+	DEBUG(1, "cs: reset done on socket %ld\n", i);
 	if (s->state & SOCKET_SUSPEND) {
 	    s->state &= ~EVENT_MASK;
 	    if (verify_cis_cache(s) != 0)
@@ -516,6 +554,10 @@ static void unreset_socket(u_long i)
 	    else
 		send_event(s, CS_EVENT_PM_RESUME, CS_EVENT_PRI_LOW);
 	} else if (s->state & SOCKET_SETUP_PENDING) {
+#ifdef CONFIG_CARDBUS
+	    if (s->state & SOCKET_CARDBUS)
+		cb_alloc(s);
+#endif
 	    send_event(s, CS_EVENT_CARD_INSERTION, CS_EVENT_PRI_LOW);
 	    s->state &= ~SOCKET_SETUP_PENDING;
 	} else {
@@ -526,14 +568,13 @@ static void unreset_socket(u_long i)
 	    s->state &= ~EVENT_MASK;
 	}
     } else {
-	DEBUG(2, ("cs: socket %ld not ready yet\n", i));
-	if (s->unreset_timeout > unreset_limit) {
+	DEBUG(2, "cs: socket %ld not ready yet\n", i);
+	if (++s->setup_timeout > unreset_limit) {
 	    printk(KERN_NOTICE "cs: socket %ld timed out during"
 		   " reset\n", i);
 	    s->state &= ~EVENT_MASK;
 	} else {
-	    s->unreset_timeout++;
-	    s->setup.expires = RUN_AT(unreset_check);
+	    s->setup.expires = jiffies + unreset_check;
 	    add_timer(&s->setup);
 	}
     }
@@ -552,8 +593,8 @@ static int send_event(socket_info_t *s, event_t event, int priority)
 {
     client_t *client = s->clients;
     int ret;
-    DEBUG(1, ("cs: send_event(sock %d, event %d, pri %d)\n",
-	      s->sock, event, priority));
+    DEBUG(1, "cs: send_event(sock %d, event %d, pri %d)\n",
+	  s->sock, event, priority);
     ret = 0;
     for (; client; client = client->next) { 
 	if (client->state & (CLIENT_UNBOUND|CLIENT_STALE))
@@ -578,11 +619,11 @@ static void do_shutdown(socket_info_t *s)
 	if (!(client->Attributes & INFO_MASTER_CLIENT))
 	    client->state |= CLIENT_STALE;
     if (s->state & (SOCKET_SETUP_PENDING|SOCKET_RESET_PENDING)) {
-	DEBUG(0, ("cs: flushing pending setup\n"));
+	DEBUG(0, "cs: flushing pending setup\n");
 	del_timer(&s->setup);
 	s->state &= ~EVENT_MASK;
     }
-    s->shutdown.expires = RUN_AT(shutdown_delay);
+    s->shutdown.expires = jiffies + shutdown_delay;
     add_timer(&s->shutdown);
     s->state &= ~SOCKET_PRESENT;
 }
@@ -602,14 +643,15 @@ static void parse_events(void *info, u_int events)
 	if (status & SS_DETECT) {
 	    if (s->state & SOCKET_SETUP_PENDING) {
 		del_timer(&s->setup);
-		DEBUG(1, ("cs: delaying pending setup\n"));
+		DEBUG(1, "cs: delaying pending setup\n");
 	    }
 	    s->state |= SOCKET_SETUP_PENDING;
 	    s->setup.function = &setup_socket;
+	    s->setup_timeout = 0;
 	    if (s->state & SOCKET_SUSPEND)
-		s->setup.expires = RUN_AT(resume_delay);
+		s->setup.expires = jiffies + resume_delay;
 	    else
-		s->setup.expires = RUN_AT(setup_delay);
+		s->setup.expires = jiffies + setup_delay;
 	    add_timer(&s->setup);
 	}
 	spin_unlock_irqrestore(&s->lock, flags);
@@ -621,7 +663,7 @@ static void parse_events(void *info, u_int events)
     if (events & SS_READY) {
 	if (!(s->state & SOCKET_RESET_PENDING))
 	    send_event(s, CS_EVENT_READY_CHANGE, CS_EVENT_PRI_LOW);
-	else DEBUG(1, ("cs: ready change during reset\n"));
+	else DEBUG(1, "cs: ready change during reset\n");
     }
 } /* parse_events */
 
@@ -634,17 +676,22 @@ static void parse_events(void *info, u_int events)
     
 ======================================================================*/
 
-#ifdef CONFIG_APM
-static int handle_apm_event(apm_event_t event)
+#ifdef CONFIG_PM
+#if (LINUX_VERSION_CODE < VERSION(2,3,43))
+static int handle_pm_event(apm_event_t rqst)
+#else
+static int handle_pm_event(struct pm_dev *dev, pm_request_t rqst,
+			   void *data)
+#endif
 {
     int i, stat;
     socket_info_t *s;
     static int down = 0;
-    
-    switch (event) {
-    case APM_SYS_SUSPEND:
-    case APM_USER_SUSPEND:
-	DEBUG(1, ("cs: received suspend notification\n"));
+
+    /* <linux/pm.h> hides a hack so this works with old APM support */
+    switch (rqst) {
+    case PM_SUSPEND:
+	DEBUG(1, "cs: received suspend notification\n");
 	if (down) {
 	    printk(KERN_DEBUG "cs: received extra suspend event\n");
 	    break;
@@ -660,9 +707,8 @@ static int handle_apm_event(apm_event_t event)
 	    }
 	}
 	break;
-    case APM_NORMAL_RESUME:
-    case APM_CRITICAL_RESUME:
-	DEBUG(1, ("cs: received resume notification\n"));
+    case PM_RESUME:
+	DEBUG(1, "cs: received resume notification\n");
 	if (!down) {
 	    printk(KERN_DEBUG "cs: received bogus resume event\n");
 	    break;
@@ -681,7 +727,7 @@ static int handle_apm_event(apm_event_t event)
 	break;
     }
     return 0;
-} /* handle_apm_event */
+} /* handle_pm_event */
 #endif
 
 /*======================================================================
@@ -691,14 +737,28 @@ static int handle_apm_event(apm_event_t event)
 ======================================================================*/
 
 static int alloc_io_space(socket_info_t *s, u_int attr, ioaddr_t *base,
-			  ioaddr_t num, char *name)
+			  ioaddr_t num, u_int lines, char *name)
 {
     int i;
-    ioaddr_t try;
-    
+    ioaddr_t try, align;
+
+    align = (*base) ? (lines ? 1<<lines : 0) : 1;
+    if (align && (align < num)) {
+	if (*base) {
+	    DEBUG(0, "odd IO request: num %04x align %04x\n",
+		  num, align);
+	    align = 0;
+	} else
+	    while (align && (align < num)) align <<= 1;
+    }
+    if (*base & ~(align-1)) {
+	DEBUG(0, "odd IO request: base %04x align %04x\n",
+	      *base, align);
+	align = 0;
+    }
     for (i = 0; i < MAX_IO_WIN; i++) {
 	if (s->io[i].NumPorts == 0) {
-	    if (find_io_region(base, num, name) == 0) {
+	    if (find_io_region(base, num, align, name) == 0) {
 		s->io[i].Attributes = attr;
 		s->io[i].BasePort = *base;
 		s->io[i].NumPorts = s->io[i].InUse = num;
@@ -710,7 +770,7 @@ static int alloc_io_space(socket_info_t *s, u_int attr, ioaddr_t *base,
 	/* Try to extend top of window */
 	try = s->io[i].BasePort + s->io[i].NumPorts;
 	if ((*base == 0) || (*base == try))
-	    if (find_io_region(&try, num, name) == 0) {
+	    if (find_io_region(&try, num, 0, name) == 0) {
 		*base = try;
 		s->io[i].NumPorts += num;
 		s->io[i].InUse += num;
@@ -719,7 +779,7 @@ static int alloc_io_space(socket_info_t *s, u_int attr, ioaddr_t *base,
 	/* Try to extend bottom of window */
 	try = s->io[i].BasePort - num;
 	if ((*base == 0) || (*base == try))
-	    if (find_io_region(&try, num, name) == 0) {
+	    if (find_io_region(&try, num, 0, name) == 0) {
 		s->io[i].BasePort = *base = try;
 		s->io[i].NumPorts += num;
 		s->io[i].InUse += num;
@@ -733,7 +793,7 @@ static void release_io_space(socket_info_t *s, ioaddr_t base,
 			     ioaddr_t num)
 {
     int i;
-    vacate_region(base, num);
+    release_region(base, num);
     for (i = 0; i < MAX_IO_WIN; i++) {
 	if ((s->io[i].BasePort <= base) &&
 	    (s->io[i].BasePort+s->io[i].NumPorts >= base+num)) {
@@ -810,6 +870,7 @@ static int bind_device(bind_req_t *req)
     s = SOCKET(req);
 
     client = (client_t *)kmalloc(sizeof(client_t), GFP_KERNEL);
+    if (!client) return CS_OUT_OF_RESOURCE;
     memset(client, '\0', sizeof(client_t));
     client->client_magic = CLIENT_MAGIC;
     strncpy(client->dev_info, (char *)req->dev_info, DEV_NAME_LEN);
@@ -821,8 +882,8 @@ static int bind_device(bind_req_t *req)
     init_waitqueue_head(&client->mtd_req);
     client->next = s->clients;
     s->clients = client;
-    DEBUG(1, ("cs: bind_device(): client 0x%p, sock %d, dev %s\n",
-	      client, client->Socket, client->dev_info));
+    DEBUG(1, "cs: bind_device(): client 0x%p, sock %d, dev %s\n",
+	  client, client->Socket, client->dev_info);
     return CS_SUCCESS;
 } /* bind_device */
 
@@ -857,8 +918,8 @@ static int bind_mtd(mtd_bind_t *req)
 	return CS_BAD_OFFSET;
     strncpy(region->dev_info, (char *)req->dev_info, DEV_NAME_LEN);
     
-    DEBUG(1, ("cs: bind_mtd(): attr 0x%x, offset 0x%x, dev %s\n",
-	  req->Attributes, req->CardOffset, (char *)req->dev_info));
+    DEBUG(1, "cs: bind_mtd(): attr 0x%x, offset 0x%x, dev %s\n",
+	  req->Attributes, req->CardOffset, (char *)req->dev_info);
     return CS_SUCCESS;
 } /* bind_mtd */
 
@@ -872,7 +933,7 @@ static int deregister_client(client_handle_t handle)
     u_long flags;
     int i, sn;
     
-    DEBUG(1, ("cs: deregister_client(%p)\n", handle));
+    DEBUG(1, "cs: deregister_client(%p)\n", handle);
     if (CHECK_HANDLE(handle))
 	return CS_BAD_HANDLE;
     if (handle->state &
@@ -903,7 +964,7 @@ static int deregister_client(client_handle_t handle)
 	    return CS_BAD_HANDLE;
 	*client = handle->next;
 	handle->client_magic = 0;
-	kfree_s(handle, sizeof(struct client_t));
+	kfree(handle);
 	spin_unlock_irqrestore(&s->lock, flags);
     } else {
 	handle->state = CLIENT_UNBOUND;
@@ -1042,6 +1103,55 @@ static int get_next_client(client_handle_t *handle, client_req_t *req)
     return CS_SUCCESS;
 } /* get_next_client */
 
+/*====================================================================*/
+
+static int get_window(window_handle_t *handle, int idx, win_req_t *req)
+{
+    socket_info_t *s;
+    window_t *win;
+    int w;
+
+    if (idx == 0)
+	s = SOCKET((client_handle_t)*handle);
+    else
+	s = (*handle)->sock;
+    if (!(s->state & SOCKET_PRESENT))
+	return CS_NO_CARD;
+    for (w = idx; w < MAX_WIN; w++)
+	if (s->state & SOCKET_WIN_REQ(w)) break;
+    if (w == MAX_WIN)
+	return CS_NO_MORE_ITEMS;
+    win = &s->win[w];
+    req->Base = win->ctl.sys_start;
+    req->Size = win->ctl.sys_stop - win->ctl.sys_start + 1;
+    req->AccessSpeed = win->ctl.speed;
+    req->Attributes = 0;
+    if (win->ctl.flags & MAP_ATTRIB)
+	req->Attributes |= WIN_MEMORY_TYPE_AM;
+    if (win->ctl.flags & MAP_ACTIVE)
+	req->Attributes |= WIN_ENABLE;
+    if (win->ctl.flags & MAP_16BIT)
+	req->Attributes |= WIN_DATA_WIDTH_16;
+    if (win->ctl.flags & MAP_USE_WAIT)
+	req->Attributes |= WIN_USE_WAIT;
+    *handle = win;
+    return CS_SUCCESS;
+} /* get_window */
+
+static int get_first_window(client_handle_t *handle, win_req_t *req)
+{
+    if ((handle == NULL) || CHECK_HANDLE(*handle))
+	return CS_BAD_HANDLE;
+    return get_window((window_handle_t *)handle, 0, req);
+}
+
+static int get_next_window(window_handle_t *win, win_req_t *req)
+{
+    if ((win == NULL) || ((*win)->magic != WINDOW_MAGIC))
+	return CS_BAD_HANDLE;
+    return get_window(win, (*win)->index+1, req);
+}
+
 /*======================================================================
 
     Get the current socket state bits.  We don't support the latched
@@ -1119,6 +1229,15 @@ static int get_status(client_handle_t handle, cs_status_t *status)
     
 ======================================================================*/
 
+static int get_mem_page(window_handle_t win, memreq_t *req)
+{
+    if ((win == NULL) || (win->magic != WINDOW_MAGIC))
+	return CS_BAD_HANDLE;
+    req->Page = 0;
+    req->CardOffset = win->ctl.card_start;
+    return CS_SUCCESS;
+} /* get_mem_page */
+
 static int map_mem_page(window_handle_t win, memreq_t *req)
 {
     socket_info_t *s;
@@ -1126,7 +1245,6 @@ static int map_mem_page(window_handle_t win, memreq_t *req)
 	return CS_BAD_HANDLE;
     if (req->Page != 0)
 	return CS_BAD_PAGE;
-
     s = win->sock;
     win->ctl.card_start = req->CardOffset;
     if (s->ss_entry(s->sock, SS_SetMemMap, &win->ctl) != 0)
@@ -1199,7 +1317,7 @@ static int modify_window(window_handle_t win, modwin_t *req)
 	win->ctl.flags |= MAP_ATTRIB;
     if (req->Attributes & WIN_ENABLE)
 	win->ctl.flags |= MAP_ACTIVE;
-    if (req->Attributes & WIN_DATA_WIDTH)
+    if (req->Attributes & WIN_DATA_WIDTH_16)
 	win->ctl.flags |= MAP_16BIT;
     if (req->Attributes & WIN_USE_WAIT)
 	win->ctl.flags |= MAP_USE_WAIT;
@@ -1279,8 +1397,8 @@ static int register_client(client_handle_t *handle, client_reg_t *req)
 	memset(s->config, 0, sizeof(config_t) * s->functions);
     }
     
-    DEBUG(1, ("cs: register_client(): client 0x%p, sock %d, dev %s\n",
-	      client, client->Socket, client->dev_info));
+    DEBUG(1, "cs: register_client(): client 0x%p, sock %d, dev %s\n",
+	  client, client->Socket, client->dev_info);
     if (client->EventMask & CS_EVENT_REGISTRATION_COMPLETE)
 	EVENT(client, CS_EVENT_REGISTRATION_COMPLETE, CS_EVENT_PRI_LOW);
     if ((socket_table[ns]->state & SOCKET_PRESENT) &&
@@ -1296,10 +1414,10 @@ static int register_client(client_handle_t *handle, client_reg_t *req)
 /*====================================================================*/
 
 static int release_configuration(client_handle_t handle,
-				 socket_t *Socket)
+				 config_req_t *req)
 {
-    pccard_io_map io;
     socket_info_t *s;
+    pccard_io_map io = { 0, 0, 0, 0, 1 };
     int i;
     
     if (CHECK_HANDLE(handle) ||
@@ -1332,8 +1450,6 @@ static int release_configuration(client_handle_t handle,
 		if (s->io[i].Config != 0)
 		    continue;
 		io.map = i;
-		s->ss_entry(s->sock, SS_GetIOMap, &io);
-		io.flags &= ~MAP_ACTIVE;
 		s->ss_entry(s->sock, SS_SetIOMap, &io);
 	    }
 	c->state &= ~CONFIG_LOCKED;
@@ -1414,8 +1530,6 @@ static int cs_release_irq(client_handle_t handle, irq_req_t *req)
     if (req->Attributes & IRQ_HANDLE_PRESENT) {
 #ifdef __LINUX__
 	bus_free_irq(s->cap.bus, req->AssignedIRQ, req->Instance);
-	if (req->Instance)
-	    IRQ_MAP(req->AssignedIRQ, NULL);
 #endif
 #ifdef __BEOS__
 	remove_io_interrupt_handler(req->AssignedIRQ, req->Handler,
@@ -1449,7 +1563,7 @@ static int release_window(window_handle_t win)
     s->state &= ~SOCKET_WIN_REQ(win->index);
 
     /* Release system memory */
-    vacate_mem_region(win->base, win->size);
+    release_mem_region(win->base, win->size);
     win->handle->state &= ~CLIENT_WIN_REQ(win->index);
 
     win->magic = 0;
@@ -1618,7 +1732,9 @@ static int request_io(client_handle_t handle, io_req_t *req)
 	return CS_UNSUPPORTED_FUNCTION;
 #endif
     }
-    
+
+    if (!req)
+	return CS_UNSUPPORTED_MODE;
     c = CONFIG(handle);
     if (c->state & CONFIG_LOCKED)
 	return CS_CONFIGURATION_LOCKED;
@@ -1631,12 +1747,14 @@ static int request_io(client_handle_t handle, io_req_t *req)
 	return CS_BAD_ATTRIBUTE;
 
     if (alloc_io_space(s, req->Attributes1, &req->BasePort1,
-		       req->NumPorts1, handle->dev_info))
+		       req->NumPorts1, req->IOAddrLines,
+		       handle->dev_info))
 	return CS_IN_USE;
 
     if (req->NumPorts2) {
 	if (alloc_io_space(s, req->Attributes2, &req->BasePort2,
-			   req->NumPorts2, handle->dev_info)) {
+			   req->NumPorts2, req->IOAddrLines,
+			   handle->dev_info)) {
 	    release_io_space(s, req->BasePort1, req->NumPorts1);
 	    return CS_IN_USE;
 	}
@@ -1718,8 +1836,6 @@ static int cs_request_irq(client_handle_t handle, irq_req_t *req)
 			     (irq == s->cap.pci_irq)) ? SA_SHIRQ : 0,
 			    handle->dev_info, req->Instance))
 	    return CS_IN_USE;
-	if (req->Instance)
-	    IRQ_MAP(irq, req->Instance);
 #endif
 #ifdef __BEOS__
 	install_io_interrupt_handler(irq, req->Handler,
@@ -1747,6 +1863,7 @@ static int request_window(client_handle_t *handle, win_req_t *req)
 {
     socket_info_t *s;
     window_t *win;
+    u_long align;
     int w;
     
     if (CHECK_HANDLE(*handle))
@@ -1757,16 +1874,26 @@ static int request_window(client_handle_t *handle, win_req_t *req)
     if (req->Attributes & (WIN_PAGED | WIN_SHARED))
 	return CS_BAD_ATTRIBUTE;
 
+    /* Window size defaults to smallest available */
+    if (req->Size == 0)
+	req->Size = s->cap.map_size;
+    align = (((s->cap.features & SS_CAP_MEM_ALIGN) ||
+	      (req->Attributes & WIN_STRICT_ALIGN)) ?
+	     req->Size : s->cap.map_size);
+    if (req->Size & (s->cap.map_size-1))
+	return CS_BAD_SIZE;
+    if ((req->Base && (s->cap.features & SS_CAP_STATIC_MAP)) ||
+	(req->Base & (align-1)))
+	return CS_BAD_BASE;
+    if (req->Base)
+	align = 0;
+
+    /* Allocate system memory window */
     for (w = 0; w < MAX_WIN; w++)
 	if (!(s->state & SOCKET_WIN_REQ(w))) break;
     if (w == MAX_WIN)
 	return CS_OUT_OF_RESOURCE;
 
-    /* Window size defaults to smallest available */
-    if (req->Size == 0)
-	req->Size = s->cap.map_size;
-    
-    /* Allocate system memory window */
     win = &s->win[w];
     win->magic = WINDOW_MAGIC;
     win->index = w;
@@ -1774,13 +1901,13 @@ static int request_window(client_handle_t *handle, win_req_t *req)
     win->sock = s;
     win->base = req->Base;
     win->size = req->Size;
-    if (find_mem_region(&win->base, win->size, (*handle)->dev_info,
-			((s->cap.features & SS_CAP_MEM_ALIGN) ?
-			 req->Size : s->cap.map_size),
+
+    if (!(s->cap.features & SS_CAP_STATIC_MAP) &&
+	find_mem_region(&win->base, win->size, align,
 			(req->Attributes & WIN_MAP_BELOW_1MB) ||
-			!(s->cap.features & SS_CAP_PAGE_REGS)))
+			!(s->cap.features & SS_CAP_PAGE_REGS),
+			(*handle)->dev_info))
 	return CS_IN_USE;
-    req->Base = win->base;
     (*handle)->state |= CLIENT_WIN_REQ(w);
 
     /* Configure the socket controller */
@@ -1791,18 +1918,19 @@ static int request_window(client_handle_t *handle, win_req_t *req)
 	win->ctl.flags |= MAP_ATTRIB;
     if (req->Attributes & WIN_ENABLE)
 	win->ctl.flags |= MAP_ACTIVE;
-    if (req->Attributes & WIN_DATA_WIDTH)
+    if (req->Attributes & WIN_DATA_WIDTH_16)
 	win->ctl.flags |= MAP_16BIT;
     if (req->Attributes & WIN_USE_WAIT)
 	win->ctl.flags |= MAP_USE_WAIT;
-    win->ctl.sys_start = req->Base;
-    win->ctl.sys_stop = req->Base + req->Size-1;
+    win->ctl.sys_start = win->base;
+    win->ctl.sys_stop = win->base + win->size-1;
     win->ctl.card_start = 0;
     if (s->ss_entry(s->sock, SS_SetMemMap, &win->ctl) != 0)
 	return CS_BAD_ARGS;
     s->state |= SOCKET_WIN_REQ(w);
 
     /* Return window handle */
+    req->Base = win->ctl.sys_start;
     *handle = (client_handle_t)win;
     
     return CS_SUCCESS;
@@ -1836,7 +1964,7 @@ static int reset_card(client_handle_t handle, client_req_t *req)
 	handle->event_callback_args.info = (void *)(u_long)ret;
 	EVENT(handle, CS_EVENT_RESET_COMPLETE, CS_EVENT_PRI_LOW);
     } else {
-	DEBUG(1, ("cs: resetting socket %d\n", i));
+	DEBUG(1, "cs: resetting socket %d\n", i);
 	send_event(s, CS_EVENT_RESET_PHYSICAL, CS_EVENT_PRI_LOW);
 	s->reset_handle = handle;
 	reset_socket(i);
@@ -1864,7 +1992,7 @@ static int suspend_card(client_handle_t handle, client_req_t *req)
     if (s->state & SOCKET_SUSPEND)
 	return CS_IN_USE;
 
-    DEBUG(1, ("cs: suspending socket %d\n", i));
+    DEBUG(1, "cs: suspending socket %d\n", i);
     send_event(s, CS_EVENT_PM_SUSPEND, CS_EVENT_PRI_LOW);
     s->ss_entry(s->sock, SS_SetSocket, &dead_socket);
     s->state |= SOCKET_SUSPEND;
@@ -1885,7 +2013,7 @@ static int resume_card(client_handle_t handle, client_req_t *req)
     if (!(s->state & SOCKET_SUSPEND))
 	return CS_IN_USE;
 
-    DEBUG(1, ("cs: waking up socket %d\n", i));
+    DEBUG(1, "cs: waking up socket %d\n", i);
     setup_socket(i);
 
     return CS_SUCCESS;
@@ -1909,7 +2037,7 @@ static int eject_card(client_handle_t handle, client_req_t *req)
     if (!(s->state & SOCKET_PRESENT))
 	return CS_NO_CARD;
 
-    DEBUG(1, ("cs: user eject request on socket %d\n", i));
+    DEBUG(1, "cs: user eject request on socket %d\n", i);
 
     ret = send_event(s, CS_EVENT_EJECTION_REQUEST, CS_EVENT_PRI_LOW);
     if (ret != 0)
@@ -1935,7 +2063,7 @@ static int insert_card(client_handle_t handle, client_req_t *req)
     if (s->state & SOCKET_PRESENT)
 	return CS_IN_USE;
 
-    DEBUG(1, ("cs: user insert request on socket %d\n", i));
+    DEBUG(1, "cs: user insert request on socket %d\n", i);
 
     spin_lock_irqsave(&s->lock, flags);
     if (!(s->state & SOCKET_SETUP_PENDING)) {
@@ -2014,7 +2142,7 @@ int CardServices(int func, void *a1, void *a2, void *a3)
 {
 
 #ifdef PCMCIA_DEBUG
-    if (pc_debug > 1) {
+    if (pc_debug > 2) {
 	int i;
 	for (i = 0; i < SERVICE_COUNT; i++)
 	    if (service_table[i].key == func) break;
@@ -2119,6 +2247,12 @@ int CardServices(int func, void *a1, void *a2, void *a3)
 	return insert_card(a1, a2); break;
     case ReplaceCIS:
 	return replace_cis(a1, a2); break;
+    case GetFirstWindow:
+	return get_first_window(a1, a2); break;
+    case GetNextWindow:
+	return get_next_window(a1, a2); break;
+    case GetMemPage:
+	return get_mem_page(a1, a2); break;
     default:
 	return CS_UNSUPPORTED_FUNCTION; break;
     }
@@ -2133,6 +2267,8 @@ int CardServices(int func, void *a1, void *a2, void *a3)
 
 #ifdef __LINUX__
 
+#include <linux/pci.h>
+
 #if (LINUX_VERSION_CODE <= VERSION(2,1,17))
 
 #undef CONFIG_MODVERSIONS
@@ -2144,65 +2280,130 @@ static struct symbol_table cs_symtab = {
     X(unregister_ss_entry),
     X(CardServices),
     X(MTDHelperEntry),
+#ifdef HAS_PROC_BUS
+    X(proc_pccard),
+#endif
 #ifndef HAVE_MEMRESERVE
     X(request_mem_region),
     X(release_mem_region),
 #endif
+#ifdef CONFIG_PNP_BIOS
+    X(check_pnp_irq),
+#endif
+#ifdef CONFIG_PCI
+    X(pci_irq_mask),
+    X(pci_devices),
+    X(pci_root),
+    X(pci_find_slot),
+    X(pci_find_class),
+    X(pci_enable_device),
+    X(pci_set_power_state),
+#endif
 #include <linux/symtab_end.h>
 };
 
 #else
 
-#define register_symtab(n)
 EXPORT_SYMBOL(register_ss_entry);
 EXPORT_SYMBOL(unregister_ss_entry);
 EXPORT_SYMBOL(CardServices);
 EXPORT_SYMBOL(MTDHelperEntry);
+#ifdef HAS_PROC_BUS
+EXPORT_SYMBOL(proc_pccard);
+#endif
 #ifndef HAVE_MEMRESERVE
 EXPORT_SYMBOL(request_mem_region);
 EXPORT_SYMBOL(release_mem_region);
 #endif
+#ifdef CONFIG_PNP_BIOS
+EXPORT_SYMBOL(check_pnp_irq);
+#endif
+#ifdef CONFIG_PCI
+EXPORT_SYMBOL(pci_irq_mask);
+#if (LINUX_VERSION_CODE < VERSION(2,3,24))
+EXPORT_SYMBOL(pci_enable_device);
+EXPORT_SYMBOL(pci_set_power_state);
+#endif
+#endif
 
 #endif
 
-int init_module(void)
+static int __init init_pcmcia_cs(void)
 {
     printk(KERN_INFO "%s\n", release);
+#ifdef UTS_RELEASE
     printk(KERN_INFO "  %s\n", kernel);
+#endif
     printk(KERN_INFO "  %s\n", options);
-    DEBUG(0, ("%s\n", version));
-#ifdef CONFIG_APM
-    apm_register_callback(&handle_apm_event);
+    DEBUG(0, "%s\n", version);
+#ifdef CONFIG_PM
+    if (do_apm)
+	pm_register(PM_SYS_DEV, PM_SYS_PCMCIA, handle_pm_event);
+#endif
+#ifdef CONFIG_PCI
+    pci_fixup_init();
+#endif
+#ifdef CONFIG_PNP_BIOS
+    if (do_pnp) {
+	pnp_bios_init();
+	pnp_proc_init();
+	pnp_rsrc_init();
+    }
 #endif
     register_symtab(&cs_symtab);
 #ifdef HAS_PROC_BUS
-    proc_pccard = create_proc_entry("pccard", S_IFDIR, proc_bus);
-#ifndef HAVE_MEMRESERVE
+    proc_pccard = proc_mkdir("pccard", proc_bus);
+#ifdef CONFIG_PNP_BIOS
     if (proc_pccard) {
-	struct proc_dir_entry *ent;
-	ent = create_proc_entry("memory", 0, proc_pccard);
-	ent->read_proc = proc_read_mem;
+	create_proc_read_entry("ioport", 0, proc_pccard,
+			       proc_read_io, NULL);
+	create_proc_read_entry("irq", 0, proc_pccard,
+			       proc_read_irq, NULL);
     }
 #endif
+#ifndef HAVE_MEMRESERVE
+    if (proc_pccard)
+	create_proc_read_entry("memory", 0, proc_pccard,
+			       proc_read_mem, NULL);
+#endif
 #endif
     return 0;
 }
 
-void cleanup_module(void)
+static void __exit exit_pcmcia_cs(void)
 {
     printk(KERN_INFO "unloading PCMCIA Card Services\n");
 #ifdef HAS_PROC_BUS
     if (proc_pccard) {
+#ifdef CONFIG_PNP_BIOS
+	remove_proc_entry("ioport", proc_pccard);
+	remove_proc_entry("irq", proc_pccard);
+#endif
+#ifndef HAVE_MEMRESERVE
 	remove_proc_entry("memory", proc_pccard);
+#endif
 	remove_proc_entry("pccard", proc_bus);
     }
 #endif
-#ifdef CONFIG_APM
-    apm_unregister_callback(&handle_apm_event);
+#ifdef CONFIG_PM
+    if (do_apm)
+	pm_unregister_all(handle_pm_event);
+#endif
+#ifdef CONFIG_PCI
+    pci_fixup_done();
+#endif
+#ifdef CONFIG_PNP_BIOS
+    if (do_pnp) {
+	pnp_proc_done();
+	pnp_rsrc_done();
+    }
 #endif
     release_resource_db();
 }
 
+module_init(init_pcmcia_cs);
+module_exit(exit_pcmcia_cs);
+
 #endif /* __LINUX__ */
 
 /*====================================================================*/
@@ -2221,7 +2422,7 @@ static status_t std_ops(int32 op)
     case B_MODULE_INIT:
 	printk(KERN_INFO "%s\n", release);
 	printk(KERN_INFO "  %s\n", options);
-	DEBUG(0, ("%s\n", version));
+	DEBUG(0, "%s\n", version);
 	init_timer();
 	if (get_module(B_ISA_MODULE_NAME, (mod_t **)&isa) != B_OK)
 	    return B_ERROR;
