@@ -1,5 +1,5 @@
 /*
- * cs_internal.h 1.39 1999/07/20 16:01:25
+ * cs_internal.h 1.51 2000/02/08 00:31:51
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.1 (the "License"); you may not use this file except in
@@ -12,8 +12,8 @@
  * limitations under the License. 
  *
  * The initial developer of the original code is David A. Hinds
- * <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
- *  are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+ * <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+ *  are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
  */
 
 #ifndef _LINUX_CS_INTERNAL_H
@@ -23,6 +23,10 @@
 #include "wchan.h"
 #endif
 
+#ifdef __LINUX__
+#include <linux/config.h>
+#endif
+
 typedef struct erase_busy_t {
     eraseq_entry_t	*erase;
     client_handle_t	client;
@@ -140,7 +144,7 @@ typedef struct socket_info_t {
     u_int			real_clients;
     client_handle_t		reset_handle;
     struct timer_list		setup, shutdown;
-    u_long			unreset_timeout;
+    u_long			setup_timeout;
     pccard_mem_map		cis_mem;
     u_char			*cis_virt;
     config_t			*config;
@@ -207,6 +211,8 @@ typedef struct socket_info_t {
     ((h)->event_handler((e), (p), &(h)->event_callback_args))
 
 /* In cardbus.c */
+int cb_alloc(socket_info_t *s);
+void cb_free(socket_info_t *s);
 int cb_config(socket_info_t *s);
 void cb_release(socket_info_t *s);
 void cb_enable(socket_info_t *s);
@@ -250,16 +256,33 @@ int copy_memory(memory_handle_t handle, copy_op_t *req);
 /* In rsrc_mgr */
 void validate_mem(int (*is_valid)(u_long), int (*do_cksum)(u_long),
 		  int force_low);
-int find_io_region(ioaddr_t *base, ioaddr_t num, char *name);
-int find_mem_region(u_long *base, u_long num, char *name,
-		    u_long align, int force_low);
+int find_io_region(ioaddr_t *base, ioaddr_t num, ioaddr_t align,
+		   char *name);
+int find_mem_region(u_long *base, u_long num, u_long align,
+		    int force_low, char *name);
 int try_irq(u_int Attributes, int irq, int specific);
 void undo_irq(u_int Attributes, int irq);
 int adjust_resource_info(client_handle_t handle, adjust_t *adj);
 void release_resource_db(void);
+int proc_read_io(char *buf, char **start, off_t pos,
+		 int count, int *eof, void *data);
 int proc_read_mem(char *buf, char **start, off_t pos,
 		  int count, int *eof, void *data);
 
+/* in pnp components */
+int proc_read_irq(char *buf, char **start, off_t pos,
+		  int count, int *eof, void *data);
+int check_pnp_irq(int n);
+void pnp_bios_init(void);
+void pnp_proc_init(void);
+void pnp_proc_done(void);
+void pnp_rsrc_init(void);
+void pnp_rsrc_done(void);
+
+/* in pci_fixup */
+void pci_fixup_init(void);
+void pci_fixup_done(void);
+
 #define MAX_SOCK 8
 extern socket_t sockets;
 extern socket_info_t *socket_table[MAX_SOCK];
@@ -278,14 +301,9 @@ extern config_manager_for_bus_module_info *cm;
 
 #ifdef PCMCIA_DEBUG
 extern int pc_debug;
-#ifdef __LINUX__
-#define _printk(args...) printk(KERN_DEBUG args)
+#define DEBUG(n, args...) do { if (pc_debug>(n)) printk(KERN_DEBUG args); } while (0)
 #else
-#define _printk printk
-#endif
-#define DEBUG(n, args) do { if (pc_debug>(n)) _printk args; } while (0)
-#else
-#define DEBUG(n, args) do { } while (0)
+#define DEBUG(n, args...) do { } while (0)
 #endif
 
 #endif /* _LINUX_CS_INTERNAL_H */
