@@ -2,7 +2,7 @@
 
     PC Card Driver Services
     
-    ds.c 1.94 1999/07/30 04:27:15
+    ds.c 1.105 2000/05/03 20:04:52
     
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,14 +15,27 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
 #include <pcmcia/config.h>
 #include <pcmcia/k_compat.h>
 
+#include <linux/module.h>
+#include <linux/init.h>
 #include <linux/kernel.h>
 #include <linux/major.h>
 #include <linux/string.h>
@@ -33,6 +46,7 @@
 #include <linux/sched.h>
 #include <linux/timer.h>
 #include <linux/ioctl.h>
+#include <linux/proc_fs.h>
 #if (LINUX_VERSION_CODE >= VERSION(2,1,23))
 #include <linux/poll.h>
 #endif
@@ -49,16 +63,19 @@ int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static const char *version =
-"ds.c 1.94 1999/07/30 04:27:15 (David Hinds)";
+"ds.c 1.105 2000/05/03 20:04:52 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
 
+MODULE_AUTHOR("David Hinds <dhinds@pcmcia.sourceforge.org>");
+MODULE_DESCRIPTION("PCMCIA Driver Services " CS_RELEASE);
+
 /*====================================================================*/
 
 typedef struct driver_info_t {
     dev_info_t		dev_info;
-    int			use_count;
+    int			use_count, status;
     dev_link_t		*(*attach)(void);
     void		(*detach)(dev_link_t *);
     struct driver_info_t *next;
@@ -108,6 +125,11 @@ static driver_info_t *root_driver = NULL;
 static int sockets = 0, major_dev = -1;
 static socket_info_t *socket_table = NULL;
 
+extern struct proc_dir_entry *proc_pccard;
+
+/* We use this to distinguish in-kernel from modular drivers */
+static int init_status = 1;
+
 /*====================================================================*/
 
 static void cs_error(client_handle_t handle, int func, int ret)
@@ -139,8 +161,10 @@ int register_pccard_driver(dev_info_t *dev_info,
 	    break;
     if (!driver) {
 	driver = kmalloc(sizeof(driver_info_t), GFP_KERNEL);
+	if (!driver) return -ENOMEM;
 	strncpy(driver->dev_info, (char *)dev_info, DEV_NAME_LEN);
 	driver->use_count = 0;
+	driver->status = init_status;
 	driver->next = root_driver;
 	root_driver = driver;
     }
@@ -176,7 +200,7 @@ int unregister_pccard_driver(dev_info_t *dev_info)
 			    DEV_NAME_LEN) != 0))
 	d = &(*d)->next;
     if (*d == NULL)
-	return -1;
+	return -ENODEV;
     
     target = *d;
     if (target->use_count == 0) {
@@ -192,6 +216,21 @@ int unregister_pccard_driver(dev_info_t *dev_info)
     return 0;
 } /* unregister_pccard_driver */
 
+/*====================================================================*/
+
+#ifdef HAS_PROC_BUS
+static int proc_read_drivers(char *buf, char **start, off_t pos,
+			     int count, int *eof, void *data)
+{
+    driver_info_t *d;
+    char *p = buf;
+    for (d = root_driver; d; d = d->next)
+	p += sprintf(p, "%-24.24s %d %d\n", d->dev_info,
+		     d->status, d->use_count);
+    return (p - buf);
+}
+#endif
+
 /*======================================================================
 
     These manage a ring buffer of events pending for one user process
@@ -272,7 +311,7 @@ static int ds_event(event_t event, int priority,
 	s->state &= ~SOCKET_PRESENT;
 	if (!(s->state & SOCKET_REMOVAL_PENDING)) {
 	    s->state |= SOCKET_REMOVAL_PENDING;
-	    s->removal.expires = RUN_AT(HZ/10);
+	    s->removal.expires = jiffies + HZ/10;
 	    add_timer(&s->removal);
 	}
 	break;
@@ -345,6 +384,7 @@ static int bind_request(int i, bind_info_t *bind_info)
 	    break;
     if (driver == NULL) {
 	driver = kmalloc(sizeof(driver_info_t), GFP_KERNEL);
+	if (!driver) return -ENOMEM;
 	strncpy(driver->dev_info, bind_info->dev_info, DEV_NAME_LEN);
 	driver->use_count = 0;
 	driver->next = root_driver;
@@ -450,11 +490,11 @@ static int unbind_request(int i, bind_info_t *bind_info)
 	    for (d = &root_driver; *d; d = &((*d)->next))
 		if (c->driver == *d) break;
 	    *d = (*d)->next;
-	    kfree_s(c->driver, sizeof(driver_info_t));
+	    kfree(c->driver);
 	}
     }
     *b = c->next;
-    kfree_s(c, sizeof(socket_bind_t));
+    kfree(c);
     
     return 0;
 } /* unbind_request */
@@ -482,8 +522,9 @@ static int ds_open(struct inode *inode, struct file *file)
 	    s->state |= SOCKET_BUSY;
     }
     
-    MOD_INC_USE_COUNT;
     user = kmalloc(sizeof(user_info_t), GFP_KERNEL);
+    if (!user) return -ENOMEM;
+    MOD_INC_USE_COUNT;
     user->event_tail = user->event_head = 0;
     user->next = s->user;
     user->user_magic = USER_MAGIC;
@@ -521,7 +562,7 @@ static FS_RELEASE_T ds_release(struct inode *inode, struct file *file)
 	return (FS_RELEASE_T)0;
     *link = user->next;
     user->user_magic = 0;
-    kfree_s(user, sizeof(user_info_t));
+    kfree(user);
     
     MOD_DEC_USE_COUNT;
     return (FS_RELEASE_T)0;
@@ -529,9 +570,9 @@ static FS_RELEASE_T ds_release(struct inode *inode, struct file *file)
 
 /*====================================================================*/
 
-static FS_SIZE_T ds_read FOPS(struct inode *inode,
-			      struct file *file, char *buf,
-			      U_FS_SIZE_T count, loff_t *ppos)
+static ssize_t ds_read FOPS(struct inode *inode,
+			    struct file *file, char *buf,
+			    size_t count, loff_t *ppos)
 {
     socket_t i = MINOR(F_INODE(file)->i_rdev);
     socket_info_t *s;
@@ -559,9 +600,9 @@ static FS_SIZE_T ds_read FOPS(struct inode *inode,
 
 /*====================================================================*/
 
-static FS_SIZE_T ds_write FOPS(struct inode *inode,
-			       struct file *file, const char *buf,
-			       U_FS_SIZE_T count, loff_t *ppos)
+static ssize_t ds_write FOPS(struct inode *inode,
+			     struct file *file, const char *buf,
+			     size_t count, loff_t *ppos)
 {
     socket_t i = MINOR(F_INODE(file)->i_rdev);
     socket_info_t *s;
@@ -663,7 +704,7 @@ static int ds_ioctl(struct inode * inode, struct file * file,
     if (size > sizeof(ds_ioctl_arg_t)) return -EINVAL;
 
     /* Permission check */
-    if (!(cmd & IOC_OUT) && !suser())
+    if (!(cmd & IOC_OUT) && !capable(CAP_SYS_ADMIN))
 	return -EPERM;
 	
     if (cmd & IOC_IN) {
@@ -733,7 +774,7 @@ static int ds_ioctl(struct inode * inode, struct file * file,
 	ret = CardServices(InsertCard, s->handle, NULL);
 	break;
     case DS_ACCESS_CONFIGURATION_REGISTER:
-	if ((buf.conf_reg.Action == CS_WRITE) && !suser())
+	if ((buf.conf_reg.Action == CS_WRITE) && !capable(CAP_SYS_ADMIN))
 	    return -EPERM;
 	ret = CardServices(AccessConfigurationRegister, s->handle,
 			   &buf.conf_reg);
@@ -744,11 +785,24 @@ static int ds_ioctl(struct inode * inode, struct file * file,
     case DS_GET_NEXT_REGION:
 	ret = CardServices(GetNextRegion, s->handle, &buf.region);
 	break;
+    case DS_GET_FIRST_WINDOW:
+	buf.win_info.handle = (window_handle_t)s->handle;
+	ret = CardServices(GetFirstWindow, &buf.win_info.handle,
+			   &buf.win_info.window);
+	break;
+    case DS_GET_NEXT_WINDOW:
+	ret = CardServices(GetNextWindow, &buf.win_info.handle,
+			   &buf.win_info.window);
+	break;
+    case DS_GET_MEM_PAGE:
+	ret = CardServices(GetMemPage, buf.win_info.handle,
+			   &buf.win_info.map);
+	break;
     case DS_REPLACE_CIS:
 	ret = CardServices(ReplaceCIS, s->handle, &buf.cisdump);
 	break;
     case DS_BIND_REQUEST:
-	if (!suser()) return -EPERM;
+	if (!capable(CAP_SYS_ADMIN)) return -EPERM;
 	err = bind_request(i, &buf.bind_info);
 	break;
     case DS_GET_DEVICE_INFO:
@@ -761,7 +815,7 @@ static int ds_ioctl(struct inode * inode, struct file * file,
 	err = unbind_request(i, &buf.bind_info);
 	break;
     case DS_BIND_MTD:
-	if (!suser()) return -EPERM;
+	if (!capable(CAP_SYS_ADMIN)) return -EPERM;
 	err = bind_mtd(i, &buf.mtd_info);
 	break;
     default:
@@ -797,21 +851,16 @@ static int ds_ioctl(struct inode * inode, struct file * file,
 /*====================================================================*/
 
 static struct file_operations ds_fops = {
-    NULL,		/* lseek */
-    ds_read,		/* read */
-    ds_write,		/* write */
-    NULL,		/* readdir */
+    open:	ds_open,
+    release:	ds_release,
+    ioctl:	ds_ioctl,
+    read:	ds_read,
+    write:	ds_write,
 #if (LINUX_VERSION_CODE < VERSION(2,1,23))
-    ds_select,		/* select */
+    select:	ds_select
 #else
-    ds_poll,		/* poll */
+    poll:	ds_poll
 #endif
-    ds_ioctl,		/* ioctl */
-    NULL,		/* mmap */
-    ds_open,		/* open */
-    NULL_FLUSH		/* flush */
-    ds_release,		/* release */
-    NULL		/* fsync */
 };
 
 #if (LINUX_VERSION_CODE <= VERSION(2,1,17))
@@ -828,7 +877,6 @@ static struct symbol_table ds_symtab = {
 
 #else
 
-#define register_symtab(n)
 EXPORT_SYMBOL(register_pccard_driver);
 EXPORT_SYMBOL(unregister_pccard_driver);
 
@@ -836,7 +884,7 @@ EXPORT_SYMBOL(unregister_pccard_driver);
 
 /*====================================================================*/
 
-int init_module(void)
+int __init init_pcmcia_ds(void)
 {
     client_reg_t client_reg;
     servinfo_t serv;
@@ -858,6 +906,7 @@ int init_module(void)
     
     sockets = serv.Count;
     socket_table = kmalloc(sockets*sizeof(socket_info_t), GFP_KERNEL);
+    if (!socket_table) return -1;
     for (i = 0, s = socket_table; i < sockets; i++, s++) {
 	s->state = 0;
 	s->user = NULL;
@@ -865,7 +914,7 @@ int init_module(void)
 	init_waitqueue_head(&s->queue);
 	init_waitqueue_head(&s->request);
 	s->handle = NULL;
-	s->removal.prev = s->removal.next = NULL;
+	init_timer(&s->removal);
 	s->removal.data = i;
 	s->removal.function = &handle_removal;
 	s->bind = NULL;
@@ -906,13 +955,30 @@ int init_module(void)
     else
 	major_dev = i;
     register_symtab(&ds_symtab);
-    
+
+#ifdef HAS_PROC_BUS
+    if (proc_pccard)
+	create_proc_read_entry("drivers", 0, proc_pccard,
+			       proc_read_drivers, NULL);
+    init_status = 0;
+#endif
     return 0;
 }
 
-void cleanup_module(void)
+#ifdef MODULE
+
+int __init init_module(void)
+{
+    return init_pcmcia_ds();
+}
+
+void __exit cleanup_module(void)
 {
     int i;
+#ifdef HAS_PROC_BUS
+    if (proc_pccard)
+	remove_proc_entry("drivers", proc_pccard);
+#endif
     if (major_dev != -1)
 	unregister_chrdev(major_dev, "pcmcia");
     for (i = 0; i < sockets; i++)
@@ -920,3 +986,5 @@ void cleanup_module(void)
     sockets = 0;
     kfree(socket_table);
 }
+
+#endif
