@@ -2,7 +2,7 @@
 
     Resource management routines
 
-    rsrc_mgr.c 1.67 1999/07/30 03:48:44
+    rsrc_mgr.c 1.77 1999/11/16 03:32:59
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,8 +15,19 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
@@ -25,6 +36,9 @@
 #include <pcmcia/k_compat.h>
 
 #ifdef __LINUX__
+#include <linux/config.h>
+#include <linux/module.h>
+#include <linux/init.h>
 #include <linux/sched.h>
 #include <linux/kernel.h>
 #include <linux/errno.h>
@@ -89,18 +103,23 @@ static irq_info_t irq_table[NR_IRQS] = { { 0, 0, 0 }, /* etc */ };
 
 #endif
 
-#ifdef USE_SPIN_LOCKS
-static spinlock_t rsrc_lock = SPIN_LOCK_UNLOCKED;
-#endif
-
 /*======================================================================
 
-    Linux resource management extensions for keeping track of memory
-    mapped devices.
+    Linux resource management extensions
     
 ======================================================================*/
 
-#if defined(__LINUX__) && !defined(HAVE_MEMRESERVE)
+#ifdef __LINUX__
+
+#ifndef CONFIG_PNP_BIOS
+#define check_io_region(b,n) (0)
+#endif
+
+#if defined(CONFIG_PNP_BIOS) || !defined(HAVE_MEMRESERVE)
+
+#ifdef USE_SPIN_LOCKS
+static spinlock_t rsrc_lock = SPIN_LOCK_UNLOCKED;
+#endif
 
 typedef struct resource_entry_t {
     u_long			base, num;
@@ -108,8 +127,13 @@ typedef struct resource_entry_t {
     struct resource_entry_t	*next;
 } resource_entry_t;
 
-/* An ordered linked list of allocated memory blocks */
+/* Ordered linked lists of allocated IO and memory blocks */
+#ifdef CONFIG_PNP_BIOS
+static resource_entry_t io_list = { 0, 0, NULL, NULL };
+#endif
+#ifndef HAVE_MEMRESERVE
 static resource_entry_t mem_list = { 0, 0, NULL, NULL };
+#endif
 
 static resource_entry_t *find_gap(resource_entry_t *root,
 				  resource_entry_t *entry)
@@ -137,6 +161,7 @@ static int register_my_resource(resource_entry_t *list,
     resource_entry_t *p, *entry;
 
     entry = kmalloc(sizeof(resource_entry_t), GFP_ATOMIC);
+    if (!entry) return -ENOMEM;
     entry->base = base;
     entry->num = num;
     entry->name = name;
@@ -145,7 +170,7 @@ static int register_my_resource(resource_entry_t *list,
     p = find_gap(list, entry);
     if (p == NULL) {
 	spin_unlock_irqrestore(&rsrc_lock, flags);
-	kfree_s(entry, sizeof(resource_entry_t));
+	kfree(entry);
 	return -EBUSY;
     }
     entry->next = p->next;
@@ -166,7 +191,7 @@ static void release_my_resource(resource_entry_t *list,
 	if (q == NULL) break;
 	if ((q->base == base) && (q->num == num)) {
 	    p->next = q->next;
-	    kfree_s(q, sizeof(resource_entry_t));
+	    kfree(q);
 	    spin_unlock_irqrestore(&rsrc_lock, flags);
 	    return;
 	}
@@ -184,6 +209,38 @@ static int check_my_resource(resource_entry_t *list,
     return 0;
 }
 
+#ifdef CONFIG_PNP_BIOS
+int check_io_region(u_long base, u_long num)
+{
+    return check_my_resource(&io_list, base, num);
+}
+void request_io_region(u_long base, u_long num, char *name)
+{
+    register_my_resource(&io_list, base, num, name);
+}
+void release_io_region(u_long base, u_long num)
+{
+    release_my_resource(&io_list, base, num);
+}
+#ifdef HAS_PROC_BUS
+int proc_read_io(char *buf, char **start, off_t pos,
+		 int count, int *eof, void *data)
+{
+    resource_entry_t *r;
+    u_long flags;
+    char *p = buf;
+    
+    spin_lock_irqsave(&rsrc_lock, flags);
+    for (r = io_list.next; r; r = r->next)
+	p += sprintf(p, "%04lx-%04lx : %s\n", r->base,
+		     r->base+r->num-1, r->name);
+    spin_unlock_irqrestore(&rsrc_lock, flags);
+    return (p - buf);
+}
+#endif
+#endif
+
+#ifndef HAVE_MEMRESERVE
 int check_mem_region(u_long base, u_long num)
 {
     return check_my_resource(&mem_list, base, num);
@@ -196,17 +253,35 @@ void release_mem_region(u_long base, u_long num)
 {
     release_my_resource(&mem_list, base, num);
 }
+#ifdef HAS_PROC_BUS
+int proc_read_mem(char *buf, char **start, off_t pos,
+		  int count, int *eof, void *data)
+{
+    resource_entry_t *r;
+    u_long flags;
+    char *p = buf;
+    
+    spin_lock_irqsave(&rsrc_lock, flags);
+    for (r = mem_list.next; r; r = r->next)
+	p += sprintf(p, "%08lx-%08lx : %s\n", r->base,
+		     r->base+r->num-1, r->name);
+    spin_unlock_irqrestore(&rsrc_lock, flags);
+    return (p - buf);
+}
+#endif
+#endif
 
-#endif /* __LINUX__ && !HAVE_MEMRESOURCE */
+#endif /* defined(CONFIG_PNP_BIOS) || !defined(HAVE_MEMRESERVE) */
+#endif /* __LINUX__ */
+
+#ifdef __BEOS__
 
 /*======================================================================
 
-    BeOS resource management functions for memory mapped devices.
+    BeOS resource management functions
     
 ======================================================================*/
 
-#ifdef __BEOS__
-
 #include "config_manager.h"
 #include "config_manager_p.h"
 
@@ -265,25 +340,6 @@ int check_resource(int type, u_long base, u_long num)
 
 #endif /* __BEOS__ */
 
-/*====================================================================*/
-
-#if defined(HAS_PROC_BUS) && !defined(HAVE_MEMRESERVE)
-int proc_read_mem(char *buf, char **start, off_t pos,
-		  int count, int *eof, void *data)
-{
-    resource_entry_t *r;
-    u_long flags;
-    char *p = buf;
-    
-    spin_lock_irqsave(&rsrc_lock, flags);
-    for (r = mem_list.next; r; r = r->next)
-	p += sprintf(p, "%08lx-%08lx : %s\n", r->base,
-		     r->base+r->num-1, r->name);
-    spin_unlock_irqrestore(&rsrc_lock, flags);
-    return (p - buf);
-}
-#endif
-
 /*======================================================================
 
     These manage the internal databases of available resources.
@@ -301,10 +357,11 @@ static int add_interval(resource_map_t *map, u_long base, u_long num)
 	    break;
     }
     q = kmalloc(sizeof(resource_map_t), GFP_KERNEL);
+    if (!q) return CS_OUT_OF_RESOURCE;
     q->base = base; q->num = num;
     q->next = p->next; p->next = q;
-    return 0;
-} /* add_interval */
+    return CS_SUCCESS;
+}
 
 /*====================================================================*/
 
@@ -321,7 +378,7 @@ static int sub_interval(resource_map_t *map, u_long base, u_long num)
 		if (q->base+q->num <= base+num) {
 		    /* Delete whole block */
 		    p->next = q->next;
-		    kfree_s(q, sizeof(resource_map_t));
+		    kfree(q);
 		    /* don't advance the pointer yet */
 		    q = p;
 		} else {
@@ -335,6 +392,7 @@ static int sub_interval(resource_map_t *map, u_long base, u_long num)
 	    } else {
 		/* Split the block into two pieces */
 		p = kmalloc(sizeof(resource_map_t), GFP_KERNEL);
+		if (!p) return CS_OUT_OF_RESOURCE;
 		p->base = base+num;
 		p->num = q->base+q->num - p->base;
 		q->num = base - q->base;
@@ -342,8 +400,8 @@ static int sub_interval(resource_map_t *map, u_long base, u_long num)
 	    }
 	}
     }
-    return 0;
-} /* sub_interval */
+    return CS_SUCCESS;
+}
 
 /*======================================================================
 
@@ -367,7 +425,8 @@ static void do_io_probe(ioaddr_t base, ioaddr_t num)
     b = kmalloc(256, GFP_KERNEL);
     memset(b, 0, 256);
     for (i = base, most = 0; i < base+num; i += 8) {
-	if (check_region(i, 8) != 0) continue;
+	if (check_region(i, 8) || check_io_region(i, 8))
+	    continue;
 	hole = inb(i);
 	for (j = 1; j < 8; j++)
 	    if (inb(i+j) != hole) break;
@@ -379,7 +438,7 @@ static void do_io_probe(ioaddr_t base, ioaddr_t num)
 
     bad = any = 0;
     for (i = base; i < base+num; i += 8) {
-	if (check_region(i, 8) != 0)
+	if (check_region(i, 8) || check_io_region(i, 8))
 	    continue;
 	for (j = 0; j < 8; j++)
 	    if (inb(i+j) != most) break;
@@ -533,79 +592,71 @@ void validate_mem(int (*is_valid)(u_long), int (*do_cksum)(u_long),
 
     These find ranges of I/O ports or memory addresses that are not
     currently allocated by other devices.
+
+    The 'align' field should reflect the number of bits of address
+    that need to be preserved from the initial value of *base.  It
+    should be a power of two, greater than or equal to 'num'.  A value
+    of 0 means that all bits of *base are significant.  *base should
+    also be strictly less than 'align'.
     
 ======================================================================*/
 
-int find_io_region(ioaddr_t *base, ioaddr_t num, char *name)
+int find_io_region(ioaddr_t *base, ioaddr_t num, ioaddr_t align,
+		   char *name)
 {
-    ioaddr_t align;
+    ioaddr_t try;
     resource_map_t *m;
     
-    if (*base != 0) {
-	for (m = io_db.next; m != &io_db; m = m->next) {
-	    if ((*base >= m->base) && (*base+num <= m->base+m->num)) {
-		if (check_region(*base, num)) {
-		    return -1;
-		} else {
-		    request_region(*base, num, name);
-		    return 0;
-		}
-	    }
-	}
-	return -1;
-    }
-    
-    for (align = 1; align < num; align *= 2) ;
     ACQUIRE_RESOURCE_LOCK;
     for (m = io_db.next; m != &io_db; m = m->next) {
-	for (*base = (m->base + align - 1) & (~(align-1));
-	     *base+align <= m->base + m->num;
-	     *base += align)
-	    if (check_region(*base, num) == 0) {
-		request_region(*base, num, name);
+	try = (m->base & ~(align-1)) + *base;
+	for (try = (try >= m->base) ? try : try+align;
+	     (try >= m->base) && (try+num <= m->base+m->num);
+	     try += align) {
+	    if ((check_region(try, num) == 0) &&
+		(check_io_region(try, num) == 0)) {
+		*base = try;
+		request_region(try, num, name);
 		RELEASE_RESOURCE_LOCK;
 		return 0;
 	    }
+	    if (!align) break;
+	}
     }
     RELEASE_RESOURCE_LOCK;
     return -1;
-} /* find_io_region */
+}
 
-int find_mem_region(u_long *base, u_long num, char *name,
-		    u_long align, int force_low)
+int find_mem_region(u_long *base, u_long num, u_long align,
+		    int force_low, char *name)
 {
+    u_long try;
     resource_map_t *m;
 
-    if (*base != 0) {
-	for (m = mem_db.next; m != &mem_db; m = m->next) {
-	    if ((*base >= m->base) && (*base+num <= m->base+m->num))
-		if (check_mem_region(*base, num) == 0) {
-		    request_mem_region(*base, num, name);
-		    return 0;
-		}
-	}
-	return -1;
-    }
-    
     ACQUIRE_RESOURCE_LOCK;
     while (1) {
 	for (m = mem_db.next; m != &mem_db; m = m->next) {
 	    /* first pass >1MB, second pass <1MB */
 	    if ((force_low != 0) ^ (m->base < 0x100000)) continue;
-	    for (*base = (m->base + align - 1) & (~(align-1));
-		 *base+num <= m->base+m->num; *base += align)
-		if (check_mem_region(*base, num) == 0) {
-		    request_mem_region(*base, num, name);
+	    try = (m->base & ~(align-1)) + *base;
+	    for (try = (try >= m->base) ? try : try+align;
+		 (try >= m->base) && (try+num <= m->base+m->num);
+		 try += align) {
+		if (check_mem_region(try, num) == 0) {
+		    request_mem_region(try, num, name);
+		    *base = try;
 		    RELEASE_RESOURCE_LOCK;
 		    return 0;
 		}
+		if (!align) break;
+	    }
 	}
 	if (force_low) break;
 	force_low++;
     }
     RELEASE_RESOURCE_LOCK;
     return -1;
-} /* find_mem_region */
+}
 
 /*======================================================================
 
@@ -621,9 +672,9 @@ int find_mem_region(u_long *base, u_long num, char *name,
 static void fake_irq IRQ(int i, void *d, struct pt_regs *r) { }
 static inline int check_irq(int irq)
 {
-    if (REQUEST_IRQ(irq, fake_irq, 0, "bogus", NULL) != 0)
+    if (request_irq(irq, fake_irq, 0, "bogus", NULL) != 0)
 	return -1;
-    FREE_IRQ(irq, NULL);
+    free_irq(irq, NULL);
     return 0;
 }
 #endif
@@ -678,7 +729,7 @@ int try_irq(u_int Attributes, int irq, int specific)
 	}
     }
     return 0;
-} /* try_irq */
+}
 
 #endif
 
@@ -720,34 +771,35 @@ void undo_irq(u_int Attributes, int irq)
 static int adjust_memory(adjust_t *adj)
 {
     u_long base, num;
-    int i;
+    int i, ret;
 
     base = adj->resource.memory.Base;
     num = adj->resource.memory.Size;
     if ((num == 0) || (base+num-1 < base))
 	return CS_BAD_SIZE;
 
+    ret = CS_SUCCESS;
     switch (adj->Action) {
     case ADD_MANAGED_RESOURCE:
-	if (add_interval(&mem_db, base, num) != 0)
-	    return CS_IN_USE;
+	ret = add_interval(&mem_db, base, num);
 	break;
     case REMOVE_MANAGED_RESOURCE:
-	sub_interval(&mem_db, base, num);
-	for (i = 0; i < sockets; i++) {
-	    release_cis_mem(socket_table[i]);
+	ret = sub_interval(&mem_db, base, num);
+	if (ret == CS_SUCCESS) {
+	    for (i = 0; i < sockets; i++) {
+		release_cis_mem(socket_table[i]);
 #ifdef CONFIG_CARDBUS
-	    cb_release_cis_mem(socket_table[i]);
+		cb_release_cis_mem(socket_table[i]);
 #endif
+	    }
 	}
 	break;
     default:
-	return CS_UNSUPPORTED_FUNCTION;
-	break;
+	ret = CS_UNSUPPORTED_FUNCTION;
     }
     
-    return CS_SUCCESS;
-} /* adjust_mem */
+    return ret;
+}
 
 /*====================================================================*/
 
@@ -780,7 +832,7 @@ static int adjust_io(adjust_t *adj)
     }
 
     return CS_SUCCESS;
-} /* adjust_io */
+}
 
 /*====================================================================*/
 
@@ -821,7 +873,7 @@ static int adjust_irq(adjust_t *adj)
     }
 #endif
     return CS_SUCCESS;
-} /* adjust_irq */
+}
 
 /*====================================================================*/
 
@@ -842,20 +894,35 @@ int adjust_resource_info(client_handle_t handle, adjust_t *adj)
 	break;
     }
     return CS_UNSUPPORTED_FUNCTION;
-} /* adjust_resource_info */
+}
 
 /*====================================================================*/
 
 void release_resource_db(void)
 {
     resource_map_t *p, *q;
+#if defined(CONFIG_PNP_BIOS) || !defined(HAVE_MEMRESERVE)
+    resource_entry_t *u, *v;
+#endif
     
     for (p = mem_db.next; p != &mem_db; p = q) {
 	q = p->next;
-	kfree_s(p, sizeof(resource_map_t));
+	kfree(p);
     }
     for (p = io_db.next; p != &io_db; p = q) {
 	q = p->next;
-	kfree_s(p, sizeof(resource_map_t));
+	kfree(p);
     }
+#ifdef CONFIG_PNP_BIOS
+    for (u = io_list.next; u; u = v) {
+	v = u->next;
+	kfree(u);
+    }
+#endif
+#ifndef HAVE_MEMRESERVE
+    for (u = mem_list.next; u; u = v) {
+	v = u->next;
+	kfree(u);
+    }
+#endif
 }
