@@ -1,5 +1,5 @@
 /*
- * smc34c90.h 1.5 1999/07/20 16:01:32
+ * smc34c90.h 1.8 2000/06/09 21:25:03
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.1 (the "License"); you may not use this file except in
@@ -12,8 +12,19 @@
  * limitations under the License. 
  *
  * The initial developer of the original code is David A. Hinds
- * <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
- * are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+ * <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+ * are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+ *
+ * Alternatively, the contents of this file may be used under the
+ * terms of the GNU Public License version 2 (the "GPL"), in which
+ * case the provisions of the GPL are applicable instead of the
+ * above.  If you wish to allow the use of your version of this file
+ * only under the terms of the GPL and not to allow others to use
+ * your version of this file under the MPL, indicate your decision by
+ * deleting the provisions above and replace them with the notice and
+ * other provisions required by the GPL.  If you do not delete the
+ * provisions above, a recipient may use your version of this file
+ * under either the MPL or the GPL.
  */
 
 #ifndef _LINUX_SMC34C90_H
@@ -38,4 +49,10 @@
 #define SMC34C90_EEINFO_RING		0x0010
 #define SMC34C90_EEINFO_LED		0x0020
 
+#define SMC_PCIC_ID \
+    IS_SMC34C90
+
+#define SMC_PCIC_INFO \
+    { "SMC 34C90", IS_CARDBUS, ID(SMC, 34C90) }
+
 #endif /* _LINUX_SMC34C90_H */
