@@ -2,7 +2,7 @@
 
     Device driver for Databook TCIC-2 PCMCIA controller
 
-    tcic.c 1.102 1999/07/20 16:01:33
+    tcic.c 1.115 2000/05/10 19:30:15
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,14 +15,27 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
-    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+
+    Alternatively, the contents of this file may be used under the
+    terms of the GNU Public License version 2 (the "GPL"), in which
+    case the provisions of the GPL are applicable instead of the
+    above.  If you wish to allow the use of your version of this file
+    only under the terms of the GPL and not to allow others to use
+    your version of this file under the MPL, indicate your decision
+    by deleting the provisions above and replace them with the notice
+    and other provisions required by the GPL.  If you do not delete
+    the provisions above, a recipient may use your version of this
+    file under either the MPL or the GPL.
     
 ======================================================================*/
 
 #include <pcmcia/config.h>
 #include <pcmcia/k_compat.h>
 
+#include <linux/module.h>
+#include <linux/init.h>
 #include <linux/types.h>
 #include <linux/fcntl.h>
 #include <linux/string.h>
@@ -39,6 +52,7 @@
 #include <linux/timer.h>
 #include <linux/ioport.h>
 #include <linux/delay.h>
+#include <linux/pci.h>
 
 #include <pcmcia/version.h>
 #include <pcmcia/cs_types.h>
@@ -50,12 +64,15 @@
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 static const char *version =
-"tcic.c 1.102 1999/07/20 16:01:33 (David Hinds)";
+"tcic.c 1.115 2000/05/10 19:30:15 (David Hinds)";
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 #else
 #define DEBUG(n, args...)
 #endif
 
+MODULE_AUTHOR("David Hinds <dhinds@pcmcia.sourceforge.org>");
+MODULE_DESCRIPTION("Databook TCIC-2 PCMCIA socket driver");
+
 /*====================================================================*/
 
 /* Parameters that can be set with 'insmod' */
@@ -97,7 +114,7 @@ MODULE_PARM(cycle_time, "i");
 
 /*====================================================================*/
 
-static void tcic_interrupt IRQ(int irq, void *dev, struct pt_regs *regs);
+static void tcic_interrupt(int irq, void *dev, struct pt_regs *regs);
 static void tcic_timer(u_long data);
 static int tcic_service(u_int sock, u_int cmd, void *arg);
 
@@ -109,11 +126,9 @@ typedef struct socket_info_t {
     u_char	id;
 } socket_info_t;
 
-static struct timer_list poll_timer;
-static int tcic_timer_pending = 0;
-
 static int sockets;
 static socket_info_t socket_table[2];
+static struct timer_list poll_timer = { function: tcic_timer };
 
 static socket_cap_t tcic_cap = {
     /* only 16-bit cards, memory windows must be size-aligned */
@@ -125,6 +140,10 @@ static socket_cap_t tcic_cap = {
 
 /*====================================================================*/
 
+/* Trick when selecting interrupts: the TCIC sktirq pin is supposed
+   to map to irq 11, but is coded as 0 or 1 in the irq registers. */
+#define TCIC_IRQ(x) ((x) ? (((x) == 11) ? 1 : (x)) : 15)
+
 #ifdef PCMCIA_DEBUG_X
 static u_char tcic_getb(u_char reg)
 {
@@ -205,7 +224,7 @@ static int to_cycles(int ns)
 	return 0;
     else
 	return 2*(ns-14)/cycle_time;
-} /* speed_convert */
+}
 
 static int to_ns(int cycles)
 {
@@ -216,32 +235,32 @@ static int to_ns(int cycles)
 
 static volatile u_int irq_hits;
 
-static void irq_count IRQ(int irq, void *dev, struct pt_regs *regs)
+static void __init irq_count(int irq, void *dev, struct pt_regs *regs)
 {
     irq_hits++;
 }
 
-static u_int try_irq(int irq)
+static u_int __init try_irq(int irq)
 {
     u_short cfg;
 
     irq_hits = 0;
-    if (REQUEST_IRQ(irq, irq_count, 0, "irq scan", NULL) != 0)
+    if (request_irq(irq, irq_count, 0, "irq scan", irq_count) != 0)
 	return -1;
     mdelay(10);
     if (irq_hits) {
-	FREE_IRQ(irq, NULL);
+	free_irq(irq, irq_count);
 	return -1;
     }
 
     /* Generate one interrupt */
     cfg = TCIC_SYSCFG_AUTOBUSY | 0x0a00;
-    tcic_aux_setw(TCIC_AUX_SYSCFG, cfg | (irq == 11 ? 1 : irq));
+    tcic_aux_setw(TCIC_AUX_SYSCFG, cfg | TCIC_IRQ(irq));
     tcic_setb(TCIC_IENA, TCIC_IENA_ERR | TCIC_IENA_CFG_HIGH);
     tcic_setb(TCIC_ICSR, TCIC_ICSR_ERR | TCIC_ICSR_JAM);
 
     udelay(1000);
-    FREE_IRQ(irq, NULL);
+    free_irq(irq, irq_count);
 
     /* Turn off interrupts */
     tcic_setb(TCIC_IENA, TCIC_IENA_CFG_OFF);
@@ -252,17 +271,13 @@ static u_int try_irq(int irq)
     return (irq_hits != 1);
 }
 
-static u_int irq_scan(u_int mask0)
+static u_int __init irq_scan(u_int mask0)
 {
     u_int mask1;
     int i;
 
-#ifdef __alpha__
-#define PIC 0x4d0
-    /* Don't probe level-triggered interrupts -- reserved for PCI */
-    int level_mask = inb_p(PIC) | (inb_p(PIC+1) << 8);
-    if (level_mask)
-	mask0 &= ~level_mask;
+#ifdef CONFIG_PCI
+    mask0 &= ~pci_irq_mask;
 #endif
 
     mask1 = 0;
@@ -282,9 +297,9 @@ static u_int irq_scan(u_int mask0)
 	/* Fallback: just find interrupts that aren't in use */
 	for (i = 0; i < 16; i++)
 	    if ((mask0 & (1 << i)) &&
-		(REQUEST_IRQ(i, irq_count, 0, "x", NULL) == 0)) {
+		(request_irq(i, irq_count, 0, "x", irq_count) == 0)) {
 		mask1 |= (1 << i);
-		FREE_IRQ(i, NULL);
+		free_irq(i, irq_count);
 	    }
 	printk("default");
     }
@@ -307,7 +322,7 @@ static u_int irq_scan(u_int mask0)
     
 ======================================================================*/
 
-static int is_active(int s)
+static int __init is_active(int s)
 {
     u_short scf1, ioctl, base, num;
     u_char pwr, sstat;
@@ -345,7 +360,7 @@ static int is_active(int s)
     
 ======================================================================*/
 
-static int get_tcic_id(void)
+static int __init get_tcic_id(void)
 {
     u_short id;
     
@@ -358,12 +373,19 @@ static int get_tcic_id(void)
 
 /*====================================================================*/
 
-int tcic_init(void)
+static int __init init_tcic(void)
 {
     int i, sock;
     u_int mask, scan;
+    servinfo_t serv;
 
     DEBUG(0, "%s\n", version);
+    CardServices(GetCardServicesInfo, &serv);
+    if (serv.Revision != CS_RELEASE_CODE) {
+	printk(KERN_NOTICE "tcic: Card Services release "
+	       "does not match!\n");
+	return -1;
+    }
     
     printk(KERN_INFO "Databook TCIC-2 PCMCIA probe: ");
     sock = 0;
@@ -423,11 +445,6 @@ int tcic_init(void)
 	printk("Unknown ID 0x%02x", socket_table[0].id);
     }
     
-    /* Set up polling */
-    poll_timer.function = &tcic_timer;
-    poll_timer.data = 0;
-    poll_timer.prev = poll_timer.next = NULL;
-
     /* Build interrupt mask */
     printk(", %d sockets\n" KERN_INFO "  irq list (", sockets);
     if (irq_list[0] == -1)
@@ -451,7 +468,8 @@ int tcic_init(void)
 	u_int cs_mask = mask & ((cs_irq) ? (1<<cs_irq) : ~(1<<12));
 	for (i = 15; i > 0; i--)
 	    if ((cs_mask & (1 << i)) &&
-		(REQUEST_IRQ(i, tcic_interrupt, 0, "tcic", NULL) == 0))
+		(request_irq(i, tcic_interrupt, 0, "tcic",
+			     tcic_interrupt) == 0))
 		break;
 	cs_irq = i;
 	if (cs_irq == 0) poll_interval = HZ;
@@ -471,23 +489,23 @@ int tcic_init(void)
     }
     
     /* jump start interrupt handler, if needed */
-    tcic_interrupt IRQ(0, NULL, NULL);
+    tcic_interrupt(0, NULL, NULL);
 
     if (register_ss_entry(sockets, &tcic_service) != 0) {
 	printk(KERN_NOTICE "tcic: register_ss_entry() failed\n");
 	release_region(tcic_base, 16);
 	if (cs_irq != 0)
-	    FREE_IRQ(cs_irq, NULL);
+	    free_irq(cs_irq, tcic_interrupt);
 	return -ENODEV;
     }
 
     return 0;
     
-} /* tcic_init */
+} /* init_tcic */
 
 /*====================================================================*/
 
-static void tcic_finish(void)
+static void __exit exit_tcic(void)
 {
     u_long flags;
     unregister_ss_entry(&tcic_service);
@@ -495,17 +513,16 @@ static void tcic_finish(void)
     cli();
     if (cs_irq != 0) {
 	tcic_aux_setw(TCIC_AUX_SYSCFG, TCIC_SYSCFG_AUTOBUSY|0x0a00);
-	FREE_IRQ(cs_irq, NULL);
+	free_irq(cs_irq, tcic_interrupt);
     }
-    if (tcic_timer_pending)
-	del_timer(&poll_timer);
+    del_timer(&poll_timer);
     restore_flags(flags);
     release_region(tcic_base, 16);
-} /* tcic_finish */
+} /* exit_tcic */
 
 /*====================================================================*/
 
-static void tcic_interrupt IRQ(int irq, void *dev, struct pt_regs *regs)
+static void tcic_interrupt(int irq, void *dev, struct pt_regs *regs)
 {
     int i, quick = 0;
     u_char latch, sstat;
@@ -543,15 +560,15 @@ static void tcic_interrupt IRQ(int irq, void *dev, struct pt_regs *regs)
 	    events |= (latch & TCIC_SSTAT_LBAT1) ? SS_BATDEAD : 0;
 	    events |= (latch & TCIC_SSTAT_LBAT2) ? SS_BATWARN : 0;
 	}
+	DEBUG(1, "tcic: socket %d event 0x%04x\n", i, events);
 	if (events)
 	    socket_table[i].handler(socket_table[i].info, events);
     }
 
     /* Schedule next poll, if needed */
-    if (((cs_irq == 0) || quick) && (!tcic_timer_pending)) {
-	poll_timer.expires = RUN_AT(quick ? poll_quick : poll_interval);
+    if (((cs_irq == 0) || quick) && !timer_pending(&poll_timer)) {
+	poll_timer.expires = jiffies + (quick ? poll_quick : poll_interval);
 	add_timer(&poll_timer);
-	tcic_timer_pending = 1;
     }
     active = 0;
     
@@ -562,9 +579,8 @@ static void tcic_interrupt IRQ(int irq, void *dev, struct pt_regs *regs)
 static void tcic_timer(u_long data)
 {
     DEBUG(2, "tcic: tcic_timer()\n");
-    tcic_timer_pending = 0;
-    tcic_interrupt IRQ(0, NULL, NULL);
-} /* tcic_timer */
+    tcic_interrupt(0, NULL, NULL);
+}
 
 /*====================================================================*/
 
@@ -709,7 +725,7 @@ static int tcic_set_socket(u_short lsock, socket_state_t *state)
     
     tcic_setw(TCIC_ADDR, TCIC_SCF1(psock));
     scf1 = TCIC_SCF1_FINPACK;
-    scf1 |= ((state->io_irq == 11) ? 1 : state->io_irq);
+    scf1 |= TCIC_IRQ(state->io_irq);
     if (state->flags & SS_IOCARD) {
 	scf1 |= TCIC_SCF1_IOSTS;
 	if (state->flags & SS_SPKR_ENA)
@@ -723,7 +739,7 @@ static int tcic_set_socket(u_short lsock, socket_state_t *state)
     reg = TCIC_WAIT_ASYNC | TCIC_WAIT_SENSE | to_cycles(250);
     tcic_aux_setb(TCIC_AUX_WCTL, reg);
     tcic_aux_setw(TCIC_AUX_SYSCFG, TCIC_SYSCFG_AUTOBUSY|0x0a00|
-		  ((cs_irq == 11) ? 1 : cs_irq));
+		  TCIC_IRQ(cs_irq));
     
     /* Card status change interrupt mask */
     tcic_setw(TCIC_ADDR, TCIC_SCF2(psock));
@@ -775,7 +791,7 @@ static int tcic_get_io_map(u_short lsock, struct pccard_io_map *io)
     default:
 	break;
     }
-    DEBUG(1, "tcic: GetIOMap(%d, %d) = %#2.2x, %d ns, "
+    DEBUG(3, "tcic: GetIOMap(%d, %d) = %#2.2x, %d ns, "
 	  "%#4.4x-%#4.4x\n", lsock, io->map, io->flags,
 	  io->speed, io->start, io->stop);
     return 0;
@@ -789,7 +805,7 @@ static int tcic_set_io_map(u_short lsock, struct pccard_io_map *io)
     u_int addr;
     u_short base, len, ioctl;
     
-    DEBUG(1, "tcic: SetIOMap(%d, %d, %#2.2x, %d ns, "
+    DEBUG(3, "tcic: SetIOMap(%d, %d, %#2.2x, %d ns, "
 	  "%#4.4x-%#4.4x)\n", lsock, io->map, io->flags,
 	  io->speed, io->start, io->stop);
     if ((io->map > 1) || (io->start > 0xffff) || (io->stop > 0xffff) ||
@@ -857,7 +873,7 @@ static int tcic_get_mem_map(u_short lsock, struct pccard_mem_map *mem)
     mem->flags |= (ctl & TCIC_MCTL_WP) ? MAP_WRPROT : 0;
     mem->speed = to_ns(ctl & TCIC_MCTL_WSCNT_MASK);
     
-    DEBUG(1, "tcic: GetMemMap(%d, %d) = %#2.2x, %d ns, "
+    DEBUG(3, "tcic: GetMemMap(%d, %d) = %#2.2x, %d ns, "
 	  "%#5.5lx-%#5.5lx, %#5.5x\n", lsock, mem->map, mem->flags,
 	  mem->speed, mem->sys_start, mem->sys_stop, mem->card_start);
     return 0;
@@ -871,7 +887,7 @@ static int tcic_set_mem_map(u_short lsock, struct pccard_mem_map *mem)
     u_short addr, ctl;
     u_long base, len, mmap;
 
-    DEBUG(1, "tcic: SetMemMap(%d, %d, %#2.2x, %d ns, "
+    DEBUG(3, "tcic: SetMemMap(%d, %d, %#2.2x, %d ns, "
 	  "%#5.5lx-%#5.5lx, %#5.5x)\n", lsock, mem->map, mem->flags,
 	  mem->speed, mem->sys_start, mem->sys_stop, mem->card_start);
     if ((mem->map > 3) || (mem->card_start > 0x3ffffff) ||
@@ -927,33 +943,10 @@ static subfn_t service_table[] = {
 
 static int tcic_service(u_int lsock, u_int cmd, void *arg)
 {
-    int err;
-
-    DEBUG(2, "tcic_service(%d, %d, 0x%p)\n", lsock, cmd, arg);
-
-    if (cmd < NFUNC)
-	err = service_table[cmd](lsock, arg);
-    else
-	err = -EINVAL;
-
-    return err;
-} /* tcic_service */
+    return (cmd < NFUNC) ? service_table[cmd](lsock, arg) : -EINVAL; 
+}
 
 /*====================================================================*/
 
-int init_module(void)
-{
-    servinfo_t serv;
-    CardServices(GetCardServicesInfo, &serv);
-    if (serv.Revision != CS_RELEASE_CODE) {
-	printk(KERN_NOTICE "tcic: Card Services release "
-	       "does not match!\n");
-	return -1;
-    }
-    return tcic_init();
-}
-
-void cleanup_module(void)
-{
-    tcic_finish();
-}
+module_init(init_tcic);
+module_exit(exit_tcic);
