@@ -1,5 +1,5 @@
 /*
- * ti113x.h 1.13 1999/07/20 16:01:34
+ * ti113x.h 1.22 2000/06/09 21:26:14
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.1 (the "License"); you may not use this file except in
@@ -12,8 +12,19 @@
  * limitations under the License. 
  *
  * The initial developer of the original code is David A. Hinds
- * <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
- * are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+ * <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+ * are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+ *
+ * Alternatively, the contents of this file may be used under the
+ * terms of the GNU Public License version 2 (the "GPL"), in which
+ * case the provisions of the GPL are applicable instead of the
+ * above.  If you wish to allow the use of your version of this file
+ * only under the terms of the GPL and not to allow others to use
+ * your version of this file under the MPL, indicate your decision by
+ * deleting the provisions above and replace them with the notice and
+ * other provisions required by the GPL.  If you do not delete the
+ * provisions above, a recipient may use your version of this file
+ * under either the MPL or the GPL.
  */
 
 #ifndef _LINUX_TI113X_H
@@ -26,12 +37,12 @@
 #ifndef PCI_DEVICE_ID_TI_1130
 #define PCI_DEVICE_ID_TI_1130		0xac12
 #endif
-#ifndef PCI_DEVICE_ID_TI_1131
-#define PCI_DEVICE_ID_TI_1131		0xac15
-#endif
 #ifndef PCI_DEVICE_ID_TI_1031
 #define PCI_DEVICE_ID_TI_1031		0xac13
 #endif
+#ifndef PCI_DEVICE_ID_TI_1131
+#define PCI_DEVICE_ID_TI_1131		0xac15
+#endif
 #ifndef PCI_DEVICE_ID_TI_1250A
 #define PCI_DEVICE_ID_TI_1250A		0xac16
 #endif
@@ -44,18 +55,27 @@
 #ifndef PCI_DEVICE_ID_TI_1210
 #define PCI_DEVICE_ID_TI_1210		0xac1a
 #endif
-#ifndef PCI_DEVICE_ID_TI_1251A
-#define PCI_DEVICE_ID_TI_1251A		0xac1d
-#endif
-#ifndef PCI_DEVICE_ID_TI_1251B
-#define PCI_DEVICE_ID_TI_1251B		0xac1f
-#endif
 #ifndef PCI_DEVICE_ID_TI_1450
 #define PCI_DEVICE_ID_TI_1450		0xac1b
 #endif
 #ifndef PCI_DEVICE_ID_TI_1225
 #define PCI_DEVICE_ID_TI_1225		0xac1c
 #endif
+#ifndef PCI_DEVICE_ID_TI_1251A
+#define PCI_DEVICE_ID_TI_1251A		0xac1d
+#endif
+#ifndef PCI_DEVICE_ID_TI_1211
+#define PCI_DEVICE_ID_TI_1211		0xac1e
+#endif
+#ifndef PCI_DEVICE_ID_TI_1251B
+#define PCI_DEVICE_ID_TI_1251B		0xac1f
+#endif
+#ifndef PCI_DEVICE_ID_TI_1410
+#define PCI_DEVICE_ID_TI_1410		0xac50
+#endif
+#ifndef PCI_DEVICE_ID_TI_1420
+#define PCI_DEVICE_ID_TI_1420		0xac51
+#endif
 
 /* Register definitions for TI 113X PCI-to-CardBus bridges */
 
@@ -84,6 +104,7 @@
 
 #define  TI122X_SCR_SER_STEP		0xc0000000
 #define  TI122X_SCR_INTRTIE		0x20000000
+#define  TI122X_SCR_P2CCLK		0x08000000
 #define  TI122X_SCR_CBRSVD		0x00400000
 #define  TI122X_SCR_MRBURSTDN		0x00008000
 #define  TI122X_SCR_MRBURSTUP		0x00004000
@@ -159,5 +180,33 @@
 /* ExCA IO offset registers */
 #define TI113X_IO_OFFSET(map)		(0x36+((map)<<1))
 
-#endif /* _LINUX_TI113X_H */
+/* Data structure for tracking vendor-specific state */
+typedef struct ti113x_state_t {
+    u_int		sysctl;		/* TI113X_SYSTEM_CONTROL */
+    u_char		cardctl;	/* TI113X_CARD_CONTROL */
+    u_char		devctl;		/* TI113X_DEVICE_CONTROL */
+    u_char		diag;		/* TI1250_DIAGNOSTIC */
+} ti113x_state_t;
 
+#define TI_PCIC_ID \
+    IS_TI1130, IS_TI1131, IS_TI1250A, IS_TI1220, IS_TI1221,	\
+    IS_TI1210, IS_TI1251A, IS_TI1251B, IS_TI1450, IS_TI1225,	\
+    IS_TI1211, IS_TI1420, IS_TI1031, IS_TI1410
+
+#define TI_PCIC_INFO \
+    { "TI 1130",  IS_TI|IS_CARDBUS, ID(TI, 1130) }, \
+    { "TI 1131",  IS_TI|IS_CARDBUS, ID(TI, 1131) }, \
+    { "TI 1250A", IS_TI|IS_CARDBUS, ID(TI, 1250A) }, \
+    { "TI 1220",  IS_TI|IS_CARDBUS, ID(TI, 1220) }, \
+    { "TI 1221",  IS_TI|IS_CARDBUS, ID(TI, 1221) }, \
+    { "TI 1210",  IS_TI|IS_CARDBUS, ID(TI, 1210) }, \
+    { "TI 1251A", IS_TI|IS_CARDBUS, ID(TI, 1251A) }, \
+    { "TI 1251B", IS_TI|IS_CARDBUS, ID(TI, 1251B) }, \
+    { "TI 1450",  IS_TI|IS_CARDBUS, ID(TI, 1450) }, \
+    { "TI 1225",  IS_TI|IS_CARDBUS, ID(TI, 1225) }, \
+    { "TI 1211",  IS_TI|IS_CARDBUS, ID(TI, 1211) }, \
+    { "TI 1420",  IS_TI|IS_CARDBUS, ID(TI, 1420) }, \
+    { "TI 1031",  IS_TI|IS_CARDBUS, ID(TI, 1031) }, \
+    { "TI 1410",  IS_TI|IS_CARDBUS, ID(TI, 1410) }
+
+#endif /* _LINUX_TI113X_H */
