@@ -1,5 +1,5 @@
 /*
- * yenta.h 1.14 1999/07/20 16:01:36
+ * yenta.h 1.18 2000/02/06 06:57:49
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.1 (the "License"); you may not use this file except in
@@ -12,8 +12,19 @@
  * limitations under the License. 
  *
  * The initial developer of the original code is David A. Hinds
- * <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
- * are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+ * <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+ * are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
+ *
+ * Alternatively, the contents of this file may be used under the
+ * terms of the GNU Public License version 2 (the "GPL"), in which
+ * case the provisions of the GPL are applicable instead of the
+ * above.  If you wish to allow the use of your version of this file
+ * only under the terms of the GPL and not to allow others to use
+ * your version of this file under the MPL, indicate your decision by
+ * deleting the provisions above and replace them with the notice and
+ * other provisions required by the GPL.  If you do not delete the
+ * provisions above, a recipient may use your version of this file
+ * under either the MPL or the GPL.
  */
 
 #ifndef _LINUX_YENTA_H
@@ -81,6 +92,7 @@
 
 #define CB_SOCKET_EVENT			0x0000
 #define  CB_SE_CSTSCHG			0x00000001
+#define  CB_SE_CCD			0x00000006
 #define  CB_SE_CCD1			0x00000002
 #define  CB_SE_CCD2			0x00000004
 #define  CB_SE_PWRCYCLE			0x00000008
@@ -92,6 +104,7 @@
 
 #define CB_SOCKET_STATE			0x0008
 #define  CB_SS_CSTSCHG			0x00000001
+#define  CB_SS_CCD			0x00000006
 #define  CB_SS_CCD1			0x00000002
 #define  CB_SS_CCD2			0x00000004
 #define  CB_SS_PWRCYCLE			0x00000008
@@ -105,6 +118,7 @@
 #define  CB_SS_3VCARD			0x00000800
 #define  CB_SS_XVCARD			0x00001000
 #define  CB_SS_YVCARD			0x00002000
+#define  CB_SS_VSENSE			0x00003c86
 #define  CB_SS_5VSOCKET			0x10000000
 #define  CB_SS_3VSOCKET			0x20000000
 #define  CB_SS_XVSOCKET			0x40000000
